/*********************************************************************
*
*    [12.VIII.86]
*
*    wwx.c-- demo windowing application.
*
*    This is a semi-deluxe shell program intended mainly as a guide
*         to window manipulations. Multiple windows and buffered
*         draw and redraw operations are demonstrated. All three
*         resolutions are supported. The program has been written so
*         as to allow relatively painless customization by the pro-
*         grammer. If the modularization seems at times excessive, the
*         user is reminded that the program was intended partly to
*         be a learning aid (for the author too!). There is plenty of 
*         room for optimization.
*
*    To add more windows, increase NR_WWS, write an initial draw func-
*         tion for the window, &c. The code will show how the window
*         structures are initialized and the windows managed.
*
*    I have tried to maintain a rough correspondence with Atari's
*         "apskel.c" program, since it is so well known. The flow of
*         control in "wwx.c" is essentially the same as in "apskel".
*         Some of the variable names also are the same.
*
*    The menu (.RSC) file was created with the Resource Construction
*         Set. The function 'make_menu()' loads this file and then
*         displays the menu.  Most of the menu is a dummy-- it is 
*         used mainly to allow access to the desk accessories and thus
*         demonstrate this program's ability to handle various window
*         tasks. Note that the resource file is expected to be in the
*         default drive. You can change this by changing the filename
*         string (#define'd as RSRC_FILE, below). The resource file is
*         called "wwmenu.rsc", and it also is included on this disk.
*
*    As a dubious bonus, the rudiments of a miserable text editor
*         are also included (cf. 'write_stuff()'). But this function
*         too is included chiefly to help demonstrate the window redraw
*         functions.
*
*    This shell can be easily transformed into a desk accessory by
*         making the usual changes for accessory open and close mes-
*         sages from the AES. However, I have not been able so far to
*         use multiple windows in a desk accessory. In other words,
*         NR_WWS must have the value 1 (one) in a desk accessory ver-
*         sion.
*         
*
*    This version compiled under Megamax C.
*    Ignore the compiler warnings about pointer type mismatches
*         in calls to 'malloc()'.
*
*    For DRI (Alcyon) C, the main change will be simply to declare
*         a 2-dimensional array 'screen' whose members will each
*         have a size of 32K bytes (Megamax doesn't allow this).
*         Otherwise everything should stay the same, unless some of
*         the '#include' files have different names. I have heard
*         rumors of problems with structure offsets in the unrevised
*         version of Alcyon, but I had no trouble with the earlier 
*         versions of this program, before Megamax appeared. However,
*         I was never able to successfully link a desk accessory version
*         of "wwx" under Alcyon, which is one (but not the only) reason
*         why I switched to Megamax.
*              
*
**********************************************************************/



/*********************         
*
* include files--    
*
*                                                 
*********************/

#include <obdefs.h>      /* common object definitions & structures     */
#include <gemdefs.h>     /* common GEM definitions & misc. structures  */
#include <stdio.h>       /* for 'malloc()'                             */
#include <osbind.h>      /* for BIOS &c. calls                         */
#include <ctype.h>       /* char-processing macros                     */

/********************
*
* defines-- (mostly) my own.    
*
*
*********************/
#define TRUE 1
#define FALSE 0

          /* main title and subtitle for main window   */
#define TOP_TITLE " WWX: ver = 1.0  //  12.VIII.86 "
#define SUB_TITLE " Multiple windows, &c.: Demo     "
#define WI2_TITLE " wwx: second window // 12.VIII.86 "

          /* max title & subtitle string length   */
          /* not currently used                   */
#define TTL_LEN 60

          /* used to set window attributes        */
#define WI1_KIND        (SIZER|MOVER|FULLER|CLOSER|NAME|INFO)
#define WI2_KIND        (SIZER|MOVER|FULLER|CLOSER|NAME)

          /* used to test GEM function return     */
#define NO_WINDOW (-1)

          /* following are used to prevent window 'underflow'     */
#define MIN_WIDTH  (2*gl_wbox_G)
#define MIN_HEIGHT (3*gl_hbox_G)

#define SCRN_SIZE 32768
          /* ... i.e., roughly 32k --for screen buffer            */

#define RES_NR 3
          /* ... nr of possible resolutions                       */
#define ONE 1
          /* ... handy indivisible graphics constant              */
#define STD 1
          /* ... type of coordinates used                         */

#define NR_WWS 2
          /* ...i.e., nr of windows allowed  */

          /* menu-related definitions:       */
#define RSRC_FILE "wwmenu.rsc"
#define MSUB_TITLE " menu title ix ==     / menu item ix ==       "
#define SUB_TF_1 18
#define SUB_TF_2 40

          /* for 'pr_menu()' debug msgs      */
#define W_TITLE " ww handle ==         "
#define W_TF_1 14

/*********************
*
* externals-- 
*
*
*********************/

extern int      gl_apid;   /* used by accessory, set by call to      */
                           /*   'appl_init()'; label is internal.    */
                           /* not used by application                */

/*********************
*
* system variables--
*
*********************/
int     contrl[12];
int     intin[128];
int     ptsin[128];
int     intout[128];
int     ptsout[128];        /* used to pass args by c bindings      */
                            /* == extern variables                  */

/*********************
*
* global variables-- identified by '_G' suffix.
*
*  
*********************/

int     gl_hchar_G;
int     gl_wchar_G;
int     gl_wbox_G;
int     gl_hbox_G;            /* ...system sizes                    */
int     screenx_G, screeny_G; /* defines top edge of screen buffer  */

int     phys_handle_G;  /* physical workstation handle              */
int     handle_G;       /* virtual workstation handle               */
int     top_window_G;   /* handle of topped window                  */

int     xdesk_G,ydesk_G,hdesk_G,wdesk_G;
                        /* dimensions of desktop window             */

int     msgbuff_G[8];   /* event message buffer                     */
int     butdown_G;      /* button state tested for, UP/DOWN         */
int     ret_G;          /* dummy return variable                    */

int     hidden_G;       /* current state of cursor                  */

int     work_in_G[11];      /* input to GEM parameter array         */
int     work_out_G[57];     /* output from GEM parameter array      */

int     d1_G,d2_G,d3_G,d4_G,d5_G,d6_G;  /* dummy vars               */

OBJECT  wwmenu_G; /* menu object from resource file                 */

          /* following are arrays containing values for each of     */
          /*   three possible resolutions                           */
int     heights_G[RES_NR] = {200,200,400};
          /* ...pixel heights of screen                             */
int     widths_G[RES_NR] = {320,640,640};
          /* ...pixel widths of screen                              */
int     planes_G[RES_NR] = {4,2,1};
          /* ...number of bit planes                                */

          /* following variables are loaded by 'fix_res()' with the */
          /*   correct values from the above arrays                 */
int     r_height_G;      /* pixel height of screen this resolution  */
int     r_width_G;       /* pixel width of screen this resolution   */
int     r_planes_G;      /* number of bit planes this resolution    */

int     main_ww_G;
          /* ... index (into 'ww[]') of program's main window       */

/************************
*
* structure definitions--
*
************************/
/* values are filled in for each window by 'prep_wind()' and by     */
/*   'open_window()'                                                */
struct  w_pack {            /* window data package                  */
        int wi_handle;      /* window handle                        */
        int fulled;         /* current state of window              */
        int xold,yold,hold,wold;      /* prev dim of window         */ 
        FDB wind_buff;      /* buffer to save window's contents     */
        int xwork,ywork,hwork,wwork;  /* work dim of window         */
        int w_kind;         /* first arg to 'wind_create()'         */
        int max_dims[4];    /* window's max dimensions              */
        int st_dims[4];     /* window's initial dimensions          */
        char t;             /* TRUE == window has title             */
        char *title;        /* title string, if any                 */
        char s;             /* TRUE == window has subtitle          */
        char *subtl;        /* subtitle string, if any              */
        int closed;         /* TRUE == window is closed             */
        };   


/*****
*
* main-- top execution level of application: performs initialization.
*
*****/
main()
{
     struct w_pack ww[NR_WWS]; /* window data-- one for each    */
     FDB wind_actv;       /* mfdb for screen                    */
     int raz_pts[8];      /* source/dest coord packet for       */
                          /*  raster functions                  */
     int *screen[NR_WWS]; /* pointers to window buffers         */
     OBJECT *wwmp;        /* points to menu tree                */
     OBJECT *make_menu(); /* returns the above                  */
     int draw_ww1();      /* declared here to access pointer    */
     int draw_ww2();      /* ditto                              */

     /* initialize GEM-- note that 'gl_apid' is not used by     */
     /*    application                                          */
     gl_apid = appl_init();

     /* initialize AES   */
     phys_handle_G = graf_handle(&gl_wchar_G,&gl_hchar_G,&gl_wbox_G,
                               &gl_hbox_G);

     /* allocate screen buffer space    */
     screen[0] = malloc(SCRN_SIZE);
     screen[1] = malloc(SCRN_SIZE); 

     /* return coordinates of work area of desktop window   */
     wind_get(0, WF_WORKXYWH, &xdesk_G, &ydesk_G, &wdesk_G, &hdesk_G);

     /* initialize VDI, &c.   */
     open_vwork();

     /* set up resolution-dependent variables     */
     fix_res();

     /* initialize screen raster structure    */
     i_scrnras(r_planes_G,&wind_actv);

     /* This is where the indices (into 'ww[]') for the various    */     
     /*   windows are determined. Note that the last window opened */
     /*   will automatically be the 'active' window at program     */
     /*   start-up. The global variable 'main_ww_G' is important:  */
     /*   when the window with that index is closed, the other     */
     /*   windows are also closed and the program ends.            */
     /* Please note also that we are concerned only with the ap-   */
     /*   plication's own windows here-- not with those of a       */
     /*   possibly active accessory or with the desktop's.         */

     /* initialize startup structure for first window  */
     prep_wind(&ww[0],WI1_KIND,xdesk_G,ydesk_G,wdesk_G,hdesk_G,
               xdesk_G,ydesk_G,wdesk_G,hdesk_G,TRUE,TOP_TITLE,
               TRUE,SUB_TITLE);

     /* establish main window == nr 0   */
     main_ww_G = 0;

     /* initialize second window structure             */
     /* note how values are expressed in resolution-independent form  */
     prep_wind(&ww[1],WI2_KIND,xdesk_G + (r_width_G / 3), ydesk_G,
               wdesk_G - (r_width_G / 3), hdesk_G,
               xdesk_G + (r_width_G / 3), ydesk_G, 
               wdesk_G - (r_width_G / 3),
               hdesk_G, TRUE,WI2_TITLE,FALSE," ");

     /* initialize screen variables     */
     init_svars();

     /* open first window      */     
     open_window(&ww[0],&wind_actv,raz_pts,screen[0],r_planes_G,
          draw_ww1);

     /* open second window     */
     open_window(&ww[1],&wind_actv,raz_pts,screen[1],r_planes_G,
          draw_ww2);

     /* set up menu          */
     wwmp = make_menu();

     /* set the mouse shape   */
     graf_mouse(ARROW,(long)0x0);

     /* mouse is hidden until after first redraw       */
     hidden_G = TRUE;    
     butdown_G = TRUE;

     /* now go wait for messages; i.e., do the main business of   */
     /*   the program                                             */
     gem_loop(ww,&wind_actv,raz_pts,wwmp);
}


/*****
*
* gem_loop-- main execution level of application.
*
*****/
int gem_loop(ww,wind_actv,raz_pts,wwmp)
struct w_pack ww[];      /* data for windows                   */
FDB *wind_actv;          /* raster info for screen             */
int raz_pts[];           /* raster source/dest coords          */
OBJECT *wwmp;            /* pointer to menu objects            */
{
     int event;          /* combined returned event bit fields */
     int keycode;        /* keycode returned by event-keyboard */
     int mx,my;          /* mouse x and y pos.                 */
     int ww_ix;          /* window index                       */
     int i;              /* index variable                     */
     int stop_it;        /* TRUE == user closed main window    */

     event = 0;
     stop_it = FALSE;
     ww_ix = NO_WINDOW;
     do 
     {
          /* check all windows     */
          for (i = 0; (i < NR_WWS) && !stop_it; i++)
          {

               /* check for mouse button and/or keyboard events,       */
               /*    as well as GEM window, &c., messages              */
               event = evnt_multi(MU_MESAG | MU_BUTTON | MU_KEYBD,
                              1,1,butdown_G, /* button parms           */
                              0,0,0,0,0, /* 1st rect parms             */
                              0,0,0,0,0, /* 2nd rect parms             */
                              msgbuff_G, /* message pipe buffer        */
                              0,0,       /* timer low and high words   */      
                              &mx,&my,   /* mouse x/y coords at event  */
                              &ret_G,    /* state of buttons at event  */
                              &ret_G,    /* state of certain keys      */
                              &keycode,  /* code of pressed key        */
                              &ret_G);   /* nr of mouse key states     */


               /* get index of window AES is sending message to   */
               if (event & (MU_MESAG | MU_KEYBD | MU_BUTTON))
                    ww_ix = next_struct(ww,msgbuff_G[3]);

               /* check for active window    */
               wind_get(d1_G,WF_TOP,&top_window_G,&d2_G,&d3_G,&d4_G);
/*               pr_window(&ww[0],top_window_G);
*/
               /* process a menu event  */
               if (msgbuff_G[0] == 10)
                    pr_menu(wwmp,ww[0].wi_handle);

               /* freeze the AES prior to fooling with windows  */          
               wind_update(TRUE);

               /* process window management messages    */
               if (event & MU_MESAG)
               {
                    /* interpret AES message     */          
                    switch (msgbuff_G[0]) 
                    {

                         case WM_REDRAW:
                              do_redraw(&ww[ww_ix],msgbuff_G[4],
                                   msgbuff_G[5],msgbuff_G[6],msgbuff_G[7],
                                   wind_actv,raz_pts);
                              show_mouse();
                              break;

                         case WM_NEWTOP:
                         case WM_TOPPED:
                              wind_set(ww[ww_ix].wi_handle,WF_TOP,0,0,0,0);
                              top_window_G = ww[ww_ix].wi_handle;
                              break;

                         case WM_SIZED:
                         case WM_MOVED:
                              do_resize(&ww[ww_ix],
                                        msgbuff_G[0],wind_actv,raz_pts);
                              break;

                         case WM_FULLED:
                              if (ww[ww_ix].fulled)
                                   do_unfull(&ww[ww_ix]);
                              else
                                   do_full(&ww[ww_ix]);
                              break;

                         case WM_CLOSED:
                              if ((ww_ix != main_ww_G) && 
                                  (msgbuff_G[3] == ww[ww_ix].wi_handle))
                                   close_window(&ww[ww_ix]);
                              else if (ww_ix == main_ww_G)
                                   stop_it = TRUE;

                    } /* switch (msgbuff_G[0]) */

               }    /* if (event & MU_MESAG)   */

               /* toggle mouse button state for 'evnt_multi()'   */
               if (event & MU_BUTTON)
                    butdown_G = !butdown_G;

               /* process keyboard input     */
               if (event & MU_KEYBD)
               {
                    write_stuff(&ww[next_struct(ww,top_window_G)],mx,my,
                                keycode,wind_actv,raz_pts,wwmp);
               }

               /* unfreeze AES     */        
               wind_update(FALSE);
               show_mouse();

          }  /* for loop   */

     }
     /* do the above until we get message that user clicked on  */
     /*   close box of main window                              */
     while (!((event & MU_MESAG) && (msgbuff_G[0] == WM_CLOSED) && 
               (ww_ix == main_ww_G)));

     /* make sure all windows are closed before leaving         */
     for (i = 0; i < NR_WWS; i++)
          if (!ww[i].closed)
               close_window(&ww[i]);

     /* shut down VDI    */
     v_clsvwk(handle_G);

     /* exit GEM         */
     appl_exit();

}


/*****
*
* hide_mouse-- hide the mouse, if it's visible.
*
*****/
int hide_mouse()
{            
     if (!hidden_G)
     {    
          graf_mouse(M_OFF,(long)0x0);
          hidden_G = TRUE;
     }
}

/*****
*
* show_mouse-- show the mouse, if it's hidden.
*
*****/
int show_mouse()
{
     if (hidden_G)
     {
          graf_mouse(M_ON,(long)0x0);
          hidden_G = FALSE;
     }
}

/*****
*
* open_vwork-- initialize the VDI.
*
*****/
int open_vwork()
{
     int i;    /* index variable   */

     for (i = 0; i < 10; work_in_G[i++] = 1)
          ;
     work_in_G[10] = 2;
     handle_G = phys_handle_G;
     v_opnvwk(work_in_G,&handle_G,work_out_G);
}


/*****
*
* set_clip-- set clipping rectangle for draw or redraw, and set up
*            coordinates for raster operation.
*
*****/
int set_clip(ww,x,y,w,h,redraw,raz_pts,wind_actv)
struct w_pack *ww;  /* window data                                  */
int x,y,w,h;        /* dimensions to be transformed into clip rect  */
int redraw;         /* TRUE => this is preparatory to a redraw      */
int raz_pts[];      /* source/dest coords for raster operation      */
FDB *wind_actv;     /* raster structure for screen                  */
{
     int clip[4];   /* coordinates of clipping rectangle            */

     /* find the region in the window buffer corresponding   */
     /*    to the area to be redrawn in this window          */
     wind_get(ww->wi_handle,WF_WORKXYWH,&(ww->xwork),&(ww->ywork),
              &(ww->wwork),&(ww->hwork));

     /* truncate dimensions extending beyond screen edge   */
     if ((x + w) > r_width_G)
          w = r_width_G - x;
     if ((y + h) > r_height_G)
          h = r_height_G - y;

     /* calculate rectangle for 'vs_clip()'  */
     clip[0] = x;
     clip[1] = y;
     clip[2] = x + w - ONE;    
     clip[3] = y + h - ONE;

     /* calculate coords for raster operation  */   
     if (redraw)
     {
          /* buffer == source      */
          raz_pts[0] = screenx_G + x - ww->xwork;      
          raz_pts[1] = screeny_G + y - ww->ywork;      
          raz_pts[2] = (raz_pts[0] + w - ONE);    
          raz_pts[3] = (raz_pts[1] + h - ONE);    
          raz_pts[4] = clip[0];
          raz_pts[5] = clip[1];
          raz_pts[6] = clip[2];    
          raz_pts[7] = clip[3];
   
     }                               
     else                           
     {                             
          /* buffer == destination */
          raz_pts[0] = clip[0];
          raz_pts[1] = clip[1];
          raz_pts[2] = clip[2];             
          raz_pts[3] = clip[3];         
          raz_pts[4] = screenx_G;
          raz_pts[5] = screeny_G;
          raz_pts[6] = (screenx_G + w - ONE);  
          raz_pts[7] = (screeny_G + h - ONE);  
     }         

     /* set clip rectangle for GEM      */
     vs_clip(handle_G,1,clip);
}


/*****
*
* open_window-- set up and open window.
*
*****/
int open_window(ww,wind_actv,raz_pts,base,res,w_init)
struct w_pack *ww;  /* window data                         */
FDB *wind_actv;     /* raster info for screen              */
int raz_pts[];      /* srce/dest raster coords             */
long base;          /* address of window buffer            */
int res;            /* nr of bit planes this res           */
int (*w_init)();    /* pointer to window startup routine   */
{

     /* set window's max size   */
     ww->wi_handle = wind_create(ww->w_kind,ww->max_dims[0],
                                 ww->max_dims[1],ww->max_dims[2],
                                 ww->max_dims[3]);

     /* set the window's title                                  */
     if (ww->t)
          wind_set(ww->wi_handle, WF_NAME, ww->title,0,0);

     /* set subtitle                                            */
     if (ww->s)
          wind_set(ww->wi_handle, WF_INFO, ww->subtl,0,0);

     /* draw an expanding box outline preparatory to drawing    */
     /*   the actual window                                     */
     graf_growbox(ww->st_dims[0] + ww->st_dims[2] / 2,
                  ww->st_dims[1] + ww->st_dims[3] / 2,
                  gl_wbox_G,gl_hbox_G,
                  ww->st_dims[0],ww->st_dims[1],ww->st_dims[2],
                  ww->st_dims[3]);

     /* draw window for first time                              */
     wind_open(ww->wi_handle,ww->st_dims[0],ww->st_dims[1],ww->st_dims[2],
               ww->st_dims[3]);

     /* get the dimensions of the window's working area         */
     wind_get(ww->wi_handle,WF_WORKXYWH,&(ww->xwork),&(ww->ywork),
              &(ww->wwork),&(ww->hwork));
     ww->fulled = FALSE;    /* because window has no 'old' dimensions  */
     ww->closed = FALSE;    /* flag for general window closer at end   */

     /* initialize window buffer   */
     ww->wind_buff.fd_addr = base;
     ww->wind_buff.fd_w = r_width_G;
     ww->wind_buff.fd_h = r_height_G;
     ww->wind_buff.fd_wdwidth = r_width_G / 16;
     ww->wind_buff.fd_stand = STD;
     ww->wind_buff.fd_nplanes = res;

     /* draw window first time and copy it into buffer */
     /* window is automatically redrawn at first entry into            */
     /*   'gem_loop()', so hide the mouse until the redraw is complete */
     hide_mouse();
     set_clip(ww,ww->xwork,ww->ywork,ww->wwork,ww->hwork,FALSE,raz_pts,
              wind_actv);
     (*w_init)(ww);    /* initial draw routine    */
     vro_cpyfm(handle_G, S_ONLY, raz_pts, wind_actv, &(ww->wind_buff));

}


/*****
*
* close_window-- close and delete a window.
*
*****/
int close_window(ww)
struct w_pack *ww;       /* window data      */
{
     wind_close(ww->wi_handle);
     graf_shrinkbox(ww->xwork + ww->wwork / 2, ww->ywork + ww->hwork / 2,
                    gl_wbox_G, gl_hbox_G, ww->xwork, ww->ywork,
                    ww->wwork, ww->hwork);
     wind_delete(ww->wi_handle);
     ww->closed = TRUE;

     return;
}


/*****
*
* do_redraw-- redraw the application window.
*
*****/
int do_redraw(ww,xc,yc,wc,hc,wind_actv,raz_pts)
struct w_pack *ww;  /* window data                                */
int xc,yc,wc,hc;    /* msgbuff_G[4..7]: dimensions of area to be  */
                    /*  redrawn                                   */
FDB *wind_actv;     /* raster info for screen                     */
int raz_pts[];      /* srce/dest coords for raster                */
{
     GRECT t1,t2;   /* from 'obdefs.h': holds rect dimensions     */
                    /* t1 = dimensions of rectangle from rect-    */
                    /*      angle list                            */
                    /* t2 = dimensions of area to be redrawn,     */
                    /*      from parms                            */

     hide_mouse();
     t2.g_x = xc;   /* transfer parms (rather clumsily-- but      */
     t2.g_y = yc;   /*   the structures are required by           */
     t2.g_w = wc;   /*   'rc_intersect()' below, and the args as  */
     t2.g_h = hc;   /*   passed are elements of 'msgbuff[]')      */          

     /* get the first rectangle from rectangle list    */
     wind_get(ww->wi_handle,WF_FIRSTXYWH,&t1.g_x,&t1.g_y,&t1.g_w,&t1.g_h);

     /* keep redrawing while there are valid rectangles in the list  */
     while (t1.g_w && t1.g_h) 
     {
          /* if the rectangles intersect, redraw that area           */
          if (rc_intersect(&t2,&t1)) 
          {
               set_clip(ww,t1.g_x,t1.g_y,t1.g_w,t1.g_h,TRUE,raz_pts,
                        wind_actv);
               
               /* raster from buffer into current window    */
               vro_cpyfm(handle_G,S_ONLY,raz_pts,&(ww->wind_buff),
                           wind_actv);
               
          }
          /* get the next rectangle from the list      */
          wind_get(ww->wi_handle,WF_NEXTXYWH,&t1.g_x,&t1.g_y,&t1.g_w,
                   &t1.g_h);
     }
     show_mouse();
}


/*****
*
* draw_ww1-- initial draw, window 1.
*
*****/
int draw_ww1(ww)
struct w_pack *ww;       /* window data      */
{
     int temp[4];        /* coords for drawing sample shape  */

     /* style = pattern  */
     vsf_interior(handle_G,2);

     /* style + style_index => solid    */
     vsf_style(handle_G,12);

     /* color = white    */
     vsf_color(handle_G,0);

     temp[0] = ww->xwork;
     temp[1] = ww->ywork;
     temp[2] = ww->xwork + ww->wwork - ONE;
     temp[3] = ww->ywork + ww->hwork - ONE;

     /* blank the interior  */
     v_bar(handle_G,temp);

     /* now set up for drawing shape    */
     /* note resolution-independent expressions   */
     temp[0] = ww->xwork + (r_width_G / 10);      
     temp[2] = ww->xwork + ww->wwork - (r_width_G / 10 + ONE);  

     /* color = black    */
     vsf_color(handle_G,1);
     v_bar(handle_G,temp);

}


/****
*
* write_stuff-- or, there must be a better way to do this.
*               But so far...
*
*               Much of the AES-driven code was hit upon by trial and 
*                   error. I'm sure there is a definitive way to do 
*                   this sort of thing but it probably is documented 
*                   only on cuneiform tablets kept under lock and key 
*                   at DRI. 
*
****/
int write_stuff(ww,x,y,ch,wind_actv,raz_pts,wwmp)
struct w_pack *ww;       /* window data                                */
int x, y;                /* where we are at start                      */
int ch;                  /* char typed that got us here                */
FDB *wind_actv;          /* raster info for screen                     */
int raz_pts[];           /* srce/dest coords for raster                */
OBJECT *wwmp;            /* pointer to menu objects                    */
{
     int effect;                        /* return from 'vst_effects'   */
     char o_s[2];                       /* output string: char + null  */
     int cell_width,l_delta,r_delta;    /* char matrix data variables  */
     int status;                        /* return from 'vqt_width'     */
     int attrib[10];                    /* filled by 'vqt_attributes'  */
     int event;                         /* return flags                */
     int mbstate, kstate;               /* mouse button & keycode      */
     int fi;                            /* 0 = first iter of 'while'   */
     int ux,uy,uw,uh;                   /* coords used for rasters     */
     int last_x,last_y;                 /* latest text coords of mouse */
     int new_x,new_y;                   /* new pos after mouse move    */
     int quit = FALSE;                  /* FALSE == keep getting text  */

     hide_mouse();

     /* make sure of correct clip area     */
     wind_get(ww->wi_handle,WF_WORKXYWH,&ux,&uy,&uw,&uh);
     set_clip(ww,ux,uy,uw,uh,FALSE,raz_pts,wind_actv);

     /* make sure off-screen window area is truncated  */
     if ((ux + uw) > r_width_G)
          uw = r_width_G - ux;
     if ((uy + uh) > r_height_G)
          uh = r_height_G - uy;

     /* make sure cursor is in legal area  */
     if ((x < ux) || (x > ux + uw))
          x = ux + ONE;
     if ((y < uy) || (y > uy + uh))
          y = uy + ONE;

     /* set text style  */
     effect = vst_effects(handle_G,1);

     /* input text and echo it-- this is the only way I figured  */
     /*   out how to do it.                                      */
     o_s[0] = (ch & 0xff);
     o_s[1] = '\0';

     /* N.B. this next call can be used with 'vst_effects()' above   */
     /*      to change text style.                                   */
     status = vqt_width(handle_G,'O',&cell_width,&l_delta,&r_delta);
     vqt_attributes(handle_G,attrib);

     /* print that first char      */
     v_gtext(handle_G,x,y,o_s);
     x += cell_width;              
     fi = 0;  /* used to prevent call to 'vg_text()' during first    */
              /*  iteration of "while" loop below                    */
     last_x = x;    
     last_y = y;

     while (TRUE)              /* mouse button press exits this loop */
     {
          if (!quit)
          {
               ch = 0;
               /* get a character  */
               vrq_locator(handle_G,x,y,&last_x,&last_y,&ch); 
          } 

          /* Watch for button press:                                 */
          event = evnt_multi(MU_BUTTON | MU_M1 | MU_MESAG | MU_KEYBD, 
                             1,1,butdown_G,        /* button parms   */
                             1,last_x,last_y,0,0,  /* 1st rect parms */
                             0,0,0,0,0,            /* 2nd rect parms */
                             msgbuff_G, /* msg pipe buffer           */
                             1,0,       /* timer low & high words    */
                             &new_x,&new_y,  /* mouse x/y at event   */
                             &d3_G,     /* buttons state at event    */
                             &d4_G,     /* state of certain keys     */
                             &d6_G,     /* code of pressed key       */
                             &d5_G);    /* nr of mouse key states    */
     

          if ((event & (MU_BUTTON | MU_MESAG)) && !(event & MU_KEYBD))
          {
               o_s[0] = '\0';
               x = new_x;
               y = new_y;
               /* make sure cursor is in legal area  */
               if ((x < ux) || (x > ux + uw))
                    x = ux + ONE;
               if ((y < uy) || (y > uy + uh))
                    y = uy + ONE;
               quit = TRUE;
               butdown_G = !butdown_G;
               continue;

          }

          /* If button pressed, leave and do mouse stuff, &c.      */
          if (quit) 
          {
               /* copy current window into buffer for use in    */
               /*   redrawing window                            */
               
               wind_get(ww->wi_handle,WF_WORKXYWH,&ux,&uy,&uw,&uh);
               set_clip(ww,ux,uy,uw,uh,FALSE,raz_pts,wind_actv);    
               vro_cpyfm(handle_G,S_ONLY,raz_pts,wind_actv,
                           &(ww->wind_buff));

               return;          

          }
          if ((event & MU_KEYBD) || (ch != 0))
               quit = FALSE;
               
          if ((event & MU_MESAG) && (msgbuff_G[0] == 10))
               pr_menu(wwmp,ww->wi_handle);

          /* insert character into print string   */
          o_s[0] = ch & 0x00ff;
          o_s[1] = '\0';

          /* this looks stupid, but I'd like to see anything that  */
          /*   doesn't and also works                              */
          if (fi != 0)
          {
               /* figure out what to do with current char    */
               if (ch != '\r')
               {
                    if ((ch == '\b') && (x > ux))
                    {
                         /* valid backspace       */
                         o_s[0] = ' ';
                         v_gtext(handle_G,x,y,o_s);
                         x -= cell_width;
                    }
                    else if ((ch != '\b') && (isprint(ch)))
                    {
                         /* not a backspace       */                                   
                         v_gtext(handle_G,x,y,o_s);
                         x += cell_width;
                    } 
               }
               else
               {
                    /* carriage return  */
                    y += attrib[9] + ONE;
                    x = ux + ONE; 
               }
          }
          else
               fi = 1;

          if (x >= (ux + uw - ONE))
          {
               /* at end of clipped text line, go down to next   */
               y += attrib[9] + ONE;
               x = ux + ONE;
          }
          /* do not go lower than bottom of clipped area    */
          if (y > (uy + uh - ONE))
               y -= attrib[9] + ONE;

          /* make sure cursor is in legal area  */
          if ((x < ux) || (x > ux + uw))
               x = ux + ONE;
          if ((y < uy) || (y > uy + uh))
               y = uy + ONE;


     }    /* while  */        

}


/*****
*
* i_scrnras-- initialize FDB for screen raster ops.
*             FDB is a GEM structure.
*
*****/
int i_scrnras(res,ras)
int res;       /* nr of bit planes this res       */
FDB *ras;      /* will hold raster info           */
{
     /* get address of video ram   */
     ras->fd_addr = Logbase();
 
     /* other data-- cf. VDI manual p. 6-2 or GPR p. 158    */    
     ras->fd_w = r_width_G;
     ras->fd_h = r_height_G;
     ras->fd_wdwidth = r_width_G / 16;
     ras->fd_stand = STD;               
     ras->fd_nplanes = res;
     
}



/*****
*
* do_resize-- resize or move a window.
*
*****/
int do_resize(ww,msg,wind_actv,raz_pts)
struct w_pack *ww;  /* window info                             */
int msg;            /* specific window op prompting this call  */
FDB *wind_actv;     /* raster info for screen                  */
int raz_pts[];      /* srce/dest coords for raster             */
{

     /* make sure minimum dimensions aren't exceeded     */     
     if (msgbuff_G[6] < MIN_WIDTH)
          msgbuff_G[6] = MIN_WIDTH;
     if (msgbuff_G[7] < MIN_HEIGHT)
          msgbuff_G[7] = MIN_HEIGHT;

     /* ditto max   */
     if (msgbuff_G[6] > ww->max_dims[2])
          msgbuff_G[6] = ww->max_dims[2];
     if (msgbuff_G[7] > ww->max_dims[3])
          msgbuff_G[7] = ww->max_dims[3];

     /* change dimensions     */
     wind_set(ww->wi_handle,WF_CURRXYWH,msgbuff_G[4],
              msgbuff_G[5],msgbuff_G[6],msgbuff_G[7]);

     /* get new dimensions of work area      */  
     wind_get(ww->wi_handle,WF_WORKXYWH,&(ww->xwork),&(ww->ywork),
              &(ww->wwork),&(ww->hwork));

     /* redraw if necessary   */
     if (msg == WM_SIZED)
          do_redraw(ww,ww->xwork,ww->ywork,ww->wwork,ww->hwork,
                    wind_actv,raz_pts);    

}


/*****
*
* do_unfull-- make window previous size before it was fulled.
*
*****/
int do_unfull(ww)
struct w_pack *ww;       /* window data      */
{
     /* calculate restored work dimensions   */
     wind_calc(WC_WORK,ww->w_kind,ww->xold,ww->yold,ww->wold,ww->hold,
               &(ww->xwork),&(ww->ywork),&(ww->wwork),&(ww->hwork));

     /* set new dimensions    */
     wind_set(ww->wi_handle,WF_CURRXYWH,ww->xold,
               ww->yold,ww->wold,ww->hold);
     ww->fulled = FALSE;

}


/*****
*
* do_full-- make window full size and save previous dimensions.
*
*****/
int do_full(ww)
struct w_pack *ww;  /* window data           */
{
     /* save old dimensions                  */
     wind_calc(WC_BORDER,ww->w_kind,ww->xwork,ww->ywork,ww->wwork,
               ww->hwork,&(ww->xold),&(ww->yold),&(ww->wold),&(ww->hold));

     /* calculate new work dimensions        */
     wind_calc(WC_WORK,ww->w_kind,xdesk_G,ydesk_G,ww->max_dims[2],
               ww->max_dims[3],      
               &(ww->xwork),&(ww->ywork),&(ww->wwork),&(ww->hwork));

     /* set new dimensions = maximum         */
     wind_set(ww->wi_handle,WF_CURRXYWH,xdesk_G,ydesk_G,ww->max_dims[2],
              ww->max_dims[3]);  
     ww->fulled = TRUE;

}


/*****
*
* make_menu-- load menu resource file, get menu tree address,
*             and display menu bar.
*
*****/ 
OBJECT *make_menu()
{
     OBJECT *wwmp;  /* pointer to menu objects    */
     
     ret_G = rsrc_load(RSRC_FILE);

     ret_G = rsrc_gaddr(1,0,&wwmp);

     ret_G = menu_bar(wwmp,1);
     return wwmp;

}


/*****
*
* pr_menu-- process menu events.
*
*****/
int pr_menu(wwmp,wi_handle)
OBJECT *wwmp;       /* pointer to menu objects         */
int wi_handle;      /* window handle                   */
{
     
     char *sub_ttl = MSUB_TITLE;   /* window sub-title string     */
     char *arg1, *arg2;            /* pointers to interior of abv */
     int i;                        /* index variable              */
     
     arg1 = &sub_ttl[SUB_TF_1];
     arg2 = &sub_ttl[SUB_TF_2];                                                 
     sprintf(arg1,"%d",msgbuff_G[3]);
     for (i = 0; arg1[i] != '\0'; i++)
          ;
     arg1[i] = ' ';
     sprintf(arg2,"%d",msgbuff_G[4]);
     for (i = 0; arg2[i] != '\0'; i++)
          ;
     arg2[i] = ' ';

     wind_set(wi_handle, WF_INFO, sub_ttl, 0, 0);

     menu_icheck(wwmp,msgbuff_G[4],1);
     menu_tnormal(wwmp,msgbuff_G[3],1);

}


/*****
*
* save_window-- save current window state into buffer.
*
*****/
int save_window(ww,wind_actv,raz_pts)
struct w_pack *ww;  /* window data                     */
FDB *wind_actv;     /* raster info for screen          */
int raz_pts[];      /* srce/dest coords for raster     */
{
     int ux,uy,uw,uh;    /* temp coords      */  
               
     wind_get(ww->wi_handle,WF_WORKXYWH,&ux,&uy,&uw,&uh);
     set_clip(ww,ux,uy,uw,uh,FALSE,raz_pts,wind_actv);    
     vro_cpyfm(handle_G,S_ONLY,raz_pts,wind_actv,&(ww->wind_buff));

     return;

}


/*****
*
* pr_window-- prints handle of active window as subtitle of appli-
*             cation window.
*             Not currently used (seems to interfere with some
*                AES functions.)
*
*****/
int pr_window(ww,wh)
struct w_pack *ww;  /* window data           */
int wh;             /* window handle         */
{
     
     char *sub_ttl = W_TITLE;      /* subtitle string for window   */
     char *arg1;                   /* pointer to interior of above */
     int i;                        /* index variable               */
     
     arg1 = &sub_ttl[W_TF_1];
     sprintf(arg1,"%d",wh);
     for (i = 0; arg1[i] != '\0'; i++)
          ;
     arg1[i] = ' ';

     wind_set(ww->wi_handle, WF_INFO, sub_ttl, 0, 0);

     return;

}


/*****
*
* init_svars-- initialize screen variables.
*
*****/
int init_svars()
{
     /* set buffer variables  */
     screenx_G = xdesk_G;
     screeny_G = ydesk_G;

}


/*****
*
* next_struct-- returns index (into 'ww[]') of window to be processed.
*
*****/
int next_struct(ww,cw)
struct w_pack ww[];      /* array of window structures   */
int cw;                  /* msgbuff[3] from AES          */
{
     int i, go;
        
     for (i = 0, go = TRUE; go && i < NR_WWS; i++)
          go = (ww[i].wi_handle == cw) ? FALSE : TRUE;

     return (go) ? NO_WINDOW : --i;

}


/*****
*
* draw_ww2-- initial draw, window 2.
*
*****/
int draw_ww2(ww)
struct w_pack *ww;       /* window data           */
{
     int temp[4];        /* coords for pattern    */

     vsf_interior(handle_G,2);
     vsf_style(handle_G,23);
     vsf_color(handle_G,1);
     temp[0] = ww->xwork;
     temp[1] = ww->ywork;
     temp[2] = ww->xwork + ww->wwork - ONE;
     temp[3] = ww->ywork + ww->hwork - ONE;
     v_bar(handle_G,temp);

}


/*****
*
* fix_res-- sets up resolution-dependent variables from arrays.
*           uses global variables.
*
*****/
int fix_res()
{
     int r_ix;      /*  0 == low, 1 == med, 2 == high resolution  */

     r_ix = Getrez();

     r_height_G = heights_G[r_ix];
     r_width_G = widths_G[r_ix];
     r_planes_G = planes_G[r_ix];

}


/*****
*
* prep_wind-- initialize structure for a window.
*
*****/
int prep_wind(ww,kind,sx,sy,sw,sh,mx,my,mw,mh,t,t_s,s,s_s)
struct w_pack *ww;  /* structure for this window            */
int kind;           /* GEM attributes for this window       */
int sx,sy,sw,sh;    /* starting dimensions                  */
int mx,my,mw,mh;    /* maximum dimensions                   */
char t;             /* TRUE==window has title               */
char *t_s;          /* title string                         */
char s;             /* TRUE==window has subtitle            */
char *s_s;          /* subtitle string                      */
{

     /* initialize startup structure for window   */
     ww->w_kind = kind;       /* GEM attributes   */
     ww->max_dims[0] = mx;    /* dimensions:      */
     ww->st_dims[0] = sx;
     ww->max_dims[1] = my; 
     ww->st_dims[1] = sy;
     ww->max_dims[2] = mw; 
     ww->st_dims[2] = sw;
     ww->max_dims[3] = mh; 
     ww->st_dims[3] = sh;
     if (t)                   /* main title       */
          ww->title = t_s;
     if (s)                   /* subtitle         */
          ww->subtl = s_s;
     ww->t = t;               /* title flags      */
     ww->s = s;

}


