
/***********************************************************************
*
* fxd.h-- constants and data declarations for 'fx.c'.
*
*
************************************************************************/


/***********************
*
* defined constants.
*
***********************/
/* some data structure constants                                       */

#define HASH_SIZE 128
     /* ...max hash index == 127                                       */
#define LINE_NR 50
     /* ...max number of dump strings                                  */
#define ARG_NR 6
     /* ...number of command line arguments required + 1               */
#define TIME_OUT 32000
     /* ...printer time-out count used by 'psend()' in "f9.c"          */

/* following are offsets into font header area...                      */
/* not all of them are used:                                           */
#define FONT_ID 0
     /* ...1 == system font                                            */
#define FONT_SIZE 2
     /* ...font size in points                                         */
#define FONT_NAME 4
     /* ...font name as an ascii string                                */
#define ASC_LOW 36
     /* ...lowest ascii val of displayable chars                       */
#define ASC_HIGH 38
     /* ...highest ascii val of displayable chars                      */
#define BIG_WIDTH 50
     /* ...width of broadest char                                      */
#define BIG_CELL_WD 52
     /* ...width of broadest char cell                                 */
#define LINK_OFF 54
     /* ...linker offset                                               */
#define FONT_FLAG 66
     /* ...flag:                                                       */
     /*          b0 == 1 => system font                                */
     /*          b1 == 1 => horiz offset table present                 */
     /*          b2 == 1 => bytes in 68000 format;                     */
     /*             == 0 => IBM format                                 */
     /*          b3 == 1 => width of all chars equal                   */

#define HORZ_OFFTAB 68
     /* ...pointer to horizontal offset table; == 0 => no table        */
#define CHAR_OFFTAB 72
     /* ...pointer to character offset table                           */
#define FONT_DATA 76
     /* ...pointer to font data                                        */
#define FORM_WIDTH 80
     /* ...sum of widths of all chars                                  */
#define FORM_HEIGHT 82
     /* ...number of scan lines for this font                          */
#define NEXT_FONT 84
     /* ...pointer to next font                                        */

/* 'fx'-specific constants follow:                                     */
#define HASH_IX 6
     /* ...element size in hash table                                  */
#define MAX_SCAN 80
     /* ...max space taken by a scan list                              */
#define DBL_SCAN 180
     /* ...double the above, plus some                                 */
#define FIRST_ASCII 32  
     /* ...(33 with 'recognize')                                       */
     /* ...lowest valid ascii char for recognition                     */
#define FIRST_VALID 32
     /* ...lowest valid ascii char for dump                            */
     /* As long as no good text has been found, 'recognize' will       */
     /*   accept blanks as valid patterns in order to speed up this    */
     /*   phase of the program. After text has been recognized,        */
     /*   blanks can no longer be 'recognized' (but they can be        */
     /*   dumped, of course). This helps keep the memory scan "in      */
     /*   sync" with good text.                                        */
#define LAST_ASCII 126
     /* ...highest valid ascii char                                    */
#define BORDER 0xaa 
     /* aaaaaa */
     /* ...end marker value for scan list: binary '101010...' is a     */
     /*   pattern not occurring in any character.                      */
#define GOOD_BLANKS 1
     /* ...nr of consecutive blanks required                           */
#define RETRIES 0
     /* ...nr of offset retries by 'dump_line'                         */
#define ABORT -2
     /* ...abort code from 'recognize'                                 */
#define LINE_LEN 80
     /* ...length of print line for dump                               */

#define GOOD 0
     /* ...completion code                                             */
#define BAD -1
     /* ...completion code                                             */
#define L_SIZE 32
     /* ...longword size                                               */
#define BYTESIZE 8
     /* ...byte size                                                   */
#define TRUE 1
#define FALSE 0
#define DOT 46
     /* ...ascii '.'                                                   */
#define B_SLASH 92
     /* ...ascii '\'                                                   */
#define SLASH 47
     /* ...ascii '/'                                                   */
#define BLANK 32
     /* ...ascii ' '                                                   */
#define COLON 58
     /* ...ascii ':'                                                   */
#define NULL 0
#define ZERO 48
     /* ...ascii '0'                                                   */
#define ONE 49
     /* ...ascii '1'                                                   */
#define QUOTE 39
     /* ...ascii ' ' '                                                 */
#define S 83
     /* ...ascii 'S'                                                   */

#define ARG_SIZE 76
     /* ...max size cmdline arg                                        */
#define PRT 0
     /* ...printer device nr                                           */
#define TIMER 32700
     /* ...timeout count == largest signed 16-bit value (almost)       */
#define MAX_DIGITS 10
     /* ...max length number string                                    */

#define CLR_C 0xfe
     /* ...use: andi #CLR_C,ccr                                        */
#define SET_C 0x01
     /* ...use: ori #SET_C,ccr                                         */
#define CLR_Z 0xfb
     /* ...use: andi #CLR_Z,ccr                                        */
#define CLR_X 0xef
     /* ...use: andi #CLR_X,ccr                                        */
#define SET_X 0x10
     /* ...use: ori #SET_X,ccr                                         */
#define HIGH_BIT 0x80000000
     /* ...use: ori.l #HIGH_BIT,__                                     */
#define HB_BIT 0x80
     /* ...use: ori.b #HB_BIT,__                                       */
#define HI_RES 80
     /* ...meaningless relic                                           */
                                                                       
/************************************************************************
*                                                                      
* variables: to simplify access, these have all been declared          
*            global and are thus all offsets from register a4.         
*                                                                      
*                                                                      
************************************************************************/

/* various messages, &c.                                               */
char B[] = " ";                                   /* 'BLANKS'          */
char c_f[] = {0x0d, 0x0a, 0x0};                   /* 'cr_lf'           */
char w_m[] = "waiting...";                        /* 'wait_msg'        */

/* arrays for resolution-independent operation                         */
long res_ix;    /* used as index into arrays below                     */
long sc[] = {40,80,80};       /* 'screens'                             */
long li[] = {320,640,1280};   /* 'lines'                               */
long bp[] = {160,160,80};     /* 'bitplanes'                           */
long tp[] = {1280,1280,1280}; /* 'bitlines'                            */
long fo[3] = {0,0,0};         /* 'fonts'                               */
long he[] = {8,8,16};         /* 'heights'                             */
long th[] = {3,3,3};          /* 'fair_nrs'                            */
long rx[] = {4,2,1};          /* y-coord plane alignment factors--     */
                              /*    'rfax'                             */

long font_used;          /* pointer to header of current font          */
long cell_ht;            /* character cell height                      */
long form_ht;            /* effective scan line height of text line    */
long resolution;         /* window scan line length in bytes           */
long line_size;          /* text line size in bytes                    */
long front;              /* added to screen line start == window start */
long back;               /* subtr'd from screen line end == window end */
long base;               /* saved window absolute start                */
long last;               /* saved window absolute end                  */
long chunk_end;          /* == above                                   */
long screen_size;        /* nr of screen words saved                   */
long screen_base;        /* first screen address for save              */
long screen_height;      /* nr of screen scan lines to save            */
long line_count;         /* nr of text lines to dump                   */
long w_xc;               /* start x-coordinate of dump                 */
long w_yc;               /* start y-coordinate of dump                 */
long w_width;            /* width of dump area                         */
long w_height;           /* height of dump                             */
long th_hold;            /* good line threshold this res               */
long bit_off;            /* initial bit offset of dump                 */
long y_align;            /* y-coordinate alignment factor this res     */
int ret_save;            /* used to save word return from bios call    */
long first_ascii;        /* lowest 'recognizable' ascii value          */
long newbase;            /* working base of screen memory for scan     */
long hash_errors;        /* number of bad hashes from 'dump_line'      */
long recg_errors;        /* number of bad recognizes from 'dump_line'  */
long hi50;               /* 0 => small font requested                  */
long scrn_text;          /* length of full-size scan line              */
long scrn_size;          /* length of full-size text line              */
int form_inc;            /* user-requested inc to 'line_size'          */
int kbd_abort;           /* TRUE => any keypress aborts dump           */
int trigger;             /* keycode that triggers TOS dump             */
int TOS_ver;             /* TRUE => this is TOS version                */
int xshift;              /* shift factor for saved window              */

char d_s[LINE_LEN];      /* current line dumped                        */


/* In order to access addresses of variables (in particular, bases     */
/*   of arrays) within Megamax's linkage system with a minimum of      */
/*   code alteration, these addresses must be assigned to pointer      */
/*   variables which can then be accessed via the 'a4' link regis-     */
/*   ter. So all variables above whose addresses are used have         */
/*   "duplicate" pointer declarations below.                           */
/* (The alternative is to leave out the pointer declarations and       */
/*   use the 68000 'lea' and 'pea' codes, which would involve even     */
/*   more alteration of the original code.)                            */
/* The bad aspect of this scheme is that an ambiguity exists between   */
/*   simple variables and base addresses of arrays-- they look the     */
/*   same in this version. However the arrays are only used as base    */
/*   addresses in the program, so the inconsistency is only latent.    */ 

/* pointers:                                                           */
char *BLANKS = B;             /* pointer to one blank (I forgot why)   */
char *cr_lf = c_f;            /* carriage return/linefeed              */
char *wait_msg = w_m;         /* printed by 'keypress'                 */
long *screens = sc;           /* --> 'resolution'                      */
long *lines = li;             /* --> 'line_size'                       */
long *bitplanes = bp;         /* --> 'scrn_text'                       */
long *bitlines = tp;          /* --> 'scrn_size'                       */
long *fonts = fo;             /* --> 'font_used'                       */
long *heights = he;           /* --> 'cell_ht'                         */
long *rfax = rx;              /* --> 'y_align'                         */
long *fair_nrs = th;          /* --> 'th_hold'                         */
char *dump_string;            /* pointer to string containing one text */
                              /*   dump line                           */
#ifndef TEST_VER
extern char *one_line;        /* pointer to processed screen data for  */
                              /*   one text line                       */
#else
char o_l[10000];
char *one_line = o_l;
#endif

long sa0,sa1,sa2,sa3,sa5;     /* space to save address registers on    */
                              /*   entry into 'dumper()'               */

long sd0,sd1,sd2,sd3,         /* space to save data registers on entry */
     sd4,sd5,sd6,sd7;         /*   into 'dumper()'                     */

char *p_ptr;                  /* pointer to dumped line storage        */
long i;                       /* index variable                        */
int stop_it;                  /* flag for 'for' loop                   */
int a_test;                   /* < 0 => no keyboard abort              */

long atol();   /* for command-line string conversion                   */        
int dumper();  /* top level of assembly language dump routines         */
long func_ret; /* return value from above                              */





