/*===========================================================================*/
/*  Digital Clock Accessory                             (c)1986 John Brochu  */
/*---------------------------------------------------------------------------*/
/*  This  accessory  is  a  non-movable  digital  clock which also sets the  */
/*  system defaults from the DESKTOP.INF file, and thus is meant to replace  */
/*  the Atari Control Panel (saving about 10K!).   Time and date can be set  */
/*  by pointing and clicking on the desired object  (or 'Set'),  and typing  */
/*  the new settings from the keyboard using the GEM editor.   The time and  */
/*  date  are  preserved  as  long  as  the system is powered up (reset and  */
/*  rez-change do not affect the clock).                                     */
/*---------------------------------------------------------------------------*/
/*  Link with: accstart,CLOCK,aesbind,osbind          Written with Alcyon C  */
/*---------------------------------------------------------------------------*/

#include "osbind.h"
#include "obdefs.h"
#include "gemdefs.h"

/*-------------------------Miscellaneous definitions-------------------------*/
#define Vsync()		xbios(37)
#define SET_MODE	((clk_tree[SET_OBJ].ob_state & SELECTED)!=0)
#define TOUCH_EDIT	(TOUCHEXIT|EDITABLE)
#define NXT_BYT		((unsigned)((*buf_ptr++)-'0'))

#define SPC		0x20
#define CR		0x000d

#define ZERO_FILL	0	/* Defs. for the get_time() routine,      */
#define SPC_FILL	1	/*  determine leading-zero space or zero. */

#define CON		0x0002	/* Defs. for Bconout() */
#define BELL		0x0007

#define LCD		((char)0x10)	/* (LCD digits)    */
#define NRM		((char)0x30)	/* (Normal digits) */

#define BOX_X		6	/* Screen location of the clock     */
#define BOX_Y		4	/*  (upper left in character units) */

#define MO		1	/* Power-up defaults for IKB clock  */
#define DAY		1	/*  (1/1/86)                        */
#define YR		6

#define DA_MSK		0x001f	/* GEMDOS time/date masks */
#define MO_MSK		0x01e0
#define YR_MSK		0xfe00
#define HR_MSK		0xf800
#define MI_MSK		0x07e0

#define OBJ_TREE	0	/* Object 'parent'           */
#define TIME_OBJ	1	/* Object # of date window   */
#define DATE_OBJ	2	/* Object # of time window   */
#define EXIT_OBJ	3	/* Object # of 'Exit' button */
#define SET_OBJ		4	/* Object # of 'Set' button  */
/*---------------------------------------------------------------------------*/

/*-----------------------Global variable declarations------------------------*/
extern int	gl_apid;		/* Global application id             */
int		contrl[12], intin[128], intout[128], ptsin[128], ptsout[128],
		cx, cy, cw, ch;		/* Clock location & dimensions       */
char		time[10]="_____", date[10]="______",
		button[6], exit_text[]="EXIT ", store_text[]="STORE";
/*---------------------------------------------------------------------------*/

/*-------------------------Object tree for the clock-------------------------*/
TEDINFO		txtinfo[] = {				/* Text structure */
/*ptext,ptmplt,pvalid,font,junk1,just,color,junk2,thickness,txtlen,tmplen*/
time,   "__:__ _m", "9999a",  3, 6, 2, 0x1200, 0x0,  0, 6,9,
date,   "__/__/__", "999999", 3, 6, 2, 0x1200, 0x0,  0, 7,9,
button, "",         "",       5, 6, 2, 0x1100, 0x0, -1, 6,1,
"SET",  "",         "",       5, 6, 2, 0x1100, 0x0, -1, 4,1};

OBJECT		clk_tree[] = {				/* Object structure */
/*next,head,tail,type,flags,                 state,    spec,         x,y,w,h*/
-1, 1, 4, G_BOX,     NONE,                   SHADOWED, 0x21141L,     0,0,23,5,
 2,-1,-1, G_FBOXTEXT,TOUCHEXIT,              OUTLINED, &txtinfo[0],  2,1, 9,1,
 3,-1,-1, G_FBOXTEXT,TOUCHEXIT,              OUTLINED, &txtinfo[1], 12,1, 9,1,
 4,-1,-1, G_BOXTEXT, EXIT|DEFAULT|SELECTABLE,SHADOWED, &txtinfo[2],  2,3,14,1,
 0,-1,-1, G_BOXTEXT, LASTOB|TOUCHEXIT|SELECTABLE,
					     SHADOWED, &txtinfo[3], 18,3, 3,1};
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/

long 	find(st_adr, end_adr, string)		/* Search routine ---        */
char	*st_adr, *end_adr, *string;		/* Returns start adr of      */
{						/*  specified string, or     */
	int	i;				/*  null ptr if not found.   */
	char	c;
	
	do {
		i = 0;
		while ((c = *(string + i))&&(c == *(st_adr + i)))   ++i;
	} while ((c)&&(++st_adr < end_adr));

	if (st_adr==end_adr)	return(0L);
	else			return(st_adr);
}

/*---------------------------------------------------------------------------*/

int	dpeek(addr)		/* Returns the integer value stored at addr */
int	*addr;
{
	int	x;
	long	save_ssp;

	save_ssp = Super(0L);			/* Get into Super mode */
	x = *addr;				/* Read the value      */
	Super(save_ssp);			/* Back to User mode   */
	return(x);
}

/*---------------------------------------------------------------------------*/

get_asc(num, string, max, ofs, typ) /* Convert an integer to a text string. */
int	num, max, typ;		/* Pass int, pointer to destination string, */
char	*string, ofs;		/*   max string length, '0'-offset (for LCD */
				/*    or normal display), & leading-zero    */
				/*   display type --    typ==0 zero-fill,   */
				/*			typ==1 space-fill.  */
{
	char	temp[5], *tmp=temp;
	int	digit, len=0, cnt=10000;

	do {
		digit=0;
		while (num >= cnt) {
			num -= cnt;
			++digit;
		}
		if ((digit)||(len)||(typ==0)) {
			++len;				/* Show trailing */
			*tmp++ = digit + ofs;		/*  zeros as '0' */
		}
		else	*tmp++ = ' ';			/* Leading zeros */
		cnt /= 10;				/*  as space.    */
	} while (cnt >= 1);

	tmp = temp + (5-max);	/* Move right-justified ascii string to  */
	for(len=max; len; *string++ = *tmp++, --len);	/* dest.string.  */
}

/*---------------------------------------------------------------------------*/

unsigned get_num(string, cnt)	/* Converts ascii string of length 'cnt' */
char	*string;		/*  to an integer, & returns the result. */
int	cnt;
{
	unsigned num=0, pwr=1;
	int	i;
	char	c;

	string += (cnt - 1);
	for (i = 0; i<cnt; ++i) {
		c = *string--;
		if ((c >= '0')&&(c <= '9'))	c -= '0';
		else	continue;
		num += (c * pwr);
		pwr *= 10;
	}
	return(num);
}

/*---------------------------------------------------------------------------*/

copy_str(src, dst)		/* Null-terminated string copy routine */
char	*src, *dst;
{
	while(*dst++ = *src++);
}

/*---------------------------------------------------------------------------*/

int	str_len(str)		/* Returns length of null-terminated string */
char	*str;
{
	int	x;

	for (x=0; *(str+x); ++x);
	return(x);
}

/*---------------------------------------------------------------------------*/

main()
{
	static char	inf_fil[]={"#:\\DESKTOP.INF"};
	int		x, handle, char_x, char_y, menu_id, msg_buf[8];
	unsigned	tim, dat;
	long		buffer, length, error, blk, bootdev = 0x00000446;


	appl_init();
	menu_id = menu_register(gl_apid, "  Digital clock");

	blk = Gettime();		/* Get IKBD's time and date.         */
	if ((blk&0x01ff0000)==0) {	/* If mo&day == 0, assume cold-start */
		blk = ((YR*512L)+(MO*32)+DAY)<<16;
		Settime(blk);		/*   and set IKBD power-up defaults. */
	}
	dat = (blk & 0xffff0000) >> 16;
	tim =  blk & 0x0000ffff;
	Tsetdate(dat); 			/* Set GEMDOS's software clock       */
	Tsettime(tim);			/*  to the IKBD's time and date.     */
						/* Look for DESKTOP.INF...   */
	inf_fil[0] = Dgetdrv() + 'A';		/* Look on current drv first.*/
	if ((error = Fopen(inf_fil, 0)) < 0) {	/* If not there, look on     */
		x = dpeek(bootdev) + 'A';	/*  boot drive if not the    */
		if (x != inf_fil[0]) {		/*  same as current drive... */
			inf_fil[0] = x;
			error = Fopen(inf_fil, 0);
		}
	}		
	if (error >= 0) {				/* If DESKTOP.INF    */
		handle = error;				/*  was found,       */
		blk = buffer = Malloc(1024L);	/*  allocate 1K block for it.*/
		if (blk==0)	Fclose(handle);	/* Not enough free memory!   */
		else {				/* If no error, load the file*/
			length = Fread(handle, 1024L, buffer);
			Fclose(handle);
			get_b_hdr(buffer, length);    /* Process '#b' header */
			get_c_hdr(buffer, length);    /* Process '#c' header */
			Mfree(blk);		/* Free our 1K memory block  */
		}
	}


	char_x = 8;				/* Lo/med-rez chars. are 8x8,*/
	char_y = 8 + ((Getrez()==2)*8);		/*  hi-rez are 8x16.         */
	for(x=0; x<5; ++x) {
		clk_tree[x].ob_x *= char_x;	/* Initialize the clock's    */
		clk_tree[x].ob_y *= char_y;	/*  location & dimensions... */
		clk_tree[x].ob_width *= char_x;
		clk_tree[x].ob_height *= char_y;
	}		
	cx = clk_tree[0].ob_x = BOX_X * char_x;
	cy = clk_tree[0].ob_y = BOX_Y * char_y;
	cw = clk_tree[0].ob_width + 5;		/* Account for drop-shadow   */
	ch = clk_tree[0].ob_height + 5;


	while (1) {			/* Loop forever (accs never end!)    */
	  evnt_mesag(msg_buf);		/* Wait for an 'Acc-open' msg for    */
					/*  our clock acc... ignore all else.*/
	  if ((msg_buf[0]==AC_OPEN)&&(msg_buf[4]==menu_id)) {
	    graf_mouse(POINT_HAND, 0L);	/* Change mouse pointer to hand      */
	    wind_update(BEG_MCTRL);	/* Take full control of the mouse    */
	    form_dial(0, 0,0,0,0, cx,cy,cw,ch);	/* Reserve screen for clock  */

	    clk_tree[TIME_OBJ].ob_flags = TOUCHEXIT;	/* Initialize some   */
	    clk_tree[DATE_OBJ].ob_flags = TOUCHEXIT;	/*  variables        */
	    copy_str(exit_text, button);
	    get_time(LCD);			/* Get the time and date     */
	    redraw(OBJ_TREE, SHADOWED);		/* Draw the clock!           */

	    do_clock();			/* Interact with the clock...        */
					/* returns when EXIT bar is clicked. */
	    redraw(EXIT_OBJ, SHADOWED);		/* Deselect the exit bar     */
	    wind_update(END_MCTRL);			/* Free the mouse    */
	    form_dial(3, 0,0,0,0, cx,cy,cw,ch); 	/* Undraw the clock  */
	    graf_mouse(ARROW, 0L);	/* Change mouse pointer back to arrow*/
	  }
	}					/* And go wait some more!    */
}

/*---------------------------------------------------------------------------*/

get_b_hdr(buf, len)		/* Process the '#b' DESKTOP.INF header */
long	buf, len;		/*  (Printer configuration)            */
{
	char		*buf_ptr;
	int		x;
	unsigned	p_config = 0;
	long		buf_end = buf + len;


	buf_ptr = find(buf, buf_end, "#b");	/* Look for '#b' header...   */
	if (buf_ptr > 0) {			/* Continue if found         */
		buf_ptr += 2;
		while(*buf_ptr==SPC)	++buf_ptr; /* Skip any leading spaces*/
		for (x=0; x<6; ++x) {		/* Read the printer info...  */
			p_config >>= 1;
			p_config |= ((NXT_BYT&1)<<5);
		}
		Setprt(p_config);		/* Set printer configuration */
	}
}

/*---------------------------------------------------------------------------*/

get_c_hdr(buf, len)		/* Process the '#c' DESKTOP.INF header */
long	buf, len;		/*  (Control Panel info)               */
{
	char		c, kclick, conbell, *buf_ptr, *conterm = 0x00000484;
	int		r, g, b, rez, reg, pal[16], mclick, kdelay, krepeat;
	static int	ofs[]={0,15,1,2,4,6,3,5,7,8,9,10,12,14,11,13};
	long		save_ssp, buf_end = buf + len;
						

	buf_ptr = find(buf, buf_end, "#c");	/* Look for '#c' header...   */
	if (buf_ptr > 0) {			/* Continue if found         */
	    buf_ptr += 2;
	    while(*buf_ptr==SPC)   ++buf_ptr;	/* Skip any leading spaces.  */
	    rez = Getrez();

	    if (rez < 2) {		/* If color system - read palette... */
		for (reg=0; reg<16; ++reg) {
			r = NXT_BYT;
			g = NXT_BYT;
			b = NXT_BYT;
			pal[ofs[reg]] = (r<<8) + (g<<4) + b;
		}	
		if (rez==1)   pal[3] = pal[15];	/* GSX kludge */
		Setpallete(pal);
		Vsync();			/* Wait for VBI to set new   */
		Vsync();			/*  color palette!           */
	    }
			
	    else	buf_ptr += 48;	/* If mono, skip color palette values*/
					/* Read in Control Panel settings... */
	    mclick = NXT_BYT;			/*   mouse-click delay,      */
	    kclick = NXT_BYT;			/*   key-click on/off,       */
	    conbell = NXT_BYT;			/*   console bell on/off,    */
	    kdelay=((*buf_ptr-'0')*10)+(*(buf_ptr+1)-'0'); 	/* key delay,*/
	    krepeat=((*(buf_ptr+2)-'0')*10)+(*(buf_ptr+3)-'0');	/* & repeat. */
					/* Set up the system defaults...     */
	    Kbrate(kdelay, krepeat);			/* Set key rates     */
	    evnt_dclick(mclick, 1);			/* Set mouse delay   */

	    save_ssp = Super(0L);			/* Get into Super to */
	    c = *conterm & 0xfa;			/* set console flags */
	    *conterm = ((c)|(conbell & 1)|((kclick & 1)<<2));
	    Super(save_ssp);				/* Back to User mode */
	}
}

/*---------------------------------------------------------------------------*/

do_clock()			/* Interact with the clock.		*/
{				/* Returns when EXIT bar is clicked.	*/

	int		x, obj, edit_obj;
	unsigned	tim;
	long		dat;


	while(1) {
		evnt_button(1,1,0,&x,&x,&x,&x);	/* Wait for l.button release */

		if (SET_MODE)	obj = form_do(clk_tree, edit_obj); /*--Edit--*/
		else		obj = wait_button(clk_tree);	   /*-Update-*/

		if ((obj==TIME_OBJ)||(obj==DATE_OBJ)) {	/* Time/date clicked.*/
		    edit_obj = obj;			/* Object to edit    */
		    if (!SET_MODE) {			/* If not in set mode*/
			redraw(SET_OBJ, (SELECTED|SHADOWED));	/* then enter*/
			update(store_text, TOUCH_EDIT, NRM);	/* set mode! */
		    }
		}			  

		else if (obj==SET_OBJ) {		/* SET button clicked*/
		    if (SET_MODE) {			/* If set mode, start*/
			edit_obj = TIME_OBJ;		/*  edit with time   */
			update(store_text, TOUCH_EDIT, NRM); 	/*   object. */
		    }
		    else				/* Otherwise quit    */
			update(exit_text, TOUCHEXIT, LCD);	/* edit mode.*/
		}

		else if (obj==EXIT_OBJ) {	/* EXIT/STORE bar clicked.   */
		    if (SET_MODE) {			/* Store was clicked,*/
			redraw(SET_OBJ, SHADOWED);  	/* save new time.    */
			set_time();			/* Set GEMDOS clock. */
			tim = Tgettime();		/* Read it back!     */
			dat = ((long)Tgetdate())&0x0000ffff;
			Settime((dat<<16) + tim);	/* Set IKBD  */
			update(exit_text, TOUCHEXIT, LCD);
		    }
		    else	return;		/* EXIT was clicked, quit!   */
		}
	}
}

/*---------------------------------------------------------------------------*/

update(btn_txt, td_flags, dsp_typ)		/* Update routine for main() */
long	btn_txt;
int	td_flags, dsp_typ;
{
	copy_str(btn_txt, button);		/* Set exit button text    */
	redraw(EXIT_OBJ, SHADOWED);		/*  and redraw the button. */
	clk_tree[TIME_OBJ].ob_flags = td_flags;	/* Set time & date object  */
	clk_tree[DATE_OBJ].ob_flags = td_flags;	/*  flags.                 */
	get_time(dsp_typ);			/* Get current time in LCD */
	redraw(TIME_OBJ, OUTLINED);		/*  or normal digits, and  */
	redraw(DATE_OBJ, OUTLINED);		/*  redraw the time & date.*/
}

/*---------------------------------------------------------------------------*/

redraw(ob, state)	/* Redraw clock objects starting with the specified */
int	ob, state;	/*  'ob'ject in the specified 'state'.              */
{
	clk_tree[ob].ob_state = state;		/* Set the object flags  */
	objc_draw(clk_tree, ob, 1, cx,cy,cw,ch);   /* & draw the object. */
}

/*---------------------------------------------------------------------------*/

set_time()	/* Verify time/date entries & set new time and/or date */
{
	int		len;
	unsigned 	hr, min, month, day, year, pm_flg=0;
	static int	months[]={31,29,31,30,31,30,31,31,30,31,30,31};


	len = str_len(time);
	if ((time[4]=='P')||(time[4]=='p'))	pm_flg = 1;	/* Pm? */

	hr = get_num(&time[0], 2);
	min = get_num(&time[2], 2);
							/* Special cases:  */
	if ((len>4)&&(pm_flg==0)&&(hr==12))	hr = 0;	/*   12am -> 0 hrs */
	if ((pm_flg)&&(hr)&&(hr<12))	hr += 12;	/* 1-11pm -> 13-23 */
	if ((len>=4)&&(hr<24)&&(min<60)) Tsettime((hr<<11)+(min<<5));
							/* Time OK - set it */
	len = str_len(date);	
	month = get_num(&date[0], 2);
	day = get_num(&date[2], 2);
	year = get_num(&date[4], 2) - 80;

	if (year%4)	months[1]=28;		/* Check for leap year. */
	else		months[1]=29;

	if ((len>=6)&&(month)&&(month<13)&&(day)&&(day<=months[month-1]))
		Tsetdate((year<<9) | (month<<5) | day);	/* Date OK - set it */
}

/*---------------------------------------------------------------------------*/

get_time(ofs)	/* Get the time and date from GEMDOS & put in object strings */
char	ofs;	/* 'ofs' determines whether LCD or normal digits are used.   */
{
	unsigned	i, tim, dat;


	tim = Tgettime();			/* Get GEMDOS'      */
	dat = Tgetdate();			/*  time & date.    */

	i = (tim & HR_MSK)>>11;			/* Get the hour.    */
	time[4] = 'a';				
						/* Cnv 24hr to 12hr format */
	if (i==0)	i = 12;			/*       0 hrs -> 12am     */
	else if (i >= 12) {			/*    1-11 hrs -> 1-11am   */
		time[4] = 'p';			/*      12 hrs -> 12pm     */
		if (i != 12)	i -= 12;	/*   13-23 hrs -> 1-11pm   */
	}
	get_asc(i, &time[0], 2, ofs, SPC_FILL);
	get_asc((tim & MI_MSK)>>5, &time[2], 2, ofs, ZERO_FILL);
	i = ((dat & YR_MSK)>>9) + 80;		/* Years start at 1980!    */
	get_asc(i, &date[4], 2, ofs, SPC_FILL);
	get_asc((dat & MO_MSK)>>5, &date[0], 2, ofs, SPC_FILL);
	get_asc(dat & DA_MSK, &date[2], 2, ofs, SPC_FILL);
	time[5] = date[6] = 0;
}

/*---------------------------------------------------------------------------*/

int	wait_button(tree_adr)	/* This routine is a form_do work-alike with */
long	tree_adr;		/* timer event used to keep the clock updated*/
{
    int		x, mx, my, minutes, dsp_mins, event, obj, xit=0,
		key, in_box, in_state, out_state;

    do {
	event = evnt_multi(MU_TIMER|MU_KEYBD|MU_BUTTON,
			1,1,1,		/* Wait for one left button press    */
			0,0,0,0,0,0,0,0,0,0,0L,	/* (Not used) */
			1000,0,			/* Timer value is 1 second   */
			&mx,&my,&x,&x,&key,&x);	/* Returns mouse pos. & key  */

	if (event & MU_TIMER) {		/* Timer event, check the time...    */
	    dsp_mins = ((time[2] & 0x0f)*10) + (time[3] & 0x0f);
	    minutes = ((Tgettime()) & MI_MSK)>>5;
	    if (dsp_mins != minutes) {
		get_time(LCD);			/* Update display if */
		redraw(TIME_OBJ, OUTLINED);		/* necessary */
		redraw(DATE_OBJ, OUTLINED);
	    }
	}
	if ((event & MU_KEYBD) && ((key&0x00ff)==CR)) {	/* Allow RETURN key  */
		obj = EXIT_OBJ;				/*  for exit         */
		xit = 1;
	}
	if (event & MU_BUTTON) {	/* Mouse button pressed! */
					/* See what object was clicked on... */
	    obj = objc_find(tree_adr, OBJ_TREE, MAX_DEPTH, mx, my);
	    if ((obj==SET_OBJ)||(obj==EXIT_OBJ)) {	/* If button,'watch' */
		out_state = clk_tree[obj].ob_state;	/*  it until button  */
		in_state = out_state ^ SELECTED;	/*  is released.     */
		xit = graf_watchbox(tree_adr, obj, in_state, out_state);
	    }		/* If xit=0 -> released outside of button, 1->inside */
	    else if (obj>0)	xit=1;			/* Valid object, exit*/
	    else if (obj<0)	Bconout(CON,BELL);	/* No object found   */
	}
    } while (xit==0);
    return(obj);		/* Return the object clicked on */
}

/*===========================================================================*/
