; ****************************************************************************
; *                             Floyd The Droid                              *
; *                              "On The Run"                                *
; *                       Low resolution version 1.0                         *
; * ------------------------------------------------------------------------ *
; *                         Programmed by Paul Lay                           *
; ****************************************************************************


; ****************************************************************************
; Labels.

gemdos      equ         $01            ; gemdos trap vector
super       equ         $20            ; gemdos supervisor entry command
xbios       equ         $0E            ; xbios trap vector
setscreen   equ         $05            ; set screen parameters
setpallette equ         $06            ; set colour registers
mfpint      equ         $0D            ; set MFP interrupts
random      equ         $11            ; xbios random number generation
scrdmp      equ         $14            ; screen dump
ikbdws      equ         $19            ; send string to keyboard processor
open        equ         $3D            ; open file
close       equ         $3E            ; close file
read        equ         $3F            ; read file
low_res     equ         $00            ; low resolution
register    equ         $FFFF8800      ; sound register select
frequency   equ         $FFFF8802      ; sound data (write)
max_down    equ         $0A            ; maximum vertical offset
max_right   equ         $12            ; maximum horizontal offset
frclock     equ         $466           ; vertical blank frame count clock
hz_200      equ         $4BA           ; 200 Hz timer
screen_mem  equ         $78000         ; start of screen memory
bottom_line equ         $7F800         ; screen address of status line
total       equ         $04            ; total number of missiles
c_address   equ         $00            ; creature's address offset
c_type      equ         $04            ; creature's type offset
c_gap       equ         $08            ; creature's dying gap
c_xpos      equ         $08            ; creature's x-position offset
c_ypos      equ         $09            ; creature's y-position offset
c_dx        equ         $0A            ; creature's dx offset
c_dy        equ         $0B            ; creature's dy offset
c_sleep     equ         $0C            ; creature's sleep offset
c_health    equ         $0D            ; creature's health offset
m_address   equ         $00            ; missile's address offset
m_xpos      equ         $04            ; missile's x-position offset
m_ypos      equ         $05            ; missile's y-position offset
m_dx        equ         $06            ; missile's dx offset
m_dy        equ         $07            ; missile's dy offset
esc_key     equ         $01            ; escape key code
space_bar   equ         $39            ; space bar code
c_key       equ         $2E            ; c key code
d_key       equ         $20            ; d key code
p_key       equ         $19            ; p key code
v_key       equ         $2F            ; v key code


; ****************************************************************************

            rorg        $00


; ****************************************************************************
; The main program

start
            bsr         load_ouch
            bsr         load_begin
            bsr         load_over
            bsr         load_pause
            bsr         load_resume
            bsr         load_intro

            move.l      #$68000,sp
            bsr         read_hi_score_file

; enter supervisor state
            clr.l       -(sp)             ; supervisor's stack becomes user's
            move.w      #super,-(sp)      ; enter supervisor mode
            trap        #gemdos           ; call gemdos
            addq.l      #$06,sp           ; restore stack
; disable mouse and enable joystick
            move.l      #kb_commands,-(sp) ; pointer to string of commands
            move.w      #$01,-(sp)        ; number of commands
            move.w      #ikbdws,-(sp)     ; send string
            trap        #xbios
            addq.l      #$08,sp           ; restore stack
; set up MFP keyboard interrupt routine
            move.l      #kb_int,-(sp)     ; address of interrupt routine
            move.w      #$06,-(sp)        ; interrupt number $06
            move.w      #mfpint,-(sp)
            trap        #xbios            ; call xbios
            addq.l      #$08,sp           ; restore stack
; disable MFP timer interrupt routine
            move.l      #timer_int,-(sp)  ; address of interrupt routine
            move.w      #$05,-(sp)        ; interrupt number $05
            move.w      #mfpint,-(sp)
            trap        #xbios            ; call xbios
            addq.l      #$08,sp           ; restore stack
            clr.b       read_stick
            clr.b       waste_one

            bsr         play_intro

; set up screen
            move.w      #low_res,-(sp)    ; select low resolution
            move.l      #screen_mem,-(sp) ; and set physical
            move.l      #screen_mem,-(sp) ; and logical addresses
            move.w      #setscreen,-(sp)
            trap        #xbios
            add.l       #$0C,sp           ; restore stack
; set colours
            move.l      #pallette,-(sp)   ; pointer to colour pallette
            move.w      #setpallette,-(sp)
            trap        #xbios
            addq.l      #$06,sp           ; restore stack
; set interrupts
            or.w        #$0300,sr         ; level 4 upwards
; wait for colours to be set
            bsr         vb_wait
; set vbi
            move.l      #vbi,$70

play_again
            bsr         wait_fer_start    ; wait for a game to be started
            bsr         initialize
            move.w      this_number,number ; initial number of creatures
            move.l      #the_wave,A0      ; initial wave setting
            move.b      this_digit_1,(A0)
            move.b      this_digit_2,$01(A0)
            move.l      #bottom_line+$58,A1
            move.w      #$06,colour_mask
            bsr         print_string      ; display wave on status line
next_wave
            bsr         init_creatures
main_loop
            bsr         shimmer_missiles  ; shimmer missile data
            bsr         check_player      ; see if any creatures hit
            bsr         vb_wait           ; wait for the vertical blank period
            bsr         handle_stars      ; draw the stars
            clr.b       flag              ; first call
            bsr         handle_creatures  ; handle the creatures
            bsr         draw_player       ; draw the player
            bsr         move_missiles     ; move the missiles
            bsr         check_player      ; see if any creatures hit

            tst.b       need_ouch
            beq.s       no_ouch
            bsr         play_ouch
            clr.b       need_ouch
no_ouch

            bsr         shimmer_energy    ; shimmer energy image
            bsr         handle_keys
            bsr         handle_sound      ; do sounds
            bsr         vb_wait           ; wait for the vertical blank period
            bsr         handle_player     ; handle the player
            bsr         check_player      ; see if any creatures hit
            move.b      #$01,flag         ; second call
            bsr         handle_creatures  ; handle the creatures
            bsr         move_missiles     ; move the missiles
            tst.b       dead              ; is the player alive
            bne.b       thats_all_folks   
            tst.w       alive             ; are there any creatures left alive
            bpl         main_loop
            bsr         up_wave           ; move onto next wave
            cmpi.w      #$FF,number       ; maximum number of creatures reached
            beq         next_wave
            addq.w      #$01,number       ; increase number of creatures
            bra         next_wave
thats_all_folks
            bsr         explode           ; explode the player
            bsr         play_over
            tst.b       doing_demo        ; if we were doing a demo
            beq.b       demo_1_skip
            move.w      number_save,number ; then restore the status of the
            move.b      digit_1_save,the_wave ; previous game
            move.b      digit_2_save,the_wave+$01
demo_1_skip
            bra         play_again        ; and play another game


; ****************************************************************************
; Attempt to fire a missile.

try_to_fire
            tst.b       sleeping          ; test number of inactive missiles
            bne.b       heavy_metal
            bra         do_move           ; abort if zero
heavy_metal
            move.l      #m_info,A4        ; pointer to missile info
loop9
            tst.l       m_address(A4)     ; look at this missile's address
            beq         found_one         ; inactive if zero
            addq.l      #$08,A4           ; else move onto next
            bra         loop9
found_one
            move.l      address,m_address(A4) ; set missile's screen address
            addi.l      #$6E0,m_address(A4)   ; and move to middle of player
            move.b      xpos,m_xpos(A4)       ; set missile's x-position
            move.b      ypos,m_ypos(A4)       ; set missile's y-position
try_fire_up
            btst.b      #$00,stick        ; look at bit #$00
            beq.b       try_fire_down     ; if clear then try fire down
            move.b      #$FF,m_dy(A4)     ; set missile's dy to $FF
            bra         try_fire_left     ; and try fire left
try_fire_down
            btst.b      #$01,stick        ; look at bit #$01
            beq.b       no_vertical       ; if clear then no vertical movement
            move.b      #$01,m_dy(A4)     ; set missile's dy to $01
            bra         try_fire_left     ; and try fire left
no_vertical
            clr.b       m_dy(A4)          ; set missile's dy to $00
try_fire_left
            btst.b      #$02,stick        ; look at bit #$02
            beq.b       try_fire_right    ; if clear then try fire right
            move.b      #$FF,m_dx(A4)     ; set missile's dx to $FF
            bra         continue          ; and continue
try_fire_right
            btst.b      #$03,stick        ; look at bit #$03
            beq.b       check_okay        ; if clear then check okay
            move.b      #$01,m_dx(A4)     ; set missile's dx to $01
            add.l       #$08,m_address(A4)
            addq.b      #$01,m_xpos(A4)
            bra         continue          ; and continue
check_okay
            tst.b       m_dy(A4)          ; test missile's dy
            bne.b       has_direction     ; if not zero then okay
            clr.l       m_address(A4)     ; set missile's address to zero
            bra         resume            ; and resume
has_direction       
            clr.b       m_dx(A4)          ; set missile's dx to $00
continue
            subq.b      #$01,sleeping     ; decrement sleeping missiles
            or.l        #$80000000,m_address(A4)
            bra         resume            ; and resume


; ****************************************************************************
; Initialize the program.

initialize

            bsr         cls               ; clear screen
; print status line
            move.l      #$4FF,correction  ; set text correct value
            clr.w       height            ; use single height text
            move.w      #$03,colour_mask  ; use normal field text
            move.l      #status_line,A0   ; pointer to string
            move.l      #bottom_line,A1   ; screen position
            bsr         print_string      ; output string
            move.w      #$05,colour_mask
            bsr         print_string
            move.w      #$08,colour_mask
            bsr         print_string
; set strength
            move.l      #the_strength,A0  ; address of strength data
            clr.b       (A0)              ; set strength to 75%
            move.b      #$07,$01(A0)
            move.b      #$05,$02(A0)
            move.w      #$04,colour_mask
            move.l      #bottom_line+$29,A1 ; screen address for strength
            bsr         print_string
; clear score
            tst.b       doing_demo        ; if doing a demo
            beq.b       clear_score
            move.l      #demo_string,A0   ; then print 'demo'
            move.l      #bottom_line+$80,A1
            move.w      #$09,colour_mask
            bsr         print_string
            bra         init_variables
clear_score
            move.l      #the_score,A0     ; address of score data
            clr.b       (A0)
            clr.b       $01(A0)
            clr.b       $02(A0)
            clr.b       $03(A0)
            clr.b       $04(A0)
            clr.b       $05(A0)
            move.w      #$09,colour_mask
            move.l      #bottom_line+$80,A1 ; screen address for score
            bsr         print_string
init_variables
; initialise variables
            move.l      #screen_mem+$3248,address ; player's screen address
            move.b      #$09,xpos         ; player's x-position
            move.b      #$05,ypos         ; player's y-position
            move.l      #face_right,frame ; player's frame
            move.b      #total,sleeping   ; number of inactive missiles
            clr.b       dead              ; the player's alive!
            clr.b       need_ouch
            move.b      #$FF,key_code     ; keyboard press
            move.b      #$FF,pressed_code
            move.l      #m_info,A4        ; pointer to missile info
            move.w      #$03,D0           ; times to loop
go_to_sleep
            clr.l       m_address(A4)     ; set address to zero
            addq.l      #$08,A4           ; look at next missile
            dbra        D0,go_to_sleep    ; and repeat loop
            bsr         init_sound        ; set sound parameters
            bra         init_stars        ; initialise the backgroung stars


; ****************************************************************************
; Handle the player.

handle_player
            tst.b       doing_demo        ; if we're doing a demo
            beq.b       demo_2_skip
            move.w      #random,-(sp)     ; then request a random number
            trap        #xbios
            addq.l      #$02,sp           ; restore stack
            move.l      #stick_values,A0  ; get an appropriate joystick value
            andi.w      #$07,D0 
            move.b      $00(A0,D0.w),stick ; and use this instead of actually
            bra         demo_3_skip       ; reading the stick
demo_2_skip
            move.b      the_stick,stick   ; read the joystick
demo_3_skip
            tst.b       stick             ; test joystick
            bpl.b       do_move
            bra         try_to_fire       ; if trigger pressed then try to fire
do_move
            bsr         erase_player      ; remove player image
up_test
            btst.b      #$00,stick        ; look at bit #$00
            beq.b       down_test         ; if clear then try down
            tst.b       ypos              ; test player's y-position
            beq.b       left_test         ; if at top then try left
            subq.b      #$01,ypos         ; decrement player's y-position
            subi.l      #$A00,address     ; update player's screen address
            move.l      #face_up,frame    ; set player's frame
            bra         left_test         ; and try left
down_test
            btst.b      #$01,stick        ; look at bit #$01
            beq.b       left_test         ; if clear then try left
            cmpi.b      #max_down,ypos    ; test player's y-position
            beq.b       left_test         ; if at bottom then try left
            addq.b      #$01,ypos         ; increment player's y-position
            addi.l      #$A00,address     ; update player's screen address
            move.l      #face_down,frame  ; set player's frame
left_test
            btst.b      #$02,stick        ; look at bit #$02
            beq.b       right_test        ; if clear then try right
            tst.b       xpos              ; test player's x-position
            beq.b       resume            ; if at left then resume
            subq.b      #$01,xpos         ; decrement player's x-position
            subq.l      #$08,address      ; update player's screen address
            move.l      #face_left,frame  ; set player's frame
            bra         resume            ; and resume
right_test
            btst.b      #$03,stick        ; look at bit #$03
            beq.b       resume            ; if clear then resume
            cmpi.b      #max_right,xpos   ; test player's x-position
            beq.b       resume            ; if at right then resume
            addq.b      #$01,xpos         ; increment player's x-position
            addq.l      #$08,address      ; update player's screen address
            move.l      #face_right,frame ; set player's frame
resume
            bra         draw_player       ; draw new player image


; ****************************************************************************
; Erase player image.

erase_player

            move.l      address,A0        ; set pointer to screen area
            move.w      #$1F,D0           ; times to loop
loop3
            clr.l       (A0)+             ; planes 0 and 1
            clr.l       (A0)+             ; planes 2 and 3
            clr.l       (A0)+             ; planes 0 and 1
            clr.l       (A0)+             ; planes 2 and 3
            add.l       #$90,A0           ; move onto next line
            dbra        D0,loop3          ; repeat loop
            rts


; ****************************************************************************
; Draw player image.

draw_player

            move.l      address,A0        ; set pointer to screen area
            move.l      frame,A1          ; set pointer to player's frame
            move.w      #$1F,D0           ; times to loop
loop4
            move.l      (A1)+,(A0)+       ; planes 0 and 1
            move.l      (A1)+,(A0)+       ; planes 2 and 3
            move.l      (A1)+,(A0)+       ; planes 0 and 1
            move.l      (A1)+,(A0)+       ; planes 2 and 3
            add.l       #$90,A0           ; move onto next line
            dbra        D0,loop4          ; repeat loop
            rts


; ****************************************************************************
; Move all active missiles.

move_missiles

            cmpi.b      #total,sleeping   ; test all missiles
            beq.b       all_moved         ; and check if all moved

            move.l      #m_info,A4        ; pointer to missile info
            move.w      #$03,D0           ; times to loop
moving_loop
            tst.l       m_address(A4)     ; look at address
            bmi         virgin_missile
            bne         move_it           ; if not zero then missile is active
            bra         next_one
virgin_missile
            and.l       #$7FFFFFFF,m_address(A4)
            bra         forced_entry
next_one
            addq.w      #$08,A4           ; else move onto next missile
            dbra        D0,moving_loop
all_moved
            rts
move_it
            bsr         erase_missile     ; erase missile image
            bsr         check_for_hits
forced_entry
            tst.b       m_dy(A4)          ; test missile's dy
            bmi.b       going_up          ; if dy<0 then missile goes up
            bne.b       going_down        ; if dy>0 then missile goes down
re_entry_1
            tst.b       m_dx(A4)          ; test missile's dx
            bmi.b       going_left        ; if dx<0 then missile goes left
            bne.b       going_right       ; if dx>0 then missile goes right
re_entry_2
            tst.b       m_ypos(A4)        ; test missile's y-position
            bmi.b       out_of_bounds     ; check if off top
            cmpi.b      #max_down,m_ypos(A4) ; test missile's y-position
            bgt.b       out_of_bounds     ; check if off bottom
            tst.b       m_xpos(A4)        ; test missile's x-position
            bmi.b       out_of_bounds     ; check if off left
            cmpi.b      #max_right,m_xpos(A4) ; test missile's x-position
            bgt.b       out_of_bounds     ; check if off right
            bsr         check_for_hits
            bsr         draw_missile      ; draw missile image
            bra         next_one
out_of_bounds
            clr.l       m_address(A4)     ; set missile's address to zero
            addq.b      #$01,sleeping     ; another missile becomes inactive
            bra         next_one
going_up
            subq.b      #$01,m_ypos(A4)   ; decrement missile's y-position
            move.b      m_ypos(A4),reg_2  ; make noise!
            subi.l      #$A00,m_address(A4) ; update missile's screen address
            bra         re_entry_1        ; and return
going_down
            addq.b      #$01,m_ypos(A4)   ; increment missile's y-position
            move.b      m_ypos(A4),reg_2  ; make noise!
            addi.l      #$A00,m_address(A4) ; update missile's screen address
            bra         re_entry_1        ; and return
going_left
            subq.b      #$01,m_xpos(A4)   ; decrement missile's x-position
            move.b      m_xpos(A4),reg_2  ; make noise!
            subq.l      #$08,m_address(A4) ; update missile's screen address
            bra         re_entry_2        ; and return
going_right
            addq.b      #$01,m_xpos(A4)   ; increment missile's x-position
            move.b      m_xpos(A4),reg_2  ; make noise!
            addq.l      #$08,m_address(A4) ; update missile's screen address
            bra         re_entry_2        ; and return


; ****************************************************************************
; Erase a missile image.

erase_missile

            move.l      m_address(A4),A0  ; set pointer to screen area
            move.w      #$0F,D1           ; times to loop
loop6
            clr.l       (A0)+             ; planes 0 and 1
            clr.l       (A0)+             ; planes 2 and 3
            add.l       #$98,A0           ; move onto next line
            dbra        D1,loop6          ; repeat loop
            rts


; ****************************************************************************
; Draw a missile image.

draw_missile

            move.l      m_address(A4),A0  ; set pointer to screen area
            move.l      #missile_data,A1  ; set pointer to missile graphics
            move.w      #$09,D1           ; times to loop
loop7
            move.w      (A1),(A0)+        ; word 1
            move.w      (A1),(A0)+        ; word 2
            move.w      (A1),(A0)+        ; word 3
            move.w      (A1)+,(A0)+       ; word 4
            add.l       #$98,A0           ; move onto next line
            dbra        D1,loop7          ; repeat loop
            rts


; ****************************************************************************
; Check current missile for collision with any creatures.

check_for_hits

            move.l      #c_info,A5        ; pointer to creature info
            move.w      number,D1         ; times to loop
hit_loop
            tst.b       c_health(A5)      ; check creature's health
            ble.b       next_hit_test     ; no check if dead or dying
            move.b      m_xpos(A4),D2     ; find missile's x-position
            sub.b       c_xpos(A5),D2     ; subtract creature's x-position
            bmi.b       next_hit_test     ; missed (too far left)
            cmpi.b      #$02,D2
            bpl.b       next_hit_test     ; missed (too far right)
            move.b      m_ypos(A4),D2     ; find missile's y-position
            sub.b       c_ypos(A5),D2     ; subtract creature's y-position
            bpl.b       hit_skip          ; find absolute value
            neg.b       D2
hit_skip
            cmpi.b      #$02,D2
            bpl.b       next_hit_test     ; missed (vertically)
            move.b      #$E0,c_health(A5) ; kill creature
            move.l      #$A0,c_gap(A5)    ; initial gap
            move.b      #$04,reg_4
            move.b      #$05,counter
            addq.l      #$04,sp           ; remove return address from stack
            bra         out_of_bounds
next_hit_test
            add.l       #$0E,A5           ; look at next creature
            dbra        D1,hit_loop       ; and repeat loop
            rts


; ****************************************************************************
; Make a missile shimmer.

shimmer_missiles

            move.l      #missile_image,A4 ; base missile graphics
            move.l      #missile_data,A5  ; missile graphics actually used
            move.w      #$09,D4           ; times to loop
loop8
            move.w      #random,-(sp)     ; request random number
            trap        #xbios            ; call xbios
            addq.l      #$02,sp           ; restore stack
            move.w      (A4)+,D5          ; get base image
            and.w       D0,D5             ; mask some of it out
            move.w      D5,(A5)+          ; and store in used image
            dbra        D4,loop8
            rts


; ****************************************************************************
; Wait for vertical blank period.

vb_wait
            move.l      frclock,D0        ; find value of frame clock
loop5
            cmp.l       frclock,D0        ; compare with current value
            beq.l       loop5             ; repeat if equal
            rts


; ****************************************************************************
; Initialise the stars.

init_stars

               move.l   #star_x,A5           ; initial star positions
               move.w   #$2E,D7
get_position
               move.w   #random,-(sp)        ; get random position
               trap     #xbios
               addq.l   #$02,sp
               andi.w   #$FE,D0              ; mask appropriately
               cmpi.w   #$A0,D0              ; and check range
               bcc      get_position
               move.w   D0,(A5)+             ; set position
               dbra     D7,get_position
               bra      draw_stars           ; draw the stars


; ****************************************************************************
; Update the stars.

handle_stars

               bsr      move_stars           ; calculate new postions


; ****************************************************************************
; Draw the stars.

draw_stars
               move.l   #screen_mem,A0       ; locate screen memory
               move.l   #star_x,A5           ; locate star positions
               move.w   #$16,D0
               move.w   even_bits,D1         ; bit image for even stars
               move.w   odd_bits,D2          ; bit image for odd stars
draw_next
               move.w   (A5)+,A1             ; locate star offset
               move.w   D1,$00(A0,A1.w)      ; store even bit image
               add.l    #$280,A0
               move.w   (A5)+,A1             ; locate star offset
               move.w   D2,$00(A0,A1.w)      ; store odd bit image
               add.l    #$280,A0
               dbra     D0,draw_next
               rts


; ****************************************************************************
; Move the stars.

move_stars
               cmpi.w   #%1000000000000000,even_bits
               bne      okay1
               bsr      course_even_scroll   ; course scroll even stars
okay1
               rol.w    even_bits            ; fine scroll even bit image

               cmpi.w   #%0100000000000000,odd_bits
               bne      okay2
               bsr      course_odd_scroll    ; course scroll odd stars
okay2
               rol.w    odd_bits             ; fine scroll odd bit image
               rol.w    odd_bits
               rts


; ****************************************************************************
; Course scroll the even stars (slow plane).

course_even_scroll
               move.l   #$00,A0              ; no offset from screen start
               move.l   #star_x,A5           ; set address of first even star
               bra      all_scroll           ; perform the scroll


; ****************************************************************************
; Course scroll the odd stars (fast plane).

course_odd_scroll
              move.l    #$280,A0             ; offset from screen start
              move.l    #star_x+$02,A5       ; set address of first odd star


; ****************************************************************************
; Perform the scroll.

all_scroll
               add.l    #screen_mem,A0
               move.w   #$16,D1
next_scroll
               move.w   (A5),D2              ; get star offset
               clr.w    $00(A0,D2.w)         ; remove old image
               add.l    #$500,A0
               cmpi.w   #$08,D2              ; look at offset
               bpl      okay3                ; check if zero
               add.w    #$A0,D2              ; reset position
okay3
               subi.w   #$08,D2              ; move left (course)
               move.w   D2,(A5)              ; store new offset
               addq.l   #$04,A5              ; move onto next star
               dbra     D1,next_scroll
               rts


; ****************************************************************************
; Initialise a wave of creatures.

init_creatures

               move.w   number,D7            ; number of creatures in wave
               move.l   #c_info,A5           ; address of creature info
init_next_creature
               move.w   #random,-(sp)        ; request random number
               trap     #xbios
               addq.l   #$02,sp
               btst.b   #$00,D0              ; look at bit #$00
               beq.b    type2                ; if zero then set other type
type1
               move.l   #directions,A0       ; pointer to available directions
               andi.w   #$03,D0
               move.b   $00(A0,D0.w),c_dy(A5) ; set creature's dy
repeat_1
               move.w   #random,-(sp)        ; request random number
               trap     #xbios
               addq.l   #$02,sp
               andi.b   #$0E,D0              ; mask off bits not required
               cmpi.b   #max_down,D0         ; check if off bottom
               bgt.b    repeat_1
               move.b   D0,c_ypos(A5)        ; set creature's y-position
               btst.b   #$01,c_dy(A5)        ; look at bit #$01
               beq.b    set2                 ; if zero then set appropriately
set1
               clr.b    c_xpos(A5)           ; zero creature's x-position
               move.b   #$01,c_dx(A5)        ; set creature's dx
               bra      rejoin
set2
               move.b   #max_right,c_xpos(A5) ; set creature's x-position
               move.b   #$FF,c_dx(A5)        ; set creature's dx
               bra      rejoin
type2
               move.l   #directions,A0       ; pointer to available directions
               andi.w   #$03,D0
               move.b   $00(A0,D0.w),c_dx(A5) ; set creature's dx
repeat_2
               move.w   #random,-(sp)        ; request random number
               trap     #xbios
               addq.l   #$02,sp
               andi.b   #$1E,D0              ; mask off bits not required
               cmpi.b   #max_right,D0        ; check if off right
               bgt.b    repeat_2
               move.b   D0,c_xpos(A5)        ; set creature's x-position
               btst.b   #$01,c_dx(A5)        ; look at bit #$01
               beq.b    set4                 ; if zero then set appropriately
set3
               clr.b    c_ypos(A5)           ; zero creature's y-position
               move.b   #$01,c_dy(A5)        ; set creature's dy
               bra      rejoin
set4
               move.b   #max_down,c_ypos(A5) ; set creature's y-position
               move.b   #$FF,c_dy(A5)        ; set creature's dy
rejoin
               move.b   c_ypos(A5),D0        ; evaluate screen memory postion
               ext.w    D0
               mulu     #$A00,D0
               addi.l   #screen_mem,D0
               move.b   c_xpos(A5),D1
               ext.w    D1
               mulu     #$08,D1
               add.l    D0,D1
               move.l   D1,c_address(A5)     ; set creature's screen memory
                                             ; address
               move.w   #random,-(sp)        ; request random number
               trap     #xbios
               addq.l   #$02,sp
               move.b   D0,c_sleep(A5)       ; set creature's sleep
               andi.b   #$1C,D0
               ext.w    D0
               move.l   #the_types,A0
               move.l   $00(A0,D0.w),c_type(A5) ; set creature's type
               move.b   #$01,c_health(A5)    ; set creature's health
               add.l    #$0E,A5
               dbra     D7,init_next_creature
               move.w   number,alive         ; set number of creatures alive
               rts


; ****************************************************************************
; Handle creatures.

handle_creatures

               move.w   number,D7            ; number of creatures in wave
               move.l   #c_info,A5           ; pointer to creature info
creature_loop
               tst.b    c_health(A5)         ; look at creature's health
               beq.b    find_next            ; do nothing if dead
               bpl.b    test_sleeping        ; if alive then test if asleep
               cmpi.l   #glowing_energy,c_type(A5)
               bne      really_dying         ; collected an energy?
               addq.b   #$01,c_health(A5)    ; and hasn't faded away yet
               bne.b    still_glowing
               bsr      erase_creature
               bra      go_to_heaven_2
still_glowing
               bsr      draw_creature        ; then draw glowing energy
               move.b   glowing_energy,preg_0
               bra      find_next
really_dying
               bsr      erase_dying          ; erase dying image
               addq.b   #$01,c_health(A5)    ; illness becomes worse
               beq.b    go_to_heaven_1       ; has the creature died?
               subi.l   #$500,c_address(A5)  ; update screen address
               addi.l   #$A0,c_gap(A5)       ; increase gap
               bsr      draw_dying           ; draw dying creature
               bra      find_next
go_to_heaven_1
               cmpi.l   #energy,c_type(A5)
               beq      go_to_heaven_2       ; no points for shooting energies
               bsr      up_score
go_to_heaven_2
               subq.w   #$01,alive
               bra      find_next
test_sleeping
               tst.b    c_sleep(A5)          ; test sleeping
               beq.b    move_creature        ; and move if awake
               subq.b   #$01,c_sleep(A5)     ; else reduce sleep
next_creature
               bsr      draw_creature        ; draw image
find_next
               add.l    #$0E,A5              ; move pointer
               dbra     D7,creature_loop     ; and repeat loop
               rts
move_creature
               tst.b    flag                 ; if this is the first call
               bne.b    continue_moving
               cmpi.w   #$27,number          ; only move if wave 40 or above
               blt      next_creature
continue_moving
               bsr      erase_creature       ; remove image
               tst.b    c_dy(A5)             ; look at y direction
               bmi.b    creature_up
               bne.b    creature_down
re_enter
               tst.b    c_dx(A5)             ; look at x_direction
               bmi.b    creature_left
               bne.b    creature_right
               bra      next_creature
creature_up
               tst.b    c_ypos(A5)           ; look at y-position
               bne.b    can_go_up            ; check whether if at top
               move.b   #$01,c_dy(A5)        ; reverse y direction
               move.b   #$20,c_sleep(A5)     ; and have a short nap
               bra      next_creature
can_go_up
               subq.b   #$01,c_ypos(A5)      ; decrement y-position
               move.b   c_ypos(A5),reg_0     ; make noise!
               subi.l   #$A00,c_address(A5)  ; update screen address
               bra      re_enter
creature_down
               cmpi.b   #max_down,c_ypos(A5) ; look at y-position
               bmi.b    can_go_down          ; check whether at bottom
               move.b   #$FF,c_dy(A5)        ; reverse y direction
               move.b   #$20,c_sleep(A5)     ; and have a short nap
               bra      next_creature
can_go_down
               addq.b   #$01,c_ypos(A5)      ; increment y-position
               move.b   c_ypos(A5),reg_0     ; make noise!
               addi.l   #$A00,c_address(A5)  ; update screen address
               bra      re_enter
creature_left
               tst.b    c_xpos(A5)           ; look at x-position
               bne.b    can_go_left          ; check whether at left
               move.b   #$01,c_dx(A5)        ; reverse x direction
               move.b   #$20,c_sleep(A5)     ; and have a short nap
               bra      next_creature
can_go_left
               subq.b   #$01,c_xpos(A5)      ; decrement x-position
               move.b   c_xpos(A5),reg_0     ; make noise!
               subq.l   #$08,c_address(A5)   ; update screen address
               bra      next_creature
creature_right
               cmpi.b   #max_right,c_xpos(A5) ; look at x-position
               bmi.b    can_go_right         ; check whether at right
               move.b   #$FF,c_dx(A5)        ; reverse x direction
               move.b   #$20,c_sleep(A5)     ; and have a short nap
               bra      next_creature
can_go_right
               addq.b   #$01,c_xpos(A5)      ; increment x-position
               move.b   c_xpos(A5),reg_0     ; make noise!
               addq.l   #$08,c_address(A5)   ; update screen address
               bra      next_creature


; ****************************************************************************
; Erase a creature.

erase_creature

               move.l   c_address(A5),A0     ; pointer to screen address
               move.w   #$1F,D0              ; times to loop
e_loop
               clr.l    (A0)+                ; planes 0 and 1
               clr.l    (A0)+                ; planes 2 and 3
               clr.l    (A0)+                ; planes 0 and 1
               clr.l    (A0)+                ; planes 2 and 3
               add.l    #$90,A0              ; update pointer
               dbra     D0,e_loop            ; repeat loop
               rts


; ****************************************************************************
; Draw a creature.

draw_creature

               move.l   c_address(A5),A0     ; pointer to screen address
               move.l   c_type(A5),A1        ; pointer to type
               move.w   #$1F,D0              ; times to loop
d_loop
               move.l   (A1)+,(A0)+          ; planes 0 and 1
               move.l   (A1)+,(A0)+          ; planes 2 and 3
               move.l   (A1)+,(A0)+          ; planes 0 and 1
               move.l   (A1)+,(A0)+          ; planes 2 and 3
               add.l    #$90,A0              ; update pointer
               dbra     D0,d_loop            ; repeat loop
               rts


; ****************************************************************************
; Check for collision between player and any creatures.

check_player

               move.l   #c_info,A0           ; pointer to creature info
               move.w   number,D0            ; number of times to loop
check_loop
               tst.b    c_health(A0)         ; check creature's health
               ble.b    check_next           ; do nothing if dead
               move.b   c_xpos(A0),D1        ; find creature's x-position
               sub.b    xpos,D1              ; subtract player's x-position
               bpl.b    check_skip_1         ; take absolute value
               neg.b    D1
check_skip_1
               cmpi.b   #$02,D1
               bpl.b    check_next           ; missed (horizontally)
               move.b   c_ypos(A0),D1        ; find creature's y-position
               sub.b    ypos,D1              ; subtract player's y-position
               bpl.b    check_skip_2         ; take absolute value
               neg.b    D1
check_skip_2
               cmpi.b   #$02,D1
               bpl.b    check_next           ; missed (vertically)
               cmpi.l   #energy,c_type(A0)   ; test type
               beq      good_one             ; collected an energy
               move.b   #$E0,c_health(A0)    ; kill creature
               move.l   #$A0,c_gap(A0)       ; set initial gap
               move.b   #$04,reg_4
               move.b   #$05,counter
               move.b   #$01,need_ouch
               bsr      down_strength        ; decrement strength
check_next
               add.l    #$0E,A0              ; update creature pointer
               dbra     D0,check_loop        ; and repeat loop
               rts
good_one
               move.b   #$D0,c_health(A0)
               move.l   #glowing_energy,c_type(A0)
               bsr      up_strength          ; increment strength
               bra      check_next


; ****************************************************************************
; Erase a dying creature image.

erase_dying

               move.l   c_address(A5),A0     ; screen memory address
               move.w   #$1F,D0              ; times to loop
ed_loop
               cmpa.l   #screen_mem,A0       ; check not off top
               blt      ed_skip
               cmpa.l   #bottom_line,A0      ; check not off bottom
               bgt      ed_exit
               clr.l    (A0)                 ; planes 0 and 1
               clr.l    $04(A0)              ; planes 2 and 3
               clr.l    $08(A0)              ; planes 0 and 1
               clr.l    $0C(A0)              ; planes 2 and 3
ed_skip
               add.l    c_gap(A5),A0         ; add gap
               dbra     D0,ed_loop           ; and repeat loop
ed_exit
               rts


; ****************************************************************************
; Draw a dying creature.

draw_dying

               move.l   c_address(A5),A0     ; screen memory address
               move.l   c_type(A5),A1        ; creature image pointer
               move.w   #$1F,D0              ; times to loop
dd_loop
               cmpa.l   #screen_mem,A0       ; check not off top
               blt      dd_skip
               cmpa.l   #bottom_line,A0      ; check not off bottom
               bgt      dd_exit
               move.l   (A1),(A0)            ; planes 0 and 1
               move.l   $04(A1),$04(A0)      ; planes 2 and 3
               move.l   $08(A1),$08(A0)      ; planes 0 and 1
               move.l   $0C(A1),$0C(A0)      ; planes 2 and 3
dd_skip
               add.l    c_gap(A5),A0         ; add gap
               add.l    #$10,A1              ; look at next line of creature
               dbra     D0,dd_loop           ; and repeat loop
dd_exit
               rts


; ****************************************************************************
; Increment the wave display.

up_wave

               move.l   #the_wave+$02,A0     ; point to wave digits
               move.w   #$01,D0
waving
               subq.l   #$01,A0              ; look at next digit
               addq.b   #$01,(A0)            ; increment digit
               cmpi.b   #$0A,(A0)            ; reached 10?
               bmi.b    print_wave
               clr.b    (A0)                 ; reset digit
               dbra     D0,waving            ; and repeat loop
print_wave
               move.l   #the_wave,A0
               move.l   #bottom_line+$58,A1
               move.w   #$06,colour_mask
               bra      print_string


; ****************************************************************************
; Increment the score display.

up_score

               tst.b    doing_demo
               beq.b    get_some_score
               rts
get_some_score
               move.l   #the_score+$05,A0    ; point to score digits
               move.w   #$04,D0
scoring
               subq.l   #$01,A0              ; look at next digit
               addq.b   #$01,(A0)            ; increment digit
               cmpi.b   #$0A,(A0)            ; reached 10?
               bmi.b    print_score
               clr.b    (A0)                 ; reset digit
               dbra     D0,scoring           ; and repeat loop
print_score
               move.l   #the_score,A0
               move.l   #bottom_line+$80,A1
               move.w   #$09,colour_mask
               bra      print_string


; ****************************************************************************
; Increase the player's strength.

up_strength

               tst.b    the_strength         ; check m.s. digit of strength
               bne.b    at_maximum           ; cannot go above 100%
               movem.l  A0-A1,-(sp)          ; save registers
               move.l   #the_strength+$02,A0 ; pointer to strength data
               addq.b   #$06,(A0)            ; update strength total by 6
               cmpi.b   #$0A,(A0)            ; if digit has exceeded 9
               bmi.b    print_strength
               sub.b    #$0A,(A0)            ; then subtract 10
               subq.l   #$01,A0              ; and move onto next
               addq.b   #$01,(A0)            ; increment digit
               cmpi.b   #$0A,(A0)            ; if digit has exceeded 9
               bmi.b    print_strength
               subq.l   #$01,A0              ; then move onto next
               clr.b    $02(A0)              ; and set maximum strength
               clr.b    $01(A0)
               move.b   #$01,(A0)
print_strength
               move.l   #the_strength,A0
               move.l   #bottom_line+$29,A1
               move.w   #$04,colour_mask
               bsr      print_string         ; output the strength
               movem.l  (sp)+,A0-A1          ; restore registers
at_maximum
               rts


; ****************************************************************************
; Decrease the player's strength.

down_strength

               movem.l  A0-A1,-(sp)          ; save registers
               move.l   #the_strength+$02,A0 ; pointer to strength data
               subq.b   #$04,(A0)            ; decrement strength total by 4
               bpl.b    output_strength      ; if digit is below zero
               add.b    #$0A,(A0)            ; then add 10
               subq.l   #$01,A0              ; and move onto next
               subq.b   #$01,(A0)            ; decrement digit
               bpl.b    output_strength      ; if digit is below zero
               add.b    #$0A,(A0)            ; then add 10
               subq.l   #$01,A0              ; then move onto next
               subq.b   #$01,(A0)            ; decrement digit
               bpl.b    output_strength      ; if digit below zero
               clr.b    $02(A0)              ; then set to minimum strength
               clr.b    $01(A0)
               clr.b    (A0)
               move.b   #$01,dead            ; and kill off player
output_strength
               move.l   #the_strength,A0
               move.l   #bottom_line+$29,A1
               move.w   #$04,colour_mask
               bsr      print_string         ; output the strength
               movem.l  (sp)+,A0-A1          ; restore registers
               rts


; ****************************************************************************
; Shimmer the energy image.

shimmer_energy

               move.l   #energy,A4           ; address of energy image
               move.l   #glowing_energy,A5   ; address of glowing energy
               move.w   #$1F,D4              ; times to loop
shimmering
               move.w   #random,-(sp)        ; request random number
               trap     #xbios
               addq.l   #$02,sp
               move.w   D0,D1                ; copy random number to D1
               swap.w   D1                   ; copy low word to high
               add.l    D1,D0                ; add to give full 32-bit random
               move.l   D0,D1
               and.l    (A4)+,D1             ; mask out random bits
               move.l   D1,(A5)+             ; store image
               move.l   D0,D1
               and.l    (A4)+,D1             ; mask out random bits
               move.l   D1,(A5)+             ; store image
               move.l   D0,D1
               and.l    (A4)+,D1             ; mask out random bits
               move.l   D1,(A5)+             ; store image
               move.l   D0,D1
               and.l    (A4)+,D1             ; mask out random bits
               move.l   D1,(A5)+             ; store image
               dbra     D4,shimmering        ; and repeat loop
               rts


; ****************************************************************************
; Make shattering sounds!

zounds
               tst.b    volume
               bne.b    its_still_noisy
               rts
its_still_noisy
               move.w   #random,-(sp)        ; request random number
               trap     #xbios
               addq.l   #$02,sp              ; restore stack
               move.b   #$00,register        ; sound channel A frequency
               move.b   D0,frequency
               move.w   #random,-(sp)        ; request random number
               trap     #xbios
               addq.l   #$02,sp              ; restore stack
               move.b   #$02,register        ; sound channel B frequency
               move.b   D0,frequency
               move.w   #random,-(sp)        ; request random number
               trap     #xbios
               addq.l   #$02,sp              ; restore stack
               move.b   #$04,register        ; sound channel C frequency
               move.b   D0,frequency
               rts


; ****************************************************************************
; Draw and erase an explosion.

draw_explosion
erase_explosion

               move.l   #e_addr_1,A0         ; pointer to first address
               move.w   #$03,D0              ; times to loop
z_loop_1
               move.l   (A0)+,A1             ; get address
               move.w   #$03,D1              ; times to loop
z_loop_2
               move.w   #$07,D2              ; times to loop
z_loop_3
               cmpa.l   #screen_mem,A1
               blt      z_skip
               cmpa.l   #bottom_line,A1      ; off bottom?
               bgt      z_exit
               eori.b   #$FF,(A1)            ; draw piece
z_skip
               add.l    #$A0,A1              ; move onto next line
               dbra     D2,z_loop_3          ; and repeat loop
               add.l    distance,A1          ; add distance between bits
               dbra     D1,z_loop_2          ; and repeat loop
z_exit
               dbra     D0,z_loop_1          ; and move onto next
               rts


; ****************************************************************************
; Explode the player.

explode

; set sound parameters
               tst.b    volume
               beq.b    skip_sounds
               move.l   #noises,A0           ; address of commands
               bsr      set_sound            ; send to sound chip
               bsr      zounds
skip_sounds
; set initial addresses
               move.l   address,A0           ; get address of player
               move.l   #e_addr_1,A1         ; pointer to addresses
               move.w   #$03,D0              ; times to loop
x_loop_1
               move.l   A0,(A1)+             ; set address
               addq.l   #$01,A0              ; update address
               dbra     D0,x_loop_1
               clr.l    distance             ; initially no gap
               bsr      vb_wait
               bsr      draw_explosion       ; draw first explosion
               bsr      shatter_player
               bsr      zounds
               move.w   #$8B,D3              ; times to loop
x_loop_2
               bsr      vb_wait
               bsr      erase_explosion      ; remove explosion
               addi.l   #$A0,distance        ; increase gap
               subi.l   #$A8,e_addr_1        ; update addresses
               subi.l   #$A4,e_addr_2
               subi.l   #$9C,e_addr_3
               subi.l   #$98,e_addr_4
               bsr      draw_explosion       ; draw new explosion
               bsr      shatter_player
               bsr      zounds
               dbra     D3,x_loop_2          ; and repeat loop
               move.l   #off_sounds,A0       ; kill noise
               bsr      set_sound
               rts


; ****************************************************************************
; Initialise sound.

init_sound

               move.l   #sounds,A0           ; address of commands
               bsr      set_sound            ; send to sound chip
               clr.b    preg_0
               clr.b    reg_0
               clr.b    reg_2
               clr.b    reg_4
               rts


; ****************************************************************************
; Handle sound.

handle_sound
               tst.b    volume            ; if the volume's turned off
               bne.b    its_noisy
               rts                        ; then do nothing
its_noisy
               move.b   preg_0,D0         ; if an energy's collected
               bne.b    sound_skip_1      ; then use this noise for channel A
               move.b   reg_0,D0          ; else use the creature's moving
               beq.b    sound_skip_2      ; noise
               lsl.b    #$02,D0
sound_skip_1
               move.b   #$00,register     ; channel A pitch
               move.b   D0,frequency
               move.b   #$0D,register     ; waveform
               move.b   #$09,frequency
sound_skip_2
               move.b   #$06,register     ; noise pitch (used by channel B)
               move.b   reg_2,D0          ; set missile firing noise
               move.b   D0,frequency
               beq.b    sound_skip_3      ; but don't set volume if no missiles
               move.b   #$0E,D0
sound_skip_3
               move.b   #$09,register     ; channel B volume
               move.b   D0,frequency

               move.b   reg_4,D0          ; if a creature's dying
               beq.b    sound_skip_4
               lsl.b    #$05,D0           ; then use this noise for channel C
               move.b   #$04,register     ; channel C pitch
               move.b   D0,frequency
               move.b   #$0A,register     ; channel C volume
               move.b   counter,D0
               beq.b    sound_skip_5      ; if 'envelope' hasn't expired
               add.b    #$0A,D0           ; then set volume
sound_skip_5
               move.b   D0,frequency
               subq.b   #$01,counter      ; decrement counter
               bpl.b    sound_skip_4      ; if it's expired
               subq.b   #$01,reg_4        ; then reduce pitch
               move.b   #$05,counter      ; and reset counter
sound_skip_4
               clr.b    preg_0            ; clear 'shadow' registers for next
               clr.b    reg_0             ; pass
               clr.b    reg_2
               rts


; ****************************************************************************
; MFP keyboard interupt.

kb_int

               move.l   D0,-(sp)          ; save registers
               move.b   $FFFFFC02,D0      ; read keyboard ACIA
               tst.b    waste_one         ; ignore it?
               bne.b    throw_away
               tst.b    read_stick        ; use as joystick value?
               bne.b    set_stick
               cmp.b    #$FF,D0           ; joystick 2 header?
               bne.b    no_joy_2
               move.b   #$01,read_stick
               bra      kb_exit
no_joy_2
               cmp.b    #$FE,D0           ; joystick 1 header?
               bne.b    no_joy_1
               move.b   #$01,waste_one
               bra      kb_exit
no_joy_1
               btst.b   #$07,D0           ; if key released
               beq.b    key_pressed
key_released
               and.b    #$7F,D0           ; then mask
               cmp.b    pressed_code,D0
               bne.b    key_error
               move.b   D0,key_code       ; and set code
               move.b   #$FF,pressed_code
kb_exit
               bclr.b   #$06,$FFFFFA11    ; clear interrupt bit
               move.l   (sp)+,D0          ; restore registers
               rte
throw_away
               clr.b    waste_one         ; ignore input from joystick in port
               bra      kb_exit           ; #1
set_stick
               move.b   D0,the_stick      ; set the joystick value
               clr.b    read_stick        ; and clear joystick request
               bra      kb_exit
key_pressed
               move.b   D0,pressed_code   ; save the keyboard code
               bra      kb_exit
key_error
               move.b   #$FF,pressed_code ; lose previous input
               bra      kb_exit


; ****************************************************************************
; MFP timer interrupt.

timer_int

               add.l    #$11,hz_200       ; update 200 Hz timer (faster than
                                          ; normal)
               bclr.b   #$05,$FFFFFA11    ; clear interrupt service bit
               rte


; ****************************************************************************
; clear screen memory

cls

            move.l      #screen_mem,A0    ; set screen address pointer
            moveq.l     #$00,D0           ; set value to be stored
            move.w      #$7FF,D1          ; times to loop
loop1
            move.l      D0,(A0)+          ; clear screen word (long)
            move.l      D0,(A0)+          ; again
            move.l      D0,(A0)+          ; and again
            move.l      D0,(A0)+          ; and yet again
            dbra        D1,loop1          ; repeat loop
            rts


; ****************************************************************************
; Check play time keys.

handle_keys

               cmpi.b   #p_key,key_code   ; dump screen?
               bne.b    dont_dump
               bsr      print_screen      ; then print screen to printer
               bra      key_exit
dont_dump
               cmpi.b   #v_key,key_code   ; volume_toggle?
               bne.b    leave_volume
               eori.b   #$01,volume       ; toggle volume
               beq.b    kill_volume       ; if volume enabled
enable_volume
               move.l   #on_sounds,A0     ; then set sound parameters
               bsr      set_sound
               clr.b    reg_4             ; clear registers possibly previously
               bra      key_exit          ; set when quiet
kill_volume
               move.l   #off_sounds,A0    ; else clear sound parameters
               bsr      set_sound
               bra      key_exit
leave_volume
               cmpi.b   #esc_key,key_code ; escape pressed?
               bne.b    no_abort
               move.b   #$01,dead         ; then abort
               bra      key_exit
no_abort
               cmpi.b   #space_bar,key_code ; space bar pressed?
               bne.b    key_exit
               bsr      play_pause
               bsr      vb_wait           ; wait for vertical blank
               bsr      clear_play_area   ; clear screen play area
               move.l   #$13FF,correction ; use 4-times height text
               move.w   #$03,height
               move.w   #$06,colour_mask
               move.l   #paused,A0
               move.l   #screen_mem+$31A1,A1
               bsr      print_string      ; print 'paused' in middle of screen
               move.l   #$4FF,correction  ; restore attributes for text
               clr.w    height
               move.b   #$FF,key_code     ; reset key press
               move.l   #%00000000111111110000111100001111,D7 ; base image
still_paused
               move.l   D7,D1             ; copy bit image
               move.l   #$78002,A0        ; pointer to start of screen
               bsr      vb_wait
               move.w   #$4F,D2           ; loop 80 times
lay_1
               move.w   #$13,D3           ; loop 20 times
lay_2
               move.l   D1,(A0)           ; copy image
               addq.l   #$08,A0
               dbra     D3,lay_2          ; and repeat loop
               rol.l    #$01,D1           ; scroll diagonally
               dbra     D2,lay_1          ; and repeat loop (next scan)
               move.w   #$1F,D2           ; loop 32 times
lay_3
               move.w   #$07,D3           ; loop 8 times (part of a scan)
lay_4
               move.l   D1,$60(A0)        ; copy image
               move.l   D1,(A0)           ; and again
               addq.l   #$08,A0
               dbra     D3,lay_4          ; and repeat loop
               rol.l    #$01,D1           ; scroll diagonally
               add.l    #$60,A0           ; update screen pointer
               dbra     D2,lay_3          ; and repeat loop (next scan)
               move.w   #$4F,D2           ; loop 80 times
lay_5
               move.w   #$13,D3           ; loop 20 times
lay_6
               move.l   D1,(A0)           ; copy image
               addq.l   #$08,A0
               dbra     D3,lay_6          ; and repeat loop
               rol.l    #$01,D1           ; scroll diagonally
               dbra     D2,lay_5          ; and repeat loop (next scan)
               rol.l    #$01,D7           ; scroll base image (horizontally)
               cmpi.b   #space_bar,key_code ; if space bar's not pressed
               bne.b    still_paused      ; then still paused!
               bsr      play_resume
               bsr      clear_play_area   ; otherwise clear the play area
               move.l   #on_sounds,A0     ; resume sound effects
               bsr      set_sound
key_exit
               move.b   #$FF,key_code     ; clear key code
               rts


; ****************************************************************************
; Clear the play area.

clear_play_area

               move.l   #$78000,A0        ; pointer to screen memory
               clr.l    D0                ; clear source
               move.w   #$77F,D1          ; times to loop
clearing
               move.l   D0,(A0)+          ; clear word (long)
               move.l   D0,(A0)+          ; again
               move.l   D0,(A0)+          ; and again
               move.l   D0,(A0)+          ; and yet again
               dbra     D1,clearing       ; and repeat loop
               rts


; ****************************************************************************
; Set sound parameters.

set_sound

               cmpi.b   #$FF,(A0)         ; list finished?
               beq.b    sounded
               move.b   (A0)+,register    ; if not then select register
               move.b   (A0)+,frequency   ; and write data
               bra      set_sound         ; and repeat
sounded
               rts


; ****************************************************************************
; Wait for user to start game.

wait_fer_start
               bsr      init_hi_score     ; initialize the high score screen
               move.b   #$FF,key_code     ; reset key code
               move.b   #$FF,pressed_code
               clr.b    doing_demo        ; clear demo flag
waiting
               bsr      vb_wait           ; sync to vertical blank
               bsr      animate_hi_score  ; animate the high score screen
               tst.b    add_high          ; if we're adding a high score
               beq.b    try_to_start
               bsr      add_a_name        ; then read in a name
               bra      waiting
try_to_start
               cmpi.b   #d_key,key_code   ; demo?
               beq.b    do_a_demo
               cmpi.b   #c_key,key_code   ; continue?
               beq.b    carry_on
               move.l   #level_info,A0    ; keys to start game
               move.w   #$05,D0           ; times to loop
looking_for_key
               move.b   (A0)+,D1          ; get key code
               cmp.b    key_code,D1       ; are they the same?
               beq.b    started_at_level  ; then start
               addq.l   #$03,A0           ; else update pointer
               dbra     D0,looking_for_key ; and repeat loop
               bra      waiting
started_at_level
               move.b   (A0)+,D0          ; get number of creatures in wave
               ext.w    D0
               move.w   D0,this_number    ; and set number
               move.b   (A0)+,this_digit_1 ; get first digit in display
               move.b   (A0)+,this_digit_2 ; get second digit in display
               bra      colour_restore
carry_on
               move.w   number,this_number ; get previous number
               move.b   the_wave,this_digit_1 ; get previous digits
               move.b   the_wave+$01,this_digit_2
               bra      colour_restore
do_a_demo
               move.w   number,number_save ; save the number of creatures
               move.b   the_wave,digit_1_save ; save digits
               move.b   the_wave+$01,digit_2_save
               move.w   #$00,this_number  ; set number to 1
               move.b   #$00,this_digit_1 ; and set digits
               move.b   #$01,this_digit_2
               move.b   #$01,doing_demo   ; set demo flag
colour_restore
               bsr      play_begin
               bsr      vb_wait
               move.l   #pallette,A0
               move.l   #$FFFF8240,A1
               move.w   #$0F,D0
scl2loop
               move.w   (A0)+,(A1)+
               dbra     D0,scl2loop
               rts


; ****************************************************************************
; Variables and data.

need_ouch      ds.b     $01

the_strength
               ds.b     $03               ; space to hold strength string
               dc.b     $FF
the_wave
               dc.b     $00               ; space to hold wave string
               dc.b     $01
               dc.b     $FF
the_score
               ds.b     $06               ; space to hold score string
               dc.b     $FF
directions
               dc.b     $00               ; possible creature directions
               dc.b     $01
               dc.b     $FF
stick_values                              ; directions for demo
               dc.b     %10000001         ; up
               dc.b     %10000010         ; down
               dc.b     %10000100         ; left
               dc.b     %10001000         ; right
               dc.b     %10000101         ; up and left
               dc.b     %10000110         ; down and left
               dc.b     %10001001         ; up and right
               dc.b     %10001010         ; down and right
this_digit_1
               ds.b     $01               ; this levels initial digits
this_digit_2
               ds.b     $01
digit_1_save
               ds.b     $01               ; digits save for demo
digit_2_save
               ds.b     $01
preg_0
               ds.b     $01               ; overide sound channel A
reg_0
               ds.b     $01               ; sound channel A frequency
reg_2
               ds.b     $01               ; sound channel B frequency
reg_4
               ds.b     $01               ; sound channel C frequency
counter
               ds.b     $01
read_stick
               ds.b     $01               ; next interrupt joystick?
waste_one
               ds.b     $01               ; waste next interrupt?
pressed_code
               ds.b     $01               ; key press save
kb_commands
               dc.b     $12,$14           ; keyboard processor commands
xpos
               ds.b     $01               ; player's x-position
ypos
               ds.b     $01               ; player's y-position
dead
               ds.b     $01               ; player's health
stick
               ds.b     $01               ; joystick value
the_stick
               ds.b     $01               ; set value
sleeping
               ds.b     $01               ; number of incative missiles
flag
               ds.b     $01               ; indicates call on creature handling
noises                                    ; noises for explosion
               dc.b     $01,$00,$03,$07,$05,$0F,$07,$F8
               dc.b     $08,$0D,$09,$0E,$0A,$0F,$FF
volume
               dc.b     $01               ; indicates volume on or off

               cnop     0,2               ; align to word boundary

glowing_energy
               ds.l     $80               ; space for glowing energy
the_types
               dc.l     skeleton          ; look up table for creatures
               dc.l     looney
               dc.l     pacman
               dc.l     gremlin
               dc.l     ghost_buster
               dc.l     nasty
               dc.l     knight
               dc.l     energy
address
               ds.l     $01               ; player's screen address
frame
               ds.l     $01               ; player's frame
e_addr_1
               ds.l     $01               ; exploding part's addresses
e_addr_2
               ds.l     $01
e_addr_3
               ds.l     $01
e_addr_4
               ds.l     $01
distance
               ds.l     $01               ; distance between exploding bits
pallette
               dc.w     $000,$420,$030,$052,$070,$750,$760,$016
               dc.w     $237,$047,$620,$741,$222,$333,$555,$777

this_number
               ds.w     $01               ; number of creatures in this wave
number_save
               ds.w     $01               ; save for demo
even_bits
               dc.w     %0000000000000001 ; even star's bit image
odd_bits
               dc.w     %0000000000000001 ; odd star's bit image
star_x
               ds.w     $30               ; star positions
number
               dc.w     $00               ; number of creatures
alive
               ds.w     $01               ; number of creatures alive
m_info
               ds.w     $10               ; space for missile info
c_info
               ds.w     $700              ; space for creature info

level_info
               dc.b     $3B,$00,$00,$01   ; lists key,number and digits
               dc.b     $3C,$09,$01,$00
               dc.b     $3D,$13,$02,$00
               dc.b     $3E,$1D,$03,$00
               dc.b     $3F,$27,$04,$00
               dc.b     $40,$31,$05,$00
status_line                               ; text for bottom line
               dc.b     $24,$24,$1C,$1D,$1B,$0E,$17,$10,$1D,$11
               dc.b     $25,$24,$24,$24,$26,$24,$24,$FF,$20,$0A,$1F
               dc.b     $0E,$25,$24,$24,$24,$24,$FF,$1C,$0C,$18,$1B
               dc.b     $0E,$25,$24,$24,$24,$24,$24,$24,$24,$24
               dc.b     $FF
paused                                    ; text for paused
               dc.b     $19,$0A,$1E,$1C,$0E,$0D,$FF
demo_string                               ; text for demo
               dc.b     $24,$0D,$0E,$16,$18,$24,$FF
sounds                                    ; sound parameters
               dc.b     $00,$00,$01,$00,$02,$00,$03,$00,$04,$00,$05,$0F
               dc.b     $06,$00,$07,$EA,$08,$10,$09,$00,$0A,$00,$0B,$C8
               dc.b     $0C,$00,$FF
off_sounds                                ; turn sounds off
               dc.b     $08,$00,$09,$00,$0A,$00,$FF
on_sounds
               dc.b     $08,$10,$FF       ; turn sounds on

               cnop     0,4


; ****************************************************************************
; Shatter the player.

shatter_player

               move.l   frame,A4             ; player's image data
               move.l   address,A5           ; player's screen address
               move.w   #$1F,D4              ; times to loop
rez_1_loop
               move.w   #random,-(sp)        ; request random number
               trap     #xbios
               addq.l   #$02,sp
               move.w   D0,D1                ; copy random to D1
               swap.w   D1                   ; copy low word to high
               add.l    D1,D0                ; add to get random long
               move.l   D0,D1                ; planes 0 and 1
               and.l    (A4)+,D1             ; mask out random bits
               move.l   D1,(A5)+             ; store image
               move.l   D0,D1                ; planes 2 and 3
               and.l    (A4)+,D1             ; mask out random bits
               move.l   D1,(A5)+             ; store image
               move.l   D0,D1                ; planes 0 and 1
               and.l    (A4)+,D1             ; mask out random bits
               move.l   D1,(A5)+             ; store image
               move.l   D0,D1                ; planes 2 and 3
               and.l    (A4)+,D1             ; mask out random bits
               move.l   D1,(A5)+             ; store image
               add.l    #$90,A5              ; update pointer
               dbra     D4,rez_1_loop
               rts


; ****************************************************************************
; Screen dump.

print_screen

               move.l   #off_sounds,A0       ; turn off sounds
               bsr      set_sound
               move.w   #scrdmp,-(sp)        ; call screen dump routine
               trap     #xbios
               addq.l   #$02,sp              ; restore stack
               move.l   #on_sounds,A0        ; and enable sounds
               bra      set_sound
               rts


; ****************************************************************************
; High score screen functions.


; ****************************************************************************
; Initialise edges.

init_hi_score

            bsr      set_up_muzik            ; set up that terrible music
            bsr      vb_wait
            bsr      cls                     ; clear the screen
            move.l   #$6FFE0,A0
            move.l   #$FFFF8240,A1
            move.w   #$0F,D0
scl1loop
            move.w   (A0)+,(A1)+
            dbra     D0,scl1loop
            move.l   #$70000,A0
            move.l   #$78000,A1
            move.w   #$7FF,D0
chss1loop
            move.l   (A0)+,(A1)+
            move.l   (A0)+,(A1)+
            move.l   (A0)+,(A1)+
            move.l   (A0)+,(A1)+
            dbra     D0,chss1loop
            bsr      check_fer_high          ; check for a new high score
; print high scores
            clr.w    height
            move.l   #$4FF,correction
            move.w   #$01,colour_mask
            move.w   #$04,D7                 ; print 5 high scores
            move.l   #the_high_scores,A6     ; pointer to high scores
            move.l   #the_high_names,A5      ; pointer to name
            move.l   #screen_mem+$55D0,A4    ; first screen address
jamaica_high
            move.l   (A5)+,A0                ; get address of name
            move.l   A4,A1                   ; get screen address
            bsr      print_string            ; print it
            add.l    #$28,A4                 ; update screen address
            move.l   (A6)+,A0                ; get address of score
            move.l   A4,A1                   ; get screen address
            bsr      print_string            ; print it
            add.l    #$4D8,A4                ; update screen address
            dbra     D7,jamaica_high         ; and repeat loop
; set up instructions
            move.w   #$0F,colour_mask
            move.l   #copyright,str_pointer  ; first instruction
            clr.w    str_message             ; clear pointer to instructions
            move.l   #screen_mem+$78A0,str_address ; initial screen address
            clr.w    str_length              ; initial length
            clr.b    str_stage               ; initial stage
            rts


; ****************************************************************************
; Move the 'edging' creatures.

animate_hi_score
            bsr      scroll_instructions
            bra      play_muzik              ; play that terrible music


; ****************************************************************************
; Scroll instructions accross the bottom of screen.

scroll_instructions
            tst.b    add_high                ; if we're adding a high score
            beq.b    do_the_scroll
            rts                              ; then don't scroll instructions
do_the_scroll
            move.w   #$0F,colour_mask
            tst.b    str_stage               ; test stage
            beq.b    stage_0                 ; if scrolling in
            cmpi.b   #$01,str_stage
            beq.b    stage_1                 ; if pausing
stage_2                                      ; else scrolling out
            addq.l   #$02,str_pointer        ; update string start
            subq.w   #$02,str_length         ; reduce string length
            bsr      print_size_string       ; and print it
            tst.w    str_length              ; end of sting reached?
            bne      instructions_exit
            move.l   #messages,A0            ; then get address of messages
            move.w   str_message,D0          ; get current pointer
            addq.w   #$04,D0                 ; update it
            andi.w   #$1C,D0
            move.w   D0,str_message          ; save it
            move.l   $00(A0,D0.w),str_pointer ; get new pointer
            move.l   #screen_mem+$78A0,str_address ; and reset screen address
            clr.w    str_length              ; clear length
            clr.b    str_stage               ; and set to scrolling in stage
            rts
stage_1
            addq.w   #$01,str_delay          ; update delay
            cmpi.w   #$80,str_delay          ; expired?
            bne      instructions_exit
            move.b   #$02,str_stage          ; then set to scrolling out stage
            rts
stage_0
            subq.l   #$08,str_address        ; update screen address
            addq.w   #$02,str_length         ; increase screen length
            bsr      print_size_string       ; and print string
            cmpi.w   #$28,str_length         ; all of string displayed?
            bne      instructions_exit
            move.b   #$01,str_stage          ; then set to pausing stage
            clr.w    str_delay               ; initial delay
instructions_exit
            rts


; ****************************************************************************
; Print a sized string.

print_size_string

            move.l   str_pointer,A0          ; get pointer to string
            move.w   str_length,D0           ; get length of string
            move.b   $00(A0,D0.w),chr_save   ; save character after end
            move.b   #$FF,$00(A0,D0.w)       ; and set it too $FF terminator
            move.l   str_address,A1          ; get screen address
            bsr      print_string            ; print string
            move.l   str_pointer,A0          ; get pointer to string
            move.w   str_length,D0           ; get length of string
            move.b   chr_save,$00(A0,D0.w)   ; restore character after end
            rts


; ****************************************************************************
; Check for high score.

check_fer_high
            clr.b    add_high                ; clear adding high score flag
            tst.b    first_game              ; test whether first game
            bne.b    skip_high_check         ; then can't be a high score
            tst.b    doing_demo              ; test whether just done a demo
            bne.b    skip_high_check         ; then can't be a high score
            move.w   #$04,D0                 ; check all five scores
            move.l   #the_high_scores,A0     ; get address of high scores
check_high_loop_1
            move.l   (A0)+,A1                ; get pointer to score
            move.l   #the_score,A2           ; get pointer to current score
            move.w   #$05,D1                 ; check all six digits
check_high_loop_2
            cmp.b    (A1)+,(A2)+             ; compare them
            bmi.b    look_at_next_score      ; if less then look at next score
            beq.b    look_at_next_digit      ; if equal then look at next digit
            bra      a_new_high_score        ; otherwise found a new high score
look_at_next_digit
            dbra     D1,check_high_loop_2    ; repeat loop
look_at_next_score
            dbra     D0,check_high_loop_1    ; repeat loop
            rts
skip_high_check
            clr.b    first_game              ; clear first game flag
            rts
a_new_high_score
            move.b   #$FF,first_entry        ; set first entry flag
            move.b   #$FF,add_high           ; set adding high score flag
            move.l   last_high_score,my_score ; save pointer to new score
            move.l   last_high_name,my_name  ; and name
            move.l   #the_high_scores+$0C,A0 ; pointer to high scores
            move.l   #the_high_names+$0C,A1  ; pointer to names
            move.l   #screen_mem+$6ED0,A2    ; screen address pointer
new_high_loop_1
            tst.w    D0                      ; found position
            beq      re_arranged
            move.l   (A0),$04(A0)            ; if not then move scores down
            move.l   (A1),$04(A1)            ; and move names down
            sub.l    #$500,A2                ; update screen address
            subq.l   #$04,A0                 ; update pointer to scores
            subq.l   #$04,A1                 ; update pointer to names
            subq.w   #$01,D0
            bra      new_high_loop_1         ; and repeat loop
re_arranged
            addq.l   #$04,A0                 ; found position, update pointers
            addq.l   #$04,A1
            sub.l    #$500,A2
            move.l   A2,my_address           ; save screen address (base)
            move.l   my_score,(A0)           ; save pointer to score
            move.l   my_name,(A1)            ; save pointer to name
            move.w   #$05,D0                 ; copy 6 digits
            move.l   my_score,A0             ; pointer to score string
            move.l   #the_score,A1           ; pointer to my score
copy_new_high_1
            move.b   (A1)+,(A0)+             ; copy digit
            dbra     D0,copy_new_high_1      ; and repeat loop
            move.w   #$07,D0                 ; clear 8 characters
            move.l   my_name,A0              ; pointer to name string
copy_new_high_2
            move.b   #$24,(A0)+              ; clear character
            dbra     D0,copy_new_high_2      ; and repeat loop
            clr.b    my_digit                ; clear digit pointer
            rts


; ****************************************************************************
; Make an entry on the high score.

add_a_name
            tst.b    first_entry             ; if just found a new high score
            beq.b    go_add_it
            move.l   my_address,A1           ; then set address
            move.b   #$28,D0                 ; print square
            move.w   #$04,colour_mask        ; select colour
            bsr      print_char              ; output cursor
            move.w   #$01,colour_mask        ; restore colour mask
            clr.b    first_entry             ; clear first entry
            move.b   #$FF,key_code           ; clear key code
            rts
go_add_it
            cmpi.b   #$1C,key_code           ; return key?
            beq.b    all_entered             ; then terminate entry
            cmpi.b   #$0E,key_code           ; backspace key?
            beq.b    gone_backwards          ; then remove a character
            cmpi.b   #$08,my_digit           ; reached end?
            beq.b    no_more_letters         ; then can't add any more
            move.w   #$24,D7                 ; else look through all keys
            move.l   #look_up_end,A6         ; address of key table
try_another
            move.b   -(A6),D6                ; get key code
            cmp.b    key_code,D6             ; found?
            beq.b    key_struck              ; then add to name
            dbra     D7,try_another          ; else repeat loop
no_more_letters
            move.b   #$FF,key_code           ; reset key code
            rts
key_struck
            move.b   my_digit,D6             ; get digit pointer
            ext.w    D6
            move.l   my_name,A6              ; get pointer to name string
            move.b   D7,$00(A6,D6)           ; store character
            move.l   my_address,A1           ; get screen address
            asl.w    #$02,D6
            move.l   #chr_updates,A6
            add.l    $00(A6,D6.w),A1
            move.b   D7,D0
            bsr      print_char              ; and print the character
            move.w   #$04,colour_mask        ; select colour
            move.b   #$28,D0                 ; square
            bsr      print_char              ; draw cursor
            move.w   #$01,colour_mask        ; restore colour mask
            addq.b   #$01,my_digit           ; increment pointer
            move.b   #$FF,key_code           ; reset key code
            rts

gone_backwards
            move.b   my_digit,D6             ; get digit pointer
            beq.b    back_weve_gone          ; if at end then can't go back
            subq.b   #$01,D6                 ; else decrement pointer
            ext.w    D6
            move.l   my_name,A6              ; get pointer to name string
            move.b   #$24,$00(A6,D6)         ; clear character
            move.l   my_address,A1           ; get screen address
            asl.w    #$02,D6
            move.l   #chr_updates,A6
            add.l    $00(A6,D6.w),A1
            move.b   #$28,D0                 ; square
            move.w   #$04,colour_mask        ; select colour
            bsr      print_char              ; draw cursor
            move.w   #$01,colour_mask        ; restore colour
            move.b   #$24,D0                 ; space
            bsr      print_char              ; print space (removes old cursor)
            subq.b   #$01,my_digit           ; move pointer backwards
back_weve_gone
            move.b   #$FF,key_code           ; reset key code
            rts
all_entered
            move.b   my_digit,D6             ; get offset
            ext.w    D6
            move.l   my_address,A1           ; get screen address
            asl.w    #$02,D6
            move.l   #chr_updates,A6
            add.l    $00(A6,D6.w),A1
            move.b   #$24,D0                 ; space
            bsr      print_char              ; print character (remove cursor)
            clr.b    add_high                ; clear adding high flag
            move.b   #$FF,key_code           ; reset key code
            rts


; ****************************************************************************

first_entry                                  ; just entering a high score
            ds.b     $01
add_high                                     ; adding a high score
            ds.b     $01
my_digit                                     ; offset in name
            ds.b     $01
first_game                                   ; playing first game
            dc.b     $FF
chr_save                                     ; save character
            ds.b     $01
edge_delay                                   ; delay for 'edging'
            ds.b     $01
str_stage
            ds.b     $01

            cnop     0,2                     ; align to word boundary

my_score
            ds.l     $01                     ; pointer to score string
my_name
            ds.l     $01                     ; pointer to name string
my_address
            ds.l     $01                     ; pointer to screen address (base)
str_pointer
            ds.l     $01                     ; string pointer
str_address
            ds.l     $01                     ; string screen address
str_length
            ds.w     $01                     ; string length
str_delay
            ds.w     $01                     ; string delay
str_message
            ds.w     $01                     ; string indicator
the_high_scores
            dc.l     hi_score_1              ; list of high scores
            dc.l     hi_score_2
            dc.l     hi_score_3
            dc.l     hi_score_4
last_high_score
            dc.l     hi_score_5
the_high_names
            dc.l     hi_name_1               ; list of high names
            dc.l     hi_name_2
            dc.l     hi_name_3
            dc.l     hi_name_4
last_high_name
            dc.l     hi_name_5


; ****************************************************************************
; Set up music? (call this music?)

set_up_muzik

            move.l   #muzik_sounds,A0        ; sound parameters
            bsr      set_sound
            move.l   #melody,mk_pointer      ; set initial pointer
            clr.b    mk_count                ; clear count
            clr.b    mk_wait                 ; clear wait
            clr.b    mk_scream               ; clear scream
            rts


; ****************************************************************************
; Play the music? (ditto)

play_muzik

            tst.b    mk_wait                 ; waiting for scream?
            beq.b    mk_waited
            subq.b   #$01,mk_wait            ; then decrement waiting time
            bra      do_more_muzik
mk_waited
            move.b   #$00,register           ; else select register 0
            move.b   mk_scream,frequency     ; set frequency
            addq.b   #$01,mk_scream          ; update scream
            cmpi.b   #$80,mk_scream          ; finished screaming?
            bne.b    do_more_muzik
            move.w   #random,-(sp)           ; then request random number
            trap     #xbios
            addq.l   #$02,sp
            move.b   D0,mk_wait              ; and set time to wait for scream
            clr.b    mk_scream               ; and reset scream pitch
            move.b   #$00,register           ; clear register 0
            move.b   #$00,frequency
do_more_muzik
            tst.b    mk_count                ; test music count
            bne.b    all_done                ; if not zero then leave alone
            move.l   mk_pointer,A0           ; else get pointer to music
            move.b   (A0)+,D0                ; get note
            cmpi.b   #$FF,D0                 ; end reached?
            bne.b    not_muzik_end
            move.l   #melody,A0              ; then reset pointer
            move.b   (A0)+,D0
not_muzik_end
            move.l   A0,mk_pointer           ; save pointer
            move.b   #$02,register           ; set register 2
            move.b   D0,frequency
            move.b   #$04,register           ; set register 4
            move.b   D0,frequency
            move.b   #$0D,register           ; set waveform
            move.b   #$09,frequency
            move.b   #$06,mk_count           ; and reset count
all_done
            subq.b   #$01,mk_count           ; decrement count
            rts


; ****************************************************************************

mk_pointer
            ds.l     $01                     ; pointer to music?
mk_count
            ds.b     $01                     ; music count
mk_wait
            ds.b     $01                     ; scream wait
mk_scream
            ds.b     $01                     ; scream pitch
muzik_sounds                                 ; the sound parameters
            dc.b     0,0,1,0,2,0,3,2,4,0,5,4,7,248,8,12,9,16,10,16,11,0,12,16
            dc.b     255
melody                                       ; the notes
            dc.b     200,200,250,250,200,200,250,250
            dc.b     200,200,250,250,200,200,250,250
            dc.b     50,50,25,25,50,50,25,25
            dc.b     50,50,25,25,50,50,25,25
            dc.b     255


; ****************************************************************************

key_code
               ds.b     $01               ; code of key pressed
doing_demo
               ds.b     $01               ; indicates whether doing a demo


; ****************************************************************************
; The actual missile image used.

               cnop     0,4

missile_data
               ds.w     $0A


; ****************************************************************************
; Barber scroll.

barber_scroll                                ; scroll alphas on high score

            move.l   #screen_mem+$1FF0,A0    ; screen address
            move.l   #alphas,A1              ; pointer to alpha graphics
            move.w   alpha_pointer,D0        ; offset
            addq.w   #$04,D0                 ; move onto next line
            andi.w   #$1C,D0                 ; force in range
            move.w   D0,alpha_pointer        ; and save
            move.w   #$07,D1                 ; loop 8 times
b1loop
            move.w   #$0F,D2                 ; loop 16 times (per scan)
b2loop
            move.l   $00(A1,D0.w),(A0)       ; copy word (long)
            addq.l   #$08,A0
            dbra     D2,b2loop               ; repeat loop
            add.l    #$20,A0                 ; update screen pointer
            addq.w   #$04,D0                 ; update alpha pointer
            andi.w   #$1C,D0                 ; force in range
            dbra     D1,b1loop               ; and repeat loop

            move.w   #$0F,D1                 ; loop 16 times
b3loop
            move.l   $00(A1,D0.w),$78(A0)    ; copy word (long)
            move.l   $00(A1,D0.w),(A0)       ; again
            add.l    #$A0,A0                 ; update screen pointer
            addq.w   #$04,D0                 ; update alpha pointer
            andi.w   #$1C,D0                 ; force in range
            dbra     D1,b3loop               ; and repeat loop

            move.w   #$07,D1                 ; loop 8 times
b5loop
            move.w   #$0F,D2                 ; loop 16 times (per scan)
b6loop
            move.l   $00(A1,D0.w),(A0)       ; copy word (long)
            addq.l   #$08,A0
            dbra     D2,b6loop               ; repeat loop
            add.l    #$20,A0                 ; update screen pointer
            addq.w   #$04,D0                 ; update alpha pointer
            andi.w   #$1C,D0                 ; force in range
            dbra     D1,b5loop               ; and repeat loop

            move.w   #$37,D1                 ; loop 56 times
b7loop
            move.w   #$02,D2                 ; loop 3 times (per scan)
b8loop
            move.l   $00(A1,D0.w),$68(A0)    ; copy word (long)
            move.l   $00(A1,D0.w),(A0)       ; again
            addq.l   #$08,A0
            dbra     D2,b8loop               ; repeat loop
            add.l    #$88,A0                 ; update screen pointer
            addq.w   #$04,D0                 ; update alpha pointer
            andi.w   #$1C,D0                 ; force in range
            dbra     D1,b7loop               ; and repeat loop

            move.w   #$07,D1                 ; loop 8 times
b9loop
            move.w   #$0F,D2                 ; loop 16 times (per scan)
b10loop
            move.l   $00(A1,D0.w),(A0)       ; copy word (long)
            addq.w   #$08,A0
            dbra     D2,b10loop              ; repeat loop
            add.l    #$20,A0                 ; update screen address
            addq.w   #$04,D0                 ; update alpha pointer
            andi.w   #$1C,D0                 ; force in range
            dbra     D1,b9loop               ; and repeat loop
            rts


alpha_pointer
            dc.w     $00


; ****************************************************************************
; Vertical blank interrupt routine

vbi
         addq.l   #$01,frclock
         rte


; ****************************************************************************
; String printing functions.


; ****************************************************************************
; Print a character, specified by a byte in D0 at screen location A1. Note -
; A1 is updated to the next screen position. All other registers remain
; intact.

print_char

            movem.l   D0-D4/A2,-(sp)      ; save registers
            move.l    chbase,A2           ; locate character set
            ext.w     D0                  ; extend character byte to a word
            lsl.w     #$03,D0             ; multiply by $08
            move.w    colour_mask,D4      ; colour of text
            move.w    #$07,D1             ; times to loop
next_row
            move.b    $00(A2,D0.w),D2     ; get bit pattern
            move.w    height,D3           ; height of text determines loop
size_loop_1
try_plane_0
            btst.l    #$00,D4
            beq       no_plane_0
            move.b    D2,(A1)             ; plane 0
            bra.s     try_plane_1
no_plane_0
            clr.b     (A1)
try_plane_1
            btst.l    #$01,D4
            beq       no_plane_1
            move.b    D2,$02(A1)          ; plane 1
            bra.s     try_plane_2
no_plane_1
            clr.b     $02(A1)
try_plane_2
            btst.l    #$02,D4
            beq       no_plane_2
            move.b    D2,$04(A1)          ; plane 2
            bra.s     try_plane_3
no_plane_2
            clr.b     $04(A1)
try_plane_3
            btst.l    #$03,D4
            beq       no_plane_3
            move.b    D2,$06(A1)          ; plane 3
            bra.s     all_planes_checked
no_plane_3
            clr.b     $06(A1)
all_planes_checked
            add.l     #$A0,A1             ; update screen address
            dbra      D3,size_loop_1      ; and repeat loop
            addq.w    #$01,D0             ; point to next row in character set
            dbra      D1,next_row         ; and repeat loop
            sub.l     correction,A1       ; point to next position
            move.l    A1,D1
            btst.l    #$01,D1
            beq       no_plane_correction
            addq.l    #$06,A1
no_plane_correction
            movem.l   (sp)+,D0-D4/A2      ; restore registers
            rts


; ****************************************************************************
; Print a string, specified by a pointer in A0 at screen location A1.
; Note - A0 is updated to the last character read (normally pointing to a $FF
; byte) and A1 is updated to the next screen position. All other registers
; remain intact.

print_string

            move.l   D0,-(sp)             ; save register
next_chr
            move.b   (A0)+,D0             ; get character code
            cmpi.b   #$FF,D0              ; termination?
            beq      string_printed
            bsr      print_char           ; output character
            bra      next_chr             ; and repeat
string_printed
            move.l   (sp)+,D0             ; restore register
            rts


; ****************************************************************************
; Text variables.

chbase
            dc.l     font                 ; character set base address
correction
            ds.l     $01                  ; correction for next character
height
            ds.w     $01                  ; height of text
colour_mask
            ds.w     $01                  ; mask enabling normal or reverse
                                          ; field text


; ****************************************************************************

messages
            dc.l     copyright               ; list of messages
            dc.l     instructions_1
            dc.l     instructions_2
            dc.l     instructions_3
            dc.l     instructions_4
            dc.l     instructions_5
            dc.l     instructions_6
            dc.l     instructions_7
alphas                                       ; graphics for scrolling alphas

            dc.l     %00111111001111110000111100001111
            dc.l     %11111111111111111111111111111111
            dc.l     %00001111000011110000111100001111
            dc.l     %11111111111111111111111111111111
            dc.l     %11110011111100111111000011110000
            dc.l     %11111111111111111111111111111111
            dc.l     %11110000111100001111000011110000
            dc.l     %11111111111111111111111111111111

look_up_start                                ; lookup key codes
            dc.b     11,2,3,4,5,6,7,8,9,10,30,48,46,32,18,33,34,35,23,36,37
            dc.b     38,50,49,24,25,16,19,31,20,22,47,17,45,21,44,57
look_up_end

string_1                                     ; floyd the droid 'on the run'
            dc.b     $0F,$15,$18,$22,$0D,$24,$1D,$11
            dc.b     $0E,$24,$0D,$1B,$18,$12,$0D,$24,$27,$18
            dc.b     $17,$24,$1D,$11,$0E,$24,$1B,$1E,$17,$27
            dc.b     $FF
string_2                                     ; today's top five
            dc.b     $1D,$18,$0D,$0A,$22,$27,$1C,$24,$1D,$18
            dc.b     $19,$24,$0F,$12,$1F,$0E,$FF
string_3                                     ; programmed by paul lay
            dc.b     $19,$1B,$18,$10,$1B,$0A,$16,$16,$0E,$0D
            dc.b     $24,$0B,$22,$24,$19,$0A,$1E,$15,$24,$15
            dc.b     $0A,$22,$FF
copyright                                    ; copyright...
            dc.b     $24,$24,$24,$24,$24,$24,$24,$24,$24,$24
            dc.b     $24,$24,$24,$0C,$18,$19,$22,$1B,$12,$10
            dc.b     $11,$1D,$24,$01,$09,$08,$06,$24,$24,$24
            dc.b     $24,$24,$24,$24,$24,$24,$24,$24,$24,$24
instructions_1                               ; F1 F2 F3 etc...
            dc.b     $24,$24,$24,$24,$24,$24,$0F,$01,$24,$0F
            dc.b     $02,$24,$0F,$03,$24,$0F,$04,$24,$0F,$05
            dc.b     $24,$0F,$06,$24,$0F,$18,$1B,$24,$15,$0E
            dc.b     $1F,$0E,$15,$1C,$24,$24,$24,$24,$24,$24
instructions_2                               ; C to continue etc...
            dc.b     $24,$24,$24,$0C,$24,$1D,$18,$24,$0C,$18
            dc.b     $17,$1D,$12,$17,$1E,$0E,$24,$0F,$1B,$18
            dc.b     $16,$24,$19,$1B,$0E,$1F,$12,$18,$1E,$1C
            dc.b     $24,$15,$0E,$1F,$0E,$15,$24,$24,$24,$24
instructions_3                               ; D for a 'crazy' demo
            dc.b     $24,$24,$24,$24,$24,$24,$24,$24,$24,$24
            dc.b     $0D,$24,$0F,$18,$1B,$24,$0A,$24,$27,$0C
            dc.b     $1B,$0A,$23,$22,$27,$24,$0D,$0E,$16,$18
            dc.b     $24,$24,$24,$24,$24,$24,$24,$24,$24,$24
instructions_4                               ; V to toggle volume etc...
            dc.b     $24,$24,$24,$24,$24,$24,$1F,$24,$1D,$18
            dc.b     $24,$1D,$18,$10,$10,$15,$0E,$24,$1F,$18
            dc.b     $15,$1E,$16,$0E,$24,$18,$17,$24,$18,$1B
            dc.b     $24,$18,$0F,$0F,$24,$24,$24,$24,$24,$24
instructions_5                               ; P to dump screen etc...
            dc.b     $24,$24,$24,$24,$24,$24,$19,$24,$1D,$18
            dc.b     $24,$0D,$1E,$16,$19,$24,$1C,$0C,$1B,$0E
            dc.b     $0E,$17,$24,$1D,$18,$24,$19,$1B,$12,$17
            dc.b     $1D,$0E,$1B,$24,$24,$24,$24,$24,$24,$24
instructions_6                               ; ESC to abort etc...
            dc.b     $24,$24,$24,$24,$24,$24,$24,$24,$24,$24
            dc.b     $0E,$1C,$0C,$24,$1D,$18,$24,$0A,$0B,$18
            dc.b     $1B,$1D,$24,$0A,$24,$10,$0A,$16,$0E,$24
            dc.b     $24,$24,$24,$24,$24,$24,$24,$24,$24,$24
instructions_7                               ; space bar to pause etc...
            dc.b     $24,$24,$1C,$19,$0A,$0C,$0E,$24,$0B,$0A
            dc.b     $1B,$24,$1D,$18,$24,$19,$0A,$1E,$1C,$0E
            dc.b     $24,$18,$1B,$24,$1B,$0E,$1C,$1E,$16,$0E
            dc.b     $24,$0A,$24,$10,$0A,$16,$0E,$24,$24,$24
hi_score_1                                   ; high scores
            dc.b     $00,$00,$03,$02,$00,$00,$FF
hi_score_2
            dc.b     $00,$00,$01,$06,$00,$00,$FF
hi_score_3
            dc.b     $00,$00,$00,$08,$00,$00,$FF
hi_score_4
            dc.b     $00,$00,$00,$04,$00,$00,$FF
hi_score_5
            dc.b     $00,$00,$00,$02,$00,$00,$FF
hi_name_1                                    ; high names
            dc.b     $0C,$18,$16,$19,$1E,$1D,$0E,$1B,$FF
hi_name_2
            dc.b     $0C,$18,$16,$19,$1E,$1D,$0E,$1B,$FF
hi_name_3
            dc.b     $0C,$18,$16,$19,$1E,$1D,$0E,$1B,$FF
hi_name_4
            dc.b     $0C,$18,$16,$19,$1E,$1D,$0E,$1B,$FF
hi_name_5
            dc.b     $0C,$18,$16,$19,$1E,$1D,$0E,$1B,$FF

            cnop     0,4


; ****************************************************************************
; Character printing updates.

chr_updates
            dc.l     $00,$01,$08,$09,$10,$11,$18,$19,$20


; ****************************************************************************
; Graphics data.


; ****************************************************************************
; The base missile image.

missile_image

         dc.w     %0000001111000000
         dc.w     %0000011111100000
         dc.w     %0000111111110000
         dc.w     %0001111111111000
         dc.w     %0001111111111000
         dc.w     %0001111111111000
         dc.w     %0001111111111000
         dc.w     %0000111111110000
         dc.w     %0000011111100000
         dc.w     %0000001111000000


; ****************************************************************************

face_down
face_up      equ    face_down+$0200
face_left    equ    face_up+$0200
face_right   equ    face_left+$0200
skeleton     equ    face_right+$0200
looney       equ    skeleton+$0200
pacman       equ    looney+$0200
gremlin      equ    pacman+$0200
ghost_buster equ    gremlin+$0200
nasty        equ    ghost_buster+$0200
knight       equ    nasty+$0200
energy       equ    knight+$0200

 dc.b  0,23,0,0,0,0,0,31
 dc.b  208,0,0,0,0,0,240,0
 dc.b  0,23,0,15,0,15,0,63
 dc.b  208,0,224,0,224,0,248,0
 dc.b  0,92,0,63,0,63,0,127
 dc.b  116,0,248,0,248,0,252,0
 dc.b  0,179,0,124,0,127,0,255
 dc.b  154,0,124,0,252,0,254,0
 dc.b  1,79,0,240,0,255,1,255
 dc.b  229,0,30,0,254,0,255,0
 dc.b  2,152,1,224,1,255,3,255
 dc.b  50,128,15,0,255,0,255,128
 dc.b  1,176,3,192,3,255,7,255
 dc.b  27,0,7,128,255,128,255,192
 dc.b  7,64,3,128,3,248,7,248
 dc.b  5,192,3,128,127,128,127,192
 dc.b  2,192,7,0,7,240,15,240
 dc.b  6,128,1,192,63,192,63,224
 dc.b  13,128,6,0,7,224,15,224
 dc.b  3,96,0,192,31,192,31,224
 dc.b  1,0,14,0,15,224,31,224
 dc.b  1,0,0,224,15,224,15,240
 dc.b  24,0,14,0,15,128,31,128
 dc.b  0,48,0,224,3,224,3,240
 dc.b  31,255,15,255,15,255,31,255
 dc.b  255,240,255,224,255,224,255,240
 dc.b  31,255,0,0,0,0,31,255
 dc.b  255,240,0,0,0,0,255,240
 dc.b  31,255,31,191,31,191,0,64
 dc.b  255,240,251,240,251,240,4,0
 dc.b  19,255,16,0,16,0,31,255
 dc.b  255,128,0,16,0,0,255,240
 dc.b  23,255,16,0,16,3,31,255
 dc.b  255,192,0,16,128,0,255,240
 dc.b  23,228,23,195,23,207,24,47
 dc.b  207,192,135,208,231,192,232,48
 dc.b  16,0,16,4,16,236,24,236
 dc.b  0,0,64,16,110,0,110,48
 dc.b  23,224,16,4,16,12,31,236
 dc.b  15,224,64,16,96,0,111,240
 dc.b  24,0,16,4,16,236,24,236
 dc.b  0,32,64,16,110,0,110,48
 dc.b  31,228,16,3,16,15,31,239
 dc.b  79,224,128,16,224,0,239,240
 dc.b  8,31,0,0,0,227,8,255
 dc.b  240,32,0,0,142,0,254,32
 dc.b  23,255,16,0,16,0,23,255
 dc.b  255,208,0,16,0,16,255,208
 dc.b  24,0,31,0,24,0,24,255
 dc.b  0,48,1,240,0,48,254,48
 dc.b  28,63,15,255,28,63,28,0
 dc.b  248,112,255,224,248,112,0,112
 dc.b  14,0,7,252,15,0,15,0
 dc.b  1,224,126,192,1,224,1,224
 dc.b  7,192,3,192,7,252,7,252
 dc.b  7,192,7,128,127,192,127,192
 dc.b  1,0,3,192,3,248,3,248
 dc.b  1,0,7,128,63,128,63,128
 dc.b  1,240,3,0,3,248,3,248
 dc.b  31,0,1,128,63,128,63,128
 dc.b  1,0,3,0,3,240,3,240
 dc.b  1,0,1,128,31,128,31,128
 dc.b  1,0,1,0,1,0,1,0
 dc.b  1,128,1,0,1,128,1,128
 dc.b  0,15,0,15,0,15,0,0
 dc.b  224,0,224,0,224,0,0,0
 dc.b  0,62,0,49,0,49,0,15
 dc.b  248,0,152,0,152,0,224,0
 dc.b  0,127,0,65,0,65,0,63
 dc.b  124,0,4,0,4,0,120,0
 dc.b  0,255,0,129,0,129,0,127
 dc.b  126,0,2,0,2,0,124,0
 dc.b  1,249,1,1,1,1,0,255
 dc.b  63,0,1,0,1,0,126,0
 dc.b  3,240,2,1,2,1,1,255
 dc.b  31,128,0,128,0,128,127,0
 dc.b  7,192,4,1,4,1,3,255
 dc.b  7,192,0,64,0,64,127,128
 dc.b  7,199,4,6,4,7,3,249
 dc.b  71,192,64,64,64,64,63,128
 dc.b  15,143,8,14,8,15,7,241
 dc.b  99,224,96,32,96,32,31,192
 dc.b  15,159,8,30,8,31,7,225
 dc.b  115,224,112,32,112,32,15,192
 dc.b  31,31,16,30,16,31,15,225
 dc.b  113,240,112,16,112,16,15,224
 dc.b  30,127,16,126,16,126,15,129
 dc.b  124,240,124,16,124,16,3,224
 dc.b  31,255,31,254,31,254,0,1
 dc.b  127,240,127,240,127,240,0,0
 dc.b  31,253,0,0,0,0,31,253
 dc.b  127,240,0,0,0,0,127,240
 dc.b  31,251,31,184,31,184,0,67
 dc.b  191,240,59,240,59,240,132,0
 dc.b  29,251,0,0,0,0,29,251
 dc.b  191,112,0,0,0,0,191,112
 dc.b  16,171,0,168,0,168,16,171
 dc.b  170,16,42,0,42,0,170,16
 dc.b  63,192,0,63,0,32,63,255
 dc.b  7,248,248,0,24,0,255,248
 dc.b  0,85,0,15,0,5,0,79
 dc.b  84,4,224,0,64,0,228,4
 dc.b  0,85,31,15,0,5,31,79
 dc.b  84,4,225,240,64,0,229,244
 dc.b  31,85,31,15,31,5,31,207
 dc.b  85,244,225,240,65,240,231,244
 dc.b  0,85,0,15,0,5,48,79
 dc.b  84,0,224,0,64,0,228,24
 dc.b  7,21,7,15,7,5,39,15
 dc.b  81,192,225,192,65,192,225,204
 dc.b  8,157,15,143,15,141,8,143
 dc.b  114,32,227,224,99,224,226,36
 dc.b  16,89,31,223,31,217,16,95
 dc.b  52,16,247,240,55,240,244,20
 dc.b  16,81,31,223,31,209,16,95
 dc.b  20,16,247,240,23,240,244,20
 dc.b  16,64,31,223,31,208,16,95
 dc.b  4,16,247,244,23,244,244,20
 dc.b  16,64,31,208,31,208,16,80
 dc.b  4,16,23,244,23,244,20,20
 dc.b  8,128,15,151,15,151,8,151
 dc.b  2,32,211,228,211,228,210,36
 dc.b  7,0,39,60,39,60,39,60
 dc.b  1,192,121,200,121,200,121,200
 dc.b  0,0,16,96,16,96,16,96
 dc.b  0,0,12,16,12,16,12,16
 dc.b  0,0,15,192,15,192,15,192
 dc.b  0,0,7,224,7,224,7,224
 dc.b  0,7,0,0,0,0,0,7
 dc.b  224,0,224,8,224,8,0,8
 dc.b  0,31,0,12,0,12,0,31
 dc.b  152,8,152,8,152,8,96,8
 dc.b  0,127,0,60,0,60,0,127
 dc.b  140,8,132,8,132,8,120,8
 dc.b  0,247,0,124,0,124,0,255
 dc.b  142,8,130,8,130,8,124,8
 dc.b  1,247,0,252,0,252,1,255
 dc.b  199,0,193,8,193,8,62,8
 dc.b  3,231,1,252,1,252,3,255
 dc.b  199,128,192,136,192,136,63,8
 dc.b  7,239,3,244,3,252,7,255
 dc.b  195,136,192,128,192,136,63,8
 dc.b  7,207,7,244,7,252,7,255
 dc.b  225,200,224,64,224,72,31,136
 dc.b  15,151,7,228,7,252,15,255
 dc.b  240,200,240,64,240,72,15,136
 dc.b  15,39,15,196,15,252,15,255
 dc.b  240,104,240,32,240,40,15,200
 dc.b  30,71,15,132,15,244,31,247
 dc.b  248,40,248,32,248,40,7,200
 dc.b  25,135,14,4,15,228,31,231
 dc.b  254,40,254,32,254,32,1,200
 dc.b  31,255,15,252,15,252,31,255
 dc.b  255,232,255,192,255,192,0,40
 dc.b  31,255,0,0,0,0,31,255
 dc.b  255,248,0,0,0,0,255,248
 dc.b  31,255,11,255,11,255,20,0
 dc.b  255,252,255,252,255,252,0,0
 dc.b  31,240,0,31,0,16,31,255
 dc.b  1,128,255,0,1,0,255,252
 dc.b  31,240,0,31,16,16,31,255
 dc.b  25,128,255,0,1,0,255,252
 dc.b  15,240,247,223,247,208,248,63
 dc.b  1,131,231,0,1,0,231,255
 dc.b  112,48,240,31,254,16,254,63
 dc.b  1,131,231,0,1,0,231,255
 dc.b  15,240,240,31,240,16,255,255
 dc.b  1,131,231,0,1,0,231,255
 dc.b  240,48,0,31,254,16,254,63
 dc.b  1,131,231,0,1,0,231,255
 dc.b  31,240,0,31,16,16,31,255
 dc.b  1,131,255,0,1,0,255,255
 dc.b  16,32,0,0,14,0,30,32
 dc.b  0,127,0,254,0,255,0,255
 dc.b  31,255,0,0,0,0,31,255
 dc.b  253,254,3,129,3,255,255,255
 dc.b  0,111,0,111,0,111,31,159
 dc.b  243,0,242,255,247,255,255,255
 dc.b  15,255,15,255,15,255,0,111
 dc.b  251,254,251,255,255,255,247,255
 dc.b  1,161,1,207,1,239,1,239
 dc.b  192,0,243,255,247,255,247,255
 dc.b  198,129,199,1,199,143,199,143
 dc.b  192,128,192,255,247,255,247,255
 dc.b  186,63,124,63,254,63,254,63
 dc.b  252,126,248,127,249,255,253,255
 dc.b  40,0,48,0,56,63,56,63
 dc.b  2,2,2,1,254,255,254,255
 dc.b  32,63,48,0,48,63,48,63
 dc.b  249,0,1,129,251,129,251,129
 dc.b  48,0,32,0,48,0,48,0
 dc.b  0,127,0,254,1,255,1,255
 dc.b  0,7,16,7,16,7,16,0
 dc.b  224,0,0,0,0,0,224,0
 dc.b  16,25,16,25,16,25,16,6
 dc.b  248,0,48,0,48,0,248,0
 dc.b  16,49,16,33,16,33,16,30
 dc.b  254,0,60,0,60,0,254,0
 dc.b  16,113,16,65,16,65,16,62
 dc.b  239,0,62,0,62,0,255,0
 dc.b  0,227,16,131,16,131,16,124
 dc.b  239,128,63,0,63,0,255,128
 dc.b  1,227,17,3,17,3,16,252
 dc.b  231,192,63,128,63,128,255,192
 dc.b  17,195,1,3,17,3,16,252
 dc.b  247,224,47,192,63,192,255,224
 dc.b  19,135,2,7,18,7,17,248
 dc.b  243,224,47,224,63,224,255,224
 dc.b  19,15,2,15,18,15,17,240
 dc.b  233,240,39,224,63,224,255,240
 dc.b  22,15,4,15,20,15,19,240
 dc.b  228,240,35,240,63,240,255,240
 dc.b  20,31,4,31,20,31,19,224
 dc.b  226,120,33,240,47,240,239,248
 dc.b  20,127,4,127,4,127,19,128
 dc.b  225,152,32,112,39,240,231,248
 dc.b  23,255,3,255,3,255,20,0
 dc.b  255,248,63,240,63,240,255,248
 dc.b  31,255,0,0,0,0,31,255
 dc.b  255,248,0,0,0,0,255,248
 dc.b  63,255,63,255,63,255,0,0
 dc.b  255,248,255,208,255,208,0,40
 dc.b  1,255,0,128,0,128,63,255
 dc.b  255,248,8,0,8,0,255,248
 dc.b  1,245,0,128,0,128,63,245
 dc.b  127,248,8,0,8,8,127,248
 dc.b  193,245,0,128,0,128,255,245
 dc.b  127,240,11,239,11,239,124,31
 dc.b  193,245,0,128,0,128,255,245
 dc.b  124,14,8,15,8,127,124,127
 dc.b  193,229,0,128,0,144,255,245
 dc.b  63,240,8,15,72,15,127,255
 dc.b  193,197,0,128,0,192,255,197
 dc.b  28,15,8,0,24,127,28,127
 dc.b  193,255,0,128,0,128,255,255
 dc.b  255,248,8,0,8,8,255,248
 dc.b  254,0,127,0,255,0,255,0
 dc.b  4,8,0,8,0,120,4,120
 dc.b  127,191,129,192,255,192,255,255
 dc.b  255,248,8,0,8,0,247,248
 dc.b  0,207,255,79,255,239,255,255
 dc.b  246,0,246,0,246,0,249,248
 dc.b  127,223,255,223,255,255,255,239
 dc.b  255,240,255,240,255,240,246,0
 dc.b  0,3,255,207,255,239,255,239
 dc.b  133,128,243,128,247,128,247,128
 dc.b  1,3,255,3,255,239,255,239
 dc.b  129,99,128,227,241,227,241,227
 dc.b  126,63,254,31,255,159,255,191
 dc.b  252,93,252,62,252,127,252,127
 dc.b  64,64,128,64,255,127,255,127
 dc.b  0,20,0,12,252,28,252,28
 dc.b  0,159,129,128,129,223,129,223
 dc.b  252,4,0,12,252,12,252,12
 dc.b  254,0,127,0,255,128,255,128
 dc.b  0,12,0,4,0,12,0,12
 dc.b  0,0,0,1,0,1,0,1
 dc.b  128,0,0,0,128,0,128,0
 dc.b  0,1,0,3,0,7,0,7
 dc.b  0,0,192,0,224,0,224,0
 dc.b  0,5,0,3,0,7,0,7
 dc.b  32,0,192,0,224,0,224,0
 dc.b  0,5,0,3,0,7,0,7
 dc.b  32,0,192,0,224,0,224,0
 dc.b  0,5,0,3,0,7,0,7
 dc.b  32,0,192,0,224,0,224,0
 dc.b  0,28,0,0,0,28,0,28
 dc.b  40,0,0,0,56,0,56,0
 dc.b  0,36,0,24,0,60,0,60
 dc.b  36,0,8,0,60,0,60,0
 dc.b  0,4,0,56,0,60,0,60
 dc.b  40,0,4,0,60,0,60,0
 dc.b  0,100,0,56,0,124,0,124
 dc.b  46,0,4,0,46,0,46,0
 dc.b  0,36,0,121,0,124,0,125
 dc.b  44,0,134,0,46,0,174,0
 dc.b  0,36,0,121,0,124,0,125
 dc.b  44,0,134,0,46,0,174,0
 dc.b  0,37,0,251,0,255,0,255
 dc.b  44,0,199,0,239,0,239,0
 dc.b  0,164,1,251,1,252,1,255
 dc.b  45,0,199,128,47,128,239,128
 dc.b  1,36,3,251,3,252,3,255
 dc.b  45,128,198,192,47,192,239,192
 dc.b  2,37,7,123,7,255,7,255
 dc.b  44,192,198,96,239,224,239,224
 dc.b  4,165,14,123,15,255,15,255
 dc.b  45,96,198,48,239,240,239,240
 dc.b  9,37,28,251,31,255,31,255
 dc.b  44,176,199,24,239,248,239,248
 dc.b  18,37,57,251,63,255,63,255
 dc.b  44,88,199,140,239,252,239,252
 dc.b  36,165,115,251,127,255,127,255
 dc.b  45,44,199,198,239,254,239,254
 dc.b  73,165,231,123,255,255,255,255
 dc.b  45,150,198,227,239,255,239,255
 dc.b  83,165,206,123,255,255,255,255
 dc.b  44,214,198,99,239,255,239,255
 dc.b  83,165,206,123,255,255,255,255
 dc.b  44,214,198,99,239,255,239,255
 dc.b  83,160,206,127,255,248,255,255
 dc.b  44,214,198,99,47,255,239,255
 dc.b  83,160,206,127,255,248,255,255
 dc.b  44,214,198,99,47,255,239,255
 dc.b  83,165,206,123,255,255,255,255
 dc.b  44,214,198,99,239,255,239,255
 dc.b  83,165,206,123,255,255,255,255
 dc.b  44,214,198,99,239,255,239,255
 dc.b  83,224,206,24,255,248,255,248
 dc.b  14,214,0,99,15,255,15,255
 dc.b  83,133,206,27,255,223,255,223
 dc.b  56,214,192,99,249,255,249,255
 dc.b  83,128,206,0,255,199,255,199
 dc.b  0,214,0,99,241,255,241,255
 dc.b  211,141,14,3,255,207,255,207
 dc.b  56,215,192,96,249,255,249,255
 dc.b  15,133,0,3,15,135,15,135
 dc.b  48,240,192,0,241,240,241,240
 dc.b  0,5,0,3,0,7,0,7
 dc.b  48,0,192,0,240,0,240,0
 dc.b  0,128,0,255,0,255,0,255
 dc.b  0,1,0,0,0,0,0,0
 dc.b  0,255,1,0,1,255,1,255
 dc.b  0,1,128,0,128,0,128,0
 dc.b  1,0,2,0,3,0,3,0
 dc.b  128,1,64,0,192,0,192,0
 dc.b  3,82,5,90,7,90,7,90
 dc.b  192,1,160,0,224,0,224,0
 dc.b  4,0,9,90,13,90,13,90
 dc.b  32,1,144,0,176,0,176,0
 dc.b  8,0,17,90,25,90,25,90
 dc.b  16,1,136,0,152,0,152,0
 dc.b  16,0,33,90,49,90,49,90
 dc.b  8,1,132,0,140,0,140,0
 dc.b  32,0,65,90,97,90,97,90
 dc.b  4,1,130,0,134,0,134,0
 dc.b  192,0,131,255,195,255,195,255
 dc.b  3,1,193,0,195,0,195,0
 dc.b  67,255,128,0,199,255,199,255
 dc.b  194,1,1,0,227,0,227,0
 dc.b  71,255,187,255,255,255,255,255
 dc.b  224,57,222,254,255,254,255,254
 dc.b  123,190,132,65,255,255,255,255
 dc.b  222,125,32,56,255,68,255,124
 dc.b  0,0,255,255,255,255,255,255
 dc.b  0,131,254,124,255,130,255,254
 dc.b  67,190,252,65,255,255,255,255
 dc.b  226,17,30,254,255,0,255,254
 dc.b  83,190,244,65,255,255,255,255
 dc.b  234,41,26,254,255,16,255,254
 dc.b  64,0,228,65,252,65,252,65
 dc.b  2,17,18,254,31,56,31,254
 dc.b  127,255,255,255,255,255,255,255
 dc.b  254,1,254,254,255,16,255,254
 dc.b  0,0,255,255,255,255,255,255
 dc.b  0,131,254,124,255,130,255,254
 dc.b  127,255,128,0,255,255,255,255
 dc.b  255,69,0,56,255,68,255,124
 dc.b  0,0,255,255,255,255,255,255
 dc.b  0,0,255,254,255,255,255,255
 dc.b  108,231,215,24,255,255,255,255
 dc.b  54,254,235,0,255,255,255,255
 dc.b  108,231,215,24,255,255,255,255
 dc.b  54,240,235,0,255,240,255,240
 dc.b  108,0,215,24,255,24,255,24
 dc.b  54,192,235,0,255,192,255,192
 dc.b  68,231,215,255,247,255,247,255
 dc.b  34,0,235,0,239,0,239,0
 dc.b  68,0,215,255,247,255,247,255
 dc.b  34,0,235,0,239,0,239,0
 dc.b  68,0,215,0,247,0,247,0
 dc.b  34,0,235,0,239,0,239,0
 dc.b  68,0,215,0,247,0,247,0
 dc.b  34,0,235,0,239,0,239,0
 dc.b  0,0,255,0,255,0,255,0
 dc.b  0,0,255,0,255,0,255,0
 dc.b  127,0,156,0,255,0,255,0
 dc.b  127,0,156,0,255,0,255,0
 dc.b  99,0,156,0,255,0,255,0
 dc.b  99,0,156,0,255,0,255,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  255,255,0,0,0,0,0,0
 dc.b  255,254,0,0,0,0,0,0
 dc.b  128,0,127,255,127,255,127,255
 dc.b  0,254,255,0,255,0,255,0
 dc.b  128,0,64,0,64,0,64,0
 dc.b  0,230,1,24,1,56,1,56
 dc.b  128,0,64,0,64,0,95,255
 dc.b  0,210,1,60,1,124,253,124
 dc.b  128,0,64,0,64,0,80,0
 dc.b  0,194,1,60,1,124,5,124
 dc.b  128,0,64,0,64,0,80,0
 dc.b  0,230,1,24,1,56,5,56
 dc.b  128,0,65,206,65,206,81,206
 dc.b  0,254,33,0,33,0,37,0
 dc.b  128,0,65,8,65,8,81,8
 dc.b  0,174,97,8,97,104,101,40
 dc.b  128,0,65,46,65,46,81,46
 dc.b  0,190,161,0,161,32,165,32
 dc.b  128,0,65,10,65,10,81,10
 dc.b  0,174,161,72,161,40,165,104
 dc.b  128,0,65,10,65,10,81,10
 dc.b  0,190,241,0,241,32,245,32
 dc.b  128,0,65,206,65,206,81,206
 dc.b  0,174,33,72,33,40,37,104
 dc.b  128,0,64,0,64,0,80,0
 dc.b  0,190,1,0,1,32,5,32
 dc.b  128,0,64,0,64,0,80,0
 dc.b  0,174,1,72,1,40,5,104
 dc.b  128,0,64,0,64,0,95,255
 dc.b  0,254,1,0,1,0,253,0
 dc.b  128,0,64,0,64,0,64,0
 dc.b  0,254,1,0,1,0,1,0
 dc.b  128,0,127,255,127,255,127,255
 dc.b  0,254,255,0,255,0,255,0
 dc.b  255,255,0,0,0,0,0,0
 dc.b  255,254,0,0,0,0,0,0
 dc.b  255,255,0,0,0,0,0,0
 dc.b  255,254,0,0,0,0,0,0
 dc.b  0,0,31,255,255,255,255,255
 dc.b  0,0,255,240,255,190,255,254
 dc.b  0,0,48,0,253,182,253,182
 dc.b  0,8,3,240,219,31,219,255
 dc.b  6,219,32,0,38,219,38,219
 dc.b  108,240,1,8,109,25,109,25
 dc.b  0,0,96,0,102,219,102,219
 dc.b  0,132,1,120,109,140,109,252
 dc.b  13,182,96,0,109,182,109,182
 dc.b  216,112,1,140,217,140,217,140
 dc.b  0,0,64,0,77,182,77,182
 dc.b  0,66,0,188,216,198,216,254
 dc.b  54,219,192,0,246,219,246,219
 dc.b  108,123,0,132,108,199,108,199
 dc.b  0,0,128,0,182,219,182,219
 dc.b  0,33,0,222,108,195,108,223
 dc.b  0,0,248,0,248,0,248,0
 dc.b  0,60,7,195,7,255,7,255
 dc.b  0,0,248,0,248,0,248,0
 dc.b  0,0,7,255,7,255,7,255
 dc.b  255,255,0,0,255,255,255,255
 dc.b  255,254,0,1,255,255,255,255
 dc.b  255,255,0,0,255,255,255,255
 dc.b  255,255,0,0,255,255,255,255
 dc.b  0,0,0,0,255,255,255,255
 dc.b  0,1,0,0,255,255,255,255
 dc.b  6,195,6,192,1,0,0,3
 dc.b  128,0,0,0,0,0,128,0
 dc.b  24,55,24,49,7,193,0,7
 dc.b  192,0,0,0,0,0,192,0
 dc.b  51,159,51,152,15,96,3,7
 dc.b  224,0,0,0,0,0,224,0
 dc.b  39,207,39,201,27,49,3,7
 dc.b  240,0,64,0,64,0,240,0
 dc.b  39,207,39,200,24,48,0,7
 dc.b  248,0,32,0,32,0,248,0
 dc.b  35,140,35,136,28,112,0,4
 dc.b  124,0,16,0,16,0,124,0
 dc.b  48,24,48,24,15,224,0,0
 dc.b  62,0,8,0,8,0,62,0
 dc.b  24,51,24,51,7,192,0,0
 dc.b  159,0,132,0,4,0,31,0
 dc.b  6,199,6,199,1,0,0,0
 dc.b  207,128,194,0,2,0,15,128
 dc.b  3,199,0,15,0,15,3,207
 dc.b  199,192,225,0,225,0,231,192
 dc.b  7,136,0,7,0,13,7,143
 dc.b  35,224,192,128,96,128,227,224
 dc.b  15,8,0,2,0,10,15,10
 dc.b  33,240,128,64,160,64,169,240
 dc.b  30,0,0,2,0,8,30,10
 dc.b  0,248,128,32,32,32,160,248
 dc.b  60,7,0,0,0,8,60,8
 dc.b  194,124,2,16,50,16,48,124
 dc.b  120,7,0,4,0,11,120,12
 dc.b  192,62,96,8,144,8,80,62
 dc.b  240,11,0,12,0,7,240,8
 dc.b  160,31,112,4,192,4,32,31
 dc.b  254,55,2,24,2,15,252,16
 dc.b  208,255,56,130,224,130,16,127
 dc.b  112,119,0,56,0,63,255,192
 dc.b  222,12,60,0,236,0,19,255
 dc.b  142,15,128,7,128,7,127,255
 dc.b  240,113,192,1,192,1,255,254
 dc.b  35,224,32,0,32,0,31,255
 dc.b  7,196,0,4,0,4,255,248
 dc.b  8,127,8,0,8,0,7,255
 dc.b  254,16,0,16,0,16,255,224
 dc.b  2,31,2,0,2,0,1,255
 dc.b  248,64,0,64,0,64,255,128
 dc.b  0,135,0,131,0,131,0,127
 dc.b  225,128,225,128,225,128,254,0
 dc.b  0,97,0,97,0,97,0,31
 dc.b  194,0,194,0,194,0,252,0
 dc.b  0,24,0,24,0,24,0,7
 dc.b  12,0,12,0,12,0,240,0
 dc.b  0,16,0,16,0,31,0,15
 dc.b  4,0,4,0,252,0,248,0
 dc.b  0,31,0,30,0,30,0,1
 dc.b  210,0,130,0,142,0,252,0
 dc.b  0,63,0,60,0,60,0,3
 dc.b  201,0,129,0,135,0,254,0
 dc.b  0,127,0,120,0,120,0,7
 dc.b  228,128,128,128,131,128,255,0
 dc.b  0,255,0,240,0,240,0,15
 dc.b  226,64,128,64,129,192,255,128
 dc.b  1,0,1,0,1,0,0,0
 dc.b  0,32,0,32,0,32,0,0
 dc.b  7,255,7,255,7,255,0,0
 dc.b  255,248,255,248,255,248,0,0
 dc.b  0,1,0,15,0,7,0,7
 dc.b  224,0,240,0,224,0,224,0
 dc.b  0,8,0,24,0,7,0,0
 dc.b  16,0,24,0,224,0,0,0
 dc.b  0,24,0,63,0,32,0,39
 dc.b  28,0,252,0,4,0,228,0
 dc.b  0,55,0,63,0,0,0,15
 dc.b  236,0,252,0,0,0,240,0
 dc.b  0,16,0,63,0,0,0,0
 dc.b  8,0,252,0,0,0,0,0
 dc.b  0,7,0,15,0,0,0,0
 dc.b  224,0,240,0,0,0,0,0
 dc.b  0,3,0,7,0,0,0,0
 dc.b  192,0,224,0,0,0,0,0
 dc.b  0,15,255,255,0,0,0,0
 dc.b  240,0,255,255,0,0,0,0
 dc.b  192,17,192,31,127,224,64,0
 dc.b  136,3,248,3,7,254,0,2
 dc.b  255,240,255,240,0,15,0,0
 dc.b  15,255,15,255,240,0,0,0
 dc.b  0,55,136,247,112,8,0,0
 dc.b  236,0,239,17,16,14,0,0
 dc.b  32,48,112,246,0,9,0,0
 dc.b  140,4,239,14,144,0,128,0
 dc.b  248,48,248,246,0,9,0,0
 dc.b  12,31,111,31,144,0,0,0
 dc.b  0,55,136,247,112,8,0,0
 dc.b  236,0,239,17,16,14,0,0
 dc.b  136,48,248,247,0,8,0,0
 dc.b  12,17,239,31,16,0,0,0
 dc.b  136,48,136,247,112,8,0,0
 dc.b  12,17,239,17,16,14,0,0
 dc.b  0,48,248,240,0,15,0,0
 dc.b  12,0,15,31,240,0,0,0
 dc.b  112,63,112,255,0,0,0,0
 dc.b  252,14,255,14,0,0,0,0
 dc.b  248,0,248,255,0,0,0,0
 dc.b  0,31,255,31,0,0,0,0
 dc.b  128,0,136,255,0,0,0,0
 dc.b  0,16,255,17,0,0,0,0
 dc.b  96,24,104,62,128,0,0,0
 dc.b  24,0,124,7,0,16,0,0
 dc.b  0,24,0,62,0,0,0,0
 dc.b  24,0,124,0,0,0,0,0
 dc.b  0,24,0,62,0,0,0,0
 dc.b  24,0,124,0,0,0,0,0
 dc.b  0,16,0,20,0,24,0,16
 dc.b  16,0,48,0,24,0,16,0
 dc.b  0,24,0,62,0,0,0,0
 dc.b  24,0,124,0,0,0,0,0
 dc.b  0,24,0,62,0,16,0,16
 dc.b  24,0,124,0,16,0,16,0
 dc.b  0,24,0,62,0,16,0,16
 dc.b  24,0,124,0,16,0,16,0
 dc.b  0,0,0,4,0,24,0,0
 dc.b  0,0,32,0,24,0,0,0
 dc.b  0,0,0,4,0,24,0,0
 dc.b  36,0,36,0,90,0,0,0
 dc.b  0,60,0,60,0,66,0,0
 dc.b  60,0,60,0,82,0,16,0
 dc.b  0,60,0,60,0,82,0,16
 dc.b  0,0,0,0,126,0,0,0
 dc.b  0,0,0,0,0,126,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,31,0,0,0,32,0,32
 dc.b  248,0,0,0,4,0,4,0
 dc.b  0,0,0,63,0,0,0,63
 dc.b  4,0,248,0,0,0,248,0
 dc.b  0,4,0,59,0,0,0,63
 dc.b  20,0,232,0,0,0,248,0
 dc.b  0,13,0,62,0,12,0,63
 dc.b  180,0,120,0,48,0,248,0
 dc.b  0,0,0,63,0,0,0,63
 dc.b  4,0,248,0,0,0,248,0
 dc.b  0,0,0,28,0,32,0,60
 dc.b  0,0,56,0,4,0,60,0
 dc.b  0,0,0,7,0,0,0,7
 dc.b  0,0,224,0,0,0,224,0
 dc.b  0,0,0,7,0,0,0,7
 dc.b  160,0,192,0,0,0,192,0
 dc.b  0,0,0,7,0,0,0,7
 dc.b  160,0,192,0,0,0,192,0
 dc.b  0,0,0,7,0,0,0,7
 dc.b  160,0,192,0,0,0,192,0
 dc.b  0,0,127,255,0,0,127,255
 dc.b  0,0,255,254,0,0,255,254
 dc.b  31,255,168,0,8,0,168,0
 dc.b  255,248,0,21,0,16,0,21
 dc.b  0,0,220,0,0,0,220,0
 dc.b  0,0,0,59,0,0,0,59
 dc.b  96,15,137,224,1,239,136,0
 dc.b  240,6,7,145,247,128,0,17
 dc.b  96,4,138,112,2,116,136,0
 dc.b  32,6,14,81,46,64,0,17
 dc.b  96,5,138,112,2,117,136,0
 dc.b  160,6,14,81,174,64,0,17
 dc.b  96,4,139,144,3,148,136,0
 dc.b  96,6,9,209,105,192,0,17
 dc.b  96,5,139,144,3,149,136,0
 dc.b  160,6,9,209,169,192,0,17
 dc.b  0,13,217,224,1,237,216,0
 dc.b  176,0,7,155,183,128,0,27
 dc.b  28,31,168,15,8,31,168,31
 dc.b  248,56,240,21,248,16,248,21
 dc.b  0,0,127,255,0,0,127,255
 dc.b  0,0,255,254,0,0,255,254
 dc.b  18,73,45,182,63,255,63,255
 dc.b  36,144,219,108,255,252,255,252
 dc.b  3,1,99,81,99,81,99,81
 dc.b  128,102,144,96,144,102,144,102
 dc.b  192,1,3,13,223,255,223,255
 dc.b  128,0,130,99,255,251,255,251
 dc.b  0,0,205,146,221,243,221,243
 dc.b  0,3,13,128,253,251,253,251
 dc.b  192,0,8,146,216,243,216,243
 dc.b  0,0,8,131,248,251,248,251
 dc.b  0,0,200,140,216,255,216,255
 dc.b  0,3,8,128,248,251,248,251
 dc.b  64,0,37,0,125,255,125,255
 dc.b  0,0,5,131,253,255,253,255
 dc.b  0,0,32,0,63,255,63,255
 dc.b  0,0,0,6,255,254,255,254
 dc.b  0,0,27,1,31,255,31,255
 dc.b  0,0,128,108,255,236,255,236
 dc.b  3,1,11,37,11,37,11,37
 dc.b  128,120,146,96,146,120,146,120
 dc.b  1,36,6,219,7,255,7,255
 dc.b  146,64,109,176,255,240,255,240
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  6,0,7,0,7,0,7,0
 dc.b  0,3,0,3,0,3,0,3
 dc.b  86,0,87,128,87,128,87,128
 dc.b  0,0,0,3,0,3,0,3
 dc.b  6,124,87,254,87,254,87,254
 dc.b  0,3,0,0,0,3,0,3
 dc.b  247,128,6,0,247,128,247,128
 dc.b  0,0,0,0,0,1,0,1
 dc.b  6,0,6,0,231,0,231,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,1,0,0,0,1,0,1
 dc.b  32,0,224,0,224,0,224,0
 dc.b  0,2,0,1,0,3,0,3
 dc.b  64,0,192,0,192,0,192,0
 dc.b  0,4,0,3,0,7,0,7
 dc.b  128,0,128,0,128,0,128,0
 dc.b  0,9,0,7,0,15,0,15
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,18,0,14,28,30,0,30
 dc.b  0,0,0,0,0,0,0,0
 dc.b  4,32,28,31,34,63,0,63
 dc.b  0,0,192,0,192,0,192,0
 dc.b  10,71,58,60,69,127,0,127
 dc.b  192,0,0,0,192,0,192,0
 dc.b  8,255,56,0,71,252,0,252
 dc.b  224,0,0,0,32,0,32,0
 dc.b  14,130,254,1,1,128,0,128
 dc.b  56,0,230,0,62,0,62,0
 dc.b  0,254,255,1,255,252,255,252
 dc.b  56,126,229,255,61,255,61,255
 dc.b  255,130,0,1,255,128,255,128
 dc.b  56,0,230,0,62,0,62,0
 dc.b  8,255,120,0,7,252,0,252
 dc.b  224,0,0,0,32,0,32,0
 dc.b  14,79,62,60,65,127,0,127
 dc.b  192,0,0,0,192,0,192,0
 dc.b  4,38,28,29,34,63,0,63
 dc.b  0,0,192,0,192,0,192,0
 dc.b  0,18,0,14,28,30,0,30
 dc.b  32,0,24,0,120,0,120,0
 dc.b  0,9,0,7,0,15,0,15
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,7,0,3,0,7,0,7
 dc.b  88,112,221,248,221,248,221,248
 dc.b  0,0,0,29,0,29,0,29
 dc.b  88,0,94,0,94,0,94,0
 dc.b  0,0,0,29,0,29,0,29
 dc.b  24,24,95,252,95,252,95,252
 dc.b  0,31,0,0,0,31,0,31
 dc.b  222,0,24,0,222,0,222,0
 dc.b  0,0,0,0,0,7,0,7
 dc.b  24,0,24,0,220,0,220,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  0,0,0,0,0,0,0,0
 dc.b  1,239,1,239,1,239,1,239
 dc.b  222,0,222,0,222,0,222,0
 dc.b  1,223,1,255,1,255,1,255
 dc.b  189,0,255,0,255,0,255,0
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,128,255,128,255,128,255,128
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,64,255,192,255,192,255,192
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,64,255,192,255,192,255,192
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,64,255,192,255,192,255,192
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,64,255,192,255,192,255,192
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,64,255,192,255,192,255,192
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,64,255,192,255,192,255,192
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,64,255,192,255,192,255,192
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,64,255,192,255,192,255,192
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,64,255,192,255,192,255,192
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,64,255,192,255,192,255,192
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,64,255,192,255,192,255,192
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,64,255,192,255,192,255,192
 dc.b  1,207,1,255,1,255,1,255
 dc.b  156,64,255,192,255,192,255,192
 dc.b  3,207,3,255,3,255,3,255
 dc.b  158,64,255,192,255,192,255,192
 dc.b  3,207,3,255,3,255,3,255
 dc.b  158,64,255,192,255,192,255,192
 dc.b  3,207,3,255,3,255,3,255
 dc.b  158,64,255,192,255,192,255,192
 dc.b  7,143,7,255,7,255,7,255
 dc.b  143,64,255,192,255,192,255,192
 dc.b  7,143,7,255,7,255,7,255
 dc.b  143,64,255,192,255,192,255,192
 dc.b  31,143,31,255,31,255,31,255
 dc.b  143,192,255,192,255,192,255,192
 dc.b  255,143,255,255,255,255,255,255
 dc.b  143,252,255,252,255,252,255,252
 dc.b  255,31,255,255,255,255,255,255
 dc.b  143,250,255,254,255,254,255,254
 dc.b  255,31,255,255,255,255,255,255
 dc.b  143,249,255,255,255,255,255,255
 dc.b  254,47,255,239,255,239,255,239
 dc.b  139,249,251,255,251,255,251,255
 dc.b  252,47,255,239,255,239,255,239
 dc.b  137,249,249,255,249,255,249,255
 dc.b  128,47,255,239,255,239,255,239
 dc.b  136,129,248,255,248,255,248,255
 dc.b  64,68,127,199,127,199,127,199
 dc.b  8,65,248,127,248,127,248,127
 dc.b  32,130,63,131,63,131,63,131
 dc.b  8,33,248,63,248,63,248,63
 dc.b  17,1,31,1,31,1,31,1
 dc.b  8,17,248,31,248,31,248,31
 dc.b  14,0,14,0,14,0,14,0
 dc.b  248,15,248,15,248,15,248,15


; ****************************************************************************
; Character set data.

font


; ****************************************************************************
; 8 by 8 high resolution font (from Atari 800 font).

            dc.b     %00000000
            dc.b     %00111100
            dc.b     %01100110
            dc.b     %01101110
            dc.b     %01110110
            dc.b     %01100110
            dc.b     %00111100
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00011000
            dc.b     %00111000
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %01111110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00111100
            dc.b     %01100110
            dc.b     %00001100
            dc.b     %00011000
            dc.b     %00110000
            dc.b     %01111110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01111110
            dc.b     %00001100
            dc.b     %00011000
            dc.b     %00001100
            dc.b     %01100110
            dc.b     %00111100
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00001100
            dc.b     %00011100
            dc.b     %00111100
            dc.b     %01101100
            dc.b     %01111110
            dc.b     %00001100
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01111110
            dc.b     %01100000
            dc.b     %01111100
            dc.b     %00000110
            dc.b     %01100110
            dc.b     %00111100
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00111100
            dc.b     %01100000
            dc.b     %01111100
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %00111100
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01111110
            dc.b     %00000110
            dc.b     %00001100
            dc.b     %00011000
            dc.b     %00110000
            dc.b     %00110000
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00111100
            dc.b     %01100110
            dc.b     %00111100
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %00111100
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00111100
            dc.b     %01100110
            dc.b     %00111110
            dc.b     %00000110
            dc.b     %00001100
            dc.b     %00111000
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00011000
            dc.b     %00111100
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01111110
            dc.b     %01100110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01111100
            dc.b     %01100110
            dc.b     %01111100
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01111100
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00111100
            dc.b     %01100110
            dc.b     %01100000
            dc.b     %01100000
            dc.b     %01100110
            dc.b     %00111100
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01111000
            dc.b     %01101100
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01101100
            dc.b     %01111000
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01111110
            dc.b     %01100000
            dc.b     %01111100
            dc.b     %01100000
            dc.b     %01100000
            dc.b     %01111110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01111110
            dc.b     %01100000
            dc.b     %01111100
            dc.b     %01100000
            dc.b     %01100000
            dc.b     %01100000
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00111110
            dc.b     %01100000
            dc.b     %01100000
            dc.b     %01101110
            dc.b     %01100110
            dc.b     %00111110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01111110
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01111110
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %01111110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00000110
            dc.b     %00000110
            dc.b     %00000110
            dc.b     %00000110
            dc.b     %01100110
            dc.b     %00111100
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01100110
            dc.b     %01101100
            dc.b     %01111000
            dc.b     %01111000
            dc.b     %01101100
            dc.b     %01100110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01100000
            dc.b     %01100000
            dc.b     %01100000
            dc.b     %01100000
            dc.b     %01100000
            dc.b     %01111110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01100011
            dc.b     %01110111
            dc.b     %01111111
            dc.b     %01101011
            dc.b     %01100011
            dc.b     %01100011
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01100110
            dc.b     %01110110
            dc.b     %01111110
            dc.b     %01111110
            dc.b     %01101110
            dc.b     %01100110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00111100
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %00111100
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01111100
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01111100
            dc.b     %01100000
            dc.b     %01100000
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00111100
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01101100
            dc.b     %00110110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01111100
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01111100
            dc.b     %01101100
            dc.b     %01100110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00111100
            dc.b     %01100000
            dc.b     %00111100
            dc.b     %00000110
            dc.b     %00000110
            dc.b     %00111100
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01111110
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01111110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %00111100
            dc.b     %00011000
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01100011
            dc.b     %01100011
            dc.b     %01101011
            dc.b     %01111111
            dc.b     %01110111
            dc.b     %01100011
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %00111100
            dc.b     %00111100
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01100110
            dc.b     %01100110
            dc.b     %00111100
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01111110
            dc.b     %00001100
            dc.b     %00011000
            dc.b     %00110000
            dc.b     %01100000
            dc.b     %01111110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00000000
            dc.b     %00000000
            dc.b     %00000000
            dc.b     %00000000
            dc.b     %00000000
            dc.b     %00000000
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00000000
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %00000000
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %01100110
            dc.b     %01101100
            dc.b     %00011000
            dc.b     %00110000
            dc.b     %01100110
            dc.b     %01000110
            dc.b     %00000000

            dc.b     %00000000
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %00011000
            dc.b     %00000000
            dc.b     %00000000
            dc.b     %00000000
            dc.b     %00000000

            dc.b     %11111111
            dc.b     %11111111
            dc.b     %11111111
            dc.b     %11111111
            dc.b     %11111111
            dc.b     %11111111
            dc.b     %11111111
            dc.b     %11111111

            cnop        0,4


; [[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[

read_hi_score_file
            move.w      #$00,-(sp)       ; open hi score file for reading
            move.l      #hi_score_file_ptr,-(sp)
            move.w      #open,-(sp)
            trap        #gemdos
            addq.l      #$08,sp
            move.w      D0,handle

            move.l      #$6FFDE,-(sp) ; read in hi score file
            move.l      #$7D22,-(sp)
            move.w      handle,-(sp)
            move.w      #read,-(sp)
            trap        #gemdos
            add.l       #$0C,sp

            move.w      handle,-(sp)     ; close logo file
            move.w      #close,-(sp)
            trap        #gemdos
            addq.l      #$04,sp

            rts

handle
            ds.w        $01

hi_score_file_ptr
            dc.b        'hiscore.lo',$00

               cnop 0,4


; ****************************************************************************

*
* load ouch sample
*
load_ouch
   move.l   #ouch_start,D5
   move.l   #ouch_filename,D6
   move.l   #$0FC0,D7
   bra      load_sample


*
* load begin game sample
*
load_begin
   move.l   #begin_start,D5
   move.l   #begin_filename,D6
   move.l   #$AE11,D7
   bra      load_sample


*
* load game over sample
*
load_over
   move.l   #over_start,D5
   move.l   #over_filename,D6
   move.l   #$8039,D7
   bra      load_sample


*
* load pause sample
*
load_pause
   move.l   #pause_start,D5
   move.l   #pause_filename,D6
   move.l   #$31A2,D7
   bra      load_sample


*
* load resume sample
*
load_resume
   move.l   #resume_start,D5
   move.l   #resume_filename,D6
   move.l   #$332D,D7
   bra      load_sample


*
* load intro sample
*
load_intro
   move.l   #intro_start,D5
   move.l   #intro_filename,D6
   move.l   #$65BC,D7


*
* load a sample
*
load_sample
   move.w   #$00,-(SP)
   move.l   D6,-(SP)
   move.w   #$3D,-(SP)
   trap     #$01
   addq.l   #$08,SP
   move.w   D0,io_handle

   move.l   D5,-(SP)
   move.l   D7,-(SP)
   move.w   io_handle,-(SP)
   move.w   #$3F,-(SP)
   trap     #$01
   add.l    #$0C,SP

   move.w   io_handle,-(SP)
   move.w   #$3E,-(SP)
   trap   #$01
   addq.l   #$04,SP

   rts


*
* play ouch
*
play_ouch
   movem.l  D0/D4-D7/A0/A3-A6,-(SP)
   move.l   #ouch_start,CURSRLO
   move.l   #ouch_start+$0FC0,CURSRHI
   bsr      play_sample
   move.l   #sounds,A0
   bsr      set_sound
   movem.l  (SP)+,D0/D4-D7/A0/A3-A6
   rts


*
* play begin
*
play_begin
   movem.l  D0/D4-D7/A0/A3-A6,-(SP)
   move.l   #begin_start,CURSRLO
   move.l   #begin_start+$AE11,CURSRHI
   bsr      play_sample
   movem.l  (SP)+,D0/D4-D7/A0/A3-A6
   rts


*
* play over
*
play_over
   movem.l  D0/D4-D7/A0/A3-A6,-(SP)
   move.l   #over_start,CURSRLO
   move.l   #over_start+$8039,CURSRHI
   bsr      play_sample
   movem.l  (SP)+,D0/D4-D7/A0/A3-A6
   rts


*
* play pause
*
play_pause
   movem.l  D0/D4-D7/A0/A3-A6,-(SP)
   move.l   #pause_start,CURSRLO
   move.l   #pause_start+$31A2,CURSRHI
   bsr      play_sample
   movem.l  (SP)+,D0/D4-D7/A0/A3-A6
   rts


*
* play resume
*
play_resume
   movem.l  D0/D4-D7/A0/A3-A6,-(SP)
   move.l   #resume_start,CURSRLO
   move.l   #resume_start+$332D,CURSRHI
   bsr      play_sample
   move.l   #sounds,A0
   bsr      set_sound
   movem.l  (SP)+,D0/D4-D7/A0/A3-A6
   rts


*
* play intro
*
play_intro
   movem.l  D0/D4-D7/A0/A3-A6,-(SP)
   move.l   #intro_start,CURSRLO
   move.l   #intro_start+$65BC,CURSRHI
   bsr      play_sample
   movem.l  (SP)+,D0/D4-D7/A0/A3-A6
   rts


*
* some equates
*
io_handle       ds.w   $01
ouch_filename   dc.b   'ouch.spl',$00
begin_filename  dc.b   'begin.spl',$00
over_filename   dc.b   'gameover.spl',$00
pause_filename  dc.b   'pause.spl',$00
resume_filename dc.b   'resume.spl',$00
intro_filename  dc.b   'intro.spl',$00

      cnop 0,4

*
*   MFP EQUATES
*
MFP   EQU   $FFFA00
IERA   EQU   MFP+$07
IERB   EQU   MFP+$09
IPRA   EQU   MFP+$0B
ISRA   EQU   MFP+$0F
IMRA   EQU   MFP+$13
IMRB   EQU   MFP+$15
VECTOR   EQU   MFP+$17
TACR   EQU   MFP+$19
TADR   EQU   MFP+$1F
ISRVEC   EQU   $134
*
*   THE SOUND CHIP
*
SCREG   EQU   $FF8800   CHIP REGISTER SELECT
SCDATA   EQU   $FF8802   REGISTER DATA

*   BDOS EQUATES (TRAP #1)

P_TERM_OLD EQU   0   
F_SUPER  EQU   $20

*   BIOS EQUATES (TRAP #13)

GETMPB    EQU   0
BCONSTAT EQU   1
BCONIN    EQU   2
BCONOUT    EQU   3

*   CARTRIDGE PORT ADDRESSES

INPUT   EQU   $FB0001
OUTPUT   EQU   $FA0000

*****************************************
*           THE PROGRAMME               *
*****************************************
*****************************************
*   SET UP STACK, MEMORY & SUPERVISOR   *
*****************************************

play_sample

   BSR   SAVEMFP      SAVE NATURAL MFP CONDITIONS
   MOVE.W   #$2700,SR
   LEA   INULL,A0
   BSR   SETINT      SET NULL INTERRUPT ADDRESS
   MOVEQ   #2,D0      SET INITIAL FREQUENCY
   BSR   SETFREQ
   BSR   ENABMFP      SET THE MFP RUNNING
   BSR   SETSND      SET UP SOUND REGISTERS
   
   SF   XTERNAL      SET THE MODE FOR INTERNAL ATARI USE
   BSR   INTREGS      SET UP INTERNAL REGISTERS
   MOVE.W   #$2500,SR   ENABLE THE INTERRUPTS

*
*  CONTROL/PROGRAM CODE CAN GO HERE
*

   MOVE.L   CURSRLO,A6   PASS START OF REPLAY
   MOVE.L   CURSRHI,D6   & END
   ST   INUSE      FLAG INTS IN USE
   SF   OPTION      WE ARE NOT SAMPLING
   LEA   IREPLAY,A0   SET UP THE ROUTINE
   BSR   SETINT      AND DO IT

sample_wait
   tst.b   INUSE
   bne.s   sample_wait

*
*  THIS TIDIES UP THE SYSTEM BEFORE LEAVING BACK TO GEM
*
   MOVE.W   #$2700,SR   DISABLE INTS.
   BSR   OLDMFP      RESTORE ORIGINAL MFP DATA

   move.w   #$2300,SR

   rts


****************************************
*   THE INTERRUPT SERVICE ROUTINES     *
****************************************
*
*   A6 = POINTER TO BASE OF REPLAY RAM
*   A5 = OUTPUT ROUTINE ADDR
*   A4 = INT. SOUND REG / EXT. PORT ADDR
*   A3 = EXT. SND LOOK UP TABLE
*   A2 = NOT USED
*   A1 = NOT USED
*   A0 = RESERVED FOR MAIN PROG.
*   D7 = BYTE FROM PORT /  OUT TO PORT
*   D6 = POINTER TO END OF REPLAY RAM
*   D5 = SOUND CHIP DATA
*   D4 = SOUND CHIP DATA
*   D3 = UNUSED
*   D2 = UNUSED
*   D1 = UNUSED
*   D0 = RESERVED FOR MAIN PROG. 
*
****************************************
*     THE NULL INTERRUPT ROUTINE       *
****************************************

INULL   RTE

****************************************
*       REPLAY FROM MEMORY             *
****************************************

IREPLAY   MOVE.B   (A6)+,D7   READ FROM RAM
   CMP.L   D6,A6      EXCEDED END STOP
   BGT.S   HALTREP      BRANCH IF SO
   JMP   (A5)      ELSE OUTPUT IT

HALTREP   MOVE.W   #$2700,SR
   LEA   INULL,A0   REMOVE REPLAY INT.
   BSR   SETINT
   SF   INUSE
   MOVE.W   #$2500,SR
   RTE

****************************************
*       THE OUTPUT ROUTINES            *
****************************************
*
*  The following code plays the sounds out through the GI sound chip.
*  We use 'A3' as the base pointer to the look up table. This is easy
*  to use here since it is spare. Position independant writers may
*  replace 'A3' with 'PC'. This relieves an extra register but the
*  table must be located within 256 BYTES of the code for it to work.
*  Our Thanks to Andy Beveridge for the improved sound tables & code.
*
INTOUTP   AND.W   #$00FF,D7   MASK OFF RUBBISH
   ADD.B   #$80,D7      COMPLENT VALUE
   LSL.W   #3,D7      DOUBLE LONG WORD OFFSET
   MOVE.L   0(A3,D7.W),D5   GET DATA #1
   MOVE.W   4(A3,D7.W),D4   GET DATA #2
   MOVEP.L   D5,0(A4)   PLAY #1
   MOVEP.W   D4,0(A4)   PLAY #2
   RTE

*****************************************
*       THE SYSTEM SUB-ROUTINES         *
*****************************************
*****************************************
*       PRESERVE THE MFP REGISTERS      *
*****************************************

SAVEMFP   MOVE.B   IERA,MFPMEM   PUSH CURRENT MFP DATA
   MOVE.B   IERB,MFPMEM+1
   MOVE.B   IMRA,MFPMEM+2
   MOVE.B   IMRB,MFPMEM+3
   MOVE.B   TADR,MFPMEM+4
   MOVE.B   TACR,MFPMEM+5
   MOVE.B   VECTOR,MFPMEM+6
   RTS

*****************************************
*   REPLACE NATURAL RUNNING MFP VALUES  *
*****************************************

OLDMFP   MOVE.B   MFPMEM+6,VECTOR   RESTORE OLD MFP VALS
   MOVE.B   MFPMEM+5,TACR
   MOVE.B   MFPMEM+4,TADR
   MOVE.B   MFPMEM+3,IMRB
   MOVE.B   MFPMEM+2,IMRA
   MOVE.B   MFPMEM+1,IERB
   MOVE.B   MFPMEM,IERA
   RTS

*****************************************
*   CHOOSE INTERRUPT VECTOR         *
*****************************************
*
*   SET UP SELECTED INTERRUPT WITH A0.L
*   CONTAINING THE NEW ROUTINE VECTOR.
*
SETINT   MOVE.W   SR,D0      SAVE SYSTEM STATUS
   MOVE.W   #$2700,SR   INTERRUPTS OFF

   MOVE.L   A0,ISRVEC   INSTALL NEW ROUTINE

   MOVE.W   D0,SR      RE-ASSERT OLD STATUS
   RTS

*****************************************
*       SET UP MFP FREQUENCY            *
*****************************************
*
*   SET UP MFP TIMER A TO GENERATE INTERRUPTS
*   ENTER WITH D0.W SET WITH A FREQUENCY NUMBER 0 TO 7
*   E.G. D0.W = $0002 GETS 3RD MFP DATA FROM TABLE
*
SETFREQ   MOVE.B   #0,TACR      DISABLE TIMER
   AND.W   #7,D0      ENSURE 0-7
   MOVE.W   D0,FREQNUM   SAVE FOR PROSPERITY
   LEA   FREQTAB,A0   GET THE FREQUENCY XREF ADDR
   LSL.W   #1,D0      CONVERT TO WORD TABLE OFFSET
   MOVE.W   0(A0,D0.W),D0   & GET THE MFP DATA
   MOVE.B   D0,TACR      LSB = CONTROL REG BYTE
   LSR.W   #8,D0      SHIFT DOWN NEXT BYTE
   MOVE.B   D0,TADR      NEXT = DATA REG BYTE
   RTS

***********************************
*       ENABLE THE MFP            *
***********************************

ENABMFP   MOVE.B   #$20,IMRA
   MOVE.B   #$40,IMRB
   MOVE.B   #$20,IERA
   MOVE.B   #$40,IERB
   BCLR.B   #3,VECTOR
   RTS

*****************************************
*    SET UP THE SOUND CHIP CHANNELS     *
*****************************************

SETSND   MOVE.B   #0,SCREG   CHANNEL A
   MOVE.B   #0,SCDATA
   MOVE.B   #1,SCREG
   MOVE.B   #0,SCDATA

   MOVE.B   #2,SCREG   CHANNEL B
   MOVE.B   #0,SCDATA
   MOVE.B   #3,SCREG
   MOVE.B   #0,SCDATA

   MOVE.B   #4,SCREG   CHANNEL C
   MOVE.B   #0,SCDATA
   MOVE.B   #5,SCREG
   MOVE.B   #0,SCDATA

   MOVE.B   #7,SCREG   SET UP CHANNEL MIXING & PORT 'A' I/O
   MOVE.B   #$FF,SCDATA

   MOVE.B   #8,SCREG   SET ALL VOLUMES TO ZERO
   MOVE.B   #0,SCDATA
   MOVE.B   #9,SCREG
   MOVE.B   #0,SCDATA
   MOVE.B   #10,SCREG
   MOVE.B   #0,SCDATA
   RTS
   
***************************************
*    SET UP REGS. FOR INTERNAL        *
***************************************

INTREGS   MOVE.W   SR,D0
   MOVE.W   #$2700,SR
   SF   XTERNAL
   LEA.L   INTOUTP,A5   INSTALL OUTPUT ROUTINE
   MOVE.L   #SCREG,A4   SOUND CHIP CNTRL REG
   LEA.L   SND_OUT,A3   LOOK UP TABLES
   MOVEQ   #0,D7
   MOVE.W   D0,SR
   RTS


***********************************
*   BLOCK STORAGE AREA        *
***************************************
*    Internal sound lookup table.     *
***************************************
   cnop 0,4
SND_OUT   dc.w   $80c,$90b,$a09,0,$80c,$90b,$a09,0
   dc.w   $80d,$908,$a08,0,$80b,$90b,$a0b,0
   dc.w   $80d,$909,$a05,0,$80c,$90b,$a08,0
   dc.w   $80d,$909,$a02,0,$80d,$908,$a06,0
   dc.w   $80c,$90b,$a07,0,$80d,$907,$a07,0
   dc.w   $80c,$90b,$a06,0,$80c,$90a,$a09,0
   dc.w   $80b,$90b,$a0a,0,$80c,$90b,$a02,0
   dc.w   $80c,$90b,$a00,0,$80c,$90a,$a08,0

   dc.w   $80d,$906,$a04,0,$80d,$905,$a05,0
   dc.w   $80d,$905,$a04,0,$80c,$909,$a09,0
   dc.w   $80d,$904,$a03,0,$80b,$90b,$a09,0
   dc.w   $80c,$90a,$a05,0,$80b,$90a,$a0a,0
   dc.w   $80c,$909,$a08,0,$80b,$90b,$a08,0
   dc.w   $80c,$90a,$a00,0,$80c,$90a,$a00,0
   dc.w   $80c,$909,$a07,0,$80b,$90b,$a07,0
   dc.w   $80c,$909,$a06,0,$80b,$90b,$a06,0

   dc.w   $80b,$90a,$a09,0,$80b,$90b,$a05,0
   dc.w   $80a,$90a,$a0a,0,$80b,$90b,$a02,0
   dc.w   $80b,$90a,$a08,0,$80c,$907,$a07,0
   dc.w   $80c,$908,$a04,0,$80c,$907,$a06,0
   dc.w   $80b,$909,$a09,0,$80c,$906,$a06,0
   dc.w   $80a,$90a,$a09,0,$80c,$907,$a03,0
   dc.w   $80b,$90a,$a05,0,$80b,$909,$a08,0
   dc.w   $80b,$90a,$a03,0,$80a,$90a,$a08,0

   dc.w   $80b,$90a,$a00,0,$80b,$909,$a07,0
   dc.w   $80b,$908,$a08,0,$80a,$90a,$a07,0
   dc.w   $80a,$909,$a09,0,$80c,$901,$a01,0
   dc.w   $80a,$90a,$a06,0,$80b,$908,$a07,0
   dc.w   $80a,$90a,$a05,0,$80a,$909,$a08,0
   dc.w   $80a,$90a,$a02,0,$80a,$90a,$a01,0
   dc.w   $80a,$90a,$a00,0,$809,$909,$a09,0
   dc.w   $80a,$908,$a08,0,$80b,$908,$a01,0

   dc.w   $80a,$909,$a06,0,$80b,$907,$a04,0
   dc.w   $80a,$909,$a05,0,$809,$909,$a08,0
   dc.w   $80a,$909,$a03,0,$80a,$908,$a06,0
   dc.w   $80a,$909,$a00,0,$809,$909,$a07,0
   dc.w   $809,$908,$a08,0,$80a,$908,$a04,0
   dc.w   $809,$909,$a06,0,$80a,$908,$a01,0
   dc.w   $809,$909,$a05,0,$809,$908,$a07,0
   dc.w   $808,$908,$a08,0,$809,$909,$a02,0

   dc.w   $809,$908,$a06,0,$809,$909,$a00,0
   dc.w   $809,$907,$a07,0,$808,$908,$a07,0
   dc.w   $809,$907,$a06,0,$809,$908,$a02,0
   dc.w   $808,$908,$a06,0,$809,$906,$a06,0
   dc.w   $808,$907,$a07,0,$808,$908,$a04,0
   dc.w   $808,$907,$a06,0,$808,$908,$a02,0
   dc.w   $807,$907,$a07,0,$808,$906,$a06,0
   dc.w   $808,$907,$a04,0,$807,$907,$a06,0

   dc.w   $808,$906,$a05,0,$808,$906,$a04,0
   dc.w   $807,$906,$a06,0,$807,$907,$a04,0
   dc.w   $808,$905,$a04,0,$806,$906,$a06,0
   dc.w   $807,$906,$a04,0,$807,$905,$a05,0
   dc.w   $806,$906,$a05,0,$806,$906,$a04,0
   dc.w   $806,$905,$a05,0,$806,$906,$a02,0
   dc.w   $806,$905,$a04,0,$805,$905,$a05,0
   dc.w   $806,$905,$a02,0,$805,$905,$a04,0

   dc.w   $805,$904,$a04,0,$805,$905,$a02,0
   dc.w   $804,$904,$a04,0,$804,$904,$a03,0
   dc.w   $804,$904,$a02,0,$804,$903,$a03,0
   dc.w   $803,$903,$a03,0,$803,$903,$a02,0
   dc.w   $803,$902,$a02,0,$802,$902,$a02,0
   dc.w   $802,$902,$a01,0,$801,$901,$a01,0
   dc.w   $802,$901,$a00,0,$801,$901,$a00,0
   dc.w   $801,$900,$a00,0,$800,$900,$a00,0

   dc.w   $80e,$90d,$a0c,0,$80f,$903,$a00,0
   dc.w   $80f,$903,$a00,0,$80f,$903,$a00,0
   dc.w   $80f,$903,$a00,0,$80f,$903,$a00,0
   dc.w   $80f,$903,$a00,0,$80e,$90d,$a0b,0
   dc.w   $80e,$90d,$a0b,0,$80e,$90d,$a0b,0
   dc.w   $80e,$90d,$a0b,0,$80e,$90d,$a0b,0
   dc.w   $80e,$90d,$a0b,0,$80e,$90d,$a0b,0
   dc.w   $80e,$90d,$a0a,0,$80e,$90d,$a0a,0

   dc.w   $80e,$90d,$a0a,0,$80e,$90d,$a0a,0
   dc.w   $80e,$90c,$a0c,0,$80e,$90d,$a00,0
   dc.w   $80d,$90d,$a0d,0,$80d,$90d,$a0d,0
   dc.w   $80d,$90d,$a0d,0,$80d,$90d,$a0d,0
   dc.w   $80d,$90d,$a0d,0,$80d,$90d,$a0d,0
   dc.w   $80e,$90c,$a0b,0,$80e,$90c,$a0b,0
   dc.w   $80e,$90c,$a0b,0,$80e,$90c,$a0b,0
   dc.w   $80e,$90c,$a0b,0,$80e,$90c,$a0b,0

   dc.w   $80e,$90c,$a0b,0,$80e,$90c,$a0b,0
   dc.w   $80e,$90c,$a0a,0,$80e,$90c,$a0a,0
   dc.w   $80e,$90c,$a0a,0,$80e,$90c,$a0a,0
   dc.w   $80d,$90d,$a0c,0,$80d,$90d,$a0c,0
   dc.w   $80e,$90c,$a09,0,$80e,$90c,$a09,0
   dc.w   $80e,$90c,$a05,0,$80e,$90c,$a00,0
   dc.w   $80e,$90c,$a00,0,$80e,$90b,$a0b,0
   dc.w   $80e,$90b,$a0b,0,$80e,$90b,$a0b,0

   dc.w   $80e,$90b,$a0b,0,$80e,$90b,$a0a,0
   dc.w   $80e,$90b,$a0a,0,$80e,$90b,$a0a,0
   dc.w   $80d,$90d,$a0b,0,$80d,$90d,$a0b,0
   dc.w   $80d,$90d,$a0b,0,$80e,$90b,$a09,0
   dc.w   $80e,$90b,$a09,0,$80e,$90b,$a09,0
   dc.w   $80d,$90c,$a0c,0,$80d,$90d,$a0a,0
   dc.w   $80e,$90b,$a07,0,$80e,$90b,$a00,0
   dc.w   $80e,$90b,$a00,0,$80d,$90d,$a09,0

   dc.w   $80d,$90d,$a09,0,$80e,$90a,$a09,0
   dc.w   $80d,$90d,$a08,0,$80d,$90d,$a07,0
   dc.w   $80d,$90d,$a04,0,$80d,$90d,$a00,0
   dc.w   $80e,$90a,$a04,0,$80e,$909,$a09,0
   dc.w   $80e,$909,$a09,0,$80d,$90c,$a0b,0
   dc.w   $80e,$909,$a08,0,$80e,$909,$a08,0
   dc.w   $80e,$909,$a07,0,$80e,$908,$a08,0
   dc.w   $80e,$909,$a01,0,$80c,$90c,$a0c,0

   dc.w   $80d,$90c,$a0a,0,$80e,$908,$a06,0
   dc.w   $80e,$907,$a07,0,$80e,$908,$a00,0
   dc.w   $80e,$907,$a05,0,$80e,$906,$a06,0
   dc.w   $80d,$90c,$a09,0,$80e,$905,$a05,0
   dc.w   $80e,$904,$a04,0,$80d,$90c,$a08,0
   dc.w   $80d,$90b,$a0b,0,$80e,$900,$a00,0
   dc.w   $80d,$90c,$a06,0,$80d,$90c,$a05,0
   dc.w   $80d,$90c,$a02,0,$80c,$90c,$a0b,0

   dc.w   $80c,$90c,$a0b,0,$80d,$90b,$a0a,0
   dc.w   $80d,$90b,$a0a,0,$80d,$90b,$a0a,0
   dc.w   $80d,$90b,$a0a,0,$80c,$90c,$a0a,0
   dc.w   $80c,$90c,$a0a,0,$80c,$90c,$a0a,0
   dc.w   $80d,$90b,$a09,0,$80d,$90b,$a09,0
   dc.w   $80d,$90a,$a0a,0,$80d,$90a,$a0a,0
   dc.w   $80d,$90a,$a0a,0,$80c,$90c,$a09,0
   dc.w   $80c,$90c,$a09,0,$80c,$90c,$a09,0

   dc.w   $80d,$90b,$a06,0,$80c,$90b,$a0b,0
   dc.w   $80c,$90c,$a08,0,$80d,$90b,$a00,0
   dc.w   $80d,$90b,$a00,0,$80c,$90c,$a07,0
   dc.w   $80c,$90c,$a06,0,$80c,$90c,$a05,0
   dc.w   $80c,$90c,$a03,0,$80c,$90c,$a01,0
   dc.w   $80c,$90b,$a0a,0,$80d,$90a,$a05,0
   dc.w   $80d,$90a,$a04,0,$80d,$90a,$a02,0
   dc.w   $80d,$909,$a08,0,$80d,$909,$a08,0

**************************************
*       MFP/FREQUENCY TABLE          *
**************************************
   cnop 0,4
FREQTAB   DC.W   $0506   4.9 KHZ
   DC.W   $0505   7.68 KHZ
   DC.W   $0405   9.6 KHZ
   DC.W   $1F01   19.8 KHZ
   DC.W   $0802   30.7 KHZ
   DC.W   $0106   
   DC.W   $0106

**************************************
*   DEFINE PROGRAM STORAGE       *
**************************************
   cnop 0,4
XTERNAL   DS.B   1   $FF = EXTERNAL O/P SELECTED
INUSE   DS.B   1   INTERRUPT IN USE FLAG
OPTION   DS.B   1   INT. OPTION  $FF = SAMPLE
   cnop 0,4
FREQNUM   DS.W   1   FREQUENCY SELECTED
MFPMEM   DS.B   8   SAVED MFP DATA

CURSRLO   ds.l   $01
CURSRHI ds.l   $01

ouch_start  equ   *
begin_start equ   ouch_start+$0FD0
over_start  equ   begin_start+$AE20
pause_start equ   over_start+$8039
resume_start equ  pause_start+$31A2
intro_start equ   resume_start+$332D


; ****************************************************************************
               end


; ****************************************************************************
