START          PEA            BUFFER         ;ADDR OF SUBROUTINE
               MOVE.W         #38,-(SP)      ;EXECUTE IN SUPERVISOR MODE
               TRAP           #14            ;XBIOS
               ADDQ.L         #6,SP          ;POP STACK

PROTOBT        CLR.W          -(SP)          ;NOT EXECUTABLE
               MOVE.W         #2,-(SP)       ;80-TRACK, SINGLE-SIDED
               MOVE.L         #$01000000,-(SP)    ;RANDOM SERIAL NUMBER
               PEA            BUFFER         ;ADDR OF TRACK 0, SECTOR 1
               MOVE.W         #18,-(SP)      ;PROTOTYPE BOOT SECTOR
               TRAP           #14            ;XBIOS
               ADD.L          #14,SP         ;POP STACK

               MOVEQ          #0,D7          ;CLEAR REGISTER
               MOVE.W         RDV_NUMCL,D7   ;(USER K-BYTES) + 2
               ADD.W          #9,D7          ;DISK OVERHEAD
               ASL.W          #1,D7          ;CONVERT TO SECTORS
               MOVE.L         D7,D6          ;TEMP STORAGE
               MOVE.B         D7,BUFFER+19   ;HIGH BYTE IN 8086 FORMAT
               ASR.W          #8,D7          ;ISOLATE LOW BYTE
               MOVE.B         D7,BUFFER+20   ;MOVE IT

               MOVE.L         #9,D7          ;SHIFT FACTOR
               ASL.L          D7,D6          ;CONVERT TO BYTES
               ADD.L          #256+BUFFER-START,D6;BYTES TO PROTECT

KEEP           CLR.W          -(SP)          ;NO ERROR CODE
               MOVE.L         D6,-(SP)       ;BYTES TO PROTECT
               MOVE.W         #$31,-(SP)     ;KEEP
               TRAP           #1             ;GEMDOS

HDV_BPB_TEST   CMPI.W         #3,4(SP)       ;RAM DISK ACCESS?
               BNE.S          HDV_BPB_CONT   ;IF NOT TAKE BRANCH
               MOVE.L         #RDV_BPB_DATA,D0    ;ADDR OF RAM DISK BPB
               RTS                           ;RETURN WITH ADDRESS

HDV_BPB_CONT   MOVEA.L        HDV_BPB_DFLT,A0     ;ADDR OF HARD DRIVE BPB
               JMP            (A0)           ;EXIT TO DEFAULT VECTOR

HDV_RW_TEST    CMPI.W         #3,14(SP)      ;RAM DISK ACCESS?
               BNE.S          HDV_RW_CONT    ;IF NOT TAKE BRANCH

               LEA            BUFFER,A0      ;ADDR OF TRACK 0, SECTOR 1
               MOVEQ          #0,D0          ;CLEAR REGISTER
               MOVE.W         12(SP),D0      ;FIRST SECTOR FOR R/W
               MOVE.L         #9,D1          ;SHIFT COUNT
               ASL.L          D1,D0          ;CONVERT SECTORS TO BYTES
               ADDA.L         D0,A0          ;ADDR OF FIRST SECTOR FOR R/W

               MOVEA.L        6(SP),A1       ;ADDR OF SYSTEM R/W BUFFER

               BTST.B         #0,5(SP)       ;READ OR WRITE?
               BNE.S          RDV_RW_CONT    ;IF WRITE TAKE BRANCH
               EXG            A0,A1          ;REVERSE DIRECTION OF MOVE

RDV_RW_CONT    MOVE.W         10(SP),D0      ;NUMBER OF SECTORS TO R/W
               SUBQ.W         #1,D0          ;ADJUST FOR DBRA
OUT_LOOP       MOVEQ          #127,D1        ;(LONG WORDS/SECTOR) - 1
IN_LOOP        MOVE.L         (A1)+,(A0)+    ;MOVE LONG WORD
               DBRA           D1,IN_LOOP     ;IF NOT DONE TAKE BRANCH
               DBRA           D0,OUT_LOOP    ;IF NOT DONE TAKE BRANCH

               MOVEQ          #0,D0          ;NO ERROR CODE
               RTS                           ;RETURN WITH PARAMETER

HDV_RW_CONT    MOVEA.L        HDV_RW_DFLT,A0 ;ADDR OF HARD DRIVE R/W
               JMP            (A0)           ;EXIT TO DEFAULT VECTOR

HDV_MEDIACH_TEST    CMPI.W    #3,4(SP)       ;RAM DISK ACCESS?
               BNE.S          HDV_MEDIACH_CONT    ;IF NOT TAKE BRANCH
               MOVEQ          #0,D0          ;SAME DRIVE NUMBER
               RTS                           ;RETURN WITH PARAMETER

HDV_MEDIACH_CONT    MOVEA.L   HDV_MEDIACH_DFLT,A0 ;ADDR OF HDV_MEDIACH
               JMP            (A0)           ;EXIT TO DEFAULT VECTOR

               DATA
RDV_BPB_DATA   DC.W           512            ;BYTES/SECTOR
               DC.W           2              ;SECTORS/CLUSTER
               DC.W           1024           ;BYTES/CLUSTER
               DC.W           7              ;SECTORS FOR DIRECTORY
               DC.W           5              ;SECTORS FOR FAT
               DC.W           6              ;SECTOR ADDR OF 2ND FAT
               DC.W           18             ;SECTOR ADDR OF START OF DATA
RDV_NUMCL      DC.W           152            ;(USER K-BYTES) + 2

               BSS
HDV_BPB_DFLT   DS.L           1              ;SYSTEM HDV_BPB VECTOR
HDV_RW_DFLT    DS.L           1              ;SYSTEM HDV_RW VECTOR
HDV_MEDIACH_DFLT    DS.L      1              ;SYSTEM HDV_MEDIACH VECTOR

               TEXT

;STORE SYSTEM HARD DRIVE VECTORS AND SUBSTITUTE RAM DISK VECTORS
BUFFER         MOVE.L         $472,HDV_BPB_DFLT
               MOVE.L         #HDV_BPB_TEST,$472
               MOVE.L         $476,HDV_RW_DFLT
               MOVE.L         #HDV_RW_TEST,$476
               MOVE.L         $47E,HDV_MEDIACH_DFLT
               MOVE.L         #HDV_MEDIACH_TEST,$47E

               OR.W           #8,$4C2+2      ;ACTIVATE RAM DISK

;CLEAR TRACK 0, SECTOR 2 THROUGH TRACK 1, SECTOR 9
               LEA            BUFFER+512,A6  ;ADDR OF TRACK 0, SECTOR 2
               MOVE.W         #2175,D7       ;(LONG WORDS/17 SECTORS) - 1
LOOP           CLR.L          (A6)+          ;INITIALIZE TO ZERO
               DBRA           D7,LOOP        ;IF NOT DONE TAKE BRANCH

               RTS                           ;RETURN TO CONTINUE INIT

               END
