/**
*                        CHK a CHKDSK work-alike
*                          version 2.3 03/05/87
*                                   by
*                             Daniel L. Moore
*                                  and
*                               David Small
*
*     Copyright (c) 1986, 1987 by Daniel L. Moore and David Small
*
* CHK is a C program that performs some of the same functions as the
* MSDOS utility CHKDSK.COM.  Basicly in verifies that all the file
* allocation chains in the FAT are accurate.  It will detect crossed
* allocation chains (2 (or more) files with the same data clusters),
* lost chains (a cluster chain that is not free or part of a file),
* bad chains (a cluster chain that links to a bad or non-existent
* cluster) and circular chains (a cluster chain that links to itself).
* If lost chains are found they are (optionally) 'recovered' by creating
* new directory entries that hook to the chain.  In addition CHK can be
* used to identify which file or files contain a given cluster.
*
* The symbol MSDOS is only defined when compiling on a MSDOS computer,
* it is not defined when compiling on the ST.  This allows the code to
* be directly ported (no changes) from IBM to ST.
*
* This program is written in 'portable' C, and should compile on any MSDOS
* or ST compiler with no changes.  (Assuming that long ints are 32 bits and
* chars are 8 bits.  Also it assumes a unsigned char can be assigned to
* an int without sign extension.)  The size of ints shouldn't affect the code
* (other than the execution speed).  The only non-portable code is the
* hexadecimal string to integer call in main() to get the value of find_cluster.
* The routines used, stch_i and strtol, are from the Lattice (for the IBM)
* Megamax (for the ST) C libraries.  Other C compilers will probably have
* different routines.  If nothing else the atoi function listed in K&R
* will work, though it won't accept hexadecimal input.
*
*                        -------------------------
*
*         Dedication:
* 
*         To Opus Croakus and his rhythm tuba.
*              He needs to be checked out.
*
**/

/*
 * defines to indicate which ST C compiler is being used.
 *
 * The program has been compiled with Megamax (versions 1.0 and 1.1) 
 * and Mark Williams (versions 1.0.4 and 1.1).  Mark Williams produces
 * the smallest program (after symbols are stripped), Megamax produces
 * the fastest running program.  CHK has not been tested with Lattice or
 * Alcyon.  (NOTE: be sure to set the sign ext define correctly when
 * changing compilers.)
 */
#define MEGAMAX 1       
#define MWC     0

/*------------------------*/
/*  Megamax defines       */
/*------------------------*/
#if MEGAMAX
#define void /**/
#endif

/*------------------------*/
/*  system header files   */
/*------------------------*/
#include <stdio.h>
#include <ctype.h>
#ifndef MSDOS
#include <osbind.h>    /* BIOS/XBIOS calls for the ST   */
#endif

/*------------------------*/
/*  external C/asm funcs  */
/*------------------------*/
#ifdef MSDOS
	extern int      sec_read();   /* external assembly routines to read */
	extern int      sec_writ();   /* and write sectors.                 */
	extern int      stch_i();     /* Lattice C hexadecimal to int       */
#else
#ifdef MEGAMAX
	extern long int strtol();     /* Megamax C hexadecimal to int       */
#else
	extern int atoi();            /* generic decimal to int             */
#endif
#endif

#endif

/* These "should" be defined in stdio.h, but just in case ... */
extern char *calloc();
extern char *malloc();

/* "smart" free that checks for null pointers first */
#define free0(addr)   if (addr) free(addr)

/*------------------------*/
/*  macro definitions     */
/*------------------------*/
#define FAT1         1      /* use fat copy number 1 or 2       */
#define SIGN_EXT     0      /* for stupid compilers that force sign extension */

   /* 
    * The following are the attribute byte bit values used by MSDOS to
    * identify file type.  (Also GEMDOS.)  All other bits are 'reserved'
    * for future use and should be ignored.
    */
#define READONLY    0x01
#define HIDDEN      0x02
#define SYSTEM      0x04
#define VOLABEL     0x08
#define SUBDIR      0x10
#define ARCHIVE     0x20       /* not maintained correctly by GEMDOS */

	/* misc constants */
#define TRUE        1
#define FALSE       0
#define BADFLG      0xff
#define EMPTYFLG    0xfe
#define BADCHAIN    1
#define CROSSCHAIN  2
#define CIR_CHAIN   3
#define GOODCHAIN   0

#ifndef MSDOS
	/* rwabs commands */
#define READ_CMD     0
#define WRITE_CMD    1
#define READ_NOCHG   2
#define WRITE_NOCHG  3
#endif

	/* fat data markers */
#define END16       0xfff7
#define END12       0x0ff7
#define BAD16       0xfff0
#define BAD12       0x0ff0
#define IMP16       0x8000

   /* 
    * get_word and get_long return word and long values from the pointer,
    * put_word and put_long store word and long values to the pointer.
    * Macros are used in order to make the port to the ST easier.  (The
    * ST uses C functions that take care of the byte order problem.)
    * The get_byte macro is used to convert a signed char to an unsigned
    * int value.  Some C compilers may require the version with the & 0xff
    * in order to convert unsigned chars to unsigned int with no sign
    * extension.  Problems will also occur if a compiler accepts
    * unsigned as a type instead of a modifier.
    */
#if SIGN_EXT       /* stupid compilier??? */
#define get_byte(x)   (*(x) & 0xff))
#else
#define get_byte(x)   ((unsigned int) *(x))
#endif

#ifdef MSDOS
#define get_word(x)     (*((unsigned int *) (x)))
#define get_long(x)     (*((long int *) (x)))
#define put_word(x, y)  *((unsigned int *) (x)) = y
#define put_long(x, y)  *((long int *) (x)) = y
#else
	unsigned int
	get_word(addr)
	register unsigned char *addr;
	/*
	 * Return the word at addr.  The word is stored in low-high order
	 * (80x8x style) so the bytes must be swapped for the 68xxx.
	 */
	{
		register unsigned int value;

		/* 
		 * must use 2 statements since evaluation order of expressions
		 * in a statement is not defined in C.
		 */
		value  = get_byte(addr++);
		value += get_byte(addr) << 8;
		return(value);
	}

	long
	get_long(addr)
	register unsigned char *addr;
	/* 
	 * Return the long word at addr.  The data is stored in low-high
	 * order, so it must be read byte by byte.
	 */
	{
		register long value;

		/* again seperate statements due to uncertainties in the
		   evaluation order.                                   */
		value  = get_byte(addr++);
		value += (long int) get_byte(addr++) <<  8;
		value += (long int) get_byte(addr++) << 16;
		value += (long int) get_byte(addr)   << 24;
		return(value);
	}

	void
	put_word(addr, val)
	register unsigned char *addr;
	register unsigned int   val;
	/* 
	 * Store val at the word at addr.  The data will be stored in low-high
	 * order, so it will be stored one byte at a time.
	 */
	{
		*(addr++) = val & 0xff;
		*addr     = (val >> 8) & 0xff;
	}

	void
	put_long(addr, val)
	register unsigned char *addr;
	register long int        val;
	/* 
	 * Store val at the word at addr.  The data will be stored in low-high
	 * order, so it will be stored one byte at a time.         
	 */
	{
		*(addr++) = val & 0xff;
		*(addr++) = (val >>  8) & 0xff;
		*(addr++) = (val >> 16) & 0xff;
		*addr     = (val >> 24) & 0xff;
	}

#if MEGAMAX
	/* Megamax doesn't have a setmem function */
	void
	setmem(p, n, c)
	register unsigned char *p;
	register unsigned int   n;
	register unsigned char  c;
	{
		for (; n; n--)
			*(p++) = c;
	}

	/* Megamax also doesn't have a movmem function */
	void
	movmem(s, d, n)
	register unsigned char *s, *d;
	register unsigned int   n;
	{
		/* NOTE: This assumes NO overlap of source and destination! */
		for (; n; n--)
			*(d++) = *(s++);
	}
#endif
#endif

/*------------------------*/
/*  global work variables */
/*------------------------*/

		/* data from boot sector  */
char         id[9];                /* disk id                           */
unsigned int sec_size;             /* sector size in bytes              */
unsigned int clst_size;            /* sectors per cluster               */
unsigned int sec_res;              /* reserved sectors at start of disk */
unsigned int fat_copies;           /* number of fats on disk            */
unsigned int root_entries;         /* number of root directory entries  */
unsigned int format_id;            /* disk format id byte               */
unsigned int sec_fat;              /* sectors per fat                   */
unsigned int sec_track;            /* sectors per track                 */
unsigned int sides;                /* number of disk sides              */
unsigned int sec_hid;              /* reserved sectors at end of disk   */
unsigned int total_sectors;        /* sectors per disk                  */

		/* misc work variables    */
unsigned int drive;                /* current drive                     */
unsigned int fat_bytes;            /* number of bytes in fat buffer     */
unsigned int clusters;             /* number of clusters on disk        */
unsigned int clst_bytes;           /* number of bytes per cluster       */
unsigned int dir_sec;              /* first directory sector            */
unsigned int dir_cnt;              /* number of root dir sectors        */
unsigned int dat_start;            /* first data sector                 */
unsigned int find_cluster;         /* look for this cluster             */
unsigned int cross_cluster;        /* cluster a file is cross linked at */
int          found_it;             /* true means it was found.          */
long int     free_bytes;           /* free bytes on drive               */
long int     bad_bytes;            /* bytes in bad sectors              */
long int     file_bytes;           /* bytes in files (clusters)         */
long int     dir_bytes;            /* bytes in directories (clusters)   */
unsigned int dirs;                 /* number of directories             */
unsigned int files;                /* number of files                   */

int          verbose   = FALSE;    /* TRUE means LOTS of messages       */
int          map       = FALSE;    /* TRUE means print the cluster map  */
int          talkitive = FALSE;    /* TRUE means medium number of msgs  */
int          stats     = FALSE;    /* TRUE means just report boot block info */
int          fix       = FALSE;    /* TRUE means fix the disk           */

/*
 * The following variables are used to test for end of cluster chains and
 * bad clusters.  They are set based on whether the disk has a 16 bit or
 * 12 bit fat.
 */
unsigned int bad_mark;             /* bad cluster mark                  */
unsigned int end_mark;             /* end of file cluster mark          */
unsigned int imp_mark;             /* 'illegal' cluster value           */
	    int fat16 = FALSE;        /* TRUE means 16 bit fat             */

		/* pointers to work buffers */
unsigned char *fat_buf;            /* disk fat buffer                   */
unsigned char *used;               /* array of cluster flags            */
unsigned char *cir_buf;            /* circular file test buffer         */
unsigned char *dir_buf;            /* directory buf, one cluster        */
unsigned char *sec_buf;            /* single sector buffer              */

/*------------------------*/
/*     program exit       */
/*------------------------*/
void
term(bye)
int bye;
/* Free all allocated memory, then exit. */
{
	/* 
	 * If bye is not 5 there was not enough memory to allocate the work
	 * buffers.  Either there isn't much memory (unlikely with 512K or more)
	 * or the disk is REALLY messed up.  (This might cause things like
	 * 400K clusters or fats.)  
	 */

	if (bye != 5)
		printf("\nUnable to allocate work buffers.\nABORTING\n");

#ifndef MSDOS
	/* pause so the user can see things if run from the desktop */
	printf("\nPress RETURN to exit.\n");
	getchar();
#endif

	/*
	 * being appropriately paranoid, check for non-zero before
	 * calling free on these pointers.  it never hurt anyone
	 * to check.  (Besides it is needed for the stats mode.)
	 */

	switch (bye) {
		case 5 :  free0(cir_buf);    /* and we all fall thru        */
		case 4 :  free0(used);
		case 3 :  free0(fat_buf);
		case 2 :  free0(dir_buf);
		case 1 :  free0(sec_buf);
		default:  exit(bye != 5);   /* exit with error status code */
	}
}

/*------------------------*/
/*      sector i/o        */
/*------------------------*/
void
read_sector(sec_num, addr, count)
unsigned int sec_num, count;
unsigned char *addr;
/* Call the BIOS to read count sectors from the disk. */
{
#ifdef MSDOS
	/* 
	 * sec_read and sec_write are assembly routines to call the IBM bios
	 * read or write sector routines.  They can't be called from C due to
	 * the status register being on the stack when an exit occurs.
	 */

	sec_read(sec_num, drive, addr, count);
#else
	/* 
	 * The READ_CMD vs READ_NOCHG is needed due to a bug in some RAM disk
	 * drivers.  (They WRITE when given the READ_NOCHG command!  Really nasty
	 * of them.) READ_NOCHG is used on floppies so the user can switch disks
	 * on a single drive system.  (A READ_CMD will error out if the disk
	 * is switched.)  Since drives beyond 2 are 'fixed' (hard drives and ram
	 * disks) it doesn't matter which read command is used on them.
	 */

	Rwabs((drive < 2 ? READ_NOCHG : READ_CMD), addr, count, sec_num, drive);
#endif
}

void
write_sector(sec_num, addr, count)
unsigned int sec_num, count;
unsigned char *addr;
/* Call the BIOS to write count sectors to the disk. */
{
#ifdef MSDOS
	sec_writ(sec_num, drive, addr, count);
#else
	Rwabs(WRITE_NOCHG, addr, count, sec_num, drive);
#endif
}

/*------------------------*/
/*      cluster i/o       */
/*------------------------*/
void
read_cluster(which)
int which;
/*
 * Read the cluster to the dir_buf.  This is used to read the sub-directory
 * files.
 */
{
	/*
	 * Since the first 2 clusters (number 0 and 1) don't exist the desired
	 * cluster number must be decremented by 2 before multiplying by the
	 * number of sectors per cluster.  
	 */

	read_sector(dat_start + ((which - 2) * clst_size), dir_buf, clst_size);
}

/*------------------------*/
/*  fat support routines  */
/*------------------------*/
unsigned int
fat_entry(which)
unsigned int which;
/* 
 * Unpack a FAT entry.   In a 12 bit FAT every 3 bytes contain 2 FAT
 * entries.  For an odd entry number the low hex digit should be
 * dropped, for an even entry number the high hex digit should be dropped.
 * In a 16 bit FAT each entry occupies a word, so just return it.
 */
{
	register unsigned int val;

	if (fat16)
		return(get_word(fat_buf + which * 2));
	else {
		val = get_word(fat_buf + ((which * 3) / 2));

		if (which & 1)           /* odd or even?    */
			val >>= 4;          /* drop low digit  */
		val &= 0x0fff;           /* drop high digit */
		return(val);
	}
}

void
put_entry(which, value)
unsigned int which, value;
/* Write a FAT entry. */
{
	register unsigned int val;

	if (fat16)
		put_word(fat_buf + which * 2, value);
	else {
		val = get_word(fat_buf + ((which * 3) / 2));

		if (which & 1) {
			value <<= 4;        /* shift to high portion */
			val &= 0x000f;
		}
		else
			val &= 0xf000;

		put_word(fat_buf + ((which * 3) / 2), val | value);
	}
}

void
read_fat()
/* 
 * Allocate the memory for the fat and used buffers, then load the first
 * disk fat to the fat_buf.  (There may be more than 1 fat on the disk, but
 * CHK doesn't look at them.  I decided not to have CHK check the 2nd fat
 * because the MSDOS CHKDSK.COM program doesn't.  This leads me to believe
 * it _may_ not always be correctly updated.  (But then why did Microsoft
 * decide to put it there?))
 */
{
	register unsigned int i, value;

	/*
	 * 12 bit fats are limited to 4079 (0x0fef) data clusters.   If there
	 * are more clusters than that it MUST be a 16 bit fat.  Check and
	 * init the global fat test variables.  fat16 may have been set to true
	 * if the BIOS getbpb call was used due to a non-readable boot sector.
	 */

	if (clusters >= 0x0ff0 || fat16) {  /* Is this a 16 bit or 12 bit fat? */
		fat16 = TRUE;
		end_mark = END16;
		bad_mark = BAD16;
		imp_mark = IMP16;
	}
	else {
		end_mark = END12;
		bad_mark = BAD12;
		imp_mark = BAD12;
	}

	/* allocate the fat buffer */
	if ((fat_buf = (unsigned char *) calloc(sec_fat, sec_size)) == NULL)
		term(2);

	/* allocate the used cluster buffer */
	if ((used = (unsigned char *) calloc(clusters + 2, 1)) == NULL)
		term(3);

	/* allocate the circular file test buffer */
	if ((cir_buf = (unsigned char *) calloc(clusters + 2, 1)) == NULL)
		term(4);

	/* read the fat */
#if FAT1
	read_sector(sec_res, fat_buf, sec_fat);
#else  /* use the second fat */
	read_sector(sec_res + sec_fat, fat_buf, sec_fat);
#endif

	/* 
	 * Look at all the fat entries and mark the empty and bad clusters
	 * in the used cluster array.  A 0 entry means the cluster is empty
	 * and available for use.   A $FF7 (12 bit) or $FFF7 (16 bit) entry
	 * means the cluster contains a bad sector.  (Entries $FF1 to $FF6
	 * can also be used to mark the cluster as unusable.)            
	 */

	*used = *(used + 1) = 1;  /* first 2 clusters don't exist */
	for (i = 2; i < clusters + 2; i++) {
		if ((value = fat_entry(i)) == 0) {   /* empty cluster? */
			free_bytes += clst_bytes;
			/* mark cluster as empty in the used array    */
			*(used + i) = EMPTYFLG;
		}
		else if (value > bad_mark && value <= end_mark) {
			bad_bytes += clst_bytes;
			/* mark cluster as unusable in the used array */
			*(used + i) = BADFLG;
		}
	}
}

int
trace_file(which, attr)
register unsigned int which, attr;
/* 
 * Trace all the entries in a files cluster allocation and mark them as
 * used.  Return an error code based on which (if any) errors were found
 * in the allocation chain.  (Possible returns are GOODCHAIN, CIR_CHAIN,
 * CROSSCHAIN and BADCHAIN.)  Each time a cluster is used its used array
 * entry is incremented until it reaches 0xFD (0xFE and 0xFF are used to
 * mark clusters as empty and unusable).                              
 */
{
	register unsigned int value;
	register int cnt = 0, cross = GOODCHAIN;

	/* Count the directories and files on the disk. */
	if (attr & SUBDIR)
		dirs++;
	else
		files++;

	if (verbose)
		printf("cluster chain:\n");

	/* 
	 * cir_buf is used to check for a circular file allocation chain.
	 * As a file is traced each cluster is marked in cir_buf. If
	 * it is already marked the file has a circular link.  Set all
	 * entries to unused.  When traceing the file, check to see if
	 * it uses a cluster twice (a circular link).                    
	 */
	setmem(cir_buf, clusters + 2, 0);

	/* init for the cluster find code */
	found_it = FALSE;

	do {
		/* count the bytes used by directories and files */
		if (attr & SUBDIR)
			dir_bytes += clst_bytes;
		else
			file_bytes += clst_bytes;

		/* see if this is the 'desired' cluster */
		if (find_cluster == which)
			found_it = TRUE;

		if (verbose) {
			printf("%4x ", which);
			if (++cnt % 15 == 0)
				putchar('\n');
		}

		if (cir_buf[which]) {    /* circular link? */
			cross = CIR_CHAIN;
			break;
		}

		if (value = get_byte(used + which)) {
			cross_cluster = which;
			cross = CROSSCHAIN;
		}

		/* increment the used cluster array entry.    */
		if (value <= EMPTYFLG)
			(*(used + which))++;

		/* mark the cluster as used in the circular test buffer */
		cir_buf[which] = 1;

		which = fat_entry(which);
		/* 
		 * Check for "good" cluster link.   Links can't be 0 or 1 and
		 * if a 16 bit fat it can't be 0x8000 to 0xffef.            
		 */
		if (which==0 || which==1 || (which>=imp_mark && which<=end_mark)) {
			cross = BADCHAIN;
			break;
		}
	}
	while (which <= end_mark);

	if (verbose)
		putchar('\n');

	return(cross);
}


/*------------------------*/
/*   boot sector i/o      */
/*------------------------*/
int
read_boot()
/*
 * Read in the disk boot sector (sector number 1).  This sector contains
 * a table that describes most of the disk format information.   After
 * the first sector is read the global disk data variabls are set.  If the
 * first sector is not correctly formatted for a GEMDOS boot sector call
 * the BIOS getbpb routine.  (This may be needed on some hard drives and
 * RAM disk programs.)   If that fails the disk is NOT a GEMDOS disk and
 * execution terminates.                                              
 */
{
	register int i;
#ifndef MSDOS
	struct BPB {
		int  recsiz,           /* sector size                       */
			clsiz,            /* cluster size in sectors           */
			clsizb,           /* cluster size in bytes             */
			rdlen,            /* root directory length in sectors  */
			fsiz,             /* FAT size  in sectors              */
			fatrec,           /* sector # of 1st sector of 2nd FAT */
			datrec,           /* sector # of 1st data sector       */
			numcl,            /* number of data clusters           */
			bflags;           /* misc. flags                       */
	} *bpb;
#endif

	/*
	 * Allow for up to 1024 byte sectors.  This is twice the sector size
	 * of any current IBM or ST drive.  (In fact the ST has a 1024 byte
	 * internal sector buffer, which makes larger sector sizes unlikely.) 
	 */
	if ((sec_buf = (unsigned char *) malloc(1024)) == NULL)
		term(0);

	read_sector(0, sec_buf, 1);         /* read boot record           */

	/* 
	 * get the sector size and cluster size.  If either is zero the boot
	 * sector in invalid.  use the bpb data instead.                    
	 */
	sec_size     = get_word(sec_buf+11);     /* bytes per sector      */
	clst_size    = get_byte(sec_buf+13);     /* sectors per cluster   */

#ifndef MSDOS
	/*
	 * read the bios parameter block from DOS.  This will be used if the boot
	 * sector is not valid.  If the boot sector is valid it will be used to
	 * check for a 16 bit fat on a disk with fewer than 4079 clusters.  (This
	 * is true on ST hard disks.   All fats are 16 bit no matter what how
	 * many clusters they have.                                      
	 */
	bpb = (struct BPB *) Getbpb(drive);
#endif

	if (sec_size != 0 && clst_size != 0) {      /* valid boot sector? */
	  /* Check if the system id is alpha/numeric.  On the ST not all disk
		have a system id.   (And the last 3 bytes are used as a 24 bit
		volume serial number.)  MSDOS has a full 8 character system id.*/
		for (i = 0; i < 8; i++)                  /* system id             */
			if (isprint(sec_buf[i+3]))
				id[i] = sec_buf[i+3];
			else
				break;
		id[i] = 0;

		sec_res       = get_word(sec_buf+14);  /* resevered sectors         */
		fat_copies    = get_byte(sec_buf+16);  /* number of FAT copies      */
		root_entries  = get_word(sec_buf+17);  /* entries in root directory */
		total_sectors = get_word(sec_buf+19);  /* sectors on disk           */
		format_id     = get_byte(sec_buf+21);  /* format id byte            */
		sec_fat       = get_word(sec_buf+22);  /* sectors per fat           */
		sec_track     = get_word(sec_buf+24);  /* sectors per track         */
		sides         = get_word(sec_buf+26);  /* sides on media            */
		sec_hid       = get_word(sec_buf+28);  /* hidden sectors on disk    */

		/* Calculate the number of directory sectors, the start of
		   the directory and the cluster size in bytes.  Each directory
		   entry is 32 bytes long.                                      */
		dir_cnt    = root_entries / (sec_size / 32);
		dir_sec    = sec_res + (fat_copies * sec_fat);
		clst_bytes = clst_size * sec_size;

		/*
		 * The first cluster is stored in the sector immediately following
		 * the end of the root directory sectors.                      
		 */
		dat_start  = dir_sec + dir_cnt;

		/*
		 * Clusters equals number of disk sectors - reserved sectors - fat
		 * sectors - root directory sectors divided by cluster size.   (This
		 * may cause some 'data' sectors to be unused.  There are NO partial
		 * clusters on the disk.)  There are 2 groups of "reserved" sectors
		 * on the disk.  The sec_res sectors are reserved from the "start"
		 * of the disk and inclucde the boot sector.  The sec_hid sectors
		 * are at the "real start" of the disk.  They are located before the
		 * current partition and are not in the total sectors count.  (This
		 * includes earlier partitions and the partion data sector.  Not all
		 * format programs correctly set this.)                         
		 */

		clusters = (total_sectors-sec_res-(fat_copies*sec_fat)-dir_cnt)/clst_size;

#ifndef MSDOS
		/* 
		 * check if the bpb says the fat is 16 bit.  
		 * this is done since some ST devices (HDs and ram disks) use
		 * 16 bit fats no matter how many clusters are on the disk.
		 */
		if (bpb)
			fat16       = bpb->bflags & 1;     /* true if 16 bit fat */
#endif
	}
	else {                /* invalid boot sector */
#ifndef MSDOS
		if (bpb) {          /* is there a bios parameter block? */
			sec_size    = bpb->recsiz;     /* bytes per sector           */
			clst_size   = bpb->clsiz;      /* sectors per cluster        */
			dir_cnt     = bpb->rdlen;      /* root dir length in sectors */
			fat16       = bpb->bflags & 1; /* 16 bit or 12 bit fat       */
			clusters    = bpb->numcl;      /* clusters on disk           */
			sec_res     = bpb->fatrec;     /* start of FAT               */
			sec_fat     = bpb->fsiz;       /* sectors per fat            */
			clst_bytes  = bpb->clsizb;     /* cluster size in bytes      */
			dat_start   = bpb->datrec;     /* first data sector          */

			/* 
			 * Calculate the number of directory entries and the start
			 * sector of the directory.                             
			 */
			root_entries = dir_cnt * (sec_size / 32);
			dir_sec      = dat_start - dir_cnt;

			/* No way to know how may fats there are.  Assume just 1. */
			fat_copies = 1;

			printf("Invalid boot sector.  Using BPB data instead.\n");
		}
		else
#endif
			return(FALSE); /* non DOS disk */
	}

	/* Allocate the sub-directory buffer. */
	if ((dir_buf = (unsigned char *) malloc(clst_bytes)) == NULL)
		term(1);

	return(TRUE);
}

/*------------------------*/
/*      directory i/o     */
/*------------------------*/
void
print_stat(name, ext, size, start, attr)
unsigned char *name, *ext;
unsigned int   attr, start;
long int       size;
/* 
 * If in verbose mode print the current file's name, size, starting cluster
 * number and the settings of it's attribute byte.                    
 */
{
	if (!verbose)
		return;

	printf("file: %s.%s length = %8ld start = %4x\n", name, ext, size, start);
	if (attr & READONLY)
		printf("   read_only\n");
	if (attr & HIDDEN)
		printf("   hidden\n");
	if (attr & SYSTEM)
		printf("   system\n");
	if (attr & VOLABEL)
		printf("   volume label\n");
	if (attr & SUBDIR)
		printf("   sub-directory\n");
	if (attr & ARCHIVE)
		printf("   archive\n");
}

int
write_dir(name, ext, size, date, start, attr)
unsigned char *name, *ext;
unsigned int   attr, start, date;
long int       size;
/* 
 * Write a new directory entry in the root directory.  If successfull
 * return a TRUE, otherwise return a false.                           
 */
{
	register unsigned int which= 0, root_sec, chr, found = FALSE;

	/* first locate an empty and/or deleted directory entry. */
	for (root_sec = 0; root_sec < dir_cnt; root_sec++) {
		read_sector(dir_sec+root_sec, sec_buf, 1);

		for (which = 0; which < sec_size; which += 32) {
			chr = get_byte(sec_buf + which);
			if (chr == 0 || chr == 0xe5) {
				found = TRUE;
				break;
			}
		}
		if (found)
			break;
	}

	if (!found)
		return(FALSE);          /* root directory is FULL   */

	/* Create the directory entry */
	movmem(name, sec_buf + which, 8);     /* set the file name        */
	movmem(ext, sec_buf + which+8, 3);    /* set the file extension   */

	*(sec_buf+which+11) = attr;         /* set the file attributes  */
	put_word(sec_buf+which+24, date);   /* set the creation date    */
	put_word(sec_buf+which+26, start);  /* set the starting cluster */
	put_long(sec_buf+which+28, size);   /* set the file size        */

	/* tell them about it */
	verbose = TRUE;
	printf("Creating ");
	print_stat(name, ext, size, start, attr);
	verbose = FALSE;

	/* Now write the directory sector out */
	write_sector(dir_sec+root_sec, sec_buf, 1);

	return(TRUE);
}

void
read_sub(first)
unsigned int first;
/* 
 * Read a sub-directory and trace all the files contained in it.  If
 * the sub-directory contains a sub-directory call read_sub again and
 * trace all the files in the sub-sub-directory (and so on).
 *
 * NOTE: Since this routine is recursive in theory it could crash the
 * system if directories are nested far enough.  Since Megamax supplies
 * an 8K stack this is unlikely (this allows 250+ recursive calls), but
 * other compilers may not supply a large enough stack.
 */
{
	register unsigned int which, start, attr, chr, status;
	unsigned char         name[9], ext[4];
	long int              size;

	name[8] = ext[3] = 0;

	for (;;) {
		read_cluster(first);

		for (which = 0; which < clst_bytes; which += 32) {
			movmem(dir_buf + which,     name, 8);  /* get file name        */
			movmem(dir_buf + which + 8, ext,  3);  /* get file extension   */

			attr  = get_byte(dir_buf+which+11);    /* get file attributes  */
			start = get_word(dir_buf+which+26);    /* get starting cluster */
			size  = get_long(dir_buf+which+28);    /* get file size        */

			/* 
			 * The first byte of the file name is used to mark directory
			 * entries as unused, deleted or as pointers to the parent
			 * or current sub-directory.  If not one of the above the file
			 * should be traced.                                   
			 */
			chr = get_byte(name);

			if (chr == 0) /* unused directory entry? */
				break;
			if (chr != 0xe5 && chr != '.') {
				if (verbose)
					putchar('\n');
				print_stat(name, ext, size, start, attr);
				if (start) {
					status = trace_file(start, attr);
					if (found_it)
						printf("file: %s.%s contains cluster %x\n", name,
							   ext, find_cluster);
					switch (status) {
						case CROSSCHAIN:
							printf("file: %s.%s is cross linked at cluster %x\n",
								  name, ext, cross_cluster);
							break;
						case BADCHAIN:
							printf("file: %s.%s has a bad allocation chain\n",
								  name, ext);
							break;
						case GOODCHAIN:
							if (attr & SUBDIR) {      /* sub directory   */
								read_sub(start);
							 /* 
							  * Reload the dir_buf with the 'current' cluster.
							  * The recursive call to read_sub has changed
							  * the contents of the buffer.              
							  */
								read_cluster(first);
							}
							break;
					}
				}
			}
		}
		/* Check if at end of the sub-directory file */
		if ((first = fat_entry(first)) > end_mark || chr == 0)
			break;
	}
}

void
read_root()
/* 
 * Basicly this is the same routine as read_sub, except it reads the
 * directory sectors instead of the file cluster reads used by read_sub. 
 */
{
	register unsigned int which, start, root_sec, attr, chr, status;
	unsigned char name[9], ext[4];
	long int size;

	name[8] = ext[3] = 0;

	for (root_sec = 0; root_sec < dir_cnt; root_sec++) {
		read_sector(dir_sec + root_sec, sec_buf, 1);

		for (which = 0; which < sec_size; which += 32) {
			movmem(sec_buf + which,     name, 8); /* get file name        */
			movmem(sec_buf + which + 8, ext,  3); /* get file extension   */

			attr  = get_byte(sec_buf+which+11);   /* get file attributes  */
			start = get_word(sec_buf+which+26);   /* get starting cluster */
			size  = get_long(sec_buf+which+28);   /* get file size        */

			chr = get_byte(name);
			if (chr == 0) /* unused directory entry? */
				break;
			if (chr != 0xe5 && chr != '.') {
				if (verbose)
					putchar('\n');
				print_stat(name, ext, size, start, attr);
				if (start) {
					status = trace_file(start, attr);
					if (found_it)
						printf("file: %s.%s contains cluster %x\n", name,
							   ext, find_cluster);
					switch (status) {
						case CROSSCHAIN:
							printf("file: %s.%s is cross linked at cluster %x\n",
								  name, ext, cross_cluster);
							break;
						case BADCHAIN:
							printf("file: %s.%s has a bad allocation chain\n",
								  name, ext);
							break;
						case GOODCHAIN:
							if (attr & SUBDIR)   /* sub directory */
								read_sub(start);
							break;
					}
				}
			}
		}
		if (chr == 0)
			break;
	}
}

/*------------------------*/
/*  lost cluster recovery */
/*------------------------*/
int
make_file(which, count)
unsigned int which, count;
/* 
 * Trace the cluster chain starting at which, mark the clusters in the
 * chain as used, and create a file that uses this cluster chain.      
 */
{
	register unsigned int last, temp, start;
	register int flag = TRUE;
	register long size;
	static char *name = "FILE    ";
	static char *ext = "CHK";

	/* 
	 * First make sure which is the first cluster in the chain, there
	 * maybe a later cluster that links back to it.                   
	 */
	start = which;
	while(flag) {
		flag = FALSE;
		for (temp = which + 1; temp< clusters + 2; temp++)
			if (*(used + temp) == 0)
				if (fat_entry(temp) == start) { /* link to current start? */
					start = temp;              /* new start of file */
					flag = TRUE;               /* say we found one  */
					break;
				}
	}

	/* start is the first cluster in the chain that contains which */
	last = temp = start;
	size = 0;

	/* 
	 * trace all the clusters in the chain, mark them as used and
	 * compute the file size.                                         
	 */
	while (*(used + temp) == 0) {
		*(used + temp) = 1;
		last = temp;
		size += clst_bytes;
		if ((temp = fat_entry(temp)) >= imp_mark)
			break;
	}
	put_entry(last, end_mark + 8); /* add the end of chain mark */

	/* 
	 * set the file name.  The orignal version just used the count
	 * to generate letters for the file name.  That was real simple and
	 * quick to write, but it did limit us to 26 recovered cluster
	 * chains.  First time I had a disk with more than that I had
	 * to change it to generate names with numbers instead.
	 *
	 * NOTE: this uses a semi-standard routine to generate the 
	 * ASCII version of the int.  Other compiler libraries may be
	 * different.  Check before compiling.
	 */
	itoa(count, &name[4]);

	/* 
	 * since that added a trailing null after the number we must 
	 * convert it to a space.  Oh, aren't things fun in computerdum.
	 */
	name[strlen(name)] = ' ';     /* nuke that space, tote that bale */

							/* now, now.  lets not get carried away */

	return(flag = write_dir(name, ext, size, 0, start, 0));
}

/*------------------------*/
/*   lost cluster check   */
/*------------------------*/
void
lost_check()
/* 
 * Check the used array to see if all clusters are in use.  If there
 * are any 0 entries then DOS has lost track of that cluster.  If lost
 * clusters are found attempt to create root directory entries pointing
 * to each cluster chain found.  The user can then check the files and
 * delete the unwanted ones which will 'free' the clusters.            
 */
{
	register unsigned int i;
	unsigned int count = 0, edited = FALSE, flag = TRUE, value;

	if (verbose)
		printf("\n");

	for (i = 2; i < clusters + 2; i++) {
		if (*(used + i) == 0) {
			printf("cluster %4x is unused links to %4x\n", i, fat_entry(i));
			if (fix && flag && count < 9999) {
				edited = TRUE;
				if (!(flag = make_file(i, ++count)))
					printf(" ERROR: root directory full\n");
		    }
		}
	}

	/* 
	 * write the edited fat out to fat #1 and possibly #2.  This routine
	 * assumes there are no more than 2 fats on the disk.                
	 */
	if (edited) {
		write_sector(sec_res, fat_buf, sec_fat);
		if (fat_copies == 2)
			write_sector(sec_res + sec_fat, fat_buf, sec_fat);
	}

	/* 
	 * If in verbose or map mode print out the used cluster array.  This 
	 * will let the user see where free clusters, file clusters and unusable
	 * clusters are located on the disk.  (NOTE: There is NO correlation
	 * between the print out and the disk tracks!)                    
	 */
	if (verbose || map) {
		printf("\ncluster usage map  cluster's 2 to $%x:\n$00002: ", clusters+1);
		for (i = 2; i < clusters + 2; i++) {
			value = get_byte(used + i);
			switch(value) {
				case BADFLG :   /* bad cluster */
					printf("  --");
					break;
				case EMPTYFLG : /* empty cluster */
					printf("  EM");
					break;
				default:
					printf("%4x", get_byte(used + i));
					break;
			}
			if (((i - 1) % 16) == 0) {
				putchar('\n');
				if (i < clusters + 1)
					printf("$%05x: ", i+1);
			}
		}
		putchar('\n');
	}
}

/*------------------------*/
/*         main           */
/*------------------------*/
void
main(argc, argv)
int  argc;
char *argv[];
/*
 * CHK expects at least one and optionally two parameters.  The first is
 * the drive letter to check, the second is an optional 'F', 'M', 'S', 'T',
 * 'V' or number.
 *
 * 'F' sets the fix mode, which tells chk to attempt to fix any errors it
 *     may detect.
 *
 * 'M' sets the map mode, this prints the cluster usage map.
 * 
 * 'S' will just print the boot block info.  Much quicker than the full
 *     test.  And it works on disks that are totally garbaged.
 *
 * 'T' will set talkitive mode, only a medium amount of messages.
 *
 * 'V' will set the verbose mode, which prints _lots_ of status messages.
 *
 * Number is the hexadecimal cluster number to 'watch' for.  If this cluster
 *     is in a file the filename will be printed out. 
 */
{
	static char *use = "usage:  chk drive: [-fmstv#]\n";

	/* Output the copyright and version number. */
	printf("\n\n    CHK version 2.3\n");
	printf("    Copyright (c) 1986, 1987 \n    by Daniel L. Moore and David Small\n\n");

	/* 
	 * First read the command line and parse the parameters. 
	 */
	if (argc < 2) {
		printf(use);
#ifndef MSDOS
		printf("\nPress RETURN to exit.\n");
		getchar();
#endif
		exit(1);
	}

	/* Set the drive number. */
	if (isalpha(*argv[1]) && argv[1][1] == ':')
		drive = tolower(*argv[1]) - 'a';
	else {
		printf("bad drive letter\n");
		printf(use);
#ifndef MSDOS
		printf("\nPress RETURN to exit.\n");
		getchar();
#endif
		exit(1);
	}

	/* 
	 * Check for the command flags
	 *
	 * This is really a very stupid (ie. unintelligent) command line
	 * parser.  But since most of the switches are mutally exclusive
	 * there isn't any need for a smarter one. 
	 */
	if (argc == 3 && *argv[2]++ == '-')
		if (isdigit(*argv[2])) {
#ifdef MSDOS   /* Lattice hex to int */
			find_cluster = stch_i(argv[2]);
#else
#ifdef MEGAMAX /* Megamax hex to int */
			find_cluster = (unsigned int) strtol(argv[2], NULL, 16);
#else          /* generic decimal to int */
			find_cluster = (unsigned int) atoi(argv[2]);
#endif
#endif
		}
		else {
			switch (tolower(*argv[2])) {
				case 'f' :
					fix       = TRUE;
					break;
				case 'm' :
					map       = TRUE;
					break;
				case 's' :
					stats     = TRUE;
					break;
				case 't' :
					talkitive = TRUE;
					break;
				case 'v' :
					verbose   = TRUE;
					break;
				default :      /* huh?  what command is this? */
					printf("bad command switch\n");
					printf(use);
					exit(1);
					break;
		    }
			find_cluster = 0;
		}

#ifndef MSDOS
	if (drive < 2) { /* drive A or B */
		/* on drive A and B allow the user to insert the desired disk. */
		printf("Press RETURN to begin checking drive %c:\n", drive + 'A');
		getchar();
	}
#endif

	  /* Finally go do all the work!       */
	if(read_boot()) {            /* read the boot sector                   */
		if (!stats) {           /* just the boot block info?              */
			read_fat();        /* read the disk fat                      */
			read_root();       /* read the root and sub directories      */
			lost_check();      /* check the used array for lost clusters */
		}
	}
	else {
		printf("Not a DOS disk.\nOperation aborted.\n");
#ifndef MSDOS
		printf("Press RETURN to exit.\n");
		getchar();
#endif
		term(1);
	}

	  /* Tell the user all about his disk. */
	if (talkitive || verbose || stats || map) {
		printf("\nsystem id            = %s\n", id);
		printf("format id            = %x\n", format_id);
		printf("sectors per track    = %-8u    sides                = %-8u\n", sec_track, sides);
		printf("sector size          = %-8u    total sectors        = %-8u\n", sec_size, total_sectors);
		printf("reserved sectors     = %-8u    hidden sectors       = %-8u\n", sec_res, sec_hid);
		printf("sectors per cluster  = %-8u    bytes per cluster    = %-8u\n", clst_size, clst_bytes);
		printf("fat copies           = %-8u    sectors per fat      = %-8u\n", fat_copies, sec_fat);
		printf("directory sectors    = %-8u    directory entries    = %-8u\n", dir_cnt,  root_entries);
		printf("total clusters       = %-8u\n", clusters);
	}

	/* 
	 * NOTE: the total bytes on disk and bytes available entries do NOT
	 * match the numbers that GEMDOS generates.  This is due to a bug in
	 * GEMDOS that causes it to ignore the last 2 clusters on the disk
	 * when writing files or calculating free space.  It will read files
	 * that use those clusters those (from MSDOS disks).  Anyway to get
	 * the same number as GEMDOS just subtract 2048 from the numbers.
	 */
	if (!stats) {
		printf("\n %8ld total bytes on disk\n",(long int) clusters * clst_bytes);
		printf(" %8ld bytes in %u files\n", file_bytes, files);
		printf(" %8ld bytes in %u directories\n", dir_bytes, dirs);
		printf(" %8ld bytes in bad sectors\n", bad_bytes);
		printf(" %8ld bytes available for use\n", free_bytes);
	}

	term(5);         /* and finally clean up and exit     */
}


