
******************************************************************
*
* A Text Enciphering Program-- Part 2
*
*   by Douglas Weir
*
*   Copyright 1988, ST-Log
*
******************************************************************

        text

        bsr             prompt                  determine mode

        move.l          #c_string,-(a7)         string space address
        bsr             x_cipher                encipher a string
        addq.l          #4,a7                   pop arg

        move.l          #p_string,-(a7)         print string address
        move.w          #9,-(a7)                code=print string
        trap            #1                      do it
        move.w          #0,-(a7)                code=exit program
        trap            #1                      do it

******************************************************************
*
* x_cipher-- encipher or decipher a text string.
*
*    at entry:
*             (a7) + 4 => space for storing processed string.
*
*    registers used:
*             d0 -- character from keyboard
*             d1 -- element from "incs"
*             d2 -- index into "incs"
*             d3 -- increment phase counter
*             d4 -- main loop counter
*             d5 -- not used
*             d6 -- encipher/decipher flag
*             d7 -- space flag
*
*             a0,a1 -- not used (affected by GEMDOS calls)
*             a2 -- not used
*             a3 -- base pointer to "ciphers"
*             a4 -- pointer to string space
*             a5 -- base pointer to "incs"
*             a6 -- frame pointer
*
*    at exit:
*             all registers preserved.
*
******************************************************************
  
x_cipher:
        link            a6,#0                   frame pointer
        movem.l         d0-d7/a0-a5,-(a7)       save registers
        
        movea.l         8(a6),a4                point to string space
        movea.l         #ciphers,a3             point to cipher table
        movea.l         #incs,a5                point to increments
        clr.l           d2                      reset inc index
        clr.l           d7                      reset space flag
        move.b          c_flag,d6               get cipher/decipher flag
        move.w          #I_PHASE,d3             reset phase counter
        move.w          #S_SIZE,d4              absolute counter
        move.b          #CR,last_ch             initialize last char
x_loop:
        move.w          #1,-(a7)                code=conin
        trap            #1                      do it
        addq.l          #2,a7                   pop arg
        cmpi.b          #CR,d0                  carriage return?
        beq             x_end                   if so

        cmpi.b          #SPACE,d0               space?
        beq.b           x_space                 if so
        cmpi.b          #TAB,d0                 else: tab?
        bne.b           x_nxt00                 if not
x_space:
        cmpi.b          #1,d7                   more than one?
        beq             x_test                  if so, skip everything
        move.b          #SPACE,last_ch          else save it
        move.b          #1,d7                   and set flag
        bra             x_raw                   and write it

x_nxt00:
        move.b          #0,d7                   reset space flag
        cmpi.b          #BS,d0                  backspace?
        bne.b           x_nxt0                  if not
        move.b          last_ch,d0              else get last char
        move.w          d0,-(a7)                push it
        move.w          #2,-(a7)                code=conout
        trap            #1                      do it
        addq.l          #4,a7                   pop args
        bra.b           x_loop                  and start over

x_nxt0:
        move.b          d0,last_ch              save raw char
        cmpi.b          #A,d0                   lower than 'A'?
        bcs.b           x_raw                   if so, just write it
        cmpi.b          #Z1,d0                  else: upper case?
        bcc.b           x_nxt1                  if not
        addi.w          #U_CONV,d0              else convert to lower case
        bra.b           x_nxt2                  and continue
x_nxt1:
        cmpi.w          #a,d0                   non-alphabetic?
        bcs.b           x_raw                   if so, write it
        cmpi.w          #z1,d0                  else: lower case?
        bcc.b           x_raw                   if not, write it as is

x_nxt2:
        subi.w          #C_CONV,d0              else make it an index
        move.b          0(a5,d2.w),d1           get current increment

        cmpi.b          #XDC,d6                 decipher?
        beq.b           x_nxt3                  if so
        add.w           d1,d0                   else encipher by adding inc
        cmpi.w          #C_MAX,d0               over maximum?
        bcs.b           x_nxt4                  if not
        subi.w          #C_MAX,d0               else correct it
        bra.b           x_nxt4                  and continue
x_nxt3:
        sub.w           d1,d0                   decipher by subtracting inc
        bcc.b           x_nxt4                  if >= 0, continue
        addi.w          #C_MAX,d0               else wrap around top

x_nxt4:
        move.b          0(a3,d0.w),d0           get enciphered code
        subq.w          #1,d3                   decrement phase counter
        bne.b           x_raw                   if phase not exhausted
        move.w          #I_PHASE,d3             else restart counter
        addq.l          #1,d2                   and increment index
        cmpi.w          #I_COUNT,d2             over maximum?
        bcs.b           x_raw                   if not
        subi.w          #I_COUNT,d2             else wrap around

x_raw:  move.b          d0,(a4)+                write it
x_test: dbra            d4,x_loop               go till end

x_end:  
        move.b          #LF,(a4)+               append line feed
        move.b          #CR,(a4)+               and carriage return
        move.b          #0,(a4)                 and null

        movem.l         (a7)+,d0-d7/a0-a5       restore registers
        unlk            a6                      deallocate frame
        rts                                     and return

******************************************************************
*
*    prompt-- determine function: encipher or decipher.
*
*    at exit:
*            registers d0-d1, a0-a1 are changed.
*
******************************************************************
 
prompt:
        move.l          #prompt_msg,-(a7)       prompt message
        move.w          #9,-(a7)                code=print string
        trap            #1                      do it
        addq.l          #6,a7                   pop args

        move.w          #1,-(a7)                code=conin
        trap            #1                      do it
        addq.l          #2,a7                   pop arg

        cmpi.b          #a,d0                   is letter < 'a'?
        bcc.b           p_nxt0                  if not
        addi.w          #U_CONV,d0              else convert
p_nxt0:
        cmpi.b          #e,d0                   'e' = encipher?
        bne.b           try_dec                 if not
        move.b          #ENC,c_flag             else set flag
        bra.b           prompt_out              and leave
try_dec:
        cmpi.b          #d,d0                   'd' = decipher?
        bne.b           prompt                  if not, keep trying
        move.b          #XDC,c_flag             else set flag
prompt_out:
        move.l          #prompt_end,-(a7)       newline
        move.w          #9,-(a7)                code=print string
        trap            #1                      do it
        addq.l          #6,a7                   pop args
        rts                                     and return

*****
*
* data area
*
*****
        
        data

prompt_msg      dc.b    'Type <e> to encipher, <d> to decipher'
prompt_end      dc.b    10,13,0

last_ch         ds.b    1               holds last char typed
c_flag          ds.b    1               signals cipher or decipher

p_string        dc.b    10,13           line feed, carriage return
c_string        ds.b    100             space for enciphered string
S_SIZE          equ     *-c_string      length of string

ciphers         dc.b    'abcdefghijklmnopqrstuvwxyz'
C_MAX           equ     *-ciphers       length of table
C_CONV          equ     97              'a' - 97 = 0
U_CONV          equ     32              'A' + 32 = 'a'

incs            dc.b    10,5,2,7,9,1    ciphering increments
I_COUNT         equ     *-incs          size of table
I_PHASE         equ     3               3 chars enciphered per inc

A               equ     65              ASCII 'A'
Z1              equ     91              ASCII 'Z' + 1
a               equ     97              ASCII 'a'
z1              equ     123             ASCII 'z' + 1
e               equ     101             ASCII 'e'
d               equ     100             ASCII 'd'

XDC             equ     77              flag value for deciphering
ENC             equ     88              flag value for enciphering
CR              equ     13              carriage return
LF              equ     10              line feed
TAB             equ     9               tab
BS              equ     8               backspace
SPACE           equ     32              space

                 

