/*********************************************************/
/* Computer Eyes 8bit Mode 9 picture loader by Mat*Rat   */
/* Developed with Megamax C, built upon CMANSHIP Issue#12*/
/* Altered STeights - Koala pictures TOO!                */
/*********************************************************/

#include <stdio.h>
#include <osbind.h>

/***********
** CONSTANTS
*/

#define uschar unsigned char
#define usint  unsigned int

/**************************
** File open type constants
*/

#define READ_ONLY  0
#define WRITE_ONLY 1
#define READ_WRITE 2

/****************************************
** FILL mode types
*/

#define REPLACE     1
#define TRANSPARENT 2
#define XOR         3
#define REVTRAN     4
/*************************************
** Screen resolution constants
*/

#define HI_REZ  2
#define MED_REZ 1
#define LO_REZ  0

/*************************************
** Mouse button definitions
*/

#define CANCEL 0
#define OK     1

/*************************************
** My error constants for error()
*/

#define A_OK                 0  /* NO errors occurred flag, general */
#define OPEN_ERROR          -1  /* Couldn't open specified file     */
#define CLOSE_ERROR         -2  /* Couldn't close a file            */
#define LEN_ERROR           -3  /* Len error, unexpected EOF        */
#define READ_ERROR          -4  /* Couldn't read from opened file   */
#define WRITE_ERROR         -5  /* Couldn't write to disk properly  */
#define CANCEL_ERROR        -6  /* Canceled file selection          */
#define ALLOC_ERROR         -7  /* Not enough RAM to run the program*/
#define REZ_ERROR           -8  /* Wrong screen resolution, HI only */
#define KOALA_HED_ER        -9  /* Koala header error               */
#define KOALA_CMP_ER       -10  /* Koala compress flag error        */
#define EXIT_PLEASE       -255  /* User wishes to EXIT the program  */

/*************************************
** Screen quadrant definitions
** for quad map selection
*/

#define TOP_LEFT     1
#define TOP_RIGHT    2
#define BOTTOM_LEFT  3
#define BOTTOM_RIGHT 4

/*************************************
** Screen side definitions for
** split_map selection
*/

#define LEFT  1
#define RIGHT 2

/*************************************
** Screen layout definition for
** maping 8bit pictures to display
*/

#define FULL  0
#define SPLIT 2
#define QUAD  4


#define TRUE  1
#define FALSE 0

/*************************************
** 8bit file format flags
*/

#define MODE8  8
#define MODE9  9
#define KOALA7 7

#define MAX_PIC 8192

#define REPEAT 0                /* Koala uncompress flag types  */
#define UNIQUE 1

/****************************************
** Mouse controls constants
*/

#define HIDE 256
#define SHOW 257
#define POINTER 3

/****************************************
** Global Array Definitions
*/

int work_in[11],
    work_out[57],
    pxyarray[10],
    contrl[12],
    intin[128],
    ptsin[128],
    intout[128],
    ptsout[128];

/* Global variables */

uschar *quad_set();     /* Set quadrant, returns unsigned char pointer */

/* Graphics handle, and DUMMY for miscellaneous use */

int handle, dum;

/* XY array for drawing screen dividers */

int pxy[8]   = { 319,0,319,399,0,199,639,199 };

/* Pin stripe the middle of the display to separate halves */

int pstripe[10] = { 316,0, 324,0, 324,398, 316,398, 316,0 };

char path[65]; /* Pathname storage     */
char file[13]; /* Filename storage     */

/* gray scales for mode9 picture files, monochrome display  */


uschar gray1[16] = { 255,251,251,238,219,218,217,170,
                     166, 36, 36, 17,  4,  4,  1,  0 };


uschar gray2[16] = { 255,223,222,183,110,103,102, 85,
                      25,153,145, 72, 65, 32, 16,  1 };

/* Gray scales for Full screen Koala mapped pictures           */

uschar grayfk_h1[4] = { 0xf0, 0x50, 0x40, 0x00 };
uschar grayfk_h2[4] = { 0xf0, 0xc0, 0x20, 0x00 };
uschar grayfk_l1[4] = { 0x0f, 0x05, 0x04, 0x00 };
uschar grayfk_l2[4] = { 0x0f, 0x0c, 0x02, 0x00 };

/* Gray scales for Split screen Koala mapped pictures          */

uschar graysk1[4]  = { 0x03, 0x02, 0x01, 0x00 };
uschar graysk2[4]  = { 0x03, 0x01, 0x00, 0x00 };

uschar lut[4]; /* lumanance look up table for Koala Pics       */

/****************************************
** DEGAS header for file saves
*/

int header[] = { 0x0002,0x0777,0x0776,0x0766,0x0755,
                 0x0655,0x0644,0x0544,0x0533,
                 0x0433,0x0422,0x0322,0x0311,
                 0x0211,0x0200,0x0100,0x0000 };

/* Save desktop palette here */

int oldpal[16];

/* "PEN" to hardware color register index table */

int inx[16] = { 1, 13, 15, 12, 14, 11, 10, 9, 8, 5, 7, 4, 6, 3, 2, 0 };

int klut[4] = { 0, 4, 10, 15 };

int rez,sx,sy;  /* Sx and Sy are string coordinates, adjusted for rez */
int plot[4];    /* Plot point array                                   */
int pmax;       /* point max x pos for some plots in lo rez           */

uschar string[480];             /* working string                     */

uschar koala [MAX_PIC];         /* koala buffer, oversized            */

uschar xebuff[7680];

usint xemode;              /* xe graphics mode indicator  */
int fhandle;               /* file handle                 */
usint area;                /* Area selected               */

/****************************************
** cbuff - actual pointer returned after
** an malloc call.
*/

long cbuff;                 /* screen backup buffer ptr  */
long crt;                   /* Screen RAM global pointer */

/****************************************
** MOUSE controls, MX, MY, KEY, and RETURN
** (RTS is used to disable mouse keyclick
** fall thrus to AES)
*/

usint mx;
usint my;
usint key;
usint ret = 0x4e75;         /* An RTS instruction        */

/* Main program */

main ()
{

long screen,len;
int status,i,c,type, first;
char erstr[2];


/*******************************************************
** Default extender, destination address, and len bytes
** to read
*/

appl_init ();                   /* Initialize application.      */
open_vwork ();                  /* Set up workstation.          */

rez = Getrez();

header [0] = rez;

for (i=0; i<16; i++)
 oldpal[i] = Setcolor(i, -1);

Setpalette( &header[1] );

first = TRUE;
crt   = Physbase();
clear(crt);
cbuff = 0;

status = allocate();
if (status == ALLOC_ERROR)
  {                             /* ALLOC_ERROR if               */
  status = error(ALLOC_ERROR);
  }                             /* ALLOC_ERROR endif            */
else
  {                             /* !ALLOC_ERROR else            */
    if (rez != MED_REZ)
    {                           /* REZ_OK if                    */
    first = TRUE;
    demo();
    status = play( first );
    status = A_OK;              /* Normal Exit                  */
    lcopy(cbuff, crt);
    Mfree(cbuff);
    }                           /* REZ_OK endif                 */
  else
    {                           /* !REZ_OK else                 */
    status = error(REZ_ERROR);
    Mfree(cbuff);
    }                           /* !REZ_OK endelse              */
  }                             /* !ALLOC_ERROR endelse         */

Setpalette(oldpal);
  
v_clsvwk (handle);              /* Close virtual workstation.   */

appl_exit ();                   /* Back to the desktop.         */
}

/*******************************************
** play ( first ) - if first, then startup
** otherwise, continue with screen type
** to load and return final status
*/

play ( first )
int first;

{ /* play function */

long screen,len;
int status,i,c,type;
uschar *dest;
usint button;

dest  = (uschar *)koala;
len   = MAX_PIC;

while (status != EXIT_PLEASE)
  {
  if (first == TRUE)
    {                           /* FIRST if                     */
    status = startup();
    first = FALSE;
    if (status == EXIT_PLEASE )
      break;
    else
      type = status;
    }                           /* FIRST endif                  */
  else
    {                           /* !FIRST else                  */
    status = mode_sel();
    if (status == EXIT_PLEASE )
      break;
     }                          /* !FIRST endelse               */

  if (xemode == MODE8)
    status = Read_File ("PI8", dest, len);
  else if (xemode == MODE9)
    status = Read_File ("PI9", dest, len);
  else
    status = Read_File ("PIC", dest, len);

  if ( status != A_OK )
    {                           /* ERROR if                     */
    status = error(status);
    first  = TRUE;
    }                           /* ERROR endif                  */
  else
    mapall ( type );
  button = 0;
  while ((button & 0x01) == 0)
    {
    button = button_wait ();    /* Wait for a mouse button pres */
    if ((button & 0x02) != 0)
      invert(type, area);       /* pic type & screen area sel   */
    }
  lcopy(crt, cbuff);
  i = form_alert(1,"[1][        Continue?         ][Again|Save|Restart]");
  if (i == 3)
    {                           /* RESTART if                   */
    status = A_OK;
    first  = TRUE;
    }                           /* RESTART endif                */
  else
    if ( i == 2 )
      {                         /* SAVE_DEGAS if                */
      status = degas_save(crt); /* always restart after SAVE    */
      status = A_OK;
      first  = TRUE;
      }                         /* SAVE_DEGAS endif             */
  else
    status = A_OK;              /* loop                         */
  }                             /* !EXIT_PLEASE endwhile        */

return ( status );

} /* play endfunction */


/**************************************
** Initialize a virtual worksation.
*/

open_vwork ()

{ /* open_vwork function */
int i;

handle = graf_handle (&dum,&dum,&dum,&dum); /* Get graphics handle.     */
for (i=0; i<10; work_in[i++] = 1);          /* Initialize GEM arrays.   */
work_in[10] = 2;
v_opnvwk (work_in, &handle, work_out);      /* Open virtual workstation.*/

} /* open_vwork endfunction */

/******************************************
** Select a file using fil_sel() function,
** default extender and open type is expected
** A -1 result is returned if an open
** error occurs.  If cancel is selected,
** a -2 is returned if [CANCEL] was pressed
*/

sel_file (ext, open_type)

int open_type;
char ext[];

{ /* sel_file function */
int button,                     /* File selector button value.  */
    i,j;                        /* Loop variable.               */
int drv;
char c, workname[65];
long k;

drv     = Dgetdrv();
path[0] = drv + 65;             /* Convert default drive to char*/
if (path[1] != ':')
  {
  path[1] = ':';
  Dgetpath ( &path[ 2 ], drv );
  strcat ( path, "*.");
  }
i=0;
while ( (c=path[i++]) != 0 && c != '.' );

if ( c == '.' )
  i--;

path[i++] = '.';
path[i]   = 0;

strcat ( path, ext );

i=0;
while ( (c=file[i++]) != 0 && c != '.' );

if ( c == '.' )
  i--;

file[i++] = '.';
file[i]   = 0;
strcat ( file, ext );

lcopy(crt, cbuff);
fsel_input (path, file, &button); /* Open the file selector box.*/
if ( button != CANCEL )
  {                               /* !CANCEL if                 */
  strcpy ( workname, path );
  for ( i=strlen(workname); i>=0; --i)
    if ( workname[i] == 92 )
      break;
  workname[ i+1 ] = 0;
  strcat ( workname, file );
  drv = workname[0];
  if ( (drv >= 'A') && (drv <= 'P') )
    dum = Dsetdrv( (usint)(drv-65) );
  if (open_type == WRITE_ONLY )
    i = Fcreate ( workname, 0 );
  else
    i = Fopen ( workname, open_type );
  if ( i < 0 )
    i = OPEN_ERROR;
  else
    fhandle = i;
  }                             /* !CANCEL endif                */
else
  i = CANCEL_ERROR;

lcopy(cbuff, crt);              /* Restore screen               */
return(i);
} /* sel_file endfunction */

/**************************************
** Wait for a button press and exit,
** after setting global button, mouse mx
** and mouse my coordinate values.
*/


button_wait()
{ /* button_wait function */

int button;
int letgo;

long psavcode, ignore;

button = 0;

/****************************************
** Point the mouse vector handler to the
** RTS instruction, in the global area.
** This completely eliminates the mouse
** button "fall throughs" to AES functions.
** This neat little trick is brought to
** you by Charles F. Johnson, West Coast
** editor of ANALOG/ST-Log, what a smart guy!
*/

vex_butv(handle, &ret, &psavcode);

while ( (button & 0x03) == 0)
  vq_mouse(handle, &button, &mx, &my);

letgo = 1;

while ( letgo != 0 )
  { /* wait while */
  letgo = 1;
  vq_mouse(handle, &letgo, &dum, &dum);
  } /* wait endwhile */

/**********************************
** Restore mouse vector for AES now
*/
  
vex_butv(handle, psavcode, &ignore);

return ( button );
} /* button_wait endfunction */

/*********************************************
** Select and read a file into dest of len bytes
** EXT is the default extender for the file
** selector.
*/

int Read_File ( ext, dest, len )
char ext[];
long dest;                      /* Destination of data          */
long len;                       /* File length to read          */

{ /* Read_File function  */

int status;
short i, j;
long fio;                       /* File I/O status              */


/************************************
** A file select and [OK] button press
** MUST have already been completed
** before calling fecth.  fetch will
** attempt to Fopen the file,
** Fread len bytes into the dest array
** and then Fclose the file.
*/

status = sel_file( ext, READ_ONLY );

while ( status >= 0 )
  {                             /* NOT_EXIT while               */
  fio = (long)Fread (fhandle, len, dest);
  if ( fio < 0 )
    {                           /* READ_ERROR if                */
    status = READ_ERROR;
    break;
    }                           /* READ_ERROR endif             */
  else
    {                           /* NO_ERRORS else               */
    koala_proc(fio);            /* Process koala file, if nec   */
    status = A_OK;
        break;
    }                           /* NO_ERRORS end_else           */
  }                             /* NOT_EXIT endwhile            */

if ( (status != OPEN_ERROR) && (status != CANCEL_ERROR) )
  {                             /* OPENED if                    */
  j = Fclose (fhandle);
  }                             /* OPENED endif                 */

if ( status >= A_OK)            /* NO_ERRORS if                 */
  if ( j < 0 )                  /* CLOSE_ERROR if               */
    status = CLOSE_ERROR;       /* NO_ERRORS, CLOSE_ERROR endif */

return (status);                /* Status result                */
} /* read_file endfunction */


/****************************************
** degas_save - routine to write a
** DEGAS.PI3/1 file from the current screen:
*/

degas_save( scr )
char *scr;
{ /* degas_save function */

int status, cstat;
char ext[4];
long count;


ext[0] = 'P';
ext[1] = 'I';

if (rez == HI_REZ)
  ext[2] = '3';
else
  ext[2] = '1';

ext[3] =   0;

status = sel_file(ext, WRITE_ONLY);

graf_mouse(HIDE, &dum);

if (status >= 0 )
  {                             /* SEL_OK if                    */
  count = Fwrite(fhandle, 34L, header);
  count = Fwrite(fhandle, 32000L, scr);
  if (count != 32000L)
    status=error(WRITE_ERROR);
  cstat = Fclose(fhandle);
  if (cstat<0)
    status=error(CLOSE_ERROR);
  }                             /* SEL_OK endif                 */

graf_mouse(SHOW, &dum);
} /* degas_save endfunction */


/****************************************
** mapall ( type ) - based on the global
** xemode, call up the correct mode
** mapping selector. TYPE indicates if
** sector_sel needs to be called, and
** which mapping algorithm needs to be fired
** up to finish the job.
*/

mapall ( type )
int type;

{ /* mapall function */

if (xemode == MODE8 )
  mapsel8 ( type );             /* graphics mode 8 map          */
else
  if (xemode == MODE9 )
    mapsel9 ( type );           /* graphics mode 9 map          */
else
    mapselk ( type );           /* koala antic mode $0E map     */


} /* mapall function */


/****************************************
** mapsel8 - select proper quadrant or
** side if necessary, then appropriate
** mapping algorithm.
*/

mapsel8 ( type )
int type;

{ /* mapsel8 function */

int sel;

switch (type)
  {                             /* LOC&MAP switch               */
  case FULL:
    if (rez == HI_REZ)
      full_map8();
    else
      full_map8c();
    area = FULL;
    break;
  case SPLIT:
    sel = sector_sel( type );
    area = sel;
    if (rez == HI_REZ)
      split_map8( sel );
    else
      sp_map8c( sel );
    stripe();
    break;
  case QUAD:
    sel = sector_sel( type );
    area = sel;
    if ( rez == HI_REZ)
      quad_map8( sel );
    else
      qd_map8c( sel );
    stripe();
    break;
  default:
    break;
  }                             /* LOC&MAP endswitch            */
}  /* mapsel8 endfunction */


/****************************************
** mapsel9 ( type ) - select side or
** quadrant if necessary, based on type,
** and initate proper mapping algorithm
*/

mapsel9 ( type )
int type;

{ /* mapsel9 function */
int sel;

switch (type)
  {                             /* LOC&MAP switch               */
  case FULL:
    area = FULL;
        if (rez == HI_REZ)
      full_map9();
        else
          full_map9c();
    break;
  case SPLIT:
    sel = sector_sel ( type );
    area = sel;
        if (rez == HI_REZ)
      split_map9 ( sel );
        else
          spl_map9c ( sel );
    stripe();
    break;
  case QUAD:
    sel = sector_sel ( type );
    area = sel;
        if (rez == HI_REZ)
      quad_map9 ( sel );
        else
          quad_map9c ( sel );
    stripe();
    break;
  default:
    break;
  }                             /* LOC&MAP endswitch            */

} /* mapsel9 endfunction */



/****************************************
** mapselk ( type ) - select side or
** quadrant if necessary, based on type,
** and initate proper mapping algorithm
*/

mapselk ( type )
int type;

{ /* mapselk function */
int sel;

switch (type)
  {                             /* LOC&MAP switch               */
  case FULL:
    if (rez == HI_REZ)
      full_mapk();
    else
      full_mapkc();
    area = FULL;
    break;
  case SPLIT:
    sel = sector_sel ( type );
    area = sel;
    if (rez == HI_REZ)
      split_mapk ( sel );
    else
      spl_mapkc( sel );
    stripe();
    break;
  case QUAD:
    sel = sector_sel ( type );
    area = sel;
    if (rez == HI_REZ)
      quad_mapk ( sel );
    else
      qd_mapkc ( sel );
    stripe();
    break;
  default:
    break;
  }                             /* LOC&MAP endswitch            */

} /* mapselk endfunction */


/**********************************************************
** full_mapk() - map a 4 gray level koala picture to the
** full ST display.
*/

full_mapk()

{ /* full_mapk function */

char *screen;
usint x, y, gray, index, i, shift;
uschar c;

screen = (char *)crt;

graf_mouse (HIDE, &dum);

index  = 0;

for (y=0; y < 40*192; y+=40)
  {                             /* y for                        */
  for (x=0; x < 40; x++)
    {                           /* x for                        */
    c = xebuff[x + y];
    shift = 6;
    for (i=0; i < 2; i++)
      {                         /* i for                        */
      gray = (usint)((c >> shift) & 0x03); /* get gray index    */
      gray = (usint)lut[gray];
      shift -=2;                /* shift for next pixel         */
      screen[index]      = grayfk_h1[gray];
      screen[index + 80] = grayfk_h2[gray];
      gray = (usint)((c >> shift) & 0x03);
      gray = (usint)lut[gray];
      shift -= 2;
      screen[index]       |= grayfk_l1[gray];
      screen[index + 80 ] |= grayfk_l2[gray];
      index++;
      }                         /* i endfor                     */
    }                           /* x endfor                     */
  index += 80;
  }                             /* y endfor                     */

bot_clr ( RIGHT );
bot_clr ( LEFT );

graf_mouse(SHOW, &dum);
} /* full_mapk endfunction */


/**********************************************************
** full_mapkc() - map a 4 gray level koala picture to the
** full ST color display.
*/

full_mapkc()

{ /* full_mapkc function */

usint x, y, shift,i,gray;
uschar c;

plot[0] = 0;
plot[1] = 0;
plot[2] = 1;
plot[3] = 0;

graf_mouse (HIDE, &dum);

for (y=0; y < 40*192; y+=40)
  { /* y for */
  for (x=0; x < 40; x++)
    { /* x for */
    c = xebuff[x + y];
    shift = 6;
    for (i=0; i < 4; i++)
      { /* i for */
      gray = (usint)((c >> shift) & 0x03); /* get gray index    */
      shift -=2;                /* shift for next pixel         */
      gray = lut[gray];
	  gray = klut[gray];
      vsl_color(handle,inx[gray]);
      v_pline(handle, 2, plot);
      plot[0]+=2;
      plot[2]+=2;
      } /* i endfor */

    } /* x endfor */
  plot[0]=0;
  plot[2]=1;
  plot[1]++;
  plot[3]++;
  } /* y endfor */

bot_clr ( RIGHT );
bot_clr ( LEFT );

graf_mouse(SHOW, &dum);
}  /* full_mapk endfunction */

/**********************************************************
** split_mapk ( side ) - perform a 4 gray scale map
** to the screen from the xebuffer and proper gray scales
*/

split_mapk ( side )
int side;

{ /* split_mapk function */

usint x, y, gray, index, shift;
char *screen;
uschar c;
int i;
uschar gray1, gray2;


graf_mouse( HIDE, &dum);

screen = (char *)crt;

if (side == RIGHT)
  screen +=40;

index = 0;

for (y=0; y <40*192; y+=40)
  { /* y for */
  for (x=0; x < 40; x++)
    { /* x for */
    c = xebuff[x + y];
    shift = 6;
    for (i=0; i < 4; i++)
      { /* i for */
      gray = (usint)((c >> shift) & 0x03); /* get gray index    */
      gray1 = graysk1[lut[gray]] << shift;
          gray2 = graysk2[lut[gray]] << shift;
      shift -=2;                /* shift for next pixel         */
      if (i == 0)
        {
        screen[index]      = gray1;
        screen[index + 80] = gray2;
        }
      else
        {
        screen[index]       |= gray1;
        screen[index + 80 ] |= gray2;
        }
      } /* i endfor */
      index++;
    } /* x endfor */
  index += 120;
  } /* y endfor */

bot_clr ( side );

graf_mouse ( SHOW, &dum);

} /* split_mapk endfunction */



/**********************************************************
** spl_mapkc ( side ) - perform a 4 gray scale map
** to the screen from the xebuffer and proper gray scales
** to lo rez ST color display.
*/

spl_mapkc ( side )
int side;

{ /* spl_mapkc function */

usint x, y, gray, shift;
uschar c;
int i;


plot[0] = 0;
plot[1] = 0;
plot[3] = 0;

if (side == RIGHT)
  plot[0] = 160;

plot[2] = plot[0];


graf_mouse( HIDE, &dum);


for (y=0; y <40*192; y+=40)
  { /* y for */
  for (x=0; x < 40; x++)
    { /* x for */
    c = xebuff[x + y];
    shift = 6;
    for (i=0; i < 4; i++)
      { /* i for */
      gray = (usint)((c >> shift) & 0x03); /* get gray index    */
      shift -=2;                /* shift for next pixel         */
      gray = lut[gray];
	  gray = klut[gray];
      vsl_color(handle,inx[gray]);
      v_pline(handle, 2, plot);
      plot[0]++;
      plot[2]++;
      } /* i endfor */
    } /* x endfor */
  plot[0] -= 160;
  plot[2] -= 160;
  plot[1]++;
  plot[3]++;
  } /* y endfor */

bot_clr ( side );

graf_mouse ( SHOW, &dum);

} /* spl_mapkc endfunction */


/**********************************************************
** quad_mapk (quadrant) - perform a 4 gray scale map
** to the screen from the xebuffer and proper gray scales
*/

quad_mapk ( quadrant )
usint quadrant;

{ /* quad_mapk function */

usint x, y, gray, index, shift;
uschar *screen;
uschar ct, cb;
int i;
uschar gray1, gray2;

graf_mouse( HIDE, &dum);

screen = quad_set( quadrant );
quad_clr(quadrant);             /* Erase the quadrant           */

index = 0;

for (y=0; y <40*192; y+=80)
  { /* y for */
  for (x=0; x < 40; x++)
    { /* x for */
    ct = xebuff[x + y];
    cb = xebuff[x + y + 40];
    shift = 6;
    for (i=0; i < 4; i++)
      { /* i for */
      gray   = (usint)((ct >> shift) & 0x03); /* get gray index    */
      gray1  = graysk1[lut[gray]] << shift;
      gray   = (usint)((cb >> shift) & 0x03);
      gray2  = graysk2[lut[gray]] << shift;
      shift -=2;                /* shift for next pixel         */
      if (i == 0)
        {
        screen[index]      = gray1;
        screen[index + 80] = gray2;
        }
      else
        {
        screen[index]       |= gray1;
        screen[index + 80 ] |= gray2;
        }
      } /* i endfor */
      index++;
    } /* x endfor */
  index += 120;
  } /* y endfor */

stripe();

graf_mouse ( SHOW, &dum);

} /* quad_mapk endfunction */


/**********************************************************
** qd_mapkc (quadrant) - perform a 4 gray scale map
** to the screen from the xebuffer and proper gray scales
** to the ST Lo Rez color display
*/

qd_mapkc ( quadrant )
usint quadrant;

{ /* qd_mapkc function */

usint x, y, gray, shift;
uschar *screen, c;
int i;

graf_mouse( HIDE, &dum);

screen = quad_set( quadrant );
quad_clr(quadrant);             /* Erase the quadrant           */

for (y=0; y <40*192; y+=80)
  { /* y for */
  for (x=0; x < 40; x++)
    { /* x for */
    c = xebuff[x + y];
    shift = 6;
    for (i=0; i < 4; i++)
      { /* i for */
      gray   = (usint)((c >> shift) & 0x03);
      shift -=2;                /* shift for next pixel         */
      gray = lut[gray];
	  gray = klut[gray];
      vsl_color(handle,inx[gray]);
      v_pline(handle, 2, plot);
      plot[0]++;
      plot[2]++;
      } /* i endfor */
    } /* x endfor */
  plot[0] -= 160;
  plot[2] -= 160;
  plot[1]++;
  plot[3]++;
  } /* y endfor */

stripe();

graf_mouse ( SHOW, &dum);

} /* qd_mapkc endfunction */

/**********************************************************
** quad_map9 ( quadrant )
** map XEBUFF into a quadrant on the monochrome display,
** numbered 1-4, clockwise - starting top left
*/

quad_map9 ( quadrant )
usint quadrant;

{ /* quad_map9 function */
uschar *screen;
uschar c, chi, clo;
usint x,y,line;

graf_mouse(HIDE, &dum);

screen = quad_set(quadrant);

quad_clr(quadrant);

line = 0;

for (y=0; y<40*192; y+=80)
  { /* y for */
  for (x=0; x<40; x++)
    { /*  x for */
    clo = xebuff[x+y] & 0x0f;
    chi = xebuff[x+y] >> 4;
    c   = (chi + clo) >> 1;
    screen[x + (y << 1)]      = gray1[c];
    screen[x + (y << 1) + 80] = gray2[c];
    } /*  x endfor */
    line++;
  } /* y endfor */

graf_mouse(SHOW, &dum);
} /* quad_map9 endfunction */

/**********************************************************
** quad_map9c ( quadrant )
** map XEBUFF into a quadrant on the lo rez color display,
** numbered 1-4, clockwise - starting top left
*/

quad_map9c ( quadrant )
usint quadrant;

{ /* quad_map9c function */

usint x, y;
uschar c1, chi, clo;
uschar *screen;

graf_mouse(HIDE, &dum);
screen = quad_set(quadrant);

for (y=0; y<40*192; y+=80)
  { /* y for */
  for (x=0; x<40; x++)
    { /*  x for */
    c1=xebuff[x+y];
    chi=c1 >> 4;
    clo=c1 & 0x0f;
    plot[2] = plot[0]+1;
    plot[3] = plot[1];
    vsl_color(handle,inx[chi]);
    v_pline(handle,2,plot);
    plot[0]+=2;
    plot[2]+=2;
    vsl_color(handle,inx[clo]);
    v_pline(handle,2,plot);
    plot[0]+=2;
    if (plot[0]>pmax)
      {
      plot[0]-=160;
      plot[1]++;
      }
    } /* x endfor */
  } /* y endfor */

graf_mouse (SHOW, &dum);

} /* quad_map9c endfunction */



/****************************************
** sector_sel ( type )
** select one half or quarter of the display
** to build the picture on.
*/

sector_sel ( type )
int type;

{ /* sector_sel function */

/******************
** Local variables:
*/

int x, y, button, sel, xmid, ymid;

/****************************************
** hide mouse, save screen, and give
** appropriate directives for screen
** half or quadrant selection:
*/

graf_mouse(HIDE, &dum);

lcopy(crt, cbuff);              /* Save screen buffer, global   */

/****************************************
** Either SPLIT in half or QUADRANTS
** Give appropriate indications, and
** allow user to select:
*/

x = (rez == HI_REZ) ? 100:50;
y = (rez == HI_REZ) ? 300:150;

v_gtext(handle, x, y, "[Point 'n Click]");
vsl_color(handle,1);
v_pline(handle, 2, pxy);

if (type == QUAD)
  v_pline(handle, 2, &pxy[4]);

graf_mouse(SHOW, &dum);

/****************************************
** Wait for LEFT mouse button, then decipher
** the area selection from x & y.
*/

button = 0;

while ((button & 0x01) == 0)    /* Left mouse button only       */
  button = button_wait();

x = mx;
y = my;

xmid = (rez == HI_REZ) ? 319:159;

ymid = (rez == HI_REZ) ? 199:99;

if (type == SPLIT)
  {                             /* SPLIT if                     */
  if (x < xmid)
    sel = LEFT;
  else
    sel = RIGHT;
  }                             /* SPLIT endif                  */
else
  {                             /* QUAD else                    */
  if ((x<xmid) && (y<ymid))
    sel = TOP_LEFT;
  if ((x<xmid) && (y>=ymid))
    sel = BOTTOM_LEFT;
  if ((x>=xmid) && (y<ymid))
    sel = TOP_RIGHT;
  if ((x>=xmid) && (y>=ymid))
    sel = BOTTOM_RIGHT;
  }                             /* QUAD endelse                 */

lcopy(cbuff, crt);
return (sel);
} /* sector_sel endfunction */

/****************************************
** allocate() - request 32016 bytes of
** workspace from the system.  Return
** status, either A_OK
** or ALLOC_ERROR flag.  If successful
** cbuff points to the first longword of
** 32000 bytes of storage for screen save
** and restore functions.
*/

allocate()
{ /* allocate function */
int status;
long cb;

cbuff = Malloc((long)32016);
if (cbuff != 0)
  { /* ENUFRAM if */
  status = A_OK;
  } /* ENUFRAM endif */
else
  { /* !ENUFRAM else */
  status = ALLOC_ERROR;
  } /* !ENUFRAM endelse */
return (status);
} /* allocate endfunction */


/****************************************
** lcopy ( src_Ptr, kep_Ptr) - longword copy
** to the kep_Ptr buffer from the src_Ptr
** 32000 bytes of information, ASAP!
*/

lcopy(src_Ptr, kep_Ptr)
register long *src_Ptr;
register long *kep_Ptr;

{ /* lcopy function */
graf_mouse(HIDE, &dum);
asm
  {
    move.l #8000-1, D0
sp: move.l (src_Ptr)+, (kep_Ptr)+
    dbf D0, sp
  }
graf_mouse(SHOW, &dum);
} /* lcopy endfunction */

/****************************************
** clear (crt_Ptr) - clear the screen
** pointed to by crt_Ptr (must begin on
** a long word boundary). Will clear
** 8000 long values.
*/

clear (crt_Ptr)
register long *crt_Ptr;
{ /* clear function */
graf_mouse(HIDE, &dum);
asm {
    move.l #8000-1, D0
cp: clr.l (crt_Ptr)+
    dbf D0, cp
     }
graf_mouse(SHOW, &dum);
} /* clear endfunction */


/*****************************************************
** split_map9 - map the Mode9 picture to LEFT or
** RIGHT half of the display, according to the
** TYPE specification.
*/

split_map9 ( type )
int type;
{ /* split_map9 function */

/******************
** Local Variables:
*/

uschar *screen;
usint x, y, z;
uschar c, chi, clo;

screen = (uschar *)crt;  /* crt is global screen pointer */
graf_mouse ( HIDE, &dum);
if ( type == RIGHT )
  screen +=  40;

/*****************************************************
** Map gray tables 1 & 2 to alternating lines of
** the ST display, a 2x8 grid generating the 16
** gray levels needed to emulate a mode 9 display
*/

for (y=0; y<40*192; y+=40)
  { /* y for */
  for (x=0; x<40; x++)
    {  /* x for */
    c   = xebuff[x+y];
    chi = c>>4;
    clo = c & 0x0f;
    c   = (chi + clo) >> 1;
    screen[x + (y << 2)]     = gray1[c];
    screen[x + (y <<2) + 80] = gray2[c];
    } /* x endfor */
  } /* y endfor */
bot_clr ( type );
graf_mouse ( SHOW, &dum );
} /* split_map9 endfunction */


/*****************************************************
** spl_map9c - map the Mode9 picture to LEFT or
** RIGHT half of the color display, according to the
** side specification.
*/


spl_map9c(loc)
int loc;

{ /* spl_map9c function */

usint x, y;
uschar c, chi, clo;

graf_mouse(HIDE, &dum);

plot[0] =   0;
plot[1] =   0;
pmax    = 159;
if ( loc == RIGHT )
  {
  plot[0] = 160;
  pmax    = 319;
  }

for (y=0; y<40*192; y+=40)
  { /* y for */
  for (x=0; x<40; x++)
    { /* x for */
    c   = xebuff[x+y];
    chi = c>>4;
    clo = c & 0x0f;

    plot[2] = plot[0]+1;
    plot[3] = plot[1];
    vsl_color(handle,inx[chi]);
    v_pline(handle,2,plot);
    plot[0]+=2;
    plot[2]+=2;
    vsl_color(handle,inx[clo]);
    v_pline(handle,2,plot);
    plot[0]+=2;
    if (plot[0] > pmax)
      {
      plot[0]-=160;
      plot[1]++;
      }
    } /* x endfor */
  } /* y endfor */
  
bot_clr(loc);
  
graf_mouse(SHOW, &dum);

} /* spl_map9c endfunction */

/*****************************************************
** full_map9 - use the full height of the original
** mode 9 graphic and stretch it horizontally to
** completely fill the st display.
*/

full_map9()
{ /* full_map9 function */

/********************
** Local Variables
*/

int x, y;
uschar *screen;
uschar c,clo,chi;

screen = (uschar *)crt;

graf_mouse ( HIDE, &dum );

for (y=0; y < 192*40; y+=40)
  { /* y for */
  for (x=0; x < 40; x++)
    { /* x for */
    c=xebuff[x + y];
    chi = c >> 4;
    clo = c &0x0f;
    screen[ (x << 1) + (y << 2) ]          = gray1[chi];
    screen[ (x << 1) + (y << 2) + 80 ]     = gray2[chi];
    screen[ (x << 1) + (y << 2) + 1 ]      = gray1[clo];
    screen[ (x << 1) + (y << 2) + 81 ]     = gray2[clo];
        }  /* x endfor */
  } /* y endfor */

bot_clr ( RIGHT );
bot_clr ( LEFT );

graf_mouse ( SHOW, &dum );
} /* full_map9 endfunciton */


/*****************************************************
** full_map9c - use the full height of the original
** mode 9 graphic and stretch it horizontally to
** completely fill the st lo res color display.
*/

full_map9c()
{ /* full_map9c function */

/********************
** Local Variables
*/

int x, y;
uschar c,clo,chi;

pmax    = 319;
plot[0] =   0;
plot[1] =   0;
plot[2] =   3;
plot[3] =   0;

graf_mouse ( HIDE, &dum );

for (y=0; y < 192*40; y+=40)
  { /* y for */
  for (x=0; x < 40; x++)
    { /* x for */
    c=xebuff[x + y];
    chi = c >> 4;
    clo = c &0x0f;
    vsl_color(handle,inx[chi]);
    v_pline(handle,2,plot);
    plot[0]+=4;
    plot[2]+=4;
    vsl_color(handle,inx[clo]);
    v_pline(handle,2,plot);
    plot[0] += 4;
    plot[2] += 4;
    if (plot[0] > pmax)
      {
      plot[0] = 0;
      plot[2] = 3;
      plot[1]++;
      plot[3]++;
      }
    } /* x endfor */
  } /* y endfor */

 graf_mouse(SHOW, &dum);

 } /* full_map9c endfunction */

/*****************************************************
** quad_map8 - map a graphics mode 8 picture into
** one of 4 quadrants selected by quad.
*/

quad_map8 ( quad )
usint quad;

{ /* quad_map8 function */

/********************
** Local Variables
*/

int x, y;
uschar *screen;

screen = (uschar *)crt;

graf_mouse( HIDE, &dum );
screen = quad_set(quad);
quad_clr(quad);

/*****************************************************
** There is a 1:1 correlation between a mode 8 display
** and one quadrant of the ST monochrome display.
*/

for (y=0; y < 40*192; y += 40)
  for (x=0; x < 40; x++)
    screen[x + ( y << 1 ) ] = xebuff [ x + y ];

graf_mouse ( SHOW, &dum );

} /* quad_map8 endfunction */



/*****************************************************
** qd_map8c - map a graphics mode 8 picture into
** one of 4 lo_rez color quadrants selected by quad.
*/

qd_map8c ( quad )
usint quad;

{ /* qd_map8c function */

/********************
** Local Variables
*/

usint x, y, i;
uschar *screen, z, c, mask;

screen = (uschar *)crt;

graf_mouse( HIDE, &dum );
screen = quad_set(quad);
quad_clr(quad);

for (y=0; y < 40*192; y += 80) /* skip every other scan line for lo rez */
  { /* y for */
  for (x=0; x < 40; x++)
    { /* x for */
    c = xebuff[x + y];
        mask = 0xc0;
    for (i=0; i < 4; i++)
      { /* i for */
      z = (c & mask);
      c <<= 2;

/****************************************
** Speed trick. If background color, no
** need to plot. Already WHITE.
*/        

      if (z != 0)
        {
        vsl_color(handle, 1);
        v_pline(handle, 2, plot);
        }
      plot[0]++;
      plot[2]++;
      } /* i endfor */
    } /* x endfor */
  plot[0] -= 160;
  plot[2] -= 160;
  plot[1]++;
  plot[3]++;
  } /* y endfor */


graf_mouse ( SHOW, &dum );

} /* qd_map8c endfunction  */


/*****************************************************
** split_map8 - map a mode 8 display to the LEFT or
** RIGHT half of the ST display as selected by side.
*/

split_map8 ( side )
int side;

{ /* split_map8 function */

/********************
** Local Variables
*/

int x,y;
char *screen;

screen = (char *)crt;

if (side == RIGHT)
  screen += 40;

graf_mouse ( HIDE, &dum );

for (y=0; y < 40*192; y+=40)
  { /* y for */
  for (x=0; x < 40; x++)
    { /* x for */
    screen[x + (y << 2)]       = xebuff[x + y];
    screen[x + (y << 2) + 80 ] = xebuff[x + y];
    } /* x endfor */
  } /* y endfor */

bot_clr( side );

graf_mouse ( SHOW, &dum );

}  /* split_map8 endfunction */


/*****************************************************
** sp_map8c - map a mode 8 display to the LEFT or
** RIGHT half of the ST color display as selected by side.
*/

sp_map8c ( side )
int side;

{ /* sp_map8c function */

/********************
** Local Variables
*/

usint x, y, i;
uschar c, z, mask;

graf_mouse ( HIDE, &dum );

plot[1] = 0;
plot[3] = 0;

if (side == RIGHT)
  {
  fillbox(0,REPLACE,320,0,639,399);
  plot[0] = 160;
  }
else
  {
  fillbox(0,REPLACE,0,0,319,399);
  plot[0] = 0;
  }
plot[2] = plot[0];

for (y=0; y < 40*192; y += 40)
  { /* y for */
  for (x=0; x < 40; x++)
    { /* x for */
    c = xebuff[x + y];
    mask = 0xc0;
    for (i=0; i < 4; i++)
      { /* i for */
      z = c & mask;
      c <<= 2;
      if (z != 0)
        {
        vsl_color(handle, 1);
        v_pline(handle, 2, plot);
        }
      plot[0]++;
      plot[2]++;
      } /* i endfor */
    } /* x endfor */
  plot[0] -= 160;
  plot[2] -= 160;
  plot[1]++;
  plot[3]++;
  } /* y endfor */

bot_clr( side );

graf_mouse(SHOW, &dum);

} /* sp_map8c endfunction */


/****************************************
** bot_clr ( side ) - clear the bottom
** 16 lines of the screen, for split_maps.
** this cleans up any _old_ quad maps
** left behind.
*/

bot_clr ( side )
usint side;
{ /* bot_clr function */


if (side == RIGHT)
  fillbox(0,REPLACE,320,400-16,639,399);
else
  fillbox(0,REPLACE,0,400-16,320,399);

} /* bot_clr endfunction */

/*******************************************
** quad_set( quad ) - set the screen address
** based on the quadrant selected
*/

uschar *quad_set ( quad )
usint quad;
{ /* quad_set function */

uschar *screen;

screen = (uschar *)crt;

switch ( quad )
  { /* quadrant select switch */

  case TOP_LEFT:
    plot[0] =   0;
    plot[1] =   0;
    pmax    = 159;
    break;

  case TOP_RIGHT:
    screen +=  40;
    plot[0] = 160;
    plot[1] =   0;
    pmax    = 319;
    break;

  case BOTTOM_LEFT:
    screen += 16000;
    plot[0] =     0;
    plot[1] =   100;
    pmax    =   159;
    break;

  case BOTTOM_RIGHT:
    screen += 16040;
    plot[0] =   160;
    plot[1] =   100;
    pmax    =   319;
    break;

  default:
    break;

  } /* QUAD select endswitch */

plot[2] = plot[0];
plot[3] = plot[1];
return(screen);
} /* quad_set endfunction */

/*******************************************************
** quad_clr ( quad ) - Clear one of the quadrants before
** mapping a new picture to it.
*/

quad_clr( quad )
usint quad;

{  /* quad_clr function */


vsf_interior(handle, 0);

switch (quad) { /* quad switch */

  case TOP_LEFT:
  fillbox(0,REPLACE,0,0,320,200);
  break;

  case TOP_RIGHT:
  fillbox(0,REPLACE,320,0,640,200);
  break;

  case BOTTOM_LEFT:
  fillbox(0,REPLACE,0,200,320,400);
  break;

  case BOTTOM_RIGHT:
  fillbox(0,REPLACE,320,200,640,400);
  break;

  } /* quad endswitch */

} /* quad_clr endfunction */


/****************************************
** fillbox(color, wrmode, x1, y1, x2, y2 )
** wrmode - WRITE mode
*/

fillbox (color, wrmode, x1, y1, x2, y2 )
usint color, wrmode, x1, y1, x2, y2;

{  /* fillbox function */

usint clxy[4];
usint shift;

vswr_mode(handle, wrmode);

shift = 0;
if (rez == LO_REZ)
 shift = 1;

clxy[0] = x1>>shift;
clxy[1] = y1>>shift;
clxy[2] = x2>>shift;
clxy[3] = y2>>shift;


vsf_interior(handle, color);

vr_recfl(handle, clxy);

vswr_mode(handle, REPLACE);

} /* fillbox endfunction */


/*****************************************************
** full_map8 - take a mode 8 picture and expand it to
** fill display horizontally and vertically.
*/

full_map8()

{ /* full_map8 function */

/********************
** Local Variables
*/

unsigned char c, cmask;
unsigned int i, x, y;
unsigned int z, mask_on;
char *screen;

screen = (char *)crt;

graf_mouse( HIDE, &dum);

for ( y=0; y < 40*192; y+=40)
  for (x=0; x<40; x++)
    {  /* MAP_TO_ST loop */
    c = xebuff[x + y];
    z = 0;
    mask_on = 0x0c000;
    cmask   = 0x00080;
    for (i=0; i<8; i++)
      { /* MASK loop */
      if ( ( c & cmask ) != 0 )
        z = (z | mask_on);
      mask_on = (mask_on >> 2);
      cmask   = (cmask >> 1);
      }  /* MASK endloop */
    c = (char) (z >> 8);
    screen[ (x << 1) + (y << 2)]      = c;
    screen[ (x << 1) + (y << 2) + 80] = c;
    c = (char)(z & 0x0ff);
    screen[ (x << 1) + (y << 2) + 1]  = c;
    screen[ (x << 1) + (y << 2) + 81] = c;
    } /* MAP_TO_ST endloop */

bot_clr ( RIGHT );
bot_clr ( LEFT );

graf_mouse ( SHOW, &dum );

} /* full_map8 endfunction */


/*****************************************************
** full_map8c - take a mode 8 picture
** fill color display horizontally and vertically.
*/

full_map8c()

{ /* full_map8c function */

/********************
** Local Variables
*/

unsigned char c, cmask;
unsigned int i, x, y;
unsigned int z;

plot[0] = 0;
plot[1] = 0;
plot[2] = 0;
plot[3] = 0;

graf_mouse( HIDE, &dum);

v_clrwk(handle);

for ( y=0; y < 40*192; y+=40)
  { /* y for */
  for (x=0; x<40; x++)
    {  /* x for */
    c = xebuff[x + y];
    z = 0;
    cmask   = 0x80;
    for (i=0; i<8; i++)
      { /* MASK for */
      if ( ( c & cmask ) != 0 )
        z = 1;
      else
        z = 0;
      cmask   = (cmask >> 1);
        if (z != 0)
          {
          vsl_color(handle, z);
          v_pline(handle,2,plot);
          }
      plot[0]++;
      plot[2]++;
      } /* MASK endfor */
    } /* x endfor */
  plot[0] = 0;
  plot[2] = 0;
  plot[1]++;
  plot[3]++;
  } /* y endfor */

bot_clr ( RIGHT );
bot_clr ( LEFT );

graf_mouse ( SHOW, &dum );

} /* full_map8c endfunction */


/*****************************************************
** error handler - enunciate error in a form_alert
** as indicated by the status passed to it. Return
** the response button selection.  Caller determines
** appropriate action based upon error condition and
** button value returned.
*/

error ( status )
int status;
{ /* error function */

/*****************
** Local variables
*/

int r;

/*****************************************************
** check the error and call up the appropriate error
** form_alert
*/

Cconout(7); /* Bing, error */

switch (status) { /* ERROR switch */

  case OPEN_ERROR:
    r=form_alert(0,"[3][  Could not OPEN | specified file!  ][Click Twice]");
    r = EXIT_PLEASE;
    break;

  case CLOSE_ERROR:
    r=form_alert(0,"[3][  Could not CLOSE | specified file!][Click Twice]");
    r = EXIT_PLEASE;
    break;

  case WRITE_ERROR:
    r=form_alert(1,"[3][  Write Error   | Out of disk space? ][Click Twice]");
    r = EXIT_PLEASE;
    break;

  case LEN_ERROR:
    r=form_alert(1,"[3][File length error |Unexpected EOF][Humm?]");
    break;

  case READ_ERROR:
    r=form_alert(2,"[3][Error occurred | During file READ][Click Twice]");
    break;

  case CANCEL_ERROR:
    r=form_alert(2,"[3][  No file Selected   ][Click Twice]");
    break;

  case ALLOC_ERROR:
    r=form_alert(1,"[Not enough RAM | to RUN program!][Sorry!]");
    r=EXIT_PLEASE;
    break;

  case REZ_ERROR:
    r=form_alert(1,"[1][LO-Rez or HIGH-Rez |      ONLY    ][Sorry!]");
    r=EXIT_PLEASE;
    break;

  case KOALA_HED_ER:
    r=form_alert(1,"[1][Koala HEADER error][Oops!]");
    break;

  case KOALA_CMP_ER:
    r=form_alert(1,"[1][Koala COMPRESS flag error][Oops!]");
    break;

  default:
    r=form_alert(1,"[2][Unexpected error][Click Twice]");
    r=EXIT_PLEASE;
    break;
  } /* ERROR endswitch */

if ( r == 2 )
  r = EXIT_PLEASE;

lcopy (cbuff, crt);
return (r);
} /* error endfunction */



/*****************************************************
** startup - select mode and screen format for
** loading and displaying pictures
*/

startup()
{ /* startup function */

/********************
** Local variables
*/

int select;
int go;

lcopy (crt, cbuff);

go  = form_alert(1,"[2][Altered STeights by Mat*Rat][Go! |Exit]");

if (go == 2)
  go = EXIT_PLEASE;

lcopy(cbuff, crt);

if (go < 0 )
  select = EXIT_PLEASE;
else
  {
  select = form_alert(2,"[2][  Clear Workarea before load?][Yes | No!]");
  if (select == 1)
    {
    clear( cbuff );             /* Clear out backup buffer      */
    clear( crt );
    }
  select = form_alert(1,"[2][   Display Format?    ][Full|Half|Quad]");
  }
if (select != EXIT_PLEASE)
  mode_sel();

switch (select)
  { /* format switch */
  case 1:
    select = FULL;
    break;
  case 2:
    select = SPLIT;
    break;
  case 3:
    select = QUAD;
    break;
  default:
    break;
  } /* format endswitch */
lcopy (cbuff, crt);
return(select);
} /* startup endfunction */


/*****************************************************
** Title and mode select for source files
*/

mode_sel()
{ /* mode_sel function */
int sel;

lcopy (crt, cbuff);

sel = form_alert(1,"[1][ Altered STeights by Mat*Rat ][Mode9|Mode8|KOALA]");

lcopy(cbuff, crt);

switch (sel)
  { /* MODE switch */
  case 1:
    xemode = MODE9;
    break;
  case 2:
    xemode = MODE8;
    break;
  case 3:
    xemode = KOALA7;
    break;
  default:
    break;
  }  /* modesel endswitch */

} /* mode_sel endfunction */


/*****************************************************
** stripe -make an 8 dot wide stripe down the center
** of the display to separate the 4 pics horizontally
** like 4 boardered photographs.
*/

stripe ( )

{ /* stripe function */

/********************
** Local variables
*/

int i;

graf_mouse (HIDE, &dum);

fillbox(0, REPLACE, 320-4, 0, 320+4, 399);

graf_mouse (SHOW, &dum);

} /* stripe endfunction */


/****************************************
** koala_proc ( len ) - check the current
** graphics mode enabled, if 8 or 9
** and 7680 bytes, then assume uncompressed
** and DONE.  If KOALA and header is correct
** for KOALA save format, then unpack it.
** If header is not KOALA format, tho' koala
** mode is enabled, and file is 7680 bytes
** then it's safe to call it a simple uncompressed
** mode 15+32 (antic mode $0E) format, and done
*/

koala_proc ( len )
long len;

{ /* koala_proc function */

int status, l, i;

l = (int)len;

if (xemode == KOALA7)
  {                             /* KOALA if                     */
  status = unpack(len);         /* now unpack it                */
  }                             /* KOALA endif                  */
else
  {
  for (i=0; i < 7680; i++)
    xebuff[i] = koala[i];
  status = A_OK;
  }
return (status);

} /* koala_proc endfunction */


/****************************************
** unpack () - assuming this is a
** koala picture, process the header and
** unpack it from the source to the destination
** check for uncompressed format, 7680 byte
** file, to allow simple 8bit screen dumps
** (i.e. D:PICTURE from ">" keypress on 8bit)
** with no KOALA data.
*/

unpack (len)
long len;


/******************
** Local Constants
** array elements A_xxxx
** flag types     F_xxxx
*/

#define F_ID1 255               /* 4 unique header bytes        */
#define F_ID2 128
#define F_ID3 201
#define F_ID4 199

#define A_ID  0                 /* ID at first array element    */

#define A_HED 4                 /* Header size indicator        */

#define A_COMPRESS 7            /* Compression type indicator   */
#define F_NOCOMP   0            /* No compression, but with head*/
#define F_VERT     1            /* Vertical compression techniq.*/
#define F_HORIZ    2            /* Horizontal compression       */

#define A_MODE     8            /* The pic's ANTIC mode, $0E    */
#define A_COLORS  13            /* Color register table, 5 bytes*/

#define A_FIL     18            /* Koala data body length total */


{ /* unpack function */


/******************
** Local Variables
*/

usint hed_ln;                   /* Source data pointer,hed len  */
usint body_ln;                  /* Dest data pointer, body lengt*/
usint sdat, ddat;
int status;                     /* Returned status from unpackin*/
usint i;                        /* Loop counter                 */
uschar *kP, *dP;
uschar luma[4];                 /* Lumanence table for koala col*/
uschar sort, j;                 /* sort required flag           */

/****************************************
** Function body: Check the header
*/

for (i=0; i<4; i++)
  lut[i] = i;                   /* start out sequential         */

kP = (uschar *)koala;
dP = (uschar *)xebuff;
if ( ( kP[0]==F_ID1) && (kP[1]==F_ID2) &&
     ( kP[2]==F_ID3) && (kP[3]==F_ID4) )
 {                              /* HEADER_OK if                 */

 hed_ln   =   (usint)kP[A_HED] + 256*(usint)kP[A_HED+1];
 body_ln  =   (usint)kP[A_FIL] + 256*(usint)kP[A_FIL+1];
 sdat     =   hed_ln+1;
 luma[1]  =   (kP[A_COLORS]   & 0x0f); /* Color 1               */
 luma[2]  =   (kP[A_COLORS+1] & 0x0f); /* Color 2               */
 luma[3]  =   (kP[A_COLORS+2] & 0x0f); /* Color 3               */
 luma[0]  =   (kP[A_COLORS+4] & 0x0f); /* Background color      */

 sort = TRUE;

 while (sort == TRUE)
   {                            /* sort while                   */
   for (i=0; i<3; i++)
     {                          /* sort for                     */
     sort = FALSE;
     if (luma[i] > luma[i+1])
       {                        /* if delta luma                */
       j = luma[i];
       luma[i] = luma[i+1];
       luma[i+1] = j;
       j = lut[i];
       lut[i] = lut[i+1];
       lut[i+1] = j;
       sort = TRUE;
       }                        /* endif delta luma             */
     }                          /* sort endfor                  */
   }                            /* sort endwhile                */

 switch (kP[A_COMPRESS])
   {                            /* Compress switch              */
   case F_NOCOMP:
     ddat = 0;
     for (i=hed_ln; i < hed_ln+7680; i++)
       dP[ddat++] = kP[i];
     break;

   case F_VERT:
     vert_unpack( &kP[sdat], &dP[0], body_ln);
     break;

   case F_HORIZ:
     horiz_unpack( &kP[sdat], &dP[0], body_ln);
     break;

   default:
     status = error(KOALA_CMP_ER);  /* koala compress flag err  */
     break;                     /* Do nothing if invalid compres*/
    }                           /* compress endswitch           */
  }                             /* HEADER_OK endif              */
else
  {                             /* BAD_HEADER else              */
  if (len != 7680)                          /* Uncompressed mode $0E ?      */
    status = error(KOALA_HED_ER); /* No, header error oops      */
  for (i=0; i<7680; i++)        /* if 7680, ok uncompressed file*/
    dP[i] = kP[i];              /* Assume raw image             */
  }                             /* BAD_HEADER endelse           */

status = A_OK;

return (status);
} /* unpack endfunction */



/******************************************
** vert_unpack ( src, dst, srcsz ) - unpack
** compressed Koala picture from
** source buffer to destination,
** which is compressed in vertical
** format
*/

vert_unpack ( src, dst, srcsz )
uschar *src, *dst;
usint srcsz;                    /* source size max              */

{  /* vert_unpack function */

usint lcount;                   /* byte counter                 */
uschar count;
usint i;                        /* general loop counter         */
usint sdat;                     /* Source data pointer          */
usint ddat;                     /* Destination data pointer     */
uschar done;                    /* Done flag                    */
uschar flag;                    /* conversion type flag         */
uschar data;                    /* data values to be moved      */
usint scan;                     /* scan line in the works       */

scan = 0;
done = FALSE;
sdat = 0;
ddat = 0;

while (done == FALSE)
  {

/**************************************
** fetch the next count value from the
** Koala buffer. Determine if it is a
** short (1byte) or long (2byte) counter
** and whether it is followed by UNIQUE
** or REPEATed data.
*/

  if (sdat > srcsz)
    break;
  count = src[sdat++];
  if ( ( (count & 0x80) == 0) && ( (count & 0x7f) !=0) )
    {
    lcount = (int)(count &0x7f);
    flag = REPEAT;
    }
  else
  if ( ( (count & 0x80) != 0) && ( (count & 0x7f) !=0) )
    {
    lcount = (int)(count &0x7f);
    flag   = UNIQUE;
    }
  else
  if ( count == 0 )
    {
    flag = REPEAT;
    lcount = 256*(usint)src[sdat++] + (usint)src[sdat++];
    }
  else
  if ( count == 0x80 )
    {
    flag = UNIQUE;
    lcount = 256*(usint)src[sdat++] + (usint)src[sdat++];
    }

  data = src[sdat++];

  for (i=0; i < lcount; i++)
    {
    dst[ddat] = data;
    if (ddat >= 7679)
      {
      done = TRUE;
      break;
      }
    if (flag == UNIQUE)
      data = src[sdat++];
    ddat += 80;
    if (ddat >= 7680)
      {
      if ( (scan & 0x01) != 0)
        {
        scan++;
        ddat -= 7719;
        }
      else
       {
        scan++;
        ddat -= 7640;
        }
      }
    if (scan > 192)
      break;
    }
  if ( (scan > 192) || (ddat >= 7680) || (sdat >= srcsz) )
    done = TRUE;

  if (flag == UNIQUE)
    sdat--;

  }                             /* !DONE endwhile               */
} /* vert_unpack endfunction */

/********************************************
** horiz_unpack ( src, dst, srcsz ) - unpack
** compressed Koala picture from
** source buffer to destination,
** which is compressed in horizontal
** format
*/

horiz_unpack ( src, dst, srcsz )
uschar *src, *dst;
usint srcsz;                    /* source size max              */

{  /* horiz_unpack function */

uschar count;
usint lcount;                   /* byte counter                 */
usint i;                        /* general loop counter         */
usint sdat;                     /* Source data pointer          */
usint ddat;                     /* Destination data pointer     */
uschar done;                    /* Done flag                    */
uschar flag;                    /* conversion type flag         */
uschar data;                    /* data values to be moved      */

done = FALSE;
sdat = 0;
ddat = 0;

while (done == FALSE)
  {                             /* !DONE while                  */

/**************************************
** fetch the next count value from the
** Koala buffer. Determine if it is a
** short (1byte) or long (2byte) counter
** and whether it is followed by UNIQUE
** or REPEATed data.
*/

  count = src[sdat++];
  if ( ( (count & 0x80) == 0) && ( (count & 0x7f) !=0) )
    {
    lcount = (usint)(count & 0x7f);
    flag = REPEAT;
    }
  else
    if ( ( (count & 0x80) != 0) && ( (count & 0x7f) !=0) )
      {
      lcount = (usint)(count & 0x7f);
      flag = UNIQUE;
      }
  else
    if ( count == 0 )
      {
      flag = REPEAT;
      lcount = 256*(usint)src[sdat++] + (usint)src[sdat++];
      }
  else
    if ( count == 0x80 )
      {
      flag = UNIQUE;
      lcount = 256*(usint)src[sdat++] + (usint)src[sdat++];
      }

  data = src[sdat++];

  for (i=0; i < lcount; i++)
    {                           /* COUNT for                    */
    dst[ddat++] = data;
    if (flag == UNIQUE)
      data = src[sdat++];
    if (ddat >= 7680)
      break;
    }                           /* COUNT endfor                 */

  if (flag == UNIQUE)
    sdat--;

  if ( (ddat >= 7680) || (sdat >= srcsz) )
    done = TRUE;
  }                             /* !DONE while                  */
} /*  horiz_unpack endfunction */


/****************************************
** invert ( type, area ) - invert an
** area of the screen. Type indicates
** if it's FULL, SPLIT, or QUAD. area is
** LEFT or RIGHT, if type is splie, or
** one of the corners if QUAD. This info
** will determine area of screen to invert.
*/

invert ( type, area )
usint type, area;

{ /* invert function */


graf_mouse( HIDE, &dum);

/*****************************************
** Use monochrome coordinates. fillbox checks
** resolution and adjusts limits accordingly.
*/

if (type == FULL)
  fillbox ( 1, XOR, 0, 0, 639, 384);
else if ((type == SPLIT) && (area == LEFT))
  fillbox ( 1, XOR, 0, 0, 316, 384);
else if ((type == SPLIT) && (area == RIGHT))
  fillbox ( 1, XOR, 324, 0, 639, 384);
else if ((type == QUAD) && (area == TOP_LEFT))
  fillbox ( 1, XOR, 0, 0, 316, 192);
else if ((type == QUAD) && (area == TOP_RIGHT))
  fillbox ( 1, XOR, 324, 0, 639, 192);
else if ((type == QUAD) && (area == BOTTOM_LEFT))
  fillbox (1, XOR, 0, 200, 316, 392);
else if ((type == QUAD) && (area == BOTTOM_RIGHT))
  fillbox ( 1, XOR, 324, 200, 639, 392);

lcopy( crt, cbuff );
graf_mouse( SHOW, &dum );

} /* invert endfunction */


/****************************************
** demo() - Take the .PI9 picture currently
** in xebuff and display it in a few
** cute ways and exit.
*/

demo()

{ /* demo function */

int i,j;
long fio;

graf_mouse ( HIDE, &dum);



if (rez == LO_REZ)
  {
  for (i=0; i < 8; i++)
    pxy[i] >>= 1;
  for (i=0; i < 9; i++)
    pstripe[i] >>= 1;
  }

xemode = MODE9;
i = Fopen("DEMO.PI9",READ_ONLY);

if (i >= 0)
    {
    fio = (long)Fread(i,7680L,xebuff);
    j = Fclose(i);
    demo9();
    }

xemode = MODE8;
i = Fopen("DEMO.PI8",READ_ONLY);

if (i >= 0)
    {
    fio = (long)Fread(i,7680L,xebuff);
    j = Fclose(i);
    demo8();
    }

xemode = KOALA7;
i = Fopen("DEMO.PIC",READ_ONLY);

if (i >= 0)
    {
    fio = (long)Fread(i,7680L,koala);
    j = Fclose(i);
    koala_proc(fio);
    demok();
    }
  
sx = (rez == HI_REZ) ? 32:13;

vst_effects (handle, 16 );       /* Bold, outline */
vst_height  (handle, sx, &dum, &dum, &dum, &dum);

sx = (rez == HI_REZ) ? 40:20;
sy = (rez == HI_REZ) ? 100:50;

v_gtext     (handle, sx, sx,"Altered STeights  By Mat Rat");
v_gtext     (handle, sx, sy," Brought to you by   ST-Log ");


vst_effects (handle, 0 );
vst_height  (handle, 13, &dum, &dum, &dum, &dum);

graf_mouse ( SHOW, &dum );

} /* demo endfunction */


/****************************************
** demo9() - demo.pi9 file found and loaded
** into xebuff. Now demo it for the
** appropriate resolution
*/

demo9()

{ /* demo9 function */

if (rez == HI_REZ)
  {
  quad_map9( TOP_LEFT );
  quad_map9( BOTTOM_RIGHT);
  quad_map9( TOP_RIGHT);
  quad_map9( BOTTOM_LEFT );
  split_map9( RIGHT );
  split_map9( LEFT  );
  full_map9();
  }
else
  {
  spl_map9c(LEFT);
  }

} /* demo9 endfunction */

/****************************************
** demo8() - demo.pi8 file found and loaded
** into xebuff. Now demo it for the
** appropriate resolution
*/

demo8()

{ /* demo8 function */

if (rez == HI_REZ)
  {
  quad_map8( TOP_LEFT );
  quad_map8( BOTTOM_RIGHT);
  quad_map8( TOP_RIGHT);
  quad_map8( BOTTOM_LEFT );
  split_map8( RIGHT );
  split_map8( LEFT  );
  full_map8();
  }
else
  {
  qd_map8c(TOP_RIGHT);
  }

} /* demo8 endfunction */

/****************************************
** demok() - demo.pic file found and loaded
** into xebuff. Now demo it for the
** appropriate resolution
*/

demok()

{ /* demok function */

if (rez == HI_REZ)
  {
  quad_mapk( TOP_LEFT );
  quad_mapk( BOTTOM_RIGHT);
  quad_mapk( TOP_RIGHT);
  quad_mapk( BOTTOM_LEFT );
  split_mapk( RIGHT );
  split_mapk( LEFT  );
  full_mapk();
  }
else
  {
  qd_mapkc(BOTTOM_RIGHT);
  }

} /* demok endfunction */
