/****************************************
** Detab utility by Mat*Rat
** calling format: DETAB file tabsize
** If no tabsize is specified
** a default of 8 is assumed.
*/

/****************************************
** INCLUDE files
*/

#include <stdio.h>
#include <osbind.h>

/****************************************
** Constant Declarations:
*/

#define BACK_SLASH 92
#define NAME_MAX   64
#define LINE_MAX   132
#define FALSE      0
#define TRUE       1
#define TAB        9
#define NULL       0
#define SPACE      32
#define CR         13
#define LF         10
#define BELL       7


/****************************************
** GLOBAL VARIABLES:
*/

int Tabs,Tab_Sum;
char Filename[NAME_MAX];

char Oops[] = "DeTab call format: filename [tab size]\n\0";
char Ferr[] = "Unexpected file ERROR.\0";
char Done[] = "DeTab FINISHED, exit now.\n\n\0";
char Temp[] = ".DET\0"; /* Detabbed filename */

/****************************************
** MAIN PROGRAM
*/


main ( argc, argv )
int argc;
char *argv[];

{ /* main begin*/

int j, i, eof;
FILE *fd, *fd1;
char outfile[NAME_MAX], line[LINE_MAX], c, k;
char *name;

for (i=0; i < NAME_MAX; i++)
  outfile[i] = 0;
  
/****************************************
** fetch filename from input line,
** if none is given remind user
** of proper call format and exit.
*/

Tabs = 8;

printf("DeTab by Mat*Rat.\n\n");

if (argc < 2)
  {
  printf( Oops );
  printf("Enter filename and optional tab size:  ");
  name == 0;
  while (name == 0)
    name = gets(Filename);
  i = Clfind(SPACE, Filename);
  if (i > 0)
    {
    Tabs = atoi(&Filename[i+1]);
	Filename[i] = 0;
	}
  }
else
  strcpy ( Filename, argv[1] );


if (argc >= 3)
  Tabs = atoi( argv[2] );

if (Tabs == 0)
  Tabs = 8;
  
i = Clfind( '.', Filename );     /* Check for EXTENDER */

if (i < 0)
  i = strlen(Filename);
  
for(j=0; j < i; j++)
  outfile[j] = Filename[j];

outfile[i] = 0;

strcat(outfile, Temp);
        
printf("Tab Size = %3d\n\n", Tabs);
printf("Detab file: %s\n\n", Filename);
printf("Output to: %s\n\n", outfile);

Tab_Sum = 0;
  
fd = fopen ( Filename, "r\0" );

if (fd == NULL)
  {
  Prompt( Ferr );
  exit(1);
  }
  
Fdelete ( outfile );    /* Erase if already exists */

fd1 = fopen( outfile, "w\0" );

if (fd1 == NULL)
  {
  Prompt( Ferr);
  exit(1);
  }
  
eof = FALSE;  

while (eof == FALSE)
  {
  eof = Get_Line ( line, fd );
  if (eof == FALSE)
    {
    Detab ( line );
    Write_Line( line, fd1 );
    }
  }

fclose( fd );
fclose( fd1 );

printf("Total tab characters removed: %d\n\n", Tab_Sum);
Prompt( Done );

exit(1);

} /* main end */

/****************************************
** Clfind ( c, str ) - find the LAST
** occurance of the character C in the
** string STR.  If found, return the index
** pointing to it. If not, return a -1.
*/

Clfind ( c, str )
char c, str[];

{ /* Clfind begin */    

int i, j, ptr;
char b;

j   = strlen ( str );
ptr = -1;

if (j > 0)
  { /* begin if */
  i = j-1;
  while( (i >= 0) && (ptr < 0) )
    { /* begin while */
    if ( ( b = str[i] ) == c)
      ptr = i;
    i--;
    } /* end while */
  } /* end if */
return ( ptr );

} /* Clfind end */


/****************************************
** Detab( line ) - convert tab characters
** in the line to the proper number
** of space characters according to the
** global tab size of Tabs.
*/

Detab ( line )
char line[];

{ /* Detab begin */

int i, j, tsz;
char str[LINE_MAX], c;

strcpy(str, line);

j   = 0;
i   = 0;
tsz = Tabs;

while ( j < LINE_MAX)
  { /* while begin */
  c = str[i++];
  switch ( c )
    { /* switch begin */
    case ( NULL ):
      line[j] = NULL;
      j = LINE_MAX;
      break;
          
     case ( TAB ):
	   Tab_Sum++;
       while ( ( tsz > 0) && (j < LINE_MAX) )
         {  /* while begin */
         line[j++] = SPACE;
         tsz--;
         } /* while end */
       tsz = Tabs;
       break;
          
        default:
          line[j++] = c;
          tsz--;
          if (tsz <= 0)
            tsz = Tabs;
          break;
        } /* switch end */
  
  } /* while end */
  
} /* Detab end */  

/****************************************
** Get_Line ( line, fd ) - fetch line from
** open input file pointed to by fd. Return
** eof flag. TRUE if EOF reached, else FALSE.
*/

Get_Line ( line, fd )
char line[];
FILE *fd;

{ /* Get_Line begin */

int i;
char *str;

str = fgets(line, (int)LINE_MAX, fd);

i = Clfind( CR, line);

if (i >= 0)
  line[i] = (char)NULL;
  
i = Clfind( LF, line);

if (i >= 0)
  line[i] = (char)NULL;

i = feof( fd );

return( i );
  
} /* Get_Line end */

/****************************************
** Write_Line ( line, fd ) - Write the
** line to the file pointed to by fd.
*/

Write_Line ( line, fd )
char line[];
FILE *fd;

{ /* Write_Line begin */

int i;

i = NULL;

i = fprintf(fd, "%s\n", line );

} /* Write_Line end */


/****************************************
** Prompt ( line ) - print line to display
** and then wait for RETURN to continue.
*/

Prompt ( line )
char line[];

{ /* Prompt begin */

char c;

printf("%s", line);

printf("\nPress [RETURN] ?");

putchar ( BELL );

c = getchar();

} /* Prompt end */
