**************************************************************************
*
* Assembly Line: File Handling part 1-- dump.s
* by Douglas Weir
*
* Copyright 1988 ST-Log
*
*
**************************************************************************

start:
        movea.l 4(a7),a5        get base page address
        lea.l   $80(a5),a2      point to command line
        clr.l   d0              clear for byte value
        move.b  (a2)+,d0        get byte count


        move.l  a2,-(sp)        push filename string address
        move.w  d0,-(sp)        push length
        bsr     open_file       open file
        addq.l  #6,sp           pop args
        tst.w   d0              error?
        bmi.s   abort           if so
        
        bsr     display_file    read file

        move.w  d0,-(sp)        push file handle
        move.w  #$3e,-(sp)      code==close file
        trap    #1              do it
        addq.l  #4,sp           pop args

out:
        move.w  #0,-(sp)        code==terminate
        trap    #1              do it
abort:
        move.l  #abort_msg,-(sp)        error message string
        move.w  #$09,-(sp)      code==print string
        trap    #1              do it
        addq.l  #6,sp           pop args
        bra     out             and leave


*****
*
* open_file-- get filename, open file
*
* at entry:
*       (a6) + 8  -> length of filename string
*       (a6) + 10 -> address of filename string
*
* at exit:
*       d0.w contains file handle or neg number if error.
*       all other registers preserved.
* 
*****
open_file:
        link    a6,#0           frame pointer
        movem.l a0-a2/d1-d2,-(sp)

        movea.l 10(a6),a0       address of string
        move.w  8(a6),d0        length of it
        movea.l #filename,a1    address of filename storage area
        bra     s_test          now start
scan:
        move.b  (a0)+,(a1)+     copy a byte
s_test: dbra    d0,scan         all of them
        move.b  #0,(a1)         null terminator

        move.w  #0,-(sp)        mode==read only
        move.l  #filename,-(sp) filename string
        move.w  #$3d,-(sp)      code==open file
        trap    #1              do it
        addq.l  #8,sp           pop args
        tst.w   d0              error?
        bmi.s   open_out        if so
        move.w  d0,handle       else save file handle
        
open_out:
        movem.l (sp)+,a0-a2/d1-d2
        unlk    a6
        rts



*****
*
* display_file-- display file contents in ASCII/symbolic/hex format
*
* at exit:
*       all registers preserved.
*
*****
display_file:
        movem.l a0-a4/d0-d5,-(sp)
        move.w  handle,d5       get file handle
        clr.l   d3              clear for byte values
        clr.l   d4

d_go:
        move.l  #cr_lf,-(sp)    carriage return/line feed string
        move.w  #$09,-(sp)      code==print string
        trap    #1              do it
        addq.l  #6,sp           pop args

        move.l  #buffer,-(sp)   buffer address
        move.l  #$1,-(sp)       byte count to read
        move.w  d5,-(sp)        file handle
        move.w  #$3f,-(sp)      code==read file
        trap    #1              do it
        add.l   #12,sp          pop args
        tst.w   d0              end of file?
        beq     d_out           if so
        bmi     d_out           if error

        movea.l #buffer,a2      point to buffer 
        move.b  (a2),d3         get byte read
        cmpi.b  #32,d3          do control conversion?
        bcs     do_ctrl         if so           
        cmpi.b  #127,d3         else: do ascii?
        bcs     do_ascii        if so
do_hex:
        movea.l #hexes,a1       base of hex table
        lsl.l   #8,d3           shift d3 left 12 bits
        lsl.l   #4,d3
        swap    d3              get low nybble
        move.b  0(a1,d3.w),(a2)+        hex digit in buffer
        rol.l   #4,d3           restore complete low byte
        andi.l  #$0f,d3         mask out high nybble
        move.b  0(a1,d3.w),(a2)+        hex digit in buffer
        move.b  #0,(a2)         null terminator
        move.l  #buffer,-(sp)   push address of string
        bra     print_it

do_ctrl:
        movea.l #ctrl,a1        base of ctrl table
        lsl.l   #2,d3           multiply index by 4
        pea     0(a1,d3.w)      push address of string
print_it:
        move.w  #$9,-(sp)       code==print string
        trap    #1              do it
        addq.l  #6,sp           pop args
        bra     d_go            and continue

do_ascii:
        move.w  d3,-(sp)        push char
        move.w  #$2,-(sp)       code==conout
        trap    #1              do it
        addq.l  #4,sp           pop args
        bra     d_go            and continue

d_out:
        movem.l (sp)+,a0-a4/d0-d5
        rts

*****
*
* data segment
*
*****
        data
        even

handle          ds.w    1
abort_msg       dc.b    $0d,$0a,'oops-- this file does not exist',$0a,$0d,0
filename        ds.b    30

cr_lf           dc.b    $0d,$0a,0
buffer          ds.b    8

hexes   equ     *               table of hex digits
        dc.b    '0123456789ABCDEF'

ctrl    equ     *               table of control codes
        dc.b    'NUL',0         0  -- null
        dc.b    'SOH',0         1  -- start of header
        dc.b    'STX',0         2  -- start of text
        dc.b    'ETX',0         3  -- end of text
        dc.b    'EOT',0         4  -- end of transmission
        dc.b    'ENQ',0         5  -- enquiry
        dc.b    'ACK',0         6  -- acknowledge
        dc.b    'BEL',0         7  -- bell
        dc.b    'BS ',0         8  -- backspace
        dc.b    'HT ',0         9  -- horizontal tab
        dc.b    'LF ',0         10 -- line feed
        dc.b    'VT ',0         11 -- vertical tab
        dc.b    'FF ',0         12 -- form feed
        dc.b    'CR ',0         13 -- carriage return
        dc.b    'SO ',0         14 -- shift out
        dc.b    'SI ',0         15 -- shift in
        dc.b    'DLE',0         16 -- data link escape
        dc.b    'DC1',0         17 -- device control 1
        dc.b    'DC2',0         18 -- device control 2
        dc.b    'DC3',0         19 -- device control 3
        dc.b    'DC4',0         20 -- device control 4
        dc.b    'NAK',0         21 -- negative acknowledge
        dc.b    'SYN',0         22 -- synchronous idle
        dc.b    'ETB',0         23 -- end of transmission block
        dc.b    'CAN',0         24 -- cancel
        dc.b    'EM ',0         25 -- end of medium
        dc.b    'SUB',0         26 -- substitute
        dc.b    'ESC',0         27 -- escape
        dc.b    'FS ',0         28 -- file separator
        dc.b    'GS ',0         29 -- group separator
        dc.b    'RS ',0         30 -- record separator
        dc.b    'US ',0         31 -- unit separator
