program menus;

{$I d:\gemsubs}

var
  menu          : Menu_Ptr;          { pointer to menu bar tree          }
  quit,                              { flag used to exit program         }
  grid_status   : boolean;           { flag to show or disable grid      }
  unusual_text,                      { menu item text string             }
  new_text      : str255;            { global var for menu text change   }
  file_title,                        { integers for the menu titles and  }
  opt_title,                         { items                             }
  screwed_title,
  open_item,
  close_item,
  dummy_line,
  grid_mode,
  drw_mode,
  erase_mode,
  too_short,
  too_long,
  nocheck_space,
  non_standard,
  about_menus	: integer;
  
procedure build_menu;

{ Call New_Menu to reserve space for our menu tree, add all the titles to
  our tree and then add the items to each title in the order the titles
  were declared. The dummy line will be disabled, items that will lead to
  dialog boxes (such as open) are suffixed with '...' to indicate some-
  thing follows this selection. The text for grid status may be changed
  to Don't Show Grid and we have to be sure to make the string long
  enough to have room for the max text we may put in there. }

  begin
    menu := New_Menu( 15, '  About Menus...  ' );
    file_title    := Add_MTitle( menu, ' File ' );
    opt_title     := Add_MTitle( menu, ' Options ');
    screwed_title := Add_MTitle( menu, ' Screwed Up ' );
    open_item     := Add_MItem( menu, file_title, '  Open...   ' );
    dummy_line    := Add_MItem( menu, file_title, '------------' );
    close_item    := Add_MItem( menu, file_title, '  Close     ' );
    grid_mode     := Add_MItem( menu, opt_title, '  Show Grid        ' );
    drw_mode      := Add_MItem( menu, opt_title, '  Draw Mode        ' );
    erase_mode    := Add_MItem( menu, opt_title, '  Erase Mode       ' );
    
    { Here we show some of the mistakes that can be made when designing
      menus. Run the program to see how these look on the screen.       }
      
    too_short     := Add_MItem( menu, screwed_title, '  Too Short' );
    too_long      := Add_MItem( menu, screwed_title, '  This text is too long to fit the screen width and wraps around. ' );
    nocheck_space := Add_MItem( menu, screwed_title, 'No Space for Check' );
    
    { This isn't a mistake, but it shows how you can use some of the non-
      standard characters within your menu.                              }
      
    unusual_text  := Concat( '  ', chr(14),chr(15),' ',chr(11),' ',chr(3),
                             ' ', chr(175),' ',chr(189),' ',chr(174) );
    non_standard  := Add_MItem( menu, screwed_title, unusual_text );
  end;
  
procedure set_up_menu;

{ Proper planning dictates we should disable all options that make no
  sense or aren't available to the user to prevent confusion. }

  begin
    grid_status := true;
    Menu_Check( menu, drw_mode, true );
    Menu_Disable( menu, dummy_line );
    Menu_Disable( menu, close_item );
    Menu_Disable( menu, grid_mode );
    Menu_Disable( menu, drw_mode );
    Menu_Disable( menu, erase_mode );
    Menu_Check( menu, nocheck_space, true );
  end;

procedure do_close;

{ All we do is set the quit flag, a real program would probably have to do
  some cleaning before it could exit properly. You could erase the menu
  here but I left it in the main block to show program progression. }

  begin
    quit := true;
  end;
  
procedure do_open;

{ Here we would open the file by displaying the file selector box and load
  the chosen file, it is important to reconfigure the menu to enable any 
  options now available to the user, also by disabling the open item in the
  menu we prevent loading another file on top of the one in memory. }

  var
    button : integer;

  begin
    Menu_Disable( menu, open_item );
    Menu_Enable( menu, close_item );
    Menu_Enable( menu, grid_mode );
    Menu_Enable( menu, drw_mode );
    Menu_Enable( menu, erase_mode );
    button := Do_Alert( '[1][Display the file selector here.][ Ok ]', 1 );
  end;
  
procedure do_grid;

{ Just a change to the menu text to change the commands available to the
  user. We don't show a grid anymore than we do anything else, but it's a
  command you may want to use one day. The grid_status flag is used to 
  keep track of the current state of the grid. }

  begin
    if grid_status = true then              { Grid was previously on, but }
      begin                                 { user must want to turn it   }
        grid_status := false;               { off since he called us here.}
        new_text := '  Don''t Show Grid  ';
      end
    else
      begin                                 { Grid was off, but user must }
        grid_status := true;                { want to see it again.       }
        new_text := '  Show Grid        ';
      end;
    Menu_Text( menu, grid_mode, new_text );
  end;
  
procedure do_draw;

{ Show the current mode by checking the menu option chosen. }

  begin
    Menu_Check( menu, erase_mode, false );
    Menu_Check( menu, drw_mode, true );
  end;
  
procedure do_erase;

{ Show the current mode my checking the menu option chosen. }

  begin
    Menu_Check( menu, drw_mode, false );
    Menu_Check( menu, erase_mode, true );
  end;

procedure show_about_box;

  var
    button : integer;

  begin
    button := Do_Alert( '[1][A dialog box goes here!][ Ok ]', 1 );
  end;
  
procedure do_menu( title, item : integer );

{ First we check to see if Desk_Title has been chosen, if so display the
  'about menus' box. Since each item has a unique number we can just
  concentrate on checking the item number after that. After the item has
  been processed, we must return the menu title to normal video. }

  begin
    if title = Desk_Title then show_about_box
      else
    if item = close_item then do_close
      else
    if item = open_item then do_open
      else
    if item = grid_mode then do_grid
      else
    if item = drw_mode then do_draw
      else
    if item = erase_mode then do_erase;
    Menu_Normal( menu, title );
  end;

procedure event_loop;

{ Most of the parameters in the event call aren't needed to monitor menu
  bar events. We pass dummy values to fill the spaces. Since we only told
  GEM to notify us of messages, we don't need to test the event value for
  other events or messages, we just assume (probably not a good idea) GEM
  will pass only messages. We use the CASE statement to operate only on
  menu messages and ignore all others to keep it simple. In our case we
  need only the values in the 3rd and 4th subscripts of the message
  buffer. }

  var
    event,
    dummy  : integer;
    msg    : Message_Buffer;
    
  begin
    quit := false;
    repeat
      event := Get_Event( E_Message, 0, 0, 0, 0,
                          false, 0, 0, 0, 0,
                          false, 0, 0, 0, 0,
                          msg,
                          dummy, dummy, dummy,
                          dummy, dummy, dummy );
      case msg[ 0 ] of
        MN_Selected	: do_menu( msg[ 3 ],     { title index }
                                   msg[ 4 ] );   { item index  }
      end; { case }
    until quit = true;
  end;
  
begin                              {    Main Program Block   }
  if Init_Gem >= 0 then            { Pretty self-explanatory }
    begin
      Set_Mouse( M_Point_Hand );
      build_menu;
      set_up_menu;
      Draw_Menu( menu );
      event_loop;
      Erase_Menu( menu );
      Exit_Gem;
    end;
end.
