/**********************************************************************/
/*                                                                    */
/*  Function Name:  COPIES                                            */
/*  Purpose:  Return a pointer to a character string which contains   */
/*            a specified number of copies of a supplied input        */
/*            string, all concatenated into a single long string.     */
/*            NULL is returned if memory allocation fails.            */
/*  Written By:  Karl Wiegers                                         */
/*  Date Written:  March, 1989                                        */
/*  Usage Syntax:  copies ( input_string, number_of_copies_desired )  */
/*  External Functions Called:  malloc, printf, strcat, strlen        */
/*                                                                    */
/**********************************************************************/

#include <stdio.h>
#include <strings.h>

extern char *malloc ();

char *copies ( instring, num_copies )
char *instring;
short num_copies;

{
	static char *string;   /* pointer to string of copies to return  */

/*--------------------------------------------------------------------*/
/*    attempt to allocate space to contain string of copies; don't    */
/*    forget a space for the trailing null character                  */
/*--------------------------------------------------------------------*/

	if ( ( string = malloc ( num_copies * strlen ( instring ) + 1 ) ) == NULL )
	{
		printf ( "Unable to allocate memory for COPIES\n" );
		return ( NULL );
	}
/*---------------------------------------------------------------------*/
/*  initialize output string to null character; append contents of     */
/*  of input string to the output string the desired number of times   */
/*---------------------------------------------------------------------*/

	*string = '\0';
	while ( num_copies-- )
		strcat ( string, instring );

	return ( string );
}
/**********************************************************************/
/*                                                                    */
/*  Function Name:  LEFT                                              */
/*  Purpose:  Return a pointer to a character string which contains   */
/*            the leftmost 'number' characters of a supplied input    */
/*            string, terminated by a null character.  NULL is        */
/*            returned if memory allocation fails.                    */
/*  Written By:  Karl Wiegers                                         */
/*  Date Written:  March, 1989                                        */
/*  Usage Syntax:  left ( input_string, number_of_left_chars)         */
/*  External Functions Called:  malloc, printf, strcat, strlen,       */
/*                              sprintf, copies                       */
/*                                                                    */
/**********************************************************************/

#include <stdio.h>
#include <strings.h>

extern char *copies ();
extern char *malloc ();

char *left ( instring, number )

char *instring;
short number;

{
	static char *temp_string; /* pointer to string of leftmost chars */
	register short length;    /* number of characters in 'instring'  */

/*--------------------------------------------------------------------*/
/*    attempt to allocate space to contain string of leftmost charac- */
/*    ters; don't forget a space for the trailing null character      */
/*--------------------------------------------------------------------*/

	length = strlen ( instring );
	if ( ( temp_string = (char *) malloc ( length + 1 ) ) == NULL )
	{
		printf ( "Unable to allocate memory for LEFT function.\n" );
		return ( NULL );
	}

/*--------------------------------------------------------------------*/
/*  do internal print of leftmost 'number' characters to the address  */
/*  pointed to by 'temp_string'; if 'number' was longer than the      */
/*  length of the input string, pad 'temp_string' with blanks to the  */
/*  desired length.                                                   */
/*--------------------------------------------------------------------*/

	sprintf ( temp_string, "%-.*s", number, instring );
	if ( number > length )
	strcat ( temp_string, copies ( " ", number - length ) );

	return ( temp_string );
}
/**********************************************************************/
/*                                                                    */
/*  Function Name:  RIGHT                                             */
/*  Purpose:  Return a pointer to a string which contains the right-  */
/*            most 'number' characters of a supplied input string.    */
/*            NULL is returned if memory allocation fails.            */
/*  Written By:  Karl Wiegers                                         */
/*  Date Written:  March, 1989                                        */
/*  Usage Syntax:  right ( input_string, number_of_characters)        */
/*  External Functions Called:  malloc, printf, strcat, strlen,       */
/*                              sprintf, strcpy, alloca, copies,      */
/*                              strrev                                */
/*                                                                    */
/**********************************************************************/

#include <stdio.h>
#include <strings.h>

extern char *copies ();
extern char *strrev ();
extern char *malloc ();

char *right ( instring, number )

char *instring;
short number;

{
	register short length;      /* number of chars in 'instring'        */
	register char *copy_string; /* temporary work space                 */
	static char *temp_string;   /* pointer to string of rightmost chars */

/*--------------------------------------------------------------------*/
/*  attempt to allocate space to contain the string returned; don't   */
/*  forget a space for the trailing null character                    */
/*--------------------------------------------------------------------*/

	length = strlen ( instring );
	if ( ( temp_string = (char *) malloc ( length + 1 ) ) == NULL )
	{
	     printf ( "Unable to allocate memory for RIGHT function\n" );
	     return ( NULL );
	}

/*--------------------------------------------------------------------*/
/* Use different algorithms, depending on whether the number of char- */
/* acters desired from the right side of the input string is greater  */
/* than, less than, or equal to the number of chars in the string.    */
/*                                                                    */
/* If length is less than number of characters desired, do internal   */
/* write of entire input string, reverse it, append the required      */
/* number of blanks to pad to desired length, then reverse it again   */
/* and append a null character.                                       */
/*--------------------------------------------------------------------*/

	if ( length < number )
	{
		sprintf ( temp_string, "%.*s", number, instring );
		strrev ( temp_string );
		strcat ( temp_string, copies ( " ", number-length ) );
		strrev ( temp_string );
		strcat ( temp_string, "\0" );
	}

/*--------------------------------------------------------------------*/
/* If equal, simply copy the input string to the output string.       */
/*--------------------------------------------------------------------*/
 
	else if ( length == number )
		strcpy ( temp_string, instring );

/*--------------------------------------------------------------------*/
/* If number of characters desired is less than the length of the     */
/* input string, allocate temporary work space on the stack, copy the */
/* input string into this temporary string, reverse it, do internal   */
/* write of desired number of characters, reverse it again, and       */
/* append a null character.                                           */
/*--------------------------------------------------------------------*/

	else if ( length > number )

	{
		copy_string = (char *) alloca ( length + 1 );
		strcpy (copy_string, instring );
		strrev (copy_string );
		sprintf ( temp_string, "%-.*s", number, copy_string );
		strrev (temp_string );
		strcat (temp_string, "\0" );
	}

	return ( temp_string );
}
/**********************************************************************/
/*                                                                    */
/*  Function Name:  STRREV                                            */
/*  Purpose:  Reverse the sequence of characters in a supplied input  */
/*            string.  A pointer to the altered input string is       */
/*            returned.  NULL is returned if memory allocation fails. */
/*  Written By:  Karl Wiegers                                         */
/*  Date Written:  March, 1989                                        */
/*  Usage Syntax:  strrev ( input_string)                             */
/*  External Functions Called:  malloc, printf, strcpy, strlen        */
/*                                                                    */
/**********************************************************************/

#include <stdio.h>
#include <strings.h>

extern char *malloc ();

char *strrev ( instring )
char *instring;

{
	register short i;       /* index variable                            */
	register short j;       /* index variable                            */
	register short length;  /* number of characters in 'instring'        */ 
	char *tempstr;          /* temporary work space                      */

/*--------------------------------------------------------------------*/
/*    attempt to allocate temporary space in which to construct the   */
/*    reversed string, including the trailing null character          */
/*--------------------------------------------------------------------*/

	length = strlen ( instring );
	if ( ( tempstr = (char *) malloc ( length + 1 ) ) == NULL )
	{
		printf ( "Unable to allocate memory for STRREV\n" );
		return ( NULL );
	}

/*--------------------------------------------------------------------*/
/*  start at the end of the input string and work backward; copy each */
/*  character into the next position of the temporary string; append  */
/*  a null character at the end of the temporary string, then copy    */
/*  the temporary string to the input string and return the pointer   */
/*--------------------------------------------------------------------*/

	for ( i = 0, j = length; j > 0; i++ )
		tempstr[i] = instring[--j];
	tempstr[++i] = '\0';
	strcpy ( instring, tempstr );

	return ( instring );
}
/**********************************************************************/
/*                                                                    */
/*  Function Name:  WORDS                                             */
/*  Purpose:  Return a short integer whose value is the number of     */
/*            blank-delimited words contained in a supplied input     */
/*            string.                                                 */
/*  Written By:  Karl Wiegers                                         */
/*  Date Written:  March, 1989                                        */
/*  Usage Syntax:  words ( input_string )                             */
/*  External Functions Called:  strlen                                */
/*                                                                    */
/**********************************************************************/

#include <strings.h>

/*--------------------------------------------------------------------*/
/*  define logical YES to integer 1 and NO to 0; this code makes sure */
/*  we don't try to redefine YES and NO if they already have been set */
/*--------------------------------------------------------------------*/


#ifndef  YES
#define  YES   1
#endif

#ifndef  NO
#define  NO    0
#endif

short words ( instring )
char *instring;

{
	register short num_words = 0;  /* counts # of words in 'instring' */
	register short inword    = NO; /* flag for whether current char   */
	                               /* is in a word (YES) or not (NO)  */

/*--------------------------------------------------------------------*/
/* If the length of the input string is zero, it contains no words.   */
/* Otherwise, the basic idea is to scan each character in the string. */
/* The variable 'inword' is set to YES if the current character is    */
/* not blank, NO if it is blank.  Each time we reach a blank char,    */
/* the 'num_words' counter is incremented, except that multiple       */
/* blanks are treated as a single blank for word separation purposes. */
/* If we're in a word when the terminating null character is reaced,  */
/* increment 'num_words' one more time.                               */
/*--------------------------------------------------------------------*/

	if ( strlen ( instring ) > 0 )
	{
		while ( *instring != '\0' )
		{
			if ( ( *instring == ' ' ) && ( inword == YES ) )
			{
				num_words++;
				inword = NO;
			}
			else if ( ( *instring != ' ' ) && ( inword == NO ) )
				inword = YES;
			instring++;
		}
	}
	if ( inword == YES )
		num_words++;

	return ( num_words );
}
/***********************************************************************/
/*                                                                     */
/*  Function Name:  WORD                                               */
/*  Purpose:  Return a pointer to a specific word in a string of       */
/*            blank-delimited words.  NULL is returned if the          */
/*            attempt to allocate memory fails, if the requested       */
/*            word position is less than 1, or if the requested        */
/*            word position is greater than the number of words        */
/*            in the string.                                           */
/*  Written by:  Karl Wiegers                                          */
/*  Date Written:  March 1989                                          */
/*  Syntax:  word ( input_string, desired_word_position )              */
/*  External Functions Called:  malloc, printf, words,strlen           */
/*                                                                     */
/***********************************************************************/

#include <stdio.h>
#include <strings.h>

#ifndef     YES
#define     YES     1
#define     NO     0
#endif

extern short words ();
extern char *malloc ();

char *word (instring, word_pos)

char  *instring;
short word_pos;

{
	register short word_ctr = 0;  /* counts # of words scanned so far */
	register short inword   = NO; /* flag for whether current char is */
	                              /* in a word (YES) or not (NO)      */
	char  *temp_ptr;              /* temporary work space             */
	static char *word_ptr;        /* points to desired word, returned */

/*-------------------------------------------------------------------*/
/* attempt to allocate space to contain the word being sought in the */
/* input string, including room for the trailing null character      */
/*-------------------------------------------------------------------*/

	free ( word_ptr );
	if ( ( word_ptr = (char *) malloc ( strlen ( instring ) + 1 ) ) == NULL)
	{
		printf ( "Unable to allocate memory for WORD function\n" );
		return ( NULL );
	}

/*---------------------------------------------------------------------*/
/* see if argument value for word position to return is okay;          */
/* if not, return NULL pointer                                         */
/*---------------------------------------------------------------------*/

	if ( word_pos < 1 || ( word_pos > words ( instring )  ) )
		return ( NULL );

/*---------------------------------------------------------------------*/
/* Initialize current character pointer to the beginning of the        */
/* input string.  Then search character by character down the          */
/* string until the terminating null character is hit.  The main       */
/* logic is to see if the current character is a blank or not, and     */
/* whether we are currently in a word (non-blank characters) or        */
/* not.  When we reach the end of a word, increment the word           */
/* counter.  When we reach the beginning of the desired word, scan     */
/* it until the next blank is found, then exit from this big loop      */
/* and append a terminating null character.  At this point, the        */
/* word_ptr is pointing to the beginning of the desired word, so       */
/* return that pointer.                                                */
/*---------------------------------------------------------------------*/

	temp_ptr = word_ptr;

	while ( *instring != '\0' )
	{
		if ( *instring != ' ' )
		{
			if ( inword == YES )
			{
				if ( word_ctr == word_pos )
					*temp_ptr++ = *instring;
			}
			else if ( inword == NO )
			{
				inword = YES;
				word_ctr++;
				if ( word_ctr == word_pos )
					*temp_ptr++ = *instring;
			}
		}
		else if ( *instring == ' ' )
		{
			if ( inword == YES )
			{
				if (word_ctr == word_pos )
					break;
				else
					inword = NO;
			}
		}
	instring++;
	}
	*temp_ptr = '\0';
	return ( word_ptr );
}
/**********************************************************************/
/*                                                                    */
/*  Function Name:  SUBSTR                                            */
/*  Purpose:  Return a pointer to a character string which contains   */
/*            a substring of a supplied input string, beginning at a  */
/*            particular character position and extending for a       */
/*            specified number of characters, with a supplied pad     */
/*            character used to fill trailing blanks.  NULL is        */
/*            returned if memory allocation fails or if starting      */
/*            position does not fall within the string.               */
/*  Written By:  Karl Wiegers                                         */
/*  Date Written:  March, 1989                                        */
/*  Usage Syntax:  substr ( input_string, start_position,             */
/*                          number_of_characters, pad_character )     */
/*  External Functions Called:  strlen, malloc, strcpy, printf,       */
/*                              copies                                */
/*                                                                    */
/**********************************************************************/

#include <stdio.h>
#include <strings.h>

extern char *copies ();
extern char *malloc ();

char *substr ( instring, startpos, length, pad )

char *instring;
short startpos;
short length;
char *pad;

{
	static char *temp_string;     /* pointer to substring to return   */
	register short i = 0;         /* index variable                   */
	register short stop = 0;      /* # of chars to copy from instring */
	register short inlength = 0;  /* # of chars in 'instring'         */

/*--------------------------------------------------------------------*/
/*  make sure starting position is inside the string; also, attempt   */
/*  to allocate space to contain the substring to be returned,        */
/*  including a space for the trailing null character                 */
/*--------------------------------------------------------------------*/

	inlength = strlen ( instring );
	if  ( ( startpos > inlength ) || ( startpos < 1 ) )
		return ( NULL );

	free ( temp_string );
	if  ( ( temp_string = (char *) malloc ( length + 1 ) ) == NULL )
	{
		printf ( "Unable to allocate memory for SUBSTR\n" );
		return ( NULL );
	}

/*--------------------------------------------------------------------*/
/*  initialize substring to pad character; if requested length goes   */\
/*  past the end of the string, then number of characters to copy     */
/*  ('stop') is number between starting position and the end of the   */
/*  string; otherwise, it's the actual number of characters requested */
/*--------------------------------------------------------------------*/

	strcpy ( temp_string, copies ( pad, length ) );
	if  ( length >  ( inlength - startpos + 1 ) )
		stop = inlength - startpos + 1;
	else
		stop = length;

/*--------------------------------------------------------------------*/
/*  now copy the correct number of characters from the input string   */
/*  into the output substring and return the pointer                  */
/*--------------------------------------------------------------------*/

	for  ( i = 0; i < stop; i++ )
		temp_string[i] = instring[i+startpos-1];

	return ( temp_string );
}
/**********************************************************************/
/*                                                                    */
/*  Function Name:  OVERLAY                                           */
/*  Purpose:  Overlay one character string of specified length,       */
/*            padded to that length with a supplied pad character if  */
/*            necessary, into a second string, beginning at a speci-  */
/*            fied character position.  Returns a pointer to the      */
/*            composite (overlaid) string.  Returns NULL if memory    */
/*            allocation fails.                                       */
/*  Written By:  Karl Wiegers                                         */
/*  Date Written:  March, 1989                                        */
/*  Usage Syntax: overlay ( string_to_overlay, string_to_be_overlaid, */
/*                          character_position_to_overlay,            */
/*                          length_of_overlaying_string, pad_char )   */
/*  External Functions Called:  strlen, malloc, printf, strcpy,       */
/*                              strcat, copies, left, right           */
/*                                                                    */
/**********************************************************************/

#include <stdio.h>
#include <strings.h>

extern char *copies ();
extern char *left ();
extern char *right ();
extern char *malloc ();

char *overlay (new, target, charno, length, pad )
char *new, *target, *pad;
short charno, length;

{
	static char *tstr;      /* points to overlaid string, is returned  */
	char *newnew;           /* holds 'new' string, padded if necessary */
	register short newlen;  /* length of string to be overlaid         */
	register short targlen; /* length of string being overla           */

	newlen = strlen ( new );
	targlen = strlen ( target );

/*--------------------------------------------------------------------*/
/*  attempt to allocate space to hold padded string being overlaid    */
/*  and final overlaid string that is being constructed               */
/*--------------------------------------------------------------------*/

	if ( (tstr = (char *) malloc ( newlen + targlen + 1 ) ) == NULL )
	{
		printf ( "Unable to allocate memory for OVERLAY\n" );
		return ( NULL );
	}

	if ( ( newnew = (char *) malloc ( strlen ( new ) + 1 ) ) == NULL )
	{
		printf ( "Unable to allocate memory for OVERLAY\n" );
		return ( NULL );
	}

/*--------------------------------------------------------------------*/
/*  first handle different values for argument 'length'; if 0, then   */
/*  set it to the length of the 'new' string, and copy 'new' into     */
/*  temporary buffer 'newnew'                                         */
/*--------------------------------------------------------------------*/

	if ( length == 0 )
	{
		length = newlen;
		strcpy ( newnew, new );
	}

/*--------------------------------------------------------------------*/
/*  if requested length is greater than length of 'new', copy 'new'   */
/*  to 'newnew' and pad it to requested length with pad character     */
/*--------------------------------------------------------------------*/

	else if ( length > newlen )
	{
		strcpy ( newnew, new );
		strcat ( newnew, copies ( pad,  ( length - newlen ) ) );
	}

/*--------------------------------------------------------------------*/
/*  if requeste length is shorter than length of 'new', just keep the */
/*  desired number of characters from the left side of 'new'          */
/*--------------------------------------------------------------------*/

	else if ( length < newlen )
		strcpy ( newnew, left ( new, length ) );

/*--------------------------------------------------------------------*/
/*  otherwise, 'length' matches the actual length of 'new', so just   */
/*  copy the 'new' string into 'newnew' work space                    */
/*--------------------------------------------------------------------*/

	else
		strcpy ( newnew, new );

/*--------------------------------------------------------------------*/
/* now build the output string 'tstr'; if the overlay character pos   */
/* is beyond the end of the string, copy the entire target into the   */
/* string 'tstr' and pad the end with 'pad' characters up to the      */
/* character position where the 'new' string will be appended         */
/*--------------------------------------------------------------------*/

	if ( charno > targlen )
	{
		strcpy ( tstr, target );
		strcat ( tstr, copies ( pad, charno - targlen - 1 ) );
	}

/*--------------------------------------------------------------------*/
/* if the 'new' string is being overlaid at char #1 of the target,    */
/* we won't be using the left part of 'target', so initialize 'tstr'  */
/* to nulls                                                           */
/*--------------------------------------------------------------------*/

	else if ( charno == 1 )
		strcpy ( tstr, "" );

/*--------------------------------------------------------------------*/
/*  otherise, copy into 'tstr' the leftmost characters from 'target'  */
/*  up to but not including the char where the overlay is to begin    */
/*--------------------------------------------------------------------*/

	else
		strcpy ( tstr, left ( target, charno - 1 ) );

/*--------------------------------------------------------------------*/
/* now concatenate 'newnew' onto 'tstr' and add on the right part of  */
/* the original 'target', if it hasn't been overwritten and if we     */
/* aren't appending 'newnew' onto the very end of 'target'            */
/*--------------------------------------------------------------------*/

	strcat ( tstr, newnew );
	if  ( targlen - charno - length > 0 )
		strcat ( tstr, right ( target, targlen - charno - length + 1 ) );

	free ( newnew );
	return ( tstr );
}
