/************************************************************************/
/*                                                                      */
/*  Program Name:  CHARTEST.C                                           */
/*  Purpose:  Exercises a series of functions that perform character    */
/*            string operations.                                        */
/*  Written By:  Karl Wiegers                                           */
/*  Date Written:  April 2, 1989                                        */
/*  External Functions Called:  COPIES, LEFT, RIGHT, STRREV, SUBSTR,    */
/*                              WORDS, WORD, OVERLAY                    */
/*                                                                      */
/************************************************************************/

#include <stdio.h>
#include <strings.h>

extern char *copies(), *left(), *right(), *strrev(), *substr(), *word();
extern char *overlay();
extern short words();
extern char *malloc();   /* don't forget this for Laser C */

main()
{
	char *string, *string2;
	short length;

/************************************************************************/
/*                                                                      */
/*  First test COPIES function, which returns a pointer to some number  */
/*  of copies of an input character string.                             */
/*  syntax:  COPIES ( *string_to_copy, number_of_copies )               */
/*                                                                      */
/************************************************************************/

	string = "Now is the time.";
	printf ( "Demonstration of COPIES function:\n\n" );
	printf ( "Twenty x's = %s\n", copies ( "x", 20 ) );
	printf ( "Three 'Now is the time.'s = %s\n", copies (string, 3 ) );

	printf ( "\nPress RETURN to continue.\n\n\n" );
	getchar ();

/***********************************************************************/
/*                                                                     */
/*  Now test STRREV function, which returns a pointer to a character   */
/*  string which consists of the characters from a supplied input      */
/*  string, but in reverse order                                       */
/*  Syntax:  STRREV ( *string_to_be_reversed)                          */
/*                                                                     */
/***********************************************************************/

	printf ( "Demonstration of STRREV function:\n\n" );
	printf ( "Original string = %s\n", string );
	printf ( "After reversal  = %s\n", strrev ( string ) );

	printf ( "\nPress RETURN to continue\n\n\n" );
	getchar ();
	
/***********************************************************************/
/*                                                                     */
/*  Now test LEFT function, which returns a pointer to the leftmost    */
/*  'n' characters of a supplied input string.                         */
/*  Syntax:  LEFT ( *input_string, number_of_characters)               */
/*                                                                     */
/***********************************************************************/

	string = "Now is the time for all good men.";
	printf ( "Demonstration of LEFT function:\n\n" );
	printf ( "Original string = %s\n", string );

	printf ( "leftmost character = %s\n", left ( string, 1 ) );
	printf ( "leftmost 10 characters = %s\n", left ( string, 10 ) );
	length = strlen ( string );
	printf ( "leftmost %d characters = %s\n", length, left ( string, length ) );
	printf ( "leftmost 50 characters = **%s**\n", left ( string, 50 ) );

	printf ( "\nPress RETURN to continue\n\n\n" );
	getchar ();

/***********************************************************************/
/*                                                                     */
/*  Now test RIGHT function, which returns a pointer to the rightmost  */
/*  'n' characters of a supplied input string.                         */
/*  Syntax:  RIGHT ( *input_string, number_of_characters )             */
/*                                                                     */
/***********************************************************************/

	printf ( "Demonstration of RIGHT function:\n\n" );

	printf ( "rightmost character = %s\n", right ( string, 1 ) );
	printf ( "rightmost 10 characters = %s\n", right ( string, 10 ) );
	printf ( "rightmost %d characters = %s\n", length, right ( string, length ));
	printf ( "rightmost 50 characters = %s\n", right ( string, 50 ) );

	printf ( "\nPress RETURN to continue\n\n\n" );
	getchar ();

/***********************************************************************/
/*                                                                     */
/*  Now test SUBSTR function, which returns a pointer to substring of  */
/*  characters of a supplied input string, beginning at a specific     */
/*  character position and extending for a specified number of         */
/*  characters.  A pad character is also supplied in case the          */
/*  substring requested extends beyond the end of the input string.    */
/*  Syntax:  SUBSTR ( *input_string, starting_position,                */
/*                    number_of_characters, *pad_character )           */
/*                                                                     */
/***********************************************************************/

	printf ( "Demonstration of SUBSTR function:\n\n" );

	printf ( "substr(12,4) = %s\n", substr ( string, 12, 4, " " ) );
	printf ( "substr(1,13) = %s\n", substr ( string, 1, 13, " " ) );
	printf ( "substr(1,33) = %s\n", substr ( string, 1, 32, " " ) );
	printf ( "substr(21,20), blank pad = **%s**\n",
		     substr ( string, 21, 20, " " ) );
	printf ( "substr(21,20), pad is 6 = %s\n", substr ( string, 21, 20, "6" ) );

	printf ( "\nPress RETURN to continue.\n\n\n" );
	getchar ();

/***********************************************************************/
/*                                                                     */
/*  Now test WORDS function, which returns the number of blank-        */
/*  delimited words in a supplied input string.                        */
/*  Syntax:  WORDS ( *input_string)                                    */
/*                                                                     */
/***********************************************************************/

	printf ( "Demonstration of WORDS function:\n\n" );

	printf ( "Should be 8:  %s = %d words\n", string, words ( string ) );
	strcpy ( string, "" );
	printf ( "Null string:  %s = %d words\n", string, words ( string ) );
	strcpy ( string, "     " );
	printf ( "Just some blanks:  %s = %d words\n", string, words ( string ) );
	strcpy ( string, "     Some leading blanks." );
	printf ( "Some leading blanks:  %s = %d words\n", string, words ( string ) );
	strcpy ( string, "Some trailing blanks.     " );
	printf ( "Some trailing blanks:  %s = %d words\n",
			 string, words ( string ) );
	strcpy ( string, "Nowis the time for allgood men." );
	printf ( "Should be 6:  %s = %d words\n", string, words ( string ) );
	strcpy ( string, "Nowisthetimeforallgoodmen." );
	printf ( "Should be 1:  %s = %d words\n", string, words ( string ) );

	printf ( "\nPress RETURN to continue\n\n\n" );
	getchar ();

/***********************************************************************/
/*                                                                     */
/*  Now test WORD function, which returns a pointer to a specific      */
/*  word in a blank-delimited supplied input string.                   */
/*  Syntax:  WORD ( *input_string, word_number_to_return)              */
/*                                                                     */
/***********************************************************************/

	printf ( "Demonstration of WORD function:\n\n" );
	string = "Now is the time for all good men.";

	printf ( "Word 1 = %s\n", word ( string, 1 ) );
	printf ( "Word 3 = %s\n", word ( string, 3 ) );
	printf ( "Word 8 = %s\n", word ( string, 8 ) );

	printf ( "\nPress RETURN to continue\n\n" );	
	getchar ();

/***********************************************************************/
/*                                                                     */
/*  Now test OVERLAY function, which returns a pointer to a string in  */
/*  which one string has been overlaid upon another, beginning in a    */
/*  particular character position and extending for a specified length */ 
/*  Syntax:  OVERLAY ( *string_to_overlay, *string_to_be_overlaid_on,  */
/*                     overlay_character_position, overlay_length,     */
/*                     *pad_character )                                */
/*                                                                     */
/***********************************************************************/

	string = "Now is the time for all good men.";
	string2 = (char *) malloc ( strlen ( string ) + 20 );
	printf ( "Original = %s\n", string );
	strcpy ( string2, overlay("week", string, 12, 0, " " ) );
	printf ( "Example 1 = %s\n", string2 );
	strcpy ( string2, overlay("weekly", string, 25, 4, " " ) );
	printf ( "Example 2 = %s\n", string2 );
	strcpy ( string2, overlay("week", string, 12, 9, "*" ) );
	printf ( "Example 2 = %s\n", string2 );
	
	printf ( "\n\nPress RETURN to exit\n" ) ;
	getchar ();
}
