/**************************************************************
*                 The Programmer's Calculator                 *
*                         Written by                          *
*                    Robert M. Birmingham                     *
*                      January 11, 1989                       *
**************************************************************/

#include <obdefs.h>
#include <define.h>
#include <osbind.h>
#include <gemdefs.h>
#include <string.h>

/* define the array positions for each key... */
#define  DECIMAL    0
#define  HEX        1
#define  BINARY     2
#define  OCTAL      3
#define  CLEARALL   4
#define  LSHIFT     5
#define  DKEY       6
#define  EKEY       7
#define  FKEY       8
#define  CLEARENT   9
#define  RSHIFT    10
#define  AKEY      11
#define  BKEY      12
#define  CKEY      13
#define  MULTIPLY  14
#define  AND       15
#define  NUMBER7   16
#define  NUMBER8   17
#define  NUMBER9   18
#define  DIVIDE    19
#define  OR        20
#define  NUMBER4   21
#define  NUMBER5   22
#define  NUMBER6   23
#define  PLUS      24
#define  XOR       25
#define  NUMBER1   26
#define  NUMBER2   27
#define  NUMBER3   28
#define  MINUS     29
#define  NOT       30
#define  NUMBER0   31
#define  DECPOINT  32
#define  SIGN      33
#define  EQUALS    34

/* define the three types of error values... */
#define  NOERROR     0
#define  OVERFLOW    1
#define  ZERODIVIDE  2

/* define mixinum and maximum for window redraw code... */
#define maximum(x,y)  (((x) > (y)) ? (x) : (y))
#define minimum(x,y)  (((x) < (y)) ? (x) : (y))

/* define the type of window to use... */
#define WI_KIND (CLOSER | MOVER | NAME | FULLER)

/* global application ID */
extern int gl_apid;

/* declare arrays for GEM bindings... */
int contrl[20];
int intin[128];
int ptsin[128];
int intout[128];
int ptsout[128];
int work_in[11];
int work_out[57];

int handle;            /* workstation handle            */
int wi_handle;         /* calculator window handle      */
int menu_id;           /* id number for the menu entry  */
int gr_hwchar;         /* width of a character cell     */
int gr_hhchar;         /* height of a character cell    */
int ax, ay, aw, ah;    /* position of accessory window  */
int xwork, ywork;      /* position of window work area  */
int wwork, hwork;      /* width and height of work area */
int boxw, boxh;        /* width and height of key box   */
int xgap, ygap;        /* X and Y distance between keys */ 
int disp_rect[4];      /* calculator display rectangle  */
int numbase;           /* currently active number base  */
int active;            /* currently active display      */
int error;             /* floating point error flag!    */


/* declare a structure for display strings #1 and #2 */
struct dispinfo
       {
       char string[34];    /* display string */
       int negative;       /* +/- flag       */
       int command;        /* math operation */
       } display[2];

/* declare an array of structures to hold the key information... */
struct keydefs
       {
       int x;
       int y;
       char *string;
       } key[] = {
       0,0,"DEC",  0,0,"HEX", 0,0,"BIN", 0,0,"OCT", 0,0,"CA",
       0,0,"<<",   0,0,"D",   0,0,"E",   0,0,"F",   0,0,"CE",
       0,0,">>",   0,0,"A",   0,0,"B",   0,0,"C",   0,0,"X",
       0,0,"AND",  0,0,"7",   0,0,"8",   0,0,"9",   0,0,"",
       0,0,"OR",   0,0,"4",   0,0,"5",   0,0,"6",   0,0,"+",
       0,0,"XOR",  0,0,"1",   0,0,"2",   0,0,"3",   0,0,"-",
       0,0,"NOT",  0,0,"0",   0,0,".",   0,0,"+/-", 0,0,"="
       };

/*
** Declare the enabled/disabled key bit-maps, and the maximum number of
** characters allowed to be entered, for each number base.  The number
** base keys (D,H,B & O) are always enabled so they're not represented.
** ( '=' is bit #31 <-- the other keys- -> CA is Bit #0 )
*/

struct keydata
       {
       long int flags;
       int max_chars;
       } keyinfo[] = {
       0x7FFFFC63L, 10,
       0x6FFFFFFFL, 8,
       0x6E718C63L, 32,
       0x6FFF9C63L, 12
       };


main()
{
    initialize();    /* do general initialization */
    do_event();      /* handle the message events */

}   /* End of main() routine */


/**********************************************
* press_key():                                *
* This routine processes the key index passed *
**********************************************/

press_key( mousex, mousey )
int mousex, mousey;
{
    int i;        /* miscellaneous variable */
    int index;    /* which key was pressed  */
    int button;   /* mouse button state     */


    /* find out which key is under the mouse */
    if( (index = scan_key( mousex, mousey )) < 0 ) return(0);

    /* if there's an error, only allow the CLEARALL key */
    if( error != NOERROR && index != CLEARALL ) return(0);

    /* do nothing if the current number base is already selected */
    if( index == numbase ) return(0);

    /* if key is disabled then return */
    if( index >= CLEARALL )
        {
        if(((keyinfo[numbase].flags >> (index-CLEARALL)) & 0x1L) == 0L)
           return(0);
        }

    /* force clipping rectangle to dimensions of screen */
    set_clip( 0, 0, work_out[0], work_out[1], 1 );

    /* highlight the selected key */
    if( index > OCTAL ) draw_key( index, 1 );

    /* find out which key was pressed */
    switch( index )
            {
            case DECIMAL:
            case HEX:
            case BINARY:
            case OCTAL:
                 change_numbase( numbase, index );
                 break;

            case LSHIFT:
            case RSHIFT:
                 shift_operator( index );
                 break;

            case AND:
            case OR:
            case XOR:
            case MULTIPLY:
            case DIVIDE:
            case PLUS:
            case MINUS:
            case EQUALS:
                 arithmetic( index );
                 break;

            case DECPOINT:
            case NUMBER0:
            case NUMBER1:
            case NUMBER2:
            case NUMBER3:
            case NUMBER4:
            case NUMBER5:
            case NUMBER6:
            case NUMBER7:
            case NUMBER8:
            case NUMBER9:
            case AKEY:
            case BKEY:
            case CKEY:
            case DKEY:
            case EKEY:
            case FKEY:
                 add_character( index );
                 break;

            case SIGN:      sign_change();   break;
            case NOT:       not_operator();  break;
            case CLEARALL:  ca_key();        break;
            case CLEARENT:  ce_key();        break;
            }

    /* wait for the mouse button to be released */
    do{ graf_mkstate( &i, &i, &button, &i ); } while( button );

    /* reset the key unless it's a number base key */
    if( index != numbase ) draw_key( index, 0 );

    /* turn the clipping rectangle off */
    set_clip( 0, 0, work_out[0], work_out[1], 0 );

}   /* End of - press_key() */


/*****************************************************
* change_numbase():                                  *
* This routine enables/disables the appropriate keys *
* keys depending on the new number base selected.    *
*****************************************************/

change_numbase( oldbase, newbase )
int oldbase, newbase;
{
    int i;             /* loop variable    */
    long int lnum0;    /* display #1 value */
    long int lnum1;    /* display #2 value */

    /* the 'display string to long' routines return long integers */
    long int dec_to_long();
    long int hbo_to_long();


    v_hide_c( handle );

    /* convert display strings to longs depending on the current base */
    if( oldbase == DECIMAL )
        {
        lnum0 = dec_to_long( 0 );
        lnum1 = dec_to_long( 1 );
        }
    else
        {
        lnum0 = hbo_to_long( 0, oldbase );
        lnum1 = hbo_to_long( 1, oldbase );
        }

    /* convert long to display string depending on the new base selected */
    long_to_string( 0, newbase, lnum0 );
    long_to_string( 1, newbase, lnum1 );

    /* set to the new number base */
    numbase = newbase;

    /* redraw each key only if it's state changes between the bases */
    for( i = CLEARALL; i <= EQUALS; i++ )
         {
         if( ((keyinfo[oldbase].flags >> (i - CLEARALL)) & 0x1L) !=
             ((keyinfo[newbase].flags >> (i - CLEARALL)) & 0x1L) )
             draw_key( i, 0 );
         }

    /* clear the old base key, and highlight the new base key */
    draw_key( oldbase, 0 );
    draw_key( newbase, 1 );

    /* show string #1 if string #2 is active and empty */
    if( active == 1 && strcmp( display[1].string, "0" ) == 0 )
        show_display( 0 );
    else
        show_display( active );

    v_show_c( handle, 0 );

}   /* End of - change_numbase() */


/********************************************************************
* arithmetic():                                                     *
* This routine is called to divert the processing of the arithmetic *
* for floating point and binary numbers to the appropriate function *
********************************************************************/

arithmetic( key )
int key;
{
    if( numbase == DECIMAL )
        do_float( key );
    else
        do_binary( key );

}   /* End of - arithmetic() */


/******************************************************
* do_float():                                         *
* This routine performs the floating point arithmetic *
* for the numbers entered into the calculator         *
******************************************************/

do_float( index )
int index;
{
    int i;               /* general purpose variable  */
    int prec0, prec1;    /* precision variables       */
    double num0, num1;   /* floats for math functions */

    long int dec_to_long();
    double atof(), fabs();


    /* remove the last character if it's a '.' */
    i = strlen( display[ active ].string ) - 1;
    if( display[ active ].string[ i ] == '.' )
        display[ active ].string[ i ] == '\0';

    /* store the command key pressed */
    display[ active ].command = index;

    /* if this is the first command return for the second string */
    if( active == 0 ){ active = 1;    return(0); }

    /* reset the error flag for a new operation */
    error = NOERROR;

    /* convert both display strings to floating point */
    num0 = atof( display[ 0 ].string );
    num1 = atof( display[ 1 ].string );
    if( display[ 0 ].negative != 0 ) num0 = -num0;
    if( display[ 1 ].negative != 0 ) num1 = -num1;

    /* show an error if dividing by zero! */
    if( num1 == 0 && display[0].command == DIVIDE )
        {
        error = ZERODIVIDE;
        active = 0;
        strcpy( display[0].string, "Division by zero is illegal!" );
        show_display(0);
        return(0);
        }

    /* calculate the number of digits to the right of the decimal point */
    prec0 = prec1 = 0;

    i = findch( '.', display[0].string );
    if( i >= 0 ) prec0 = strlen( display[0].string ) - i - 1;
    i = findch( '.', display[1].string );
    if( i >= 0 ) prec1 = strlen( display[1].string ) - 1 - 1;

    /* do the math operation! */
    switch( display[0].command )
            {
            case MULTIPLY:
                 num0 = num0 * num1;
                 prec0 = prec0 + prec1;
                 break;

            case DIVIDE:
                 num0 = num0 / num1;
                 prec0 = 5;
                 break;

            case PLUS:
                 num0 = num0 + num1;
                 if( prec1 > prec0 ) prec0 = prec1;
                 break;

            case MINUS:
                 num0 = num0 - num1;
                 if( prec1 > prec0 ) prec0 = prec1;
                 break;

            case AND:
            case OR:
            case XOR:
                 /* handle as an integer operation! */
                 do_binary( index );

                 /* set the sign flag for the display string */
                 if( dec_to_long(0) < 0L )
                     display[0].negative = 1;
                 else
                     display[0].negative = 0;

                 /* all done, get out of here! */
                 return(0);
                 break;
            }

    /* set the error flag if the number is out of range */
    if( fabs( num0 ) > 9999999999.0 )
        {
        error = OVERFLOW;
        active = 0;
        strcpy( display[0].string, "Overflow!" );
        show_display(0);
        return(0);
        }

    /* check if the precision variable is out of range */
    if( prec0 > 10 ) prec0 = 10;

    /* convert the number to ASCII */
    sprintf( display[0].string, "%.10f", fabs( num0 ) );

    /* there will always be a '.', just find its position */
    i = findch( '.', display[0].string );

    /* set the precision of the number */
    if( prec0 > 0 )
        display[0].string[ i + prec0 + 1 ] = '\0';
    else
        display[0].string[ i ] = '\0';

    /* set the sign flag for the display string */
    if( num0 < 0 )
        display[0].negative = 1;
    else
        display[0].negative = 0;

    /* clear display string #2 */
    strcpy( display[1].string, "0" );
    display[1].negative = 0;

    /* copy the pending arithmetic command */
    display[0].command = display[1].command;

    /* display the number */
    show_display( 0 );

}   /* End of - do_float() */


/*****************************************************
* fabs():                                            *
* Quick routine to get the absolute value of a float *
*****************************************************/

double fabs( num )
double num;
{
    if( num < 0.0 ) num = -num;
    return( num );
}


/***********************************************************************
* do_binary():                                                         *
* This routine performs the integer arithmetic for the numbers entered *
* into the calculator.  Used for HEX, BINARY, OCTAL, sometimes DECIMAL *
***********************************************************************/

do_binary( index )
int index;
{
    int i;                /* miscellaneous variable */
    long int num0, num1;  /* display to long conversion variables */

    /* 'display string to long' conversions return long integers */
    long int dec_to_long();
    long int hbo_to_long();


    /* store the command key pressed */
    display[ active ].command = index;

    /* if this is the first command return for the second string */
    if( active == 0 ){ active = 1;    return(0); }

    /* reset the math error flag for new operation */
    error = NOERROR;

    /* truncate the strings at the '.' */
    i = findch( '.', display[0].string );
    if( i >= 0 ) display[0].string[i] = '\0';

    i = findch( '.', display[1].string );
    if( i >= 0 ) display[1].string[i] = '\0';

    /* convert display strings to long integers depending on the base */
    if( numbase == DECIMAL )
        {
        num0 = dec_to_long( 0 );
        num1 = dec_to_long( 1 );
        }
    else
        {
        num0 = hbo_to_long( 0, numbase );
        num1 = hbo_to_long( 1, numbase );
        }

    /* check for a divide by zero error */
    if( num1 == 0L && display[0].command == DIVIDE )
        {
        error = ZERODIVIDE;
        active = 0;
        strcpy( display[0].string, "Division by zero is illegal!" );
        show_display( 0 );
        return(0);
        }

    /* do the arithmetic operation the user selected */
    switch( display[0].command )
            {
            case MULTIPLY:  num0 = num0 * num1;   break;
            case DIVIDE:    num0 = num0 / num1;   break;
            case PLUS:      num0 = num0 + num1;   break;
            case MINUS:     num0 = num0 - num1;   break;
            case AND:       num0 = num0 & num1;   break;
            case OR:        num0 = num0 | num1;   break;
            case XOR:       num0 = num0 ^ num1;   break;
            }

    /* convert number back to original base */
    long_to_string( 0, numbase, num0 );

    /* clear display string #2 */
    strcpy( display[1].string, "0" );
    display[1].negative = 0;

    /* copy the pending command */
    display[0].command = display[1].command;

    /* display the number */
    show_display( 0 );

}   /* End of - do_binary() */


/***************************************************
* ca_key():                                        *
* This resets the calculator to it's initial state *
***************************************************/

ca_key()
{
    active = 0;
    error = NOERROR;

    display[0].command = display[0].negative = 0;
    strcpy( display[0].string, "0" );

    display[1].command = display[1].negative = 0;
    strcpy( display[1].string, "0" );

    show_display( active );

}   /* End of - ca_key() */


/**************************************************
* ce_key():                                       *
* This routine negates the last operation entered *
**************************************************/

ce_key()
{
    /* CA is the only way to clear an error! */
    if( error ) return(0);

    /* if first display string is active then just do a CA */
    if( active == 0 )
        ca_key();
    else
        {
        /* if the display string is empty, clear the last command */
        if( strcmp( display[1].string, "0" ) == 0 )
            {
            display[1].command = 0;
            return(0);
            }

        /* otherwise, clear the string in the display */
        strcpy( display[1].string, "0" );
        display[1].negative = 0;
        show_display( 1 );
        }

}   /* End of - ce_key() */


/******************************************************************
* sign_change():                                                  *
* This routine changes the sign of the currently displayed number *
******************************************************************/

sign_change()
{
    int dispnum;       /* temporary display # variable */
    long int lnum;     /* number to change the sign of */

    long int hbo_to_long();


    /* use display #1, if display #2 is active and empty */
    if( active == 1 && strcmp( display[1].string, "0" ) == 0 )
        dispnum = 0;
    else
        dispnum = active;

    /* change the sign for decimal, and other bases separately */
    if( numbase == DECIMAL )
        display[dispnum].negative = !( display[dispnum].negative );
    else
        {
        lnum = hbo_to_long( dispnum, numbase );
        long_to_string( dispnum, numbase, -lnum );
        }

    show_display( dispnum );

}   /* End of - change_sign() */


/*******************************************************************
* not_operator():                                                  *
* This function performs a one's compliment on the current display *
*******************************************************************/

not_operator()
{
    int i;           /* miscellaneous variable  */
    int dispnum;     /* which display to NOT    */
    long int lnum;   /* value of display string */

    long int dec_to_long();
    long int hbo_to_long();


    /* use display #1, if display #2 is active and empty */
    if( active == 1 && strcmp( display[1].string, "0" ) == 0 )
        dispnum = 0;
    else
        dispnum = active;

    /* NOT a number in decimal, and the other modes separately */
    if( numbase == DECIMAL )
        {
        i = findch( '.', display[dispnum].string );
        if( i >= 0 ) display[dispnum].string[i] = '\0';

        lnum = dec_to_long( dispnum );
        }
    else
        lnum = hbo_to_long( dispnum, numbase );

    /* convert NOT'ed number to a string, then show it */
    long_to_string( dispnum, numbase, ~lnum );
    show_display( dispnum );

}   /* End of not_operator() */


/*************************************************************
* shift_operator():                                          *
* This function shifts the number 1 bit to the right or left *
*************************************************************/

shift_operator( index )
int index;
{
    int i;          /* miscellaneous variable  */
    int dispnum;    /* which display to shift  */
    long int lnum;  /* value of display string */

    long int dec_to_long();
    long int hbo_to_long();


    /* use display #1 if display #2 is active and empty */
    if( active == 1 && strcmp( display[1].string, "0" ) == 0 )
        dispnum = 0;
    else
        dispnum = active;

    /* convert number from current base to a long integer */
    if( numbase == DECIMAL )
        {
        i = findch( '.', display[dispnum].string );
        if( i >= 0 ) display[dispnum].string[i] = '\0';

        lnum = dec_to_long( dispnum );
        }
    else
        lnum = hbo_to_long( dispnum, numbase );

    /* shift the number in the selected direction */
    if( index == LSHIFT )
        lnum = lnum << 1;
    else
        lnum = lnum >> 1;

    /* convert number back to a string, then display it */
    long_to_string( dispnum, numbase, lnum );
    show_display( dispnum );

}   /* End of shift_operator() */


/**********************************************
* add_character():                            *
* This routine adds the key string pointed to *
* by 'index' to the current display string.   *
**********************************************/

add_character( index )
int index;
{
    /* don't allow two '.' characters in the same string */
    if( index == DECPOINT )
        {
        if( findch( '.', display[active].string ) >= 0 ) return(0);
        }

    /* return if the string is the maximum length for the current base */
    if( strlen( display[active].string ) >= keyinfo[numbase].max_chars )
        return(0);

    /* handle the first character and subsequent characters separately */
    if( strcmp( display[active].string, "0" ) == 0 )
        strcpy( display[active].string, key[index].string );
    else
        strcat( display[active].string, key[index].string );

    /* reset whole display if this is the first character */
    if( strlen( display[active].string ) == 1 )
        show_display( active );
    else
        print_display( active );

}   /* End of - add_character */


/**************************************************
* scan_key():                                     *
* Returns the key under the mouse position passed *
**************************************************/

scan_key( x, y )
int x, y;
{
    int i;


    for( i = DECIMAL; i <= EQUALS; i++ )
         {
         if( x >= xwork + key[i].x &&
             y >= ywork + key[i].y &&
             x <= xwork + key[i].x + boxw &&
             y <= ywork + key[i].y + boxh )
             return(i);
         }

    return(-1);

}   /* End of - scan_key() */


/*********************************************
* draw_calc():                               *
* This routine redraws the calculator window *
*********************************************/

draw_calc( w )
int w[];
{
    int i;             /* general purpose variable  */
    int pxyarray[4];   /* used to clear the window  */
    int clip[4];       /* clipping rectangle to set */


    /* set the clipping rectangle */
    clip[0] = w[0];
    clip[1] = w[1];
    clip[2] = w[0] + w[2] - 1;
    clip[3] = w[1] + w[3] - 1;
    vs_clip( handle, 1, clip );

    /* clear the work area */
    pxyarray[0] = xwork;
    pxyarray[1] = ywork;
    pxyarray[2] = xwork + wwork - 1;
    pxyarray[3] = ywork + hwork - 1;

    v_hide_c( handle );
    vsf_color( handle, 0 );
    vsf_interior( handle, 2 );
    vsf_style( handle, 8 );
    vr_recfl( handle, pxyarray );

    /* draw the numeric display */
    show_display( active );

    /* redraw each key (highlight the active number base key) */
    for( i = DECIMAL; i <= EQUALS; i++ )
         {
         if( i == numbase )
             draw_key( i, 1 );
         else
             draw_key( i, 0 );
         }

    /* reset the clipping rectangle */
    vs_clip( handle, 0, clip );
    v_show_c( handle, 0 );

}   /* End of - draw_calc() */


/******************************************************************
* draw_key():                                                     *
* This routine redraws the key whose array index number is passed *
******************************************************************/

draw_key( idx, flag )
int idx, flag;
{
    int pxyarray[4];   /* box for drawing key  */
    int shadow[4];     /* bow for key shadow   */
    int x, y;          /* position of key text */
    long int lret;     /* temporary key bitmap */


    /* use the flags to see if the key is enabled/disabled */
    vst_effects( handle, 0 );

    if( idx >= CLEARALL )
        {
        lret = (keyinfo[ numbase ].flags >> ( idx - CLEARALL )) & 0x1L;
        if( lret == 0L ) vst_effects( handle, 2 );
        }

    /* get the key box coordinates */
    pxyarray[0] = xwork + key[ idx ].x;
    pxyarray[1] = ywork + key[ idx ].y;
    pxyarray[2] = pxyarray[0] + boxw;
    pxyarray[3] = pxyarray[1] + boxh;

    /* set the shadow box coodinates */
    shadow[0] = pxyarray[0] + 2;
    shadow[1] = pxyarray[1] + 2;
    shadow[2] = pxyarray[2] + 2;
    shadow[3] = pxyarray[3] + 2;

    v_hide_c( handle );
    vswr_mode( handle, 1 );
    vsf_perimeter( handle, 1 );
    vsf_color( handle, 1 );

    vsf_interior( handle, 1 );
    v_bar( handle, shadow );

    vsf_interior( handle, flag );
    v_bar( handle, pxyarray );

    /* print the string in the box */
    x = pxyarray[0] + (boxw/2) - ((strlen(key[idx].string)*gr_hwchar)/2);
    y = pxyarray[3] - 2;

    vswr_mode( handle, 3 );
    v_gtext( handle, x, y, key[idx].string );
    vswr_mode( handle, 1 );
    vst_effects( handle, 0 );
    v_show_c( handle, 1 );

}  /* End of - draw_key() */


/******************************************************
* show_display():                                     *
* This routine draws the calculator's numeric display *
******************************************************/

show_display( active )
int active;
{
    int x, y;           /* used to display string */
    int pxyarray[4];    /* used to draw the display box */


    /* draw the outline of the box */
    pxyarray[0] = xwork + disp_rect[0];
    pxyarray[1] = ywork + disp_rect[1];
    pxyarray[2] = pxyarray[0] + disp_rect[2];
    pxyarray[3] = pxyarray[1] + disp_rect[3];

    v_hide_c( handle );
    vswr_mode( handle, 1 );
    vsf_perimeter( handle, 1 );
    vsf_color( handle, 1 );
    vsf_interior( handle, 0 );
    v_bar( handle, pxyarray );

    vswr_mode( handle, 1 );
    print_display( active );
    v_show_c( handle, 1 );

}   /* End of - show_display() */


/***************************************************
* print_display():                                 *
* This function prints the current display string  *
* without clearing the area as show_display() does *
***************************************************/

print_display( active )
int active;
{
    int temp;           /* temporary display number string    */
    int x, y;           /* coordinates for printing string    */
    int pxyarray[4];    /* array for the display box position */


    /* draw the outline of the box */
    pxyarray[0] = xwork + disp_rect[0];
    pxyarray[1] = ywork + disp_rect[1];
    pxyarray[2] = pxyarray[0] + disp_rect[2];
    pxyarray[3] = pxyarray[1] + disp_rect[3];

    v_hide_c( handle );
    vswr_mode( handle, 1 );

    /* set flag to print display #1 if display #2 is active yet empty */
    if( active == 1 && strcmp( display[active].string, "0" ) == 0 )
        temp = 0;
    else
        temp = active;

    /* if the number base is decimal then print the number's sign */
    if( numbase == DECIMAL )
        {
        x = pxyarray[0] + 2;
        y = pxyarray[3] - 4;

        if( display[ temp ].negative == 0 )
            v_gtext( handle, x, y, "+" );
        else
            v_gtext( handle, x, y, "-" );
        }

    /* print the current display string */
    x = pxyarray[2] - (strlen( display[temp].string) * gr_hwchar)-2;
    y = pxyarray[3] - 4;
    v_gtext( handle, x, y, display[ temp ].string );
    v_show_c( handle, 1 );

}   /* End of - print_display() */


/********************************************************
* do_event():                                           *
* This routine waits for an event, then calls a routine *
* to handle the message event that was returned.        *
********************************************************/

do_event()
{
    int i;                /* miscellaneous                  */
    int mousex, mousey;   /* mouse position variables       */
    int event;            /* event returned by evnt_multi() */
    int msgbuff[8];       /* buffer for the event messages  */


    graf_mouse( ARROW, 0x0L );

    /* good accessories never die! */
    while( TRUE )
           {
           /* wait for a message or button event */
           event = evnt_multi(MU_BUTTON|MU_MESAG,
                              1,1,1,0,0,0,0,0,
                              0,0,0,0,0,&msgbuff,0,0,
                              &mousex,&mousey,
                              &i,&i,&i,&i);

           /* process the event returned */
           if( event & MU_MESAG ) do_message( &msgbuff );
           if( event & MU_BUTTON ) press_key( mousex, mousey );
           }

}   /* End of - do_event() */


/*************************************************************
* do_message():                                              *
* This routine calls the appropriate window event handler    *
* with the information contained in the event message buffer *
*************************************************************/

do_message( msgbuff )
int msgbuff[];
{
    switch( msgbuff[0] )
            {
            case AC_OPEN:     acc_open( msgbuff );      break;
            case AC_CLOSE:    acc_close( msgbuff );     break;
            case WM_MOVED:    move_window(msgbuff);     break;
            case WM_CLOSED:   close_window( msgbuff );  break;
            case WM_NEWTOP:   top_window( msgbuff );    break;
            case WM_TOPPED:   top_window( msgbuff );    break;
            case WM_FULLED:   show_author( msgbuff );   break;
            case WM_REDRAW:   redraw( msgbuff );        break;
            }

}   /* End of - do_message() */


/*************************************
* acc_open():                        *
* This routine opens the accessory   *
* and ask DE for the necessary info. *
*************************************/

acc_open( msgbuff )
int msgbuff[];
{
    /* return if the open command is not for us! */
    if( msgbuff[4] != menu_id ) return(0);

    /* if the window exists, we're already open, just top it and leave! */
    if( wi_handle != -1 )
        {
        wind_set( wi_handle, WF_TOP, 0, 0, 0, 0 );
        return(0);
        }

    /* open the accessory */
    open_vwork();
    open_window();

}   /* End of - acc_open() */


/**************************************
* acc_close():                        *
* This routine closes the workstation *
**************************************/

acc_close( msgbuff )
int msgbuff[];
{
    if( msgbuff[3] == menu_id && wi_handle != -1 )
        {
        v_clsvwk( handle );
        wi_handle = -1;
        }
}


/***********************************************************************
* move_window():                                                       *
* This routine moves the window to the position specified in msgbuff[] *
***********************************************************************/

move_window( msgbuff )
int msgbuff[];
{
    /* return if move message is not for my window */
    if( msgbuff[3] != wi_handle ) return(0);

    /* update the accessory position variables */
    ax = msgbuff[4];
    ay = msgbuff[5];
    aw = msgbuff[6];
    ah = msgbuff[7];

    /* set the window position */
    wind_set( msgbuff[3], WF_CURRXYWH, ax, ay, aw, ah );

    /* call routine to store work area position in global variables */
    get_workarea();
}


/************************************************************
* close_window():                                           *
* This routine closes the window and closes the workstation *
************************************************************/

close_window( msgbuff )
int msgbuff[];
{
    /* return if the close message is not for this window */
    if( msgbuff[3] != wi_handle ) return(0);

    /* otherwise, close it! */
    wind_close( wi_handle );
    wind_delete( wi_handle );
    v_clsvwk( handle );
    graf_shrinkbox(ax+aw/2,ay+ah/2,3*gr_hwchar,3*gr_hhchar,ax,ay,aw,ah);
    wi_handle = -1;
}


/****************************************************
* top_window():                                     *
* This routine sets the accessory window to the top *
****************************************************/

top_window( msgbuff )
int msgbuff[];
{
    /* return if the top command is not for my window */
    if( msgbuff[3] != wi_handle ) return(0);

    wind_set( wi_handle, WF_TOP, 0, 0, 0, 0 );
}


/*********************************************************
* show_author():                                         *
* This routine resorts to some shameless self-promotion! *
*********************************************************/

show_author( msgbuff )
int msgbuff[];
{
static char author[] = "[1][\
   Programmer's Calculator|\
          Written by|\
     Robert M. Birmingham|\
       January 11, 1989|\
(c) 1989, All Rights Reserved ][ Ok ]";

    /* return if the top command is not for my window */
    if( msgbuff[3] != wi_handle ) return(0);

    form_alert( 1, author );
}


/*************************************************
* redraw():                                      *
* This routine redraws sections of the window by *
* stepping through the window rectangle list.    *
*************************************************/

redraw( msgbuff )
int msgbuff[];
{
    int w[4];    /* holds rectangle coord's from window list */


    /* return if the redraw message isn't for this window */
    if( msgbuff[3] != wi_handle ) return(0);

    wind_update(1);

    /* get the first rectangle from the window list */
    wind_get( msgbuff[3], WF_FIRSTXYWH, &w[0], &w[1], &w[2], &w[3] );

    while( w[2] && w[3] )
           {
           if( b_intersect( &msgbuff[4], &w ) )  draw_calc( &w );
           wind_get( msgbuff[3], WF_NEXTXYWH, &w[0], &w[1], &w[2], &w[3] );
           }

    wind_update(0);

}   /* End of - redraw() */


/******************************************************
* get_workarea():                                     *
* This routine gets the window's work area dimensions *
******************************************************/

get_workarea()
{
    wind_get( wi_handle, WF_WORKXYWH, &xwork, &ywork, &wwork, &hwork );
}


/*********************************************************
* rectangle intersection routines for the window redraws *
*********************************************************/

int b_intersect( bin, bout )
int bin[], bout[];
{
return r_intersect(bin[0],bin[1],bin[2],bin[3],
                   &bout[0],&bout[1],&bout[2],&bout[3] );
}


r_intersect( bx, by, bw, bh, ox, oy, ow, oh )
int bx, by, bw, bh, *ox, *oy, *ow, *oh;
{
    int x, y, w, h;


    w = minimum( bx + bw, *ox + *ow );
    h = minimum( by + bh, *oy + *oh );
    x = maximum( *ox, bx );
    y = maximum( *oy, by );

    *ox = x;
    *oy = y;
    *ow = w - x;
    *oh = h - y;

    return w > x && h > y;
}


/***********************************************************
* long_to_string():                                        *
* This routine calls the appropriate function to convert a *
* long integer into a string for the display number passed *
***********************************************************/

long_to_string( display, base, lnum )
int display;
int base;
long int lnum;
{
   if( base == DECIMAL ) ltod( display, lnum );
   if( base == HEX )     ltoh( display, lnum );
   if( base == BINARY )  ltob( display, lnum );
   if( base == OCTAL )   ltoo( display, lnum );
}


/********************************************************************
* ltod():                                                           *
* This function converts a long integer to a decimal display string *
********************************************************************/

ltod( dispnum, lnum )
int dispnum;
long int lnum;
{
    /* assume a positive number */
    display[ dispnum ].negative = 0;

    /* if less than zero, strip sign and set negative flag */
    if( lnum < 0L )
        {
        lnum = -lnum;
        display[ dispnum ].negative = 1;
        }

    /* sprintf chokes on 2147483648, so... */
    if( lnum == 2147483648L )
        {
        strcpy( display[ dispnum ].string, "2147483648" );
        display[ dispnum ].negative = 1;
        return(0);
        }

    /* convert the number to the display string */
    sprintf( display[ dispnum ].string, "%ld", lnum );

}   /* End of - ltod() */


/****************************************************************
* ltoh():                                                       *
* This function converts a long integer to a hex display string *
****************************************************************/

ltoh( dispnum, lnum )
int dispnum;
long int lnum;
{
    char conversion[17];  /* characters for conversion   */
    char string[9];       /* temporary conversion string */
    int index;            /* index for conversion string */


    /* initialize hex conversion strings, etc. */
    strcpy( conversion, "0123456789ABCDEF" );
    strcpy( string, "00000000" );
    index = strlen( string ) - 1;

    /* convert the number to the hex string */
    do{ string[ index-- ] = conversion[ lnum & 15L ];
        lnum = lnum >> 4;
      } while( lnum != 0 && index >= 0 );

    /* copy the conversion string to the display string */
    strcpy( display[ dispnum ].string, &string[ index ] + 1L );

}   /* End of - ltoh() */


/*******************************************************************
* ltob():                                                          *
* This function converts a long integer to a binary display string *
*******************************************************************/

ltob( dispnum, lnum )
int dispnum;
long int lnum;
{
    char string[33];   /* temporary conversion string */
    int index;         /* index for conversion string */


    /* initialize binary conversion string */
    strcpy( string, "00000000000000000000000000000000" );
    index = strlen( string ) - 1;

    /* convert the number to the binary string */
    do{ string[ index-- ] = (lnum & 1L) + '0';
        lnum = lnum >> 1;
      } while( lnum != 0 && index >= 0 );

    /* copy the conversion string to the display string */
    strcpy( display[ dispnum ].string, &string[ index ] + 1L );

}   /* End of - ltob() */


/****************************************************************
* ltoh():                                                       *
* This function converts a long integer to a hex display string *
****************************************************************/

ltoo( dispnum, lnum )
int dispnum;
long int lnum;
{
    char string[13];  /* temporary conversion string */
    int index;        /* index for conversion string */


    /* initialize the octal conversion string */
    strcpy( string, "000000000000" );
    index = strlen( string ) - 1;

    /* convert the number to an octal string */
    do{ string[ index-- ] = ( lnum & 7L ) + '0';
        lnum = lnum >> 3;
      } while( lnum != 0 && index >= 0 );

    /* copy the conversion string to the display string */
    strcpy( display[ dispnum ].string, &string[ index ] + 1L );

}   /* End of - ltoo() */


/*******************************************************************
* dec_to_long():                                                   *
* This routine converts a decimal display string to a long integer *
*******************************************************************/

long int dec_to_long( dispnum )
int dispnum;
{
    int i;
    long int lnum;
    long int atol();


    /* truncate at the '.' if it exists */
    i = findch( '.', display[ dispnum ].string );
    if( i >= 0 ) display[ dispnum ].string[ i ] = '\0';

    /* convert to a long and set the negative flag if < zero */
    lnum = atol( display[ dispnum ].string );
    if( display[ dispnum ].negative == 1 ) lnum = -lnum;
    return( lnum );

}   /* End of - dec_to_long() */


/********************************************************
* hbo_to_long():                                        *
* This function converts a hex, binary, or octal string *
* to a long integer depending on the base value passed  *
********************************************************/

long int hbo_to_long( dispnum, base )
int dispnum;
int base;
{
    static int shift[] = { 0, 4, 1, 3 };
    int i;
    long int lnum;
    long int x, position;


    i = strlen( display[ dispnum ].string ) - 1;
    lnum = 0L;
    position = 1L;

    do{
      /* convert individual character to a long integer */
      x = (long int)findch( display[dispnum].string[i], "0123456789ABCDEF" );
      if( x < 0L ) break;

      lnum = lnum + ( x * position );
      position = position << shift[ base ];
      } while( i-- != 0 );

    return( lnum );

}   /* End of - hbo_to_long() */


/**********************************************************
* findch():                                               *
* This routine scans for a character in the string passed *
**********************************************************/

findch( chr, string )
char chr, string[];
{
    int pos = 0;

    do{ if( chr == string[ pos ] ) return( pos );
      } while( string[ ++pos ] );

    return(-1);

}   /* End of - findch() */


/************************************************************
* initialize():                                             *
* This routine initializes the key array and opens a window *
************************************************************/

initialize()
{
    int i, a, b;    /* miscellaneous variables */


    /* tell GEM we're here, and install our DESK item... */
    appl_init();
    handle = graf_handle( &gr_hwchar, &gr_hhchar, &i, &i );
    menu_id = menu_register( gl_apid, "  Calculator..." );

    numbase = DECIMAL;   /* set initial number base */
    active = 0;          /* set the active display  */
    wi_handle = -1;      /* set to no window open   */

    /* initialize the key position array... */

    boxw = gr_hwchar * 6;           /* width of calculator key   */
    boxh = gr_hhchar + 2;           /* height of calculator key  */
    xgap = gr_hwchar;               /* space between keys,x-axis */
    ygap = (gr_hhchar / 2) + 2;     /* space between keys,y-axis */

    disp_rect[0] = gr_hwchar;       /* x position of display box */
    disp_rect[1] = gr_hhchar / 2;   /* y position of display box */
    disp_rect[2] = gr_hwchar * 34;  /* width of the display box  */
    disp_rect[3] = boxh + 4;        /* height of the display box */

    /* calculate the X and Y positions for the remaining keys */
    for( a = 0; a < 7; a++ )
         {
         for( b = 0; b < 5; b++  )
              {
              key[ (a*5) + b ].x = xgap + (b * (boxw+xgap));
              key[ (a*5) + b ].y = (boxh*2) + 4 + (a * (boxh+ygap));
              }
         }

    /* calculate the initial window position and dimension */
    ax = 3 * gr_hwchar;
    ay = 2 * gr_hhchar;
    aw = key[ EQUALS ].x + boxw + xgap + 2;
    ah = key[ EQUALS ].y + (boxh*2) + ygap + 2;


    strcpy( display[0].string, "0" );
    display[0].negative = display[0].command = 0;

    strcpy( display[1].string, "0" );
    display[1].negative = display[1].command = 0;

}   /* End of - initialize() */


/*******************************************************
* open_vwork():                                        *
* This routine opens the workstation for the accessory *
*******************************************************/

open_vwork()
{
    int i;

    for( i = 0; i < 10; work_in[ i++ ] = 1 );
    work_in[ 2 ] = 2;
    v_opnvwk( work_in, &handle, work_out );
}


/*******************************************************
* open_window():                                       *
* This routine opens the window used for the accessory *
*******************************************************/

open_window()
{
    wi_handle = wind_create( WI_KIND, 0, 0, aw, ah );
    wind_set( wi_handle, WF_NAME, " Programmer's Calculator ", 0, 0 );
    graf_growbox(ax+aw/2,ay+ah/2,3*gr_hwchar,3*gr_hhchar,ax,ay,aw,ah);
    wind_open( wi_handle, ax, ay, aw, ah );
    get_workarea();
}


/**********************************************************
* set_clip():                                             *
* This routine enables or disables the clipping rectangle *
**********************************************************/

set_clip( x, y, w, h, flag )
int x, y, w, h, flag;
{
    int clip[4];

    clip[0] = x;
    clip[1] = y;
    clip[2] = x + w;
    clip[3] = y + h;

    vs_clip( handle, flag, clip );
}

